@@ -15,6 +15,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+// TODO: Clear Q subchannel data on disc change and manual track change, add Q subchannel mode have variable(s).
+
 //#include <mednafen/mednafen.h>
 #include "mednafen.h"
 #include "cdrom/cdromif.h"
@@ -31,8 +33,7 @@ using namespace CDUtility;
 namespace MDFN_IEN_CDPLAY
 {
 
-static std::vector<double> sqrt_lut; //[65536];
-static std::vector<double> sin_lut; //[65536];
+static std::vector<float> sin_lut; //[65536];
 static SpeexResamplerState *resampler = NULL;
 
 static uint8 *controller_ptr;
@@ -48,13 +49,12 @@ enum
 };
 
 static int PlayMode;
-static uint32 PlaySector;
+static int32 PlaySector;
 static int16 CDDABuffer[588 * 2];
 
 static int16 ResampBuffer[588 * 2][2];	// Resampler input buffer, * 2 for resampler leftovers
 static uint32 ResampBufferPos;
 static uint32 PrevRate;
-static unsigned int CurrentDisc;
 
 static std::vector<CDIF *> *cdifs;
 
@@ -116,8 +116,6 @@ static int LoadCD(std::vector<CDIF *> *CDInterfaces)
   resampler = speex_resampler_init(2, 44100, (int)48000, 5, &err);
   PrevRate = 48000;
  }
- //resampler.buffer_size(588 * 2 + 100);
- //resampler.time_ratio((double)44100 / 48000, 0.9965);
  ResampBufferPos = 0;
 
  InitLUT();
@@ -150,10 +148,11 @@ static void CloseGame(void)
   resampler = NULL;
  }
  sin_lut.resize(0);
- sqrt_lut.resize(0);
 }
 
 static uint8 SubQBuf[3][0xC];
+//static bool SubQHave[3];
+static uint8 SubQBuf_LastValid[0xC];
 
 static void GenSubQFromSubPW(uint8 *SubPWBuf)
 {
@@ -172,21 +171,19 @@ static void GenSubQFromSubPW(uint8 *SubPWBuf)
 
   if(adr <= 0x3)
    memcpy(SubQBuf[adr], sq, 0xC);
+
+  memcpy(SubQBuf_LastValid, sq, 0xC);
  }
 }
 static const int lobes = 2;
-static const int oversample_shift = 7;	//1; //7;
+static const int oversample_shift = 5;	//1; //7;
 static const int oversample = 1 << oversample_shift;
 static const int oversample_mo = oversample - 1;
 
 static void InitLUT(void)
 {
- sqrt_lut.resize(65536);
  sin_lut.resize(65536);
 
- for(int i = 0; i < 65536; i++)
-  sqrt_lut[i] = sqrt((double)i / 65536);
-
  for(int i = 0; i < 65536; i++)
   sin_lut[i] = sin((double)i * M_PI * 2 / 65536);
 }
@@ -200,7 +197,21 @@ static void Emulate(EmulateSpecStruct *espec)
 
  //printf("%d %d\n", toc.tracks[100].lba, AudioTrackList[AudioTrackList.size() - 1] + 1);
 
- if(PlaySector > AudioTrackList[CurrentATLI].final_lba)
+ if(PlaySector < AudioTrackList[CurrentATLI].lba)	// Reverse-scanning handling.
+ {
+  if(CurrentATLI > 0)
+  {
+   CurrentATLI--;
+   PlaySector = AudioTrackList[CurrentATLI].final_lba;
+  }
+  else
+  {
+   CurrentATLI = 0;
+   PlayMode = PLAYMODE_STOP;
+   PlaySector = AudioTrackList[CurrentATLI].lba;
+  }
+ }
+ else if(PlaySector > AudioTrackList[CurrentATLI].final_lba)
  {
   if((CurrentATLI + 1) < AudioTrackList.size())
    CurrentATLI++;
@@ -208,11 +219,11 @@ static void Emulate(EmulateSpecStruct *espec)
   {
    CurrentATLI = 0;
    PlayMode = PLAYMODE_STOP;
-  }   
+  }
 
   PlaySector = AudioTrackList[CurrentATLI].lba;
  }
- 
+
  if(PlayMode == PLAYMODE_STOP || PlayMode == PLAYMODE_PAUSE)
  {
   //memset(CDDABuffer, 0, sizeof(CDDABuffer));
@@ -274,6 +285,9 @@ static void Emulate(EmulateSpecStruct *espec)
    espec->SoundBufSize = out_len;
   }
  }
+ else
+  ResampBufferPos = 0;
+
 // for(int i = 0; i < espec->SoundBufSize * 2; i++)
 //  espec->SoundBuf[i] = (rand() & 0x7FFF) - 0x4000;	//(rand() * 192) >> 8
 
@@ -284,28 +298,19 @@ static void Emulate(EmulateSpecStruct *espec)
   uint32 *pixels = espec->surface->pixels;
   uint32 text_color = format.MakeColor(0xE0, 0xE0, 0xE0);
   uint32 text_shadow_color = format.MakeColor(0x20, 0x20, 0x20);
+  uint32 wf_color = format.MakeColor(0xE0, 0x00, 0xE0);
   uint32 cur_sector = PlaySector;
 
   espec->DisplayRect.x = 0;
   espec->DisplayRect.y = 0;
 
-  espec->DisplayRect.w = 320;
-  espec->DisplayRect.h = 240;
+  espec->DisplayRect.w = 192;
+  espec->DisplayRect.h = 144;
 
   espec->surface->Fill(0, 0, 0, 0);
 
+  if(MDFN_GetSettingB("cdplay.visualization"))
   {
-   uint32 color_table[256];
-
-   for(int i = 0; i < 170; i++)
-   {
-    int m = 255 * i / 170;
-
-    color_table[i] = format.MakeColor(m, 0, m);
-
-    //printf("%d %d %08x\n", m, i, color_table[i]);
-   }
-
   for(int i = 0; i < 588; i++)
   {
    int32 unip_samp;
@@ -316,20 +321,20 @@ static void Emulate(EmulateSpecStruct *espec)
 
    for(int osi = 0; osi < oversample; osi++)
    {
-    double sample;
+    float sample;
     int x;
     int y;
-    double x_raw, y_raw;
-    double x_raw2, y_raw2;
-    double x_raw_prime, y_raw_prime;
+    float x_raw, y_raw;
+    float x_raw2, y_raw2;
+    float x_raw_prime, y_raw_prime;
 
-    sample = (double)(unip_samp * (oversample - osi) + next_unip_samp * osi) / (oversample * 65536);
+    sample = (float)(unip_samp * (oversample - osi) + next_unip_samp * osi) / (oversample * 65536);
 
     int32 theta_i = (int64)65536 * (i * oversample + osi) / (oversample * 588);
     int32 theta_i_alt = (int64)65536 * (i * oversample + osi) / (oversample * 588);
 
-    double radius = sin_lut[(lobes * theta_i) & 0xFFFF];	//	 * sin_lut[(16384 + (theta_i)) & 0xFFFF];
-    double radius2 = sin_lut[(lobes * (theta_i + 1)) & 0xFFFF];	// * sin_lut[(16384 + ((theta_i + 1))) & 0xFFFF];
+    float radius = sin_lut[(lobes * theta_i) & 0xFFFF];
+    float radius2 = sin_lut[(lobes * (theta_i + 1)) & 0xFFFF];
 
     x_raw = radius * sin_lut[(16384 + theta_i_alt) & 0xFFFF];
     y_raw = radius * sin_lut[theta_i_alt & 0xFFFF];
@@ -341,7 +346,7 @@ static void Emulate(EmulateSpecStruct *espec)
     x_raw_prime = (x_raw2 - x_raw) / (1 * M_PI * 2 / 65536);
     y_raw_prime = (y_raw2 - y_raw) / (1 * M_PI * 2 / 65536);
 
-//    printf("%f %f\n", y_raw_prime, sin_lut[(16384 + lobes * theta_i_alt) & 0xFFFF] + sin_lut[(16384 + theta_i_alt) & 0xFFFF]);
+    //printf("%f %f\n", y_raw_prime, sin_lut[(16384 + lobes * theta_i_alt) & 0xFFFF] + sin_lut[(16384 + theta_i_alt) & 0xFFFF]);
 
     if(x_raw_prime || y_raw_prime)
     {
@@ -352,63 +357,91 @@ static void Emulate(EmulateSpecStruct *espec)
     x_raw += (sample - 0.5) * y_raw_prime / 2;
     y_raw += (sample - 0.5) * -x_raw_prime / 2;
 
-    x = 160 + 100 * x_raw;
-    y = 120 + 100 * y_raw;
+    x = 96 + 60 * x_raw;
+    y = 72 + 60 * y_raw;
 
-    if((x >= 0 && x < 320) && (y >= 0 && y < 240))
-     pixels[x + y * espec->surface->pitch32] = format.MakeColor(255, 255, 255);	//color_table[(int)(sample * 150)];	//x * x + y * y; //format.MakeColor(sample * 255, 0, sample * 255);
+    if((x >= 0 && x < 192) && (y >= 0 && y < 144))
+     pixels[x + y * espec->surface->pitch32] = wf_color;
    }
   }
   }
 
   {
    TOC toc;
+   const char *disctype_string = "";
 
    (*cdifs)[AudioTrackList[CurrentATLI].disc]->ReadTOC(&toc);
 
-   trio_snprintf(tmpbuf, 256, "Disc: %d/%d", AudioTrackList[CurrentATLI].disc + 1, cdifs->size());
-   DrawTextTransShadow(pixels, espec->surface->pitch32 * 4, 320, (UTF8 *)tmpbuf, text_color, text_shadow_color, 0, MDFN_FONT_9x18_18x18);
-   pixels += 22 * espec->surface->pitch32;
+   if(toc.disc_type == 0x10)
+    disctype_string = "(CD-i)";
+   else if(toc.disc_type == 0x20)
+    disctype_string = "(CD-ROM XA)";
+   else if(toc.disc_type != 0x00)
+    disctype_string = "(unknown type)";
+
+   trio_snprintf(tmpbuf, 256, "Disc: %d/%d %s", AudioTrackList[CurrentATLI].disc + 1, cdifs->size(), disctype_string);
+   DrawTextTransShadow(pixels, espec->surface->pitch32 * 4, 192, (UTF8 *)tmpbuf, text_color, text_shadow_color, 0, MDFN_FONT_6x13_12x13);
+   pixels += 13 * espec->surface->pitch32;
 
    trio_snprintf(tmpbuf, 256, "Track: %d/%d", AudioTrackList[CurrentATLI].track, toc.last_track);
-   DrawTextTransShadow(pixels, espec->surface->pitch32 * 4, 320, (UTF8 *)tmpbuf, text_color, text_shadow_color, 0, MDFN_FONT_9x18_18x18);
-   pixels += 22 * espec->surface->pitch32;
+   DrawTextTransShadow(pixels, espec->surface->pitch32 * 4, 192, (UTF8 *)tmpbuf, text_color, text_shadow_color, 0, MDFN_FONT_6x13_12x13);
+   pixels += 13 * espec->surface->pitch32;
 
    trio_snprintf(tmpbuf, 256, "Sector: %d/%d", cur_sector, toc.tracks[100].lba - 1);
-   DrawTextTransShadow(pixels, espec->surface->pitch32 * 4, 320, (UTF8 *)tmpbuf, text_color, text_shadow_color, 0, MDFN_FONT_9x18_18x18);
-   pixels += 22 * espec->surface->pitch32;
+   DrawTextTransShadow(pixels, espec->surface->pitch32 * 4, 192, (UTF8 *)tmpbuf, text_color, text_shadow_color, 0, MDFN_FONT_6x13_12x13);
+   pixels += 13 * espec->surface->pitch32;
   }
 
-  pixels += 22 * espec->surface->pitch32;
+  pixels += 13 * espec->surface->pitch32;
 
 
-  DrawTextTransShadow(pixels, espec->surface->pitch32 * 4, 320, (UTF8 *)"SubQ", text_color, text_shadow_color, 0, MDFN_FONT_9x18_18x18);
-  pixels += 22 * espec->surface->pitch32;
-
   //trio_snprintf(tmpbuf, 256, "Q-Mode: %01x", SubQBuf[1][0] & 0xF);
-  //DrawTextTransShadow(pixels, espec->surface->pitch32 * 4, 320, (UTF8 *)tmpbuf, text_color, text_shadow_color, 0, MDFN_FONT_9x18_18x18);
-  //pixels += 22 * espec->surface->pitch32;
+  //DrawTextTransShadow(pixels, espec->surface->pitch32 * 4, 192, (UTF8 *)tmpbuf, text_color, text_shadow_color, 0, MDFN_FONT_6x13_12x13);
+  //pixels += 13 * espec->surface->pitch32;
 
   trio_snprintf(tmpbuf, 256, "Track: %d", BCD_to_U8(SubQBuf[1][1]));
-  DrawTextTransShadow(pixels, espec->surface->pitch32 * 4, 320, (UTF8 *)tmpbuf, text_color, text_shadow_color, 0, MDFN_FONT_9x18_18x18);
-  pixels += 22 * espec->surface->pitch32;
+  DrawTextTransShadow(pixels, espec->surface->pitch32 * 4, 192, (UTF8 *)tmpbuf, text_color, text_shadow_color, 0, MDFN_FONT_6x13_12x13);
+  pixels += 13 * espec->surface->pitch32;
 
   trio_snprintf(tmpbuf, 256, "Index: %d", BCD_to_U8(SubQBuf[1][2]));
-  DrawTextTransShadow(pixels, espec->surface->pitch32 * 4, 320, (UTF8 *)tmpbuf, text_color, text_shadow_color, 0, MDFN_FONT_9x18_18x18);
-  pixels += 22 * espec->surface->pitch32;
+  DrawTextTransShadow(pixels, espec->surface->pitch32 * 4, 192, (UTF8 *)tmpbuf, text_color, text_shadow_color, 0, MDFN_FONT_6x13_12x13);
+  pixels += 13 * espec->surface->pitch32;
 
 
   trio_snprintf(tmpbuf, 256, "Relative: %02d:%02d:%02d", BCD_to_U8(SubQBuf[1][3]), BCD_to_U8(SubQBuf[1][4]), BCD_to_U8(SubQBuf[1][5]));
-  DrawTextTransShadow(pixels, espec->surface->pitch32 * 4, 320, (UTF8 *)tmpbuf, text_color, text_shadow_color, 0, MDFN_FONT_9x18_18x18);
-  pixels += 22 * espec->surface->pitch32;
+  DrawTextTransShadow(pixels, espec->surface->pitch32 * 4, 192, (UTF8 *)tmpbuf, text_color, text_shadow_color, 0, MDFN_FONT_6x13_12x13);
+  pixels += 13 * espec->surface->pitch32;
 
   trio_snprintf(tmpbuf, 256, "Absolute: %02d:%02d:%02d", BCD_to_U8(SubQBuf[1][7]), BCD_to_U8(SubQBuf[1][8]), BCD_to_U8(SubQBuf[1][9]));
-  DrawTextTransShadow(pixels, espec->surface->pitch32 * 4, 320, (UTF8 *)tmpbuf, text_color, text_shadow_color, 0, MDFN_FONT_9x18_18x18);
-  pixels += 22 * espec->surface->pitch32;
+  DrawTextTransShadow(pixels, espec->surface->pitch32 * 4, 192, (UTF8 *)tmpbuf, text_color, text_shadow_color, 0, MDFN_FONT_6x13_12x13);
+  pixels += 13 * espec->surface->pitch32;
+
+  trio_snprintf(tmpbuf, 256, "Control: 0x%02x", (SubQBuf_LastValid[0] >> 4) & 0xF);
+  DrawTextTransShadow(pixels, espec->surface->pitch32 * 4, 192, (UTF8 *)tmpbuf, text_color, text_shadow_color, 0, MDFN_FONT_6x13_12x13);
+  pixels += 13 * espec->surface->pitch32;
+
+
+  // Catalog
+  trio_snprintf(tmpbuf, 256, "Catalog: %x%x%x%x%x%x%x%x%x%x%x%x%x",
+						(SubQBuf[2][1] >> 4) & 0xF, (SubQBuf[2][1] >> 0) & 0xF, (SubQBuf[2][2] >> 4) & 0xF, (SubQBuf[2][2] >> 0) & 0xF,
+					        (SubQBuf[2][3] >> 4) & 0xF, (SubQBuf[2][3] >> 0) & 0xF, (SubQBuf[2][4] >> 4) & 0xF, (SubQBuf[2][4] >> 0) & 0xF,
+					        (SubQBuf[2][5] >> 4) & 0xF, (SubQBuf[2][5] >> 0) & 0xF, (SubQBuf[2][6] >> 4) & 0xF, (SubQBuf[2][6] >> 0) & 0xF,
+						(SubQBuf[2][7] >> 4) & 0xF);
+  DrawTextTransShadow(pixels, espec->surface->pitch32 * 4, 192, (UTF8 *)tmpbuf, text_color, text_shadow_color, 0, MDFN_FONT_6x13_12x13);
+  pixels += 13 * espec->surface->pitch32;
  }
 
  if(PlayMode != PLAYMODE_STOP && PlayMode != PLAYMODE_PAUSE)
-  PlaySector++;
+ {
+  const int scan_amount = 4; //16;
+
+  if(new_controller & 0x40)
+   PlaySector += scan_amount;
+  else if(new_controller & 0x80)
+   PlaySector -= scan_amount;
+  else
+   PlaySector++;
+ }
 
  if(!(last_controller & 0x1) && (new_controller & 1))
  {
@@ -474,6 +507,7 @@ static void DoSimpleCommand(int cmd)
 
 static MDFNSetting CDPlaySettings[] =
 {
+ { "cdplay.visualization", MDFNSF_NOFLAGS, gettext_noop("Enable simple waveform visualization."), NULL, MDFNST_BOOL, "1" },
  { NULL }
 };
 
@@ -502,6 +536,7 @@ static InputDeviceInfoStruct InputDeviceInfo[] =
   "controller",
   "Controller",
   NULL,
+  NULL,
   sizeof(IDII) / sizeof(InputDeviceInputInfoStruct),
   IDII,
  }
@@ -509,7 +544,7 @@ static InputDeviceInfoStruct InputDeviceInfo[] =
 
 static const InputPortInfoStruct PortInfo[] =
 {
- { 0, "builtin", "Built-In", sizeof(InputDeviceInfo) / sizeof(InputDeviceInfoStruct), InputDeviceInfo }
+ { "builtin", "Built-In", sizeof(InputDeviceInfo) / sizeof(InputDeviceInfoStruct), InputDeviceInfo }
 };
 
 static InputInfoStruct InputInfo =
@@ -535,10 +570,14 @@ MDFNGI EmulatedCDPlay =
  TestMagicCD,
  CloseGame,
  NULL,
- "",            // Layer names, null-delimited
+ NULL,            // Layer names, null-delimited
  NULL,
  NULL,
  NULL,
+ NULL,
+ NULL,
+ NULL,
+ false,
  NULL, //StateAction,
  Emulate,
  SetInput,
@@ -548,15 +587,16 @@ MDFNGI EmulatedCDPlay =
  75 * 65536 * 256,
  false, // Multires possible?
 
- 320,   // lcm_width
- 240,   // lcm_height           
+ 192,   // lcm_width
+ 144,   // lcm_height
  NULL,  // Dummy
 
 
- 320,   // Nominal width
- 240,    // Nominal height
- 512, 			// Framebuffer width
- 256,                  	// Framebuffer height
+ 192,   // Nominal width
+ 144,    // Nominal height
+
+ 192, 			// Framebuffer width
+ 144 + 1,                  	// Framebuffer height
 
  2,     // Number of output sound channels
 };
