@@ -30,7 +30,6 @@
 	Trying to read sectors at an LBA of less than 0 is not supported.  TODO: support it(at least up to -150).
 */
 
-#define _CDROMFILE_INTERNAL
 #include "../mednafen.h"
 
 #include <sys/types.h>
@@ -42,8 +41,10 @@
 #include <trio/trio.h>
 
 #include "../general.h"
+#include "../string/trim.h"
 #include "../endian.h"
-#include "../FileWrapper.h"
+#include "../FileStream.h"
+#include "../MemoryStream.h"
 
 #include "CDAccess.h"
 #include "CDAccess_Image.h"
@@ -109,125 +110,128 @@ static const char *DI_CUE_Strings[7] =
  "MODE2/2352"
 };
 
-static char *UnQuotify(char *src, char *dest)
+// Should return an offset to the start of the next argument(past any whitespace), or if there isn't a next argument,
+// it'll return the length of the src string.
+static size_t UnQuotify(const std::string &src, size_t source_offset, std::string &dest, bool parse_quotes = true)
 {
+ const size_t source_len = src.length();
  bool in_quote = 0;
  bool already_normal = 0;
 
- while(*src)
+ dest.clear();
+
+ while(source_offset < source_len)
  {
-  if(*src == ' ' || *src == '\t')
+  if(src[source_offset] == ' ' || src[source_offset] == '\t')
   {
    if(!in_quote)
    {
-    if(already_normal)
+    if(already_normal)	// Trailing whitespace(IE we're done with this argument)
      break;
-    else
+    else		// Leading whitespace, ignore it.
     {
-     src++;
+     source_offset++;
      continue;
     }
    }
   }
 
-  if(*src == '"')
+  if(src[source_offset] == '"' && parse_quotes)
   {
    if(in_quote)
    {
-    src++;
+    source_offset++;
+// Not sure which behavior is most useful(or correct :b).
+#if 0
+    in_quote = false;
+    already_normal = true;
+#else
     break;
+#endif
    }
    else
     in_quote = 1;
   }
   else
   {
-   *dest = *src;
+   dest.push_back(src[source_offset]);
    already_normal = 1;
-   dest++;
   }
-  src++;
+  source_offset++;
  }
 
- *dest = 0;
- return(src);
+ while(source_offset < source_len)
+ {
+  if(src[source_offset] != ' ' && src[source_offset] != '\t')
+   break;
+
+  source_offset++;
+ }
+
+ return source_offset;
 }
 
 uint32 CDAccess_Image::GetSectorCount(CDRFILE_TRACK_INFO *track)
 {
- // - track->FileOffset is really only meaningful for TOC files
- // ...and the last track for CUE/BIN
  if(track->DIFormat == DI_FORMAT_AUDIO)
  {
   if(track->AReader)
    return(((track->AReader->FrameCount() * 4) - track->FileOffset) / 2352);
   else
   {
-   struct stat stat_buf;
-   fstat(fileno(track->fp), &stat_buf);
+   const int64 size = track->fp->size();
 
    //printf("%d %d %d\n", (int)stat_buf.st_size, (int)track->FileOffset, (int)stat_buf.st_size - (int)track->FileOffset);
    if(track->SubchannelMode)
-    return((stat_buf.st_size - track->FileOffset) / (2352 + 96));
+    return((size - track->FileOffset) / (2352 + 96));
    else
-    return((stat_buf.st_size - track->FileOffset) / 2352);
+    return((size - track->FileOffset) / 2352);
   }
  }
  else
  {
-  struct stat stat_buf;
-
-  if(fstat(fileno(track->fp), &stat_buf))
-  {
-   ErrnoHolder ene(errno);
-
-   throw MDFN_Error(ene.Errno(), "%s", ene.StrError());
-  }
-
-  return((stat_buf.st_size - track->FileOffset) / DI_Size_Table[track->DIFormat]);
+  const int64 size = track->fp->size();
+  
+  return((size - track->FileOffset) / DI_Size_Table[track->DIFormat]);
  }
 
  return(0);
 }
 
-CDAccess_Image::~CDAccess_Image()
-{
- for(int32 track = FirstTrack; track < (FirstTrack + NumTracks); track++)
- {
-  CDRFILE_TRACK_INFO *this_track = &Tracks[track];
-
-  if(this_track->FirstFileInstance)
-  {
-   if(Tracks[track].AReader)
-   {
-    delete Tracks[track].AReader;
-    Tracks[track].AReader = NULL;
-   }
-   else if(this_track->fp)
-    fclose(this_track->fp);
-  }
- }
-}
-
-void CDAccess_Image::ParseTOCFileLineInfo(CDRFILE_TRACK_INFO *track, const int tracknum, const char *filename, const char *binoffset, const char *msfoffset, const char *length)
+void CDAccess_Image::ParseTOCFileLineInfo(CDRFILE_TRACK_INFO *track, const int tracknum, const std::string &filename, const char *binoffset, const char *msfoffset, const char *length, bool image_memcache, std::map<std::string, Stream*> &toc_streamcache)
 {
  long offset = 0; // In bytes!
  long tmp_long;
  int m, s, f;
  uint32 sector_mult;
  long sectors;
- std::string efn;
+ std::map<std::string, Stream*>::iterator ribbit;
 
- efn = MDFN_EvalFIP(base_dir, filename);
+ ribbit = toc_streamcache.find(filename);
 
- if(NULL == (track->fp = fopen(efn.c_str(), "rb")))
+ if(ribbit != toc_streamcache.end())
  {
-  ErrnoHolder ene(errno);
+  track->FirstFileInstance = 0;
 
-  throw MDFN_Error(ene.Errno(), _("Could not open referenced file \"%s\": %s\n"), efn.c_str(), ene.StrError());
+  track->fp = ribbit->second;
+ }
+ else
+ {
+  std::string efn;
+
+  track->FirstFileInstance = 1;
+
+  efn = MDFN_EvalFIP(base_dir, filename);
+
+  if(image_memcache)
+   track->fp = new MemoryStream(new FileStream(efn.c_str(), FileStream::MODE_READ));
+  else
+   track->fp = new FileStream(efn.c_str(), FileStream::MODE_READ);
+
+  toc_streamcache[filename] = track->fp;
  }
 
- if(strlen(filename) >= 4 && !strcasecmp(filename + strlen(filename) - 4, ".wav"))
+ if(filename.length() >= 4 && !strcasecmp(filename.c_str() + filename.length() - 4, ".wav"))
  {
   track->AReader = AR_Open(track->fp);
 
@@ -283,82 +287,137 @@ void CDAccess_Image::ParseTOCFileLineInfo(CDRFILE_TRACK_INFO *track, const int t
   sectors = tmp_long;
  }
 
- track->FirstFileInstance = 1;
  track->sectors = sectors;
 }
 
 
-
-void CDAccess_Image::ImageOpen(const char *path)
+static void MDFN_strtoupper(char *str)
 {
- FileWrapper fp(path, FileWrapper::MODE_READ);
+ for(size_t x = 0; str[x]; x++)
+ {
+  if(str[x] >= 'a' && str[x] <= 'z')
+  {
+   str[x] = str[x] - 'a' + 'A';
+  }
+ }
+}
+
+static void MDFN_strtoupper(std::string &str)
+{
+ const size_t len = str.length();
+
+ for(size_t x = 0; x < len; x++)
+ {
+  if(str[x] >= 'a' && str[x] <= 'z')
+  {
+   str[x] = str[x] - 'a' + 'A';
+  }
+ }
+}
+
+#if 0
+std::string MDFN_toupper(const std::string &str)
+{
+ const size_t len = str.length();
+ std::string new_str;
+
+ new_str.reserve(len);
+
+ for(size_t x = 0; x < len; x++)
+ {
+  int c = str[x];
+
+  if(c >= 'a' && c <= 'z')
+   c = c - 'a' + 'A';
+
+  new_str.push_back(c);
+ }
+}
+#endif
+
+void CDAccess_Image::ImageOpen(const char *path, bool image_memcache)
+{
+ MemoryStream fp(new FileStream(path, FileStream::MODE_READ));
+ static const unsigned max_args = 4;
+ std::string linebuf;
+ std::string cmdbuf, args[max_args];
  bool IsTOC = FALSE;
- char linebuf[512];
  int32 active_track = -1;
  int32 AutoTrackInc = 1; // For TOC
  CDRFILE_TRACK_INFO TmpTrack;
  std::string file_base, file_ext;
+ std::map<std::string, Stream*> toc_streamcache;
 
+ disc_type = DISC_TYPE_CDDA_OR_M1;
  memset(&TmpTrack, 0, sizeof(TmpTrack));
 
  MDFN_GetFilePathComponents(path, &base_dir, &file_base, &file_ext);
 
  if(!strcasecmp(file_ext.c_str(), ".toc"))
  {
-  puts("TOC file detected.");
+  MDFN_printf(_("TOC file detected.\n"));
   IsTOC = true;
  }
+
+ // Check for annoying UTF-8 BOM.
+ if(!IsTOC)
+ {
+  uint8 bom_tmp[3];
+
+  if(fp.read(bom_tmp, 3, false) == 3 && bom_tmp[0] == 0xEF && bom_tmp[1] == 0xBB && bom_tmp[2] == 0xBF)
+  {
+   // Print an annoying error message, but don't actually error out.
+   MDFN_PrintError(_("UTF-8 BOM detected at start of CUE sheet."));
+  }
+  else
+   fp.seek(0, SEEK_SET);
+ }
+
+
  // Assign opposite maximum values so our tests will work!
  FirstTrack = 99;
  LastTrack = 0;
 
- while(fp.get_line(linebuf, 512) != NULL)
+ linebuf.reserve(1024);
+ while(fp.get_line(linebuf) >= 0)
  {
-   char cmdbuf[512], raw_args[512], args[4][512];
-   int argcount = 0;
-
-   raw_args[0] = 0;
-   cmdbuf[0] = 0;
-
-   args[0][0] = args[1][0] = args[2][0] = args[3][0] = 0;
-
-   MDFN_trim(linebuf);
+   unsigned argcount = 0;
 
    if(IsTOC)
    {
     // Handle TOC format comments
-    char *ss_loc = strstr(linebuf, "//");
-    if(ss_loc)
-    {
-     ss_loc[0] = '\n';
-     ss_loc[1] = 0;
-    }
+    size_t ss_loc = linebuf.find("//");
+
+    if(ss_loc != std::string::npos)
+     linebuf.resize(ss_loc);
    }
 
-   if(trio_sscanf(linebuf, "%s %[^\r\n]", cmdbuf, raw_args) < 1)
-    continue;	// Skip blank lines
-   
-   UnQuotify(UnQuotify(UnQuotify(UnQuotify(raw_args, args[0]), args[1]), args[2]), args[3]);
-   if(args[0][0])
+   // Call trim AFTER we handle TOC-style comments, so we'll be sure to remove trailing whitespace in lines like: MONKEY  // BABIES
+   MDFN_trim(linebuf);
+
+   if(linebuf.length() == 0)	// Skip blank lines.
+    continue;
+
+   // Grab command and arguments.
    {
-    argcount++;
-    if(args[1][0])
-    {
-     argcount++;
-     if(args[2][0])
-     {
-      argcount++;
-      if(args[3][0])
-      {
-       argcount++;
-      }
-     }
-    } 
+    size_t offs = 0;
+
+    offs = UnQuotify(linebuf, offs, cmdbuf, false);
+    for(argcount = 0; argcount < max_args && offs < linebuf.length(); argcount++)
+     offs = UnQuotify(linebuf, offs, args[argcount]);
+
+    // Make sure unused arguments are cleared out so we don't have inter-line leaks!
+    for(unsigned x = argcount; x < max_args; x++)
+     args[x].clear();
+
+    MDFN_strtoupper(cmdbuf);
    }
 
+   //printf("%s\n", cmdbuf.c_str()); //: %s %s %s %s\n", cmdbuf.c_str(), args[0].c_str(), args[1].c_str(), args[2].c_str(), args[3].c_str());
+
    if(IsTOC)
    {
-    if(!strcasecmp(cmdbuf, "TRACK"))
+    if(cmdbuf == "TRACK")
     {
      if(active_track >= 0)
      {
@@ -381,7 +440,7 @@ void CDAccess_Image::ImageOpen(const char *path)
      int format_lookup;
      for(format_lookup = 0; format_lookup < _DI_FORMAT_COUNT; format_lookup++)
      {
-      if(!strcasecmp(args[0], DI_CDRDAO_Strings[format_lookup]))
+      if(!strcasecmp(args[0].c_str(), DI_CDRDAO_Strings[format_lookup]))
       {
        TmpTrack.DIFormat = format_lookup;
        break;
@@ -390,92 +449,142 @@ void CDAccess_Image::ImageOpen(const char *path)
 
      if(format_lookup == _DI_FORMAT_COUNT)
      {
-      throw(MDFN_Error(0, _("Invalid track format: %s"), args[0]));
+      throw(MDFN_Error(0, _("Invalid track format: %s"), args[0].c_str()));
      }
 
      if(TmpTrack.DIFormat == DI_FORMAT_AUDIO)
       TmpTrack.RawAudioMSBFirst = TRUE; // Silly cdrdao...
 
-     if(!strcasecmp(args[1], "RW"))
+     if(!strcasecmp(args[1].c_str(), "RW"))
      {
       TmpTrack.SubchannelMode = CDRF_SUBM_RW;
       throw(MDFN_Error(0, _("\"RW\" format subchannel data not supported, only \"RW_RAW\" is!")));
      }
-     else if(!strcasecmp(args[1], "RW_RAW"))
+     else if(!strcasecmp(args[1].c_str(), "RW_RAW"))
       TmpTrack.SubchannelMode = CDRF_SUBM_RW_RAW;
 
     } // end to TRACK
-    else if(!strcasecmp(cmdbuf, "SILENCE"))
+    else if(cmdbuf == "SILENCE")
     {
-
+     //throw MDFN_Error(0, _("Unsupported directive: %s"), cmdbuf.c_str());
     }
-    else if(!strcasecmp(cmdbuf, "ZERO"))
+    else if(cmdbuf == "ZERO")
     {
-
+     //throw MDFN_Error(0, _("Unsupported directive: %s"), cmdbuf.c_str());
     }
-    else if(!strcasecmp(cmdbuf, "FILE") || !strcasecmp(cmdbuf, "AUDIOFILE"))
+    else if(cmdbuf == "FIFO")
+    {
+     throw MDFN_Error(0, _("Unsupported directive: %s"), cmdbuf.c_str());
+    }
+    else if(cmdbuf == "FILE" || cmdbuf == "AUDIOFILE")
     {
      const char *binoffset = NULL;
      const char *msfoffset = NULL;
      const char *length = NULL;
 
-     if(args[1][0] == '#')
+     if(args[1].c_str()[0] == '#')
      {
-      binoffset = args[1] + 1;
-      msfoffset = args[2];
-      length = args[3];
+      binoffset = args[1].c_str() + 1;
+      msfoffset = args[2].c_str();
+      length = args[3].c_str();
      }
      else
      {
-      msfoffset = args[1];
-      length = args[2];
+      msfoffset = args[1].c_str();
+      length = args[2].c_str();
      }
-     //printf("%s, %s, %s, %s\n", args[0], binoffset, msfoffset, length);
-     ParseTOCFileLineInfo(&TmpTrack, active_track, args[0], binoffset, msfoffset, length);
+     //printf("%s, %s, %s, %s\n", args[0].c_str(), binoffset, msfoffset, length);
+     ParseTOCFileLineInfo(&TmpTrack, active_track, args[0], binoffset, msfoffset, length, image_memcache, toc_streamcache);
     }
-    else if(!strcasecmp(cmdbuf, "DATAFILE"))
+    else if(cmdbuf == "DATAFILE")
     {
      const char *binoffset = NULL;
      const char *length = NULL;
   
-     if(args[1][0] == '#') 
+     if(args[1].c_str()[0] == '#') 
      {
-      binoffset = args[1] + 1;
-      length = args[2];
+      binoffset = args[1].c_str() + 1;
+      length = args[2].c_str();
      }
      else
-      length = args[1];
+      length = args[1].c_str();
 
-     ParseTOCFileLineInfo(&TmpTrack, active_track, args[0], binoffset, NULL, length);
+     ParseTOCFileLineInfo(&TmpTrack, active_track, args[0], binoffset, NULL, length, image_memcache, toc_streamcache);
     }
-    else if(!strcasecmp(cmdbuf, "INDEX"))
+    else if(cmdbuf == "INDEX")
     {
 
     }
-    else if(!strcasecmp(cmdbuf, "PREGAP"))
+    else if(cmdbuf == "PREGAP")
     {
      if(active_track < 0)
      {
-      throw(MDFN_Error(0, _("Command %s is outside of a TRACK definition!\n"), cmdbuf));
+      throw(MDFN_Error(0, _("Command %s is outside of a TRACK definition!\n"), cmdbuf.c_str()));
      }
      int m,s,f;
-     trio_sscanf(args[0], "%d:%d:%d", &m, &s, &f);
+     trio_sscanf(args[0].c_str(), "%d:%d:%d", &m, &s, &f);
      TmpTrack.pregap = (m * 60 + s) * 75 + f;
     } // end to PREGAP
-    else if(!strcasecmp(cmdbuf, "START"))
+    else if(cmdbuf == "START")
     {
      if(active_track < 0)
      {
-      throw(MDFN_Error(0, _("Command %s is outside of a TRACK definition!\n"), cmdbuf));
+      throw(MDFN_Error(0, _("Command %s is outside of a TRACK definition!\n"), cmdbuf.c_str()));
      }
      int m,s,f;
-     trio_sscanf(args[0], "%d:%d:%d", &m, &s, &f);
+     trio_sscanf(args[0].c_str(), "%d:%d:%d", &m, &s, &f);
      TmpTrack.pregap = (m * 60 + s) * 75 + f;
     }
+    else if(cmdbuf == "TWO_CHANNEL_AUDIO")
+    {
+     TmpTrack.subq_control &= ~SUBQ_CTRLF_4CH;
+    }
+    else if(cmdbuf == "FOUR_CHANNEL_AUDIO")
+    {
+     TmpTrack.subq_control |= SUBQ_CTRLF_4CH;
+    }
+    else if(cmdbuf == "NO")
+    {
+     MDFN_strtoupper(args[0]);
+
+     if(args[0] == "COPY")
+     {
+      TmpTrack.subq_control &= ~SUBQ_CTRLF_DCP; 
+     }
+     else if(args[0] == "PRE_EMPHASIS")
+     {
+      TmpTrack.subq_control &= ~SUBQ_CTRLF_PRE;
+     }
+     else
+     {
+      throw MDFN_Error(0, _("Unsupported argument to \"NO\" directive: %s"), args[0].c_str());
+     }
+    }
+    else if(cmdbuf == "COPY")
+    {
+     TmpTrack.subq_control |= SUBQ_CTRLF_DCP;
+    } 
+    else if(cmdbuf == "PRE_EMPHASIS")
+    {
+     TmpTrack.subq_control |= SUBQ_CTRLF_PRE;
+    }
+    // TODO: Confirm that these are taken from the TOC of the disc, and not synthesized by cdrdao.
+    else if(cmdbuf == "CD_DA")
+     disc_type = DISC_TYPE_CDDA_OR_M1;
+    else if(cmdbuf == "CD_ROM")
+     disc_type = DISC_TYPE_CDDA_OR_M1;
+    else if(cmdbuf == "CD_ROM_XA")
+     disc_type = DISC_TYPE_CD_XA;
+    else
+    {
+     //throw MDFN_Error(0, _("Unsupported directive: %s"), cmdbuf.c_str());
+    }
+    // TODO: CATALOG
+
    } /*********** END TOC HANDLING ************/
    else // now for CUE sheet handling
    {
-    if(!strcasecmp(cmdbuf, "FILE"))
+    if(cmdbuf == "FILE")
     {
      if(active_track >= 0)
      {
@@ -486,40 +595,38 @@ void CDAccess_Image::ImageOpen(const char *path)
 
      if(!MDFN_IsFIROPSafe(args[0]))
      {
-      throw(MDFN_Error(0, _("Referenced path \"%s\" is potentially unsafe.  See \"filesys.untrusted_fip_check\" setting.\n"), args[0]));
+      throw(MDFN_Error(0, _("Referenced path \"%s\" is potentially unsafe.  See \"filesys.untrusted_fip_check\" setting.\n"), args[0].c_str()));
      }
 
      std::string efn = MDFN_EvalFIP(base_dir, args[0]);
-
-     if(NULL == (TmpTrack.fp = fopen(efn.c_str(), "rb")))
-     {
-      ErrnoHolder ene(errno);
-
-      throw(MDFN_Error(ene.Errno(), _("Could not open referenced file \"%s\": %s\n"), efn.c_str(), ene.StrError()));
-     }
+     TmpTrack.fp = new FileStream(efn.c_str(), FileStream::MODE_READ);
      TmpTrack.FirstFileInstance = 1;
-     if(!strcasecmp(args[1], "BINARY"))
+
+     if(image_memcache)
+      TmpTrack.fp = new MemoryStream(TmpTrack.fp);
+
+     if(!strcasecmp(args[1].c_str(), "BINARY"))
      {
       //TmpTrack.Format = TRACK_FORMAT_DATA;
       //struct stat stat_buf;
       //fstat(fileno(TmpTrack.fp), &stat_buf);
       //TmpTrack.sectors = stat_buf.st_size; // / 2048;
      }
-     else if(!strcasecmp(args[1], "OGG") || !strcasecmp(args[1], "VORBIS") || !strcasecmp(args[1], "WAVE") || !strcasecmp(args[1], "WAV") || !strcasecmp(args[1], "PCM")
-	|| !strcasecmp(args[1], "MPC") || !strcasecmp(args[1], "MP+"))
+     else if(!strcasecmp(args[1].c_str(), "OGG") || !strcasecmp(args[1].c_str(), "VORBIS") || !strcasecmp(args[1].c_str(), "WAVE") || !strcasecmp(args[1].c_str(), "WAV") || !strcasecmp(args[1].c_str(), "PCM")
+	|| !strcasecmp(args[1].c_str(), "MPC") || !strcasecmp(args[1].c_str(), "MP+"))
      {
       TmpTrack.AReader = AR_Open(TmpTrack.fp);
       if(!TmpTrack.AReader)
       {
-       throw(MDFN_Error(0, _("Unsupported audio track file format: %s\n"), args[0]));
+       throw(MDFN_Error(0, _("Unsupported audio track file format: %s\n"), args[0].c_str()));
       }
      }
      else
      {
-      throw(MDFN_Error(0, _("Unsupported track format: %s\n"), args[1]));
+      throw(MDFN_Error(0, _("Unsupported track format: %s\n"), args[1].c_str()));
      }
     }
-    else if(!strcasecmp(cmdbuf, "TRACK"))
+    else if(cmdbuf == "TRACK")
     {
      if(active_track >= 0)
      {
@@ -531,7 +638,7 @@ void CDAccess_Image::ImageOpen(const char *path)
       TmpTrack.index[0] = -1;
       TmpTrack.index[1] = 0;
      }
-     active_track = atoi(args[0]);
+     active_track = atoi(args[0].c_str());
 
      if(active_track < FirstTrack)
       FirstTrack = active_track;
@@ -541,7 +648,7 @@ void CDAccess_Image::ImageOpen(const char *path)
      int format_lookup;
      for(format_lookup = 0; format_lookup < _DI_FORMAT_COUNT; format_lookup++)
      {
-      if(!strcasecmp(args[1], DI_CUE_Strings[format_lookup]))
+      if(!strcasecmp(args[1].c_str(), DI_CUE_Strings[format_lookup]))
       {
        TmpTrack.DIFormat = format_lookup;
        break;
@@ -550,59 +657,99 @@ void CDAccess_Image::ImageOpen(const char *path)
 
      if(format_lookup == _DI_FORMAT_COUNT)
      {
-      throw(MDFN_Error(0, _("Invalid track format: %s\n"), args[0]));
+      throw(MDFN_Error(0, _("Invalid track format: %s\n"), args[1].c_str()));
      }
 
-
-     TmpTrack.sectors = GetSectorCount(&TmpTrack);
      if(active_track < 0 || active_track > 99)
      {
       throw(MDFN_Error(0, _("Invalid track number: %d\n"), active_track));
      }
     }
-    else if(!strcasecmp(cmdbuf, "INDEX"))
+    else if(cmdbuf == "INDEX")
     {
      if(active_track >= 0)
      {
-      int m,s,f;
-      trio_sscanf(args[1], "%d:%d:%d", &m, &s, &f);
+      unsigned int m,s,f;
 
-      if(!strcasecmp(args[0], "01") || !strcasecmp(args[0], "1"))
+      if(trio_sscanf(args[1].c_str(), "%u:%u:%u", &m, &s, &f) != 3)
+      {
+       throw MDFN_Error(0, _("Malformed m:s:f time in \"%s\" directive: %s"), cmdbuf.c_str(), args[0].c_str());
+      }
+
+      if(!strcasecmp(args[0].c_str(), "01") || !strcasecmp(args[0].c_str(), "1"))
        TmpTrack.index[1] = (m * 60 + s) * 75 + f;
-      else if(!strcasecmp(args[0], "00") || !strcasecmp(args[0], "0"))
+      else if(!strcasecmp(args[0].c_str(), "00") || !strcasecmp(args[0].c_str(), "0"))
        TmpTrack.index[0] = (m * 60 + s) * 75 + f;
      }
     }
-    else if(!strcasecmp(cmdbuf, "PREGAP"))
+    else if(cmdbuf == "PREGAP")
     {
      if(active_track >= 0)
      {
-      int m,s,f;
-      trio_sscanf(args[0], "%d:%d:%d", &m, &s, &f);
+      unsigned int m,s,f;
+
+      if(trio_sscanf(args[0].c_str(), "%u:%u:%u", &m, &s, &f) != 3)
+      {
+       throw MDFN_Error(0, _("Malformed m:s:f time in \"%s\" directive: %s"), cmdbuf.c_str(), args[0].c_str());
+      }
+
       TmpTrack.pregap = (m * 60 + s) * 75 + f;
      }
     }
-    else if(!strcasecmp(cmdbuf, "POSTGAP"))
+    else if(cmdbuf == "POSTGAP")
     {
      if(active_track >= 0)
      {
-      int m,s,f;
-      trio_sscanf(args[0], "%d:%d:%d", &m, &s, &f);
+      unsigned int m,s,f;
+
+      if(trio_sscanf(args[0].c_str(), "%u:%u:%u", &m, &s, &f) != 3)
+      {
+       throw MDFN_Error(0, _("Malformed m:s:f time in \"%s\" directive: %s"), cmdbuf.c_str(), args[0].c_str());
+      }      
+
       TmpTrack.postgap = (m * 60 + s) * 75 + f;
      }
     }
-    else if(!strcasecmp(cmdbuf, "REM"))
+    else if(cmdbuf == "REM")
     {
 
     }
-    else if(!strcasecmp(cmdbuf, "CDTEXTFILE") || !strcasecmp(cmdbuf, "FLAGS") || !strcasecmp(cmdbuf, "CATALOG") || !strcasecmp(cmdbuf, "ISRC") ||
-	!strcasecmp(cmdbuf, "TITLE") || !strcasecmp(cmdbuf, "PERFORMER") || !strcasecmp(cmdbuf, "SONGWRITER"))
+    else if(cmdbuf == "FLAGS")
     {
-     MDFN_printf(_("Unsupported CUE sheet directive: \"%s\".\n"), cmdbuf);	// FIXME, generic logger passed by pointer to constructor
+     TmpTrack.subq_control &= ~(SUBQ_CTRLF_PRE | SUBQ_CTRLF_DCP | SUBQ_CTRLF_4CH);
+     for(unsigned i = 0; i < argcount; i++)
+     {
+      if(args[i] == "DCP")
+      {
+       TmpTrack.subq_control |= SUBQ_CTRLF_DCP;
+      }
+      else if(args[i] == "4CH")
+      {
+       TmpTrack.subq_control |= SUBQ_CTRLF_4CH;
+      }
+      else if(args[i] == "PRE")
+      {
+       TmpTrack.subq_control |= SUBQ_CTRLF_PRE;
+      }
+      else if(args[i] == "SCMS")
+      {
+	// Not implemented, likely pointless.  PROBABLY indicates that the copy bit of the subchannel Q control field is supposed to
+	// alternate between 1 and 0 at 9.375 Hz(four 1, four 0, four 1, four 0, etc.).
+      }
+      else
+      {
+       throw MDFN_Error(0, _("Unknown CUE sheet \"FLAGS\" directive flag \"%s\".\n"), args[i].c_str());
+      }
+     }
+    }
+    else if(cmdbuf == "CDTEXTFILE" || cmdbuf == "CATALOG" || cmdbuf == "ISRC" ||
+	    cmdbuf == "TITLE" || cmdbuf == "PERFORMER" || cmdbuf == "SONGWRITER")
+    {
+     MDFN_printf(_("Unsupported CUE sheet directive: \"%s\".\n"), cmdbuf.c_str());	// FIXME, generic logger passed by pointer to constructor
     }
     else
     {
-     throw MDFN_Error(0, _("Unknown CUE sheet directive \"%s\".\n"), cmdbuf);
+     throw MDFN_Error(0, _("Unknown CUE sheet directive \"%s\".\n"), cmdbuf.c_str());
     }
    } // end of CUE sheet handling
  } // end of fgets() loop
@@ -625,9 +772,24 @@ void CDAccess_Image::ImageOpen(const char *path)
  for(int x = FirstTrack; x < (FirstTrack + NumTracks); x++)
  {
   if(Tracks[x].DIFormat == DI_FORMAT_AUDIO)
-   Tracks[x].Format = CD_TRACK_FORMAT_AUDIO;
+   Tracks[x].subq_control &= ~SUBQ_CTRLF_DATA;
   else
-   Tracks[x].Format = CD_TRACK_FORMAT_DATA;
+   Tracks[x].subq_control |= SUBQ_CTRLF_DATA;
+
+  if(!IsTOC)	// TOC-format disc_type calculation is handled differently.
+  {
+   switch(Tracks[x].DIFormat)
+   {
+    default: break;
+
+    case DI_FORMAT_MODE2:
+    case DI_FORMAT_MODE2_FORM1:
+    case DI_FORMAT_MODE2_FORM2:
+    case DI_FORMAT_MODE2_RAW:
+	disc_type = DISC_TYPE_CD_XA;	
+	break;
+   }
+  }
 
   if(IsTOC)
   {
@@ -657,24 +819,17 @@ void CDAccess_Image::ImageOpen(const char *path)
 
    Tracks[x].LBA = RunningLBA;
 
-   // Make sure this is set before the call to GetSectorCount() for the last track sector count fix.
+   // Make sure FileOffset this is set before the call to GetSectorCount()
    Tracks[x].FileOffset = FileOffset;
+   Tracks[x].sectors = GetSectorCount(&Tracks[x]);
 
-   if((x + 1) >= (FirstTrack + NumTracks))
+   if((x + 1) >= (FirstTrack + NumTracks) || Tracks[x+1].FirstFileInstance)
    {
-    if(!(Tracks[x].FirstFileInstance))
-    {
-     // This will fix the last sector count for CUE+BIN
-     Tracks[x].sectors = GetSectorCount(&Tracks[x]);
-    }
-   }
-   else if(Tracks[x+1].FirstFileInstance)
-   {
-    //RunningLBA += Tracks[x].sectors;
+
    }
    else
    { 
-    // Fix the sector count if we're CUE+BIN
+    // Fix the sector count if we have multiple tracks per one binary image file.
     if(Tracks[x + 1].index[0] == -1)
      Tracks[x].sectors = Tracks[x + 1].index[1] - Tracks[x].index[1];
     else
@@ -694,9 +849,47 @@ void CDAccess_Image::ImageOpen(const char *path)
  total_sectors = RunningLBA;
 }
 
-CDAccess_Image::CDAccess_Image(const char *path)
+void CDAccess_Image::Cleanup(void)
 {
- ImageOpen(path);
+ for(int32 track = 0; track < 100; track++)
+ {
+  CDRFILE_TRACK_INFO *this_track = &Tracks[track];
+
+  if(this_track->FirstFileInstance)
+  {
+   if(Tracks[track].AReader)
+   {
+    delete Tracks[track].AReader;
+    Tracks[track].AReader = NULL;
+   }
+
+   if(this_track->fp)
+   {
+    delete this_track->fp;
+    this_track->fp = NULL;
+   }
+  }
+ }
+}
+
+CDAccess_Image::CDAccess_Image(const char *path, bool image_memcache) : NumTracks(0), FirstTrack(0), LastTrack(0), total_sectors(0)
+{
+ memset(Tracks, 0, sizeof(Tracks));
+
+ try
+ {
+  ImageOpen(path, image_memcache);
+ }
+ catch(...)
+ {
+  Cleanup();
+  throw;
+ }
+}
+
+CDAccess_Image::~CDAccess_Image()
+{
+ Cleanup();
 }
 
 void CDAccess_Image::Read_Raw_Sector(uint8 *buf, int32 lba)
@@ -755,29 +948,29 @@ void CDAccess_Image::Read_Raw_Sector(uint8 *buf, int32 lba)
       if(ct->SubchannelMode)
        SeekPos += 96 * (lba - ct->LBA);
 
-      fseek(ct->fp, SeekPos, SEEK_SET);
+      ct->fp->seek(SeekPos, SEEK_SET);
 
       switch(ct->DIFormat)
       {
 	case DI_FORMAT_AUDIO:
-		fread(buf, 1, 2352, ct->fp);
+		ct->fp->read(buf, 2352);
 
 		if(ct->RawAudioMSBFirst)
 		 Endian_A16_Swap(buf, 588 * 2);
 		break;
 
 	case DI_FORMAT_MODE1:
-		fread(buf + 12 + 3 + 1, 1, 2048, ct->fp);
+		ct->fp->read(buf + 12 + 3 + 1, 2048);
 		encode_mode1_sector(lba + 150, buf);
 		break;
 
 	case DI_FORMAT_MODE1_RAW:
 	case DI_FORMAT_MODE2_RAW:
-		fread(buf, 1, 2352, ct->fp);
+		ct->fp->read(buf, 2352);
 		break;
 
 	case DI_FORMAT_MODE2:
-		fread(buf + 16, 1, 2336, ct->fp);
+		ct->fp->read(buf + 16, 2336);
 		encode_mode2_sector(lba + 150, buf);
 		break;
 
@@ -785,19 +978,19 @@ void CDAccess_Image::Read_Raw_Sector(uint8 *buf, int32 lba)
 	// FIXME: M2F1, M2F2, does sub-header come before or after user data(standards say before, but I wonder
 	// about cdrdao...).
 	case DI_FORMAT_MODE2_FORM1:
-		fread(buf + 24, 1, 2048, ct->fp);
+		ct->fp->read(buf + 24, 2048);
 		//encode_mode2_form1_sector(lba + 150, buf);
 		break;
 
 	case DI_FORMAT_MODE2_FORM2:
-		fread(buf + 24, 1, 2324, ct->fp);
+		ct->fp->read(buf + 24, 2324);
 		//encode_mode2_form2_sector(lba + 150, buf);
 		break;
 
       }
 
       if(ct->SubchannelMode)
-       fread(buf + 2352, 1, 96, ct->fp);
+       ct->fp->read(buf + 2352, 96);
      }
     } // end if audible part of audio track read.
     break;
@@ -839,6 +1032,9 @@ void CDAccess_Image::Read_Raw_Sector(uint8 *buf, int32 lba)
  //printf("%02x\n", buf[12 + 3]);
 }
 
+//
+// Note: this function makes use of the current contents(as in |=) in SubPWBuf.
+//
 void CDAccess_Image::MakeSubPQ(int32 lba, uint8 *SubPWBuf)
 {
  uint8 buf[0xC];
@@ -877,7 +1073,7 @@ void CDAccess_Image::MakeSubPQ(int32 lba, uint8 *SubPWBuf)
  ma = ((lba + 150) / 75 / 60);
 
  uint8 adr = 0x1; // Q channel data encodes position
- uint8 control = (Tracks[track].Format == CD_TRACK_FORMAT_AUDIO) ? 0x00 : 0x04;
+ uint8 control = Tracks[track].subq_control;
 
  // Handle pause(D7 of interleaved subchannel byte) bit, should be set to 1 when in pregap or postgap.
  if((lba < Tracks[track].LBA) || (lba >= Tracks[track].LBA + Tracks[track].sectors))
@@ -891,14 +1087,16 @@ void CDAccess_Image::MakeSubPQ(int32 lba, uint8 *SubPWBuf)
   int32 pg_offset = (int32)lba - Tracks[track].LBA;
 
   // If we're more than 2 seconds(150 sectors) from the real "start" of the track/INDEX 01, and the track is a data track,
-  // and the preceding track is an audio track, encode it as audio.
+  // and the preceding track is an audio track, encode it as audio(by taking the SubQ control field from the preceding track).
+  //
+  // TODO: Look into how we're supposed to handle subq control field in the four combinations of track types(data/audio).
+  //
   if(pg_offset < -150)
   {
-   if(Tracks[track].Format == CD_TRACK_FORMAT_DATA && (FirstTrack < track) && 
-	Tracks[track - 1].Format == CD_TRACK_FORMAT_AUDIO)
+   if((Tracks[track].subq_control & SUBQ_CTRLF_DATA) && (FirstTrack < track) && !(Tracks[track - 1].subq_control & SUBQ_CTRLF_DATA))
    {
     //printf("Pregap part 1 audio->data: lba=%d track_lba=%d\n", lba, Tracks[track].LBA);
-    control = 0x00;
+    control = Tracks[track - 1].subq_control;
    }
   }
  }
@@ -937,28 +1135,25 @@ void CDAccess_Image::Read_TOC(TOC *toc)
 
  toc->first_track = FirstTrack;
  toc->last_track = FirstTrack + NumTracks - 1;
- toc->disc_type = DISC_TYPE_CDDA_OR_M1;	// FIXME
+ toc->disc_type = disc_type;
 
  for(int i = toc->first_track; i <= toc->last_track; i++)
  {
   toc->tracks[i].lba = Tracks[i].LBA;
   toc->tracks[i].adr = ADR_CURPOS;
-  toc->tracks[i].control = 0x0;
-
-  if(Tracks[i].Format != CD_TRACK_FORMAT_AUDIO)
-   toc->tracks[i].control |= 0x4;
+  toc->tracks[i].control = Tracks[i].subq_control;
  }
 
  toc->tracks[100].lba = total_sectors;
  toc->tracks[100].adr = ADR_CURPOS;
- toc->tracks[100].control = 0x00;	// Audio...
+ toc->tracks[100].control = toc->tracks[toc->last_track].control & 0x4;
 
  // Convenience leadout track duplication.
  if(toc->last_track < 99)
   toc->tracks[toc->last_track + 1] = toc->tracks[100];
 }
 
-bool CDAccess_Image::Is_Physical(void)
+bool CDAccess_Image::Is_Physical(void) throw()
 {
  return(false);
 }
