@@ -15,25 +15,39 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+/*
+ TODO:
+
+  CDDAReadPos == 588, GetCDDAValues(), mess.
+
+*/
+
 #include "../mednafen.h"
 #include "../clamp.h"
 #include <math.h>
+#include <algorithm>
 #include <trio/trio.h>
 #include "scsicd.h"
 #include "cdromif.h"
 #include "SimpleFIFO.h"
 
-#define SCSIDBG(format, ...) { printf("SCSI: " format "\n",  ## __VA_ARGS__); }
+#define SCSIDBG(format, ...) { printf("[SCSICD] " format "\n",  ## __VA_ARGS__); }
 //#define SCSIDBG(format, ...) { }
 
 using namespace CDUtility;
 
+static const unsigned CDDA_Filter_NumConvolutions = 7;
+
+static const int16 CDDA_Filter[1 + 256 + 1][CDDA_Filter_NumConvolutions] =
+{
+ #include "scsicd_cdda_filter.inc"
+};
 
 static uint32 CD_DATA_TRANSFER_RATE;
 static uint32 System_Clock;
 static void (*CDIRQCallback)(int);
 static void (*CDStuffSubchannels)(uint8, int);
-static Blip_Buffer *sbuf[2];
+static int32* HRBufs[2];
 static int WhichSystem;
 static CDIF *Cur_CDIF;
 
@@ -87,10 +101,10 @@ typedef struct
  // Used for multiple sector CD reads.
  bool data_transfer_done;
 
- // To target(the cd unit), for "MODE SELECT" mainly
- uint8 data_out[8192];
- uint32 data_out_pos;
- uint32 data_out_size;
+ // To target(the cd unit); for "MODE SELECT".
+ uint8 data_out[256];	// Technically it only needs to be 255, but powers of 2 are better than those degenerate powers of 2 minus one goons.
+ uint8 data_out_pos;	// Current index for writing into data_out.
+ uint8 data_out_want;	// Total number of bytes to buffer into data_out.
 
  bool TrayOpen;
  bool DiscChanged;
@@ -102,8 +116,6 @@ typedef struct
 
 } scsicd_t;
 
-typedef Blip_Synth < /*64*/blip_good_quality, 1 > CDSynth;
-
 enum
 {
  CDDASTATUS_PAUSED = -1,
@@ -122,19 +134,18 @@ enum
 
 typedef struct
 {
- int32 CDDADivAcc;
+ uint32 CDDADivAcc;
+ uint8 CDDADivAccVolFudge;	// For PC-FX CD-DA rate control RE impulses and resampling; 100 = 1.0.
  uint32 scan_sec_end;
 
  uint8 PlayMode;
- CDSynth CDDASynth[2];
- int32 CDDAVolume[2];
- int16 last_sample[2];
+ int32 CDDAVolume[2];	// 32768 = 1.0, maximum
  int16 CDDASectorBuffer[1176];
  uint32 CDDAReadPos;
 
  int8 CDDAStatus;
  uint8 ScanMode;
- int32 CDDADiv;
+ int64 CDDADiv;
  int CDDATimeDiv;
 
  uint8 OutPortChSelect[2];
@@ -196,7 +207,11 @@ static void FixOPV(void)
 {
  for(int port = 0; port < 2; port++)
  {
-  cdda.OutPortVolumeCache[port] = cdda.CDDAVolume[port];
+  int32 tmpvol = cdda.CDDAVolume[port] * 100 / cdda.CDDADivAccVolFudge;
+
+  //printf("TV: %d\n", tmpvol);
+
+  cdda.OutPortVolumeCache[port] = tmpvol;
 
   if(cdda.OutPortChSelect[port] & 0x01)
    cdda.OutPortChSelectCache[port] = 0;
@@ -216,7 +231,6 @@ static void VirtualReset(void)
 
  din->Flush();
 
- cdda.CDDADivAcc = (int64)System_Clock * 65536 / 44100;
  CDReadTimer = 0;
 
  pce_lastsapsp_timestamp = monotonic_timestamp;
@@ -234,7 +248,10 @@ static void VirtualReset(void)
  cdda.scan_sec_end = 0;
 
 
- cd.data_out_pos = cd.data_out_size = 0;
+ memset(cd.data_out, 0, sizeof(cd.data_out));
+ cd.data_out_pos = 0;
+ cd.data_out_want = 0;
+
 
  FixOPV();
 
@@ -437,7 +454,7 @@ static void SendStatusAndMessage(uint8 status, uint8 message)
  // This should never ever happen, but that doesn't mean it won't. ;)
  if(din->CanRead())
  {
-  printf("BUG: %d bytes still in SCSI CD FIFO\n", din->CanRead());
+  printf("[SCSICD] BUG: %d bytes still in SCSI CD FIFO\n", din->CanRead());
   din->Flush();
  }
 
@@ -497,7 +514,7 @@ void SCSICD_SetDisc(bool tray_open, CDIF *cdif, bool no_emu_side_effects)
 
 static void CommandCCError(int key, int asc = 0, int ascq = 0)
 {
- printf("CC Error: %02x %02x %02x\n", key, asc, ascq);
+ printf("[SCSICD] CC Error: %02x %02x %02x\n", key, asc, ascq);
 
  cd.key_pending = key;
  cd.asc_pending = asc;
@@ -529,8 +546,8 @@ static void DoMODESELECT6(const uint8 *cdb)
  if(cdb[4])
  {
   cd.data_out_pos = 0;
-  cd.data_out_size = cdb[4];
-  //printf("Switch to DATA OUT phase, len: %d\n", cd.data_out_size);
+  cd.data_out_want = cdb[4];
+  //printf("Switch to DATA OUT phase, len: %d\n", cd.data_out_want);
 
   ChangePhase(PHASE_DATA_OUT);
  }
@@ -616,8 +633,9 @@ struct ModePage
 	0x2B:
 	0x3F(Yes, not really a mode page but a fetch method)
 */
+// Remember to update the code in StateAction() if we change the number or layout of modepages here.
 static const int NumModePages = 5;
-static ModePage ModePages[] =
+static ModePage ModePages[NumModePages] =
 {
  // Unknown
  { 0x28,
@@ -679,16 +697,8 @@ static ModePage ModePages[] =
  },
 };
 
-static void UpdateMPCache(uint8 code)
+static void UpdateMPCacheP(const ModePage* mp)
 {
- for(int pi = 0; pi < NumModePages; pi++)
- { 
-  const ModePage *mp = &ModePages[pi];
-  //const ModePageParam *params = &ModePages[pi].params[0];
-
-  if(mp->code != code)
-   continue;
-
   switch(mp->code)
   {
    case 0x0E:
@@ -712,15 +722,48 @@ static void UpdateMPCache(uint8 code)
 
    case 0x2B:
 	    {
-             int8 speed = mp->current_value[0];
-             double rate = 44100 + (double)44100 * speed / 100;
-             //printf("Speed: %d %f\n", speed, rate);
-             cdda.CDDADivAcc = (int32)((int64)System_Clock * 65536 / rate);
+             int speed;
+             int rate;
+
+	     //
+	     // Not sure what the actual limits are, or what happens when exceeding them, but these will at least keep the
+	     // CD-DA playback system from imploding in on itself.
+	     //
+	     // The range of speed values accessible via the BIOS CD-DA player is apparently -10 to 10.
+	     //
+	     // No game is known to use the CD-DA playback speed control.  It may be useful in homebrew to lower the rate for fitting more CD-DA onto the disc,
+	     // is implemented on the PC-FX in such a way that it degrades audio quality, so it wouldn't really make sense to increase the rate in homebrew.
+	     //
+	     // Due to performance considerations, we don't even try to emulate the CD-DA oversampling filters used on the PC Engine and PC-FX, and instead
+	     // blast impulses into the 1.78MHz buffer, relying on the final sound resampler to kill spectrum mirrors.  This is less than ideal, but generally
+	     // works well in practice, except when lowering CD-DA playback rate...which causes the spectrum mirrors to enter the audible, non-murder zone, causing
+	     // harshness and the sound output amplitude to approach overflow levels.
+	     // But, until there's a killer PC-FX homebrew game that necessitates more computationally-expensive CD-DA handling,
+	     // I don't see a good reason to change how CD-DA resampling is currently implemented.
+	     // 
+	     speed = std::max<int>(-32, std::min<int>(32, (int8)mp->current_value[0]));
+	     rate = 44100 + 441 * speed;
+
+             //printf("[SCSICD] Speed: %d(pre-clamped=%d) %d\n", speed, (int8)mp->current_value[0], rate);
+             cdda.CDDADivAcc = ((int64)System_Clock * (1024 * 1024) / rate);
+	     cdda.CDDADivAccVolFudge = 100 + speed;
+	     FixOPV();	// Resampler impulse amplitude volume adjustment(call after setting cdda.CDDADivAccVolFudge)
 	    }
 	    break;
   }
+}
 
-  break;
+static void UpdateMPCache(uint8 code)
+{
+ for(int pi = 0; pi < NumModePages; pi++)
+ { 
+  const ModePage* mp = &ModePages[pi];
+
+  if(mp->code == code)
+  {
+   UpdateMPCacheP(mp);
+   break;
+  }
  }
 }
 
@@ -734,18 +777,18 @@ static void InitModePages(void)
   for(int parami = 0; parami < mp->param_length; parami++)
    mp->current_value[parami] = params[parami].default_value;
 
-  UpdateMPCache(mp->code);
+  UpdateMPCacheP(mp);
  }
 }
 
-static void FinishMODESELECT6(const uint8 *data, const uint32 data_len)
+static void FinishMODESELECT6(const uint8 *data, const uint8 data_len)
 {
 	uint8 mode_data_length, medium_type, device_specific, block_descriptor_length;
 	uint32 offset = 0;
 
-        printf("Mode Select (6) Data: 0x%08x, ", data_len);
-        for(unsigned int i = 0; i < cd.data_out_size; i++)
-         printf("0x%02x ", cd.data_out[i]);
+        printf("[SCSICD] Mode Select (6) Data: Length=0x%02x, ", data_len);
+        for(uint32 i = 0; i < data_len; i++)
+         printf("0x%02x ", data[i]);
         printf("\n");
 
         if(data_len < 4)
@@ -759,6 +802,11 @@ static void FinishMODESELECT6(const uint8 *data, const uint32 data_len)
 	device_specific = data[offset++];
 	block_descriptor_length = data[offset++];
 
+	// For now, shut up gcc.
+	(void)mode_data_length;
+	(void)medium_type;
+	(void)device_specific;
+
 	if(block_descriptor_length & 0x7)
 	{
 	 CommandCCError(SENSEKEY_ILLEGAL_REQUEST, NSE_INVALID_PARAMETER);
@@ -829,7 +877,7 @@ static void FinishMODESELECT6(const uint8 *data, const uint32 data_len)
 	    mp->current_value[parami] |= (data[offset++]) & mp->params[parami].real_mask;
 	   }
 
-	   UpdateMPCache(mp->code);
+	   UpdateMPCacheP(mp);
 	   break;
 	  }
 	 }
@@ -855,7 +903,7 @@ static void DoMODESENSE6(const uint8 *cdb)
  uint8 PageMatchOR = 0x00;
  bool AnyPageMatch = false;
 
- SCSIDBG("Mode sense 6: %02x %d %d %d\n", PageCode, PC, DBD, AllocSize);
+ SCSIDBG("Mode sense 6: %02x %d %d %d", PageCode, PC, DBD, AllocSize);
 
  if(!AllocSize)
  {
@@ -1716,8 +1764,8 @@ static void DoPATI(const uint8 *cdb)
  //  problematic when Mednafen is used with a physical CD.
  int StartTrack = cdb[4];
  int EndTrack = cdb[7];
- int StartIndex = cdb[5];
- int EndIndex = cdb[8];
+ //int StartIndex = cdb[5];
+ //int EndIndex = cdb[8];
 
  if(!StartTrack || StartTrack < toc.first_track || StartTrack > toc.last_track)
  {
@@ -1956,7 +2004,7 @@ static void DoPREFETCH(const uint8 *cdb)
 
  // Note: This command appears to lock up the CD unit to some degree on a real PC-FX if the (lba + len) >= leadout_track_lba,
  // more testing is needed if we ever try to fully emulate this command.
- if(lba >= (int32)toc.tracks[100].lba)
+ if(lba >= toc.tracks[100].lba)
  {
   CommandCCError(SENSEKEY_ILLEGAL_REQUEST, NSE_END_OF_VOLUME);
   return;
@@ -2022,7 +2070,7 @@ static void DoREADSUBCHANNEL(const uint8 *cdb)
  uint8 data_in[8192];
  int DataFormat = cdb[3];
  int TrackNum = cdb[6];
- int AllocSize = (cdb[7] << 8) | cdb[8];
+ unsigned AllocSize = (cdb[7] << 8) | cdb[8];
  bool WantQ = cdb[2] & 0x40;
  bool WantMSF = cdb[1] & 0x02;
  uint32 offset = 0;
@@ -2365,16 +2413,17 @@ static SCSICH PCECommandDefs[] =
  { 0xFF, 0, 0, NULL, NULL },
 };
 
-void SCSICD_ResetTS(void)
+void SCSICD_ResetTS(uint32 ts_base)
 {
- lastts = 0;
+ lastts = ts_base;
 }
 
+// FIXME: pause/stopped clear clear cleaaaar?
 void SCSICD_GetCDDAValues(int16 &left, int16 &right)
 {
  if(cdda.CDDAStatus)
  {
-  left = cdda.CDDASectorBuffer[cdda.CDDAReadPos * 2];
+  left = cdda.CDDASectorBuffer[cdda.CDDAReadPos * 2 + 0];
   right = cdda.CDDASectorBuffer[cdda.CDDAReadPos * 2 + 1];
  }
  else
@@ -2385,12 +2434,18 @@ static INLINE void RunCDDA(uint32 system_timestamp, int32 run_time)
 {
  if(cdda.CDDAStatus == CDDASTATUS_PLAYING || cdda.CDDAStatus == CDDASTATUS_SCANNING)
  {
-  int32 sample[2];
+  int32 sample_va[2];
 
-  cdda.CDDADiv -= run_time << 16;
+  cdda.CDDADiv -= (int64)run_time << 20;
 
   while(cdda.CDDADiv <= 0)
   {
+   const uint64 synthtime_ex = ((((uint64)system_timestamp << 20) + (int64)cdda.CDDADiv) / cdda.CDDATimeDiv) >> (4 + 2);
+   const int synthtime = synthtime_ex >> 16;
+   const int synthtime_phase = (int)(synthtime_ex & 0xFFFF) - 0x80;
+   const int synthtime_phase_int = synthtime_phase >> 8;
+   const int synthtime_phase_fract = synthtime_phase & 0xFF;
+
    cdda.CDDADiv += cdda.CDDADivAcc;
 
    //MDFN_DispMessage("%d %d %d\n", read_sec_start, read_sec, read_sec_end);
@@ -2483,15 +2538,66 @@ static INLINE void RunCDDA(uint32 system_timestamp, int32 run_time)
 
    // If the last valid sub-Q data decoded indicate that the corresponding sector is a data sector, don't output the
    // current sector as audio.
-   sample[0] = sample[1] = 0;
+   sample_va[0] = 0;
+   sample_va[1] = 0;
 
    if(!(cd.SubQBuf_Last[0] & 0x40) && cdda.PlayMode != PLAYMODE_SILENT)
    {
-    sample[0] += (cdda.CDDASectorBuffer[cdda.CDDAReadPos * 2 + cdda.OutPortChSelectCache[0]] * cdda.OutPortVolumeCache[0]) >> 16;
-    sample[1] += (cdda.CDDASectorBuffer[cdda.CDDAReadPos * 2 + cdda.OutPortChSelectCache[1]] * cdda.OutPortVolumeCache[1]) >> 16;
+#if 0
+    static double phase = 0;
+    static double phase_inc = 0;
+    static const double phase_inc_inc = 0.000003;
+
+ #if 0
+    sample_va[0] = 32767 * 0.75 * sin(phase);
+    sample_va[1] = 32767 * 0.75 * sin(phase);
+
+    phase += phase_inc;
+    phase_inc += phase_inc_inc;
+ #else
+    {
+     static unsigned counter = 0;
+     static int32 wv = 0x7000 * 1.5;
+
+     if(!counter)
+      wv = -wv;
+
+     counter = (counter + 1) & 1;
+
+     sample_va[0] = sample_va[1] = wv;
+    }
+ #endif
+
+    
+#else
+    // OPVC can have a maximum value of 65536, which is equivalent to 2.0.
+    // raw CD-DA samples are -32768 through 32767
+    // Output of this stage will be (max ranges) -131072 through 131068.
+    {
+     int16 sr[2];
+
+     sr[0] = cdda.CDDASectorBuffer[cdda.CDDAReadPos * 2 + cdda.OutPortChSelectCache[0]];
+     sr[1] = cdda.CDDASectorBuffer[cdda.CDDAReadPos * 2 + cdda.OutPortChSelectCache[1]];
+
+#if 0
+     {
+      static int16 wv = 0x7000;
+      static unsigned counter = 0;
+
+      sr[0] = wv;
+      sr[1] = wv;
+      if(counter == 0)
+       wv = -wv;
+      counter = (counter + 1) & 1;
+     }
+#endif
+
+     sample_va[0] = (sr[0] * cdda.OutPortVolumeCache[0]) >> (15 - 1);
+     sample_va[1] = (sr[1] * cdda.OutPortVolumeCache[1]) >> (15 - 1);
+    }
+#endif
    }
 
-   uint32 synthtime = ((system_timestamp + (cdda.CDDADiv >> 16))) / cdda.CDDATimeDiv;
    if(!(cdda.CDDAReadPos % 6))
    {
     int subindex = cdda.CDDAReadPos / 6 - 2;
@@ -2502,15 +2608,40 @@ static INLINE void RunCDDA(uint32 system_timestamp, int32 run_time)
      CDStuffSubchannels(0x00, subindex);
    }
 
-   if(sbuf[0] && sbuf[1])
+   //
+   //
+   //  YEAH YEAH, I know resampling like this isn't quite 100% awesome-good, as the upper part of the CD-DA spectrum is going to be mirrored on the other
+   //  side of the Nyquist frequency(we're simulating zero-stuffing style resampling, using a fairly flat-response filter for phase correction).
+   //  But it's not that big of a deal for reasons I'm too tired to elucidate at the moment!
+   //
+   //
+   if(HRBufs[0] && HRBufs[1])
    {
-    cdda.CDDASynth[0].offset_inline(synthtime, sample[0] - cdda.last_sample[0], sbuf[0]);
-    cdda.CDDASynth[1].offset_inline(synthtime, sample[1] - cdda.last_sample[1], sbuf[1]);
+    //printf("%4d %4d\n", synthtime_phase_int, synthtime_phase_fract);
+    for(unsigned ch = 0; ch < 2; ch++)
+    {
+     //int32 prev = 0;
+     int32* tb = &HRBufs[ch][synthtime & 0xFFFF];
+
+     for(unsigned c = 0; c < CDDA_Filter_NumConvolutions; c++)
+     {
+      int32 coeff;
+      int32 mr;
+
+      //coeff = CDDA_Filter[1 + synthtime_phase_int + 0][c];
+      coeff = (CDDA_Filter[1 + synthtime_phase_int + 0][c] * (256 - synthtime_phase_fract) +
+              CDDA_Filter[1 + synthtime_phase_int + 1][c] * (synthtime_phase_fract)) >> 8;
+
+      // sample_va == -131072, -131072 * 16383 = -2147352576.
+
+      mr = (sample_va[ch] * coeff) >> (16 - 6 - 8 + 1);	// +1 for sample_va extra bit.
+
+      tb[c] += mr; // - prev;
+      //prev = mr;
+     }
+     //tb[CDDA_Filter_NumConvolutions] += 0 - prev;
+    }
    }
-
-   cdda.last_sample[0] = sample[0];
-   cdda.last_sample[1] = sample[1];
-
    cdda.CDDAReadPos++;
   }
  }
@@ -2736,12 +2867,12 @@ uint32 SCSICD_Run(scsicd_timestamp_t system_timestamp)
     }
     else if(!REQ_signal && !ACK_signal && cd.data_out_pos)
     {
-     if(cd.data_out_pos == cd.data_out_size)
+     if(cd.data_out_pos == cd.data_out_want)
      {
       cd.data_out_pos = 0;
 
       if(cd.command_buffer[0] == 0x15)
-	FinishMODESELECT6(cd.data_out, cd.data_out_size);
+	FinishMODESELECT6(cd.data_out, cd.data_out_want);
       else	// Error out here?  It shouldn't be reached:
        SendStatusAndMessage(STATUS_GOOD, 0x00);
      }
@@ -2763,16 +2894,16 @@ uint32 SCSICD_Run(scsicd_timestamp_t system_timestamp)
     //if(cd_bus.DB == 0x6)		// ABORT message!
     if(1)
     {
-     puts("ABORT");
+     printf("[SCSICD] Abort Received(DB=0x%02x)\n", cd_bus.DB);
      din->Flush();
-     cd.data_out_pos = cd.data_out_size = 0;
+     cd.data_out_pos = cd.data_out_want = 0;
 
      CDReadTimer = 0;
      cdda.CDDAStatus = CDDASTATUS_STOPPED;
      ChangePhase(PHASE_BUS_FREE);
     }
     else
-     printf("Message to target: %02x\n", cd_bus.DB);
+     printf("[SCSICD] Message to target: 0x%02x\n", cd_bus.DB);
    }
    break;
 
@@ -2844,7 +2975,7 @@ uint32 SCSICD_Run(scsicd_timestamp_t system_timestamp)
 
  if(cdda.CDDAStatus == CDDASTATUS_PLAYING || cdda.CDDAStatus == CDDASTATUS_SCANNING)
  {
-  int32 cdda_div_sexytime = (cdda.CDDADiv + 0xFFFF) >> 16;
+  int32 cdda_div_sexytime = (cdda.CDDADiv + ((1 << 20) - 1)) >> 20;
   if(cdda_div_sexytime > 0 && cdda_div_sexytime < next_time)
    next_time = cdda_div_sexytime;
  }
@@ -2858,99 +2989,6 @@ void SCSICD_SetLog(void (*logfunc)(const char *, const char *, ...))
 {
  SCSILog = logfunc;
 }
-#if 0
-static void kaiser_window( double* io, int count, double beta )
-{
-        int const accuracy = 20; //12;
-
-        double* end = io + count;
-
-        double beta2    = beta * beta * (double) -0.25;
-        double to_fract = beta2 / ((double) count * count);
-        double i        = 0;
-        double rescale;
-        for ( ; io < end; ++io, i += 1 )
-        {
-                double x = i * i * to_fract - beta2;
-                double u = x;
-                double k = x + 1;
-
-                double n = 2;
-                do
-                {
-                        u *= x / (n * n);
-                        n += 1;
-                        k += u;
-                }
-                while ( k <= u * (1 << accuracy) );
-
-                if ( !i )
-                        rescale = 1 / k; // otherwise values get large
-
-                *io *= k * rescale;
-        }
-}
-
-static void gen_sinc( double* out, int size, double cutoff, double kaiser )
-{
-        assert( size % 2 == 0 ); // size must be enev
-
-        int const half_size = size / 2;
-        double* const mid = &out [half_size];
-
-        // Generate right half of sinc
-        for ( int i = 0; i < half_size; i++ )
-        {
-                double angle = (i * 2 + 1) * (M_PI / 2);
-                mid [i] = sin( angle * cutoff ) / angle;
-        }
-
-        kaiser_window( mid, half_size, kaiser );
-
-        // Mirror for left half
-        for ( int i = 0; i < half_size; i++ )
-                out [i] = mid [half_size - 1 - i];
-}
-
-static void normalize( double* io, int size, double gain = 1.0 )
-{
-        double sum = 0;
-        for ( int i = 0; i < size; i++ )
-                sum += io [i];
-
-        double scale = gain / sum;
-        for ( int i = 0; i < size; i++ )
-                io [i] *= scale;
-}
-
-struct my_kernel : blip_eq_t
-{
-        double cutoff;
-
-        my_kernel( double cutoff ) : cutoff( cutoff )
-        {
-        }
-
-        void generate( double* out, int count ) const
-        {
-                double tmp_buf[count * 2];
-                double moo = 0;
-
-		printf("%f %f\n", (double)cutoff, (double)oversample);
-
-                gen_sinc(tmp_buf, count * 2, cutoff / oversample * 2, 10);
-                normalize(tmp_buf, count * 2, 1); //189216615);
-
-                for(int i = 0; i < count; i++)
-                {
-                 out[i] = tmp_buf[count + i];
-                 moo += out[i];
-                }
-                printf("Moo: %f\n", moo);
-        }
-};
-#endif
-
 
 void SCSICD_SetTransferRate(uint32 TransferRate)
 {
@@ -2966,11 +3004,13 @@ void SCSICD_Close(void)
  }
 }
 
-void SCSICD_Init(int type, int cdda_time_div, Blip_Buffer *leftbuf, Blip_Buffer *rightbuf, uint32 TransferRate, uint32 SystemClock, void (*IRQFunc)(int), void (*SSCFunc)(uint8, int))
+void SCSICD_Init(int type, int cdda_time_div, int32* left_hrbuf, int32* right_hrbuf, uint32 TransferRate, uint32 SystemClock, void (*IRQFunc)(int), void (*SSCFunc)(uint8, int))
 {
  Cur_CDIF = NULL;
  cd.TrayOpen = false;
 
+ assert(SystemClock < 30000000);	// 30 million, sanity check.
+
  monotonic_timestamp = 0;
  lastts = 0;
 
@@ -2982,23 +3022,17 @@ void SCSICD_Init(int type, int cdda_time_div, Blip_Buffer *leftbuf, Blip_Buffer
   din = new SimpleFIFO<uint8>(2048); //8192); //1024); /2048);
 
  WhichSystem = type;
+
+ cdda.CDDADivAcc = (int64)System_Clock * (1024 * 1024) / 44100;
+ cdda.CDDADivAccVolFudge = 100;
  cdda.CDDATimeDiv = cdda_time_div;
-
- for(int i = 0; i < 2; i++)
- {
-  //double meow = (((double)22050 / 48000) - (6.4 / 64 / 2));
-  //my_kernel eq(meow);
-
-  cdda.CDDASynth[i].volume(1.0f / 65536);
-
-  cdda.CDDAVolume[i] = 65536; //1.0;
-  //CDDASynth[i].treble_eq(eq);
- }
+ cdda.CDDAVolume[0] = 32768;
+ cdda.CDDAVolume[1] = 32768;
 
  FixOPV();
 
- sbuf[0] = leftbuf;
- sbuf[1] = rightbuf;
+ HRBufs[0] = left_hrbuf;
+ HRBufs[1] = right_hrbuf;
 
  CD_DATA_TRANSFER_RATE = TransferRate;
  System_Clock = SystemClock;
@@ -3008,15 +3042,15 @@ void SCSICD_Init(int type, int cdda_time_div, Blip_Buffer *leftbuf, Blip_Buffer
 
 void SCSICD_SetCDDAVolume(double left, double right)
 {
- cdda.CDDAVolume[0] = 65536 * left;
- cdda.CDDAVolume[1] = 65536 * right;
+ cdda.CDDAVolume[0] = 32768 * left;
+ cdda.CDDAVolume[1] = 32768 * right;
 
  for(int i = 0; i < 2; i++)
  {
-  if(cdda.CDDAVolume[i] > 65536)
+  if(cdda.CDDAVolume[i] > 32768)
   {
-   printf("Debug Warning: CD-DA volume %d too large: %d\n", i, cdda.CDDAVolume[i]);
-   cdda.CDDAVolume[i] = 65536;
+   printf("[SCSICD] Debug Warning: CD-DA volume %d too large: %d\n", i, cdda.CDDAVolume[i]);
+   cdda.CDDAVolume[i] = 32768;
   }
  }
 
@@ -3031,16 +3065,6 @@ int SCSICD_StateAction(StateMem * sm, int load, int data_only, const char *sname
   SFVARN(cd_bus.signals, "Signals"),
   SFVAR(CurrentPhase),
 
-  //SFVARN(cd_bus.BSY, "BSY"),
-  //SFVARN(cd_bus.MSG, "MSG"),
-  //SFVARN(cd_bus.CD, "CD"),
-  //SFVARN(cd_bus.REQ, "REQ"),
-  //SFVARN(cd_bus.IO, "IO"),
-  //SFVARN(cd_bus.kingACK, "kingACK"),
-  //SFVARN(cd_bus.kingRST, "kingRST"),
-  //SFVARN(cd_bus.kingSEL, "kingSEL"),
-  //SFVARN(cd_bus.kingATN, "kingATN"),
-
   SFVARN(cd.last_RST_signal, "last_RST"),
   SFVARN(cd.message_pending, "message_pending"),
   SFVARN(cd.status_sent, "status_sent"),
@@ -3060,9 +3084,9 @@ int SCSICD_StateAction(StateMem * sm, int load, int data_only, const char *sname
   SFVARN(din->in_count, "din_in_count"),
   SFVARN(cd.data_transfer_done, "data_transfer_done"),
 
-  SFARRAYN(cd.data_out, 8192, "data_out"),
+  SFARRAYN(cd.data_out, sizeof(cd.data_out), "data_out"),
   SFVARN(cd.data_out_pos, "data_out_pos"),
-  SFVARN(cd.data_out_size, "data_out_size"),
+  SFVARN(cd.data_out_want, "data_out_want"),
 
   SFVARN(cd.TrayOpen, "TrayOpen"),
   SFVARN(cd.DiscChanged, "DiscChanged"),
@@ -3090,6 +3114,14 @@ int SCSICD_StateAction(StateMem * sm, int load, int data_only, const char *sname
   SFVAR(monotonic_timestamp),
   SFVAR(pce_lastsapsp_timestamp),
 
+  //
+  //
+  //
+  SFARRAY(ModePages[0].current_value, ModePages[0].param_length),
+  SFARRAY(ModePages[1].current_value, ModePages[1].param_length),
+  SFARRAY(ModePages[2].current_value, ModePages[2].param_length),
+  SFARRAY(ModePages[3].current_value, ModePages[3].param_length),
+  SFARRAY(ModePages[4].current_value, ModePages[4].param_length),
   SFEND
  };
 
@@ -3101,6 +3133,12 @@ int SCSICD_StateAction(StateMem * sm, int load, int data_only, const char *sname
   din->read_pos &= din->size - 1;
   din->write_pos = (din->read_pos + din->in_count) & (din->size - 1);
   //printf("%d %d %d\n", din->in_count, din->read_pos, din->write_pos);
+
+  if(cdda.CDDADiv <= 0)
+   cdda.CDDADiv = 1;
+
+  for(int i = 0; i < NumModePages; i++)
+   UpdateMPCacheP(&ModePages[i]);
  }
 
  return (ret);
