@@ -60,12 +60,6 @@ typedef struct
 	// The number of address bits for this address space.
 	uint32 TotalBits;
 
-	// (Segmentation bits TODO)
-	bool IsSegmented;
-	uint32 SegmentBits;
-	uint32 OffsetBits;
-	uint32 BitsOverlapped; // For future use, maybe.
-
 	// Non-power-of-2 size.  Normally 0, unless the size of the address space isn't a power of 2!
 	uint32 NP2Size;
 
@@ -132,7 +126,7 @@ typedef struct
 
  uint32 DefaultWatchAddr;
 
- uint32 ZPAddr; // Set to ~0 to disable
+ uint32 ZPAddr; // Set to ~0U to disable
 
  // If logical is true, then do the peek/poke on logical address A, else do the
  // peek/poke on physical address A.  For now, this distinction only exists
@@ -159,9 +153,17 @@ typedef struct
 
  void (*AddBreakPoint)(int type, unsigned int A1, unsigned int A2, bool logical);
 
- void (*SetCPUCallback)(void (*callb)(uint32 PC));
+ // 'bpoint' to callb() will be true if the instruction at this PC triggered a breakpoint(or previous instruction maybe in some
+ // more complex breakpoint cases where we don't have a time machine?).
+ //
+ // If 'continuous' is true, then callb() is called for every instruction; if it's false, it's only called for instructions
+ // that trigger breakpoints AND all instructions afterwards until this function is called with 'continuous' set to false again.
+ //
+ // This function IS legal to call from within callb(), and even necessary for good performance.
+ //
+ void (*SetCPUCallback)(void (*callb)(uint32 PC, bool bpoint), bool continuous);
 
- void (*SetBPCallback)(void (*callb)(uint32 PC));
+ void (*EnableBranchTrace)(bool enable);
 
  std::vector<BranchTraceResult> (*GetBranchTrace)(void);
 
@@ -183,7 +185,7 @@ typedef struct
 // An address space should only be added during Load() or LoadCD().
 int ASpace_Add(void (*gasb)(const char *name, uint32 Address, uint32 Length, uint8 *Buffer),
         void (*pasb)(const char *name, uint32 Address, uint32 Length, uint32 Granularity, bool hl, const uint8 *Buffer), const char *name, const char *long_name,
-        uint32 TotalBits, uint32 NP2Size = 0, bool IsSegmented = 0, uint32 SegmentBits = 0, uint32 OffsetBits = 0, uint32 BitsOverlapped = 0);
+        uint32 TotalBits, uint32 NP2Size = 0);
 
 int ASpace_Add(const AddressSpaceType &);
 
