@@ -15,11 +15,14 @@ MDFNGI EmulatedDEMO =
  NULL,
  NULL,
  CloseGame,
- ToggleLayer,
+ SetLayerEnableMask,
  "Background\0Sprites\0",
+ NULL,
+ NULL,
  NULL, //InstallReadPatch,
  NULL, //RemoveReadPatches,
  NULL, //MemRead,
+ false,
  NULL, //StateAction,
  Emulate,
  SetInput,
