@@ -19,6 +19,7 @@
 #include "memdebugger.h"
 #include "debugger.h"
 #include "prompt.h"
+#include "../FileStream.h"
 
 #include <ctype.h>
 #include <trio/trio.h>
@@ -41,8 +42,6 @@ static int CurASpace; // Current address space number
 static bool LowNib = FALSE;
 static bool InEditMode = FALSE;
 static bool InTextArea = FALSE; // Right side text vs left side numbers, selected via TAB
-static bool InMarkMode = FALSE;
-static uint32 MarkModeBegin;
 
 static std::string BSS_String, RS_String, TS_String;
 static char *error_string;
@@ -151,7 +150,7 @@ bool MemDebuggerPrompt::DoBSSearch(uint32 byte_count, uint8 *thebytes)
 	 bool found = FALSE;
 	 uint8 *bbuffer = (uint8 *)calloc(1, byte_count);
 
-	 LockGameMutex(1);
+	 
          do
          {
           ASpace->GetAddressSpaceBytes(ASpace->name, a, byte_count, bbuffer);
@@ -163,7 +162,7 @@ bool MemDebuggerPrompt::DoBSSearch(uint32 byte_count, uint8 *thebytes)
           }
           a = (a + 1) % zemod;
          } while(a != start_a);
-	 LockGameMutex(0);
+	 
 
 	 free(bbuffer);
 	 return(found);
@@ -177,7 +176,7 @@ bool MemDebuggerPrompt::DoRSearch(uint32 byte_count, uint8 *the_bytes)
 	 bool found = FALSE;
 	 uint8 *bbuffer = (uint8 *)calloc(1, byte_count);
 
-	 LockGameMutex(1);
+	 
          do
          {
           ASpace->GetAddressSpaceBytes(ASpace->name, a, byte_count + 1, bbuffer);
@@ -199,7 +198,7 @@ bool MemDebuggerPrompt::DoRSearch(uint32 byte_count, uint8 *the_bytes)
           }
           a = (a + 1) % zemod;
          } while(a != start_a);
-	 LockGameMutex(0);
+	 
 	 free(bbuffer);
 
 	 return(found);
@@ -277,9 +276,9 @@ void MemDebuggerPrompt::TheEnd(const std::string &pstring)
 	 {
 	  if(ICV_Init(pstring.c_str()))
 	  {
-	   LockGameMutex(1);
+	   
 	   MDFNI_SetSetting(std::string(std::string(CurGame->shortname) + "." + std::string("debugger.memcharenc")).c_str(), pstring.c_str());
-	   LockGameMutex(0);
+	   
 	  }
 	 }
          else if(InPrompt == DumpMem)
@@ -288,7 +287,7 @@ void MemDebuggerPrompt::TheEnd(const std::string &pstring)
           char fname[256];
 	  bool acceptable = FALSE;
 
-	  
+
           if(trio_sscanf(pstring.c_str(), "%08x %08x %255[^\r\n]", &A1, &A2, fname) == 3)
 	   acceptable = TRUE;
           else if(trio_sscanf(pstring.c_str(), "%08x +%08x %255[^\r\n]", &A1, &tmpsize, fname) == 3)
@@ -296,16 +295,18 @@ void MemDebuggerPrompt::TheEnd(const std::string &pstring)
 	   acceptable = TRUE;
 	   A2 = A1 + tmpsize - 1;
 	  }
+
 	  if(acceptable)
           {
-           FILE *fp = fopen(fname, "wb");
-           if(fp)
-           {
-	    uint8 write_buffer[256];
-	    uint64 a = A1;
+           
+	   try
+	   {
+	    FileStream fp(fname, FileStream::MODE_WRITE);
+            uint8 write_buffer[256];
+            uint64 a = A1;
+
 	    //printf("%08x %08x\n", A1, A2);
 
-            LockGameMutex(1);
 	    while(a <= A2)
 	    {
              size_t to_write;
@@ -314,23 +315,22 @@ void MemDebuggerPrompt::TheEnd(const std::string &pstring)
 
 	     ASpace->GetAddressSpaceBytes(ASpace->name, a, to_write, write_buffer);
 
-	     if(fwrite(write_buffer, 1, to_write, fp) != to_write)
-	     {
-              error_string = trio_aprintf("File write error: %m", errno);
-              error_time = SDL_GetTicks();
-	      break;
-	     }
+	     fp.write(write_buffer, to_write);
 	     a += to_write;
 	    }
-            LockGameMutex(0);
-            fclose(fp);
            }
-	   else
+	   catch(std::exception &e)
 	   {
-            error_string = trio_aprintf("File open error: %m", errno);
+            error_string = trio_aprintf("%s", e.what());
             error_time = SDL_GetTicks();
 	   }
+           
           }
+	  else
+	  {
+	   error_string = trio_aprintf("Invalid memory dump specification.");
+	   error_time = SDL_GetTicks();
+	  }
          }
          else if(InPrompt == LoadMem)
          {
@@ -338,7 +338,6 @@ void MemDebuggerPrompt::TheEnd(const std::string &pstring)
           char fname[256];
           bool acceptable = FALSE;
 
-
           if(trio_sscanf(pstring.c_str(), "%08x %08x %255[^\r\n]", &A1, &A2, fname) == 3)
            acceptable = TRUE;
           else if(trio_sscanf(pstring.c_str(), "%08x +%08x %255[^\r\n]", &A1, &tmpsize, fname) == 3)
@@ -346,13 +345,14 @@ void MemDebuggerPrompt::TheEnd(const std::string &pstring)
            acceptable = TRUE;
            A2 = A1 + tmpsize - 1;
           }
+
           if(acceptable)
           {
-           FILE *fp = fopen(fname, "rb");
-           if(fp)
-           {
-            LockGameMutex(1);
+	   
 
+	   try
+	   {
+            FileStream fp(fname, FileStream::MODE_READ);
 	    uint8 read_buffer[256];
 	    uint64 a = A1;
 
@@ -364,7 +364,7 @@ void MemDebuggerPrompt::TheEnd(const std::string &pstring)
 	     to_read = A2 - a + 1;
 	     if(to_read > 256) to_read = 256;
 
-	     read_len = fread(read_buffer, 1, to_read, fp);
+	     read_len = fp.read(read_buffer, to_read, false);
 
 	     if(read_len > 0)
               ASpace->PutAddressSpaceBytes(ASpace->name, a, read_len, 1, TRUE, read_buffer);
@@ -373,27 +373,23 @@ void MemDebuggerPrompt::TheEnd(const std::string &pstring)
 
 	     if(read_len != to_read)
 	     {
-	      if(ferror(fp))
-	      {
-	       error_string = trio_aprintf("File read error: %m", errno);
-       	       error_time = SDL_GetTicks();
-              }
-	      else if(feof(fp))
-	      {
-	       error_string = trio_aprintf("Warning: unexpected EOF(short by %08x byte(s))", A2 - a + 1);
-	       error_time = SDL_GetTicks();
-	      }
+	      error_string = trio_aprintf("Warning: unexpected EOF(short by %08x byte(s))", A2 - a + 1);
+	      error_time = SDL_GetTicks();
 	      break;
 	     }
 	    }
-            LockGameMutex(0);
-            fclose(fp);
            }
-	   else
+	   catch(std::exception &e)
 	   {
-            error_string = trio_aprintf("File open error: %m", errno);
+            error_string = trio_aprintf("%s", e.what());
             error_time = SDL_GetTicks();
 	   }
+	   
+          }
+          else
+          {
+           error_string = trio_aprintf("Invalid memory load specification.");
+           error_time = SDL_GetTicks();
           }
          }
 	 else if(InPrompt == TextSearch)
@@ -482,37 +478,35 @@ void MemDebuggerPrompt::TheEnd(const std::string &pstring)
 
 static MemDebuggerPrompt *myprompt = NULL;
 
-// Call this function from either thread.
+// Call this function from the game thread.
 void MemDebugger_SetActive(bool newia)
 {
  if(CurGame->Debugger)
  {
-  LockGameMutex(1);
-
   IsActive = newia;
   if(!newia)
   {
    InEditMode = FALSE;
    LowNib = FALSE;
   }
-  LockGameMutex(0);
  }
 }
 
-#define MK_COLOR_A(r,g,b,a) ( ((a)<<surface->format->Ashift) | ((r)<<surface->format->Rshift) | ((g) << surface->format->Gshift) | ((b) << surface->format->Bshift))
+#define MK_COLOR_A(r,g,b,a) (pf_cache.MakeColor(r, g, b, a))
 
-// Call this function from the main thread
-void MemDebugger_Draw(SDL_Surface *surface, const SDL_Rect *rect, const SDL_Rect *screen_rect)
+// Call this function from the game thread
+void MemDebugger_Draw(MDFN_Surface *surface, const MDFN_Rect *rect, const MDFN_Rect *screen_rect)
 {
  if(!IsActive) return;
 
- uint32 * pixels = (uint32 *)surface->pixels;
- uint32 pitch32 = surface->pitch >> 2;
+ const MDFN_PixelFormat pf_cache = surface->format;
+ uint32 *pixels = surface->pixels;
+ uint32 pitch32 = surface->pitchinpix;
  const uint64 zemod = SizeCache[CurASpace];
 
- LockGameMutex(1);
+ 
 
- DrawTextTrans(pixels, surface->pitch, rect->w, (UTF8*)ASpace->long_name, MK_COLOR_A(0x20, 0xFF, 0x20, 0xFF), 1, 1);
+ DrawTextTrans(pixels, surface->pitchinpix << 2, rect->w, (UTF8*)ASpace->long_name, MK_COLOR_A(0x20, 0xFF, 0x20, 0xFF), 1, 1);
  pixels += 10 * pitch32;
 
  uint32 A;
@@ -554,7 +548,7 @@ void MemDebugger_Draw(SDL_Surface *surface, const SDL_Rect *rect, const SDL_Rect
   if(Ameow == (ASpacePos[CurASpace] & ~0xF))
    addr_color = MK_COLOR_A(0x80, 0xB0, 0xFF, 0xFF);
 
-  alen = DrawTextTrans(pixels, surface->pitch, rect->w, (UTF8*)abuf, addr_color, 0, 1);
+  alen = DrawTextTrans(pixels, surface->pitchinpix << 2, rect->w, (UTF8*)abuf, addr_color, 0, 1);
   alen += 3;
 
   for(int x = 0; x < 16; x++)
@@ -576,22 +570,6 @@ void MemDebugger_Draw(SDL_Surface *surface, const SDL_Rect *rect, const SDL_Rect
 
    test_match_pos = ASpacePos[CurASpace] % zemod;
 
-   if(InMarkMode)
-   {
-    if(MarkModeBegin < test_match_pos && Ameow < test_match_pos && Ameow >= MarkModeBegin)
-    {
-     if(InTextArea)
-     {
-      acolor = MK_COLOR_A(0xFF, 0x00, 0x00, 0xFF);
-      bcolor = MK_COLOR_A(0xFF, 0x80, 0x80, 0xFF);
-     }
-     else
-     { 
-      acolor = MK_COLOR_A(0xFF, 0x80, 0x80, 0xFF);
-      bcolor = MK_COLOR_A(0xFF, 0x00, 0x00, 0xFF);
-     }
-    }
-   }
    if(Ameow == test_match_pos)
    {
     if(InEditMode)
@@ -604,7 +582,7 @@ void MemDebugger_Draw(SDL_Surface *surface, const SDL_Rect *rect, const SDL_Rect
       else
        pix_offset += (LowNib ? 5 : 0) + x * 12;
 
-      DrawTextTrans(pixels + pix_offset, surface->pitch, rect->w, (UTF8*)"▉", MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF), 0, 1);
+      DrawTextTrans(pixels + pix_offset, surface->pitchinpix << 2, rect->w, (UTF8*)"▉", MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF), 0, 1);
      }
     if(InTextArea)
     {
@@ -619,10 +597,10 @@ void MemDebugger_Draw(SDL_Surface *surface, const SDL_Rect *rect, const SDL_Rect
    }
 
    // hex display
-   DrawTextTrans(pixels + alen + x * 12, surface->pitch, rect->w, (UTF8*)quickbuf, bcolor, 0, 1);
+   DrawTextTrans(pixels + alen + x * 12, surface->pitchinpix << 2, rect->w, (UTF8*)quickbuf, bcolor, 0, 1);
 
    // ASCII display
-   DrawTextTrans(pixels + alen + 16 * 12 + x * 5, surface->pitch, rect->w, (UTF8 *)ascii_str, acolor, 0, 1);
+   DrawTextTrans(pixels + alen + 16 * 12 + x * 5, surface->pitchinpix << 2, rect->w, (UTF8 *)ascii_str, acolor, 0, 1);
    Ameow++;
   }
   pixels += 9 * pitch32;
@@ -650,8 +628,8 @@ void MemDebugger_Draw(SDL_Surface *surface, const SDL_Rect *rect, const SDL_Rect
   uint32 cpplen;
   uint32 cplen;
 
-  cpplen = DrawTextTrans(pixels, surface->pitch, rect->w, (UTF8*)"Cursor position: ", MK_COLOR_A(0xa0, 0xa0, 0xFF, 0xFF), 0, 1);
-  cplen = DrawTextTrans(pixels + cpplen, surface->pitch, rect->w, (UTF8*)cpstr , MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF), 0, 1);
+  cpplen = DrawTextTrans(pixels, surface->pitchinpix << 2, rect->w, (UTF8*)"Cursor position: ", MK_COLOR_A(0xa0, 0xa0, 0xFF, 0xFF), 0, 1);
+  cplen = DrawTextTrans(pixels + cpplen, surface->pitchinpix << 2, rect->w, (UTF8*)cpstr , MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF), 0, 1);
   if(GoGoPowerDD[CurASpace])
   {
    char ggddstr[32];
@@ -663,14 +641,14 @@ void MemDebugger_Draw(SDL_Surface *surface, const SDL_Rect *rect, const SDL_Rect
    else
     trio_snprintf(ggddstr, 32, "%08X", curpos - GoGoPowerDD[CurASpace]);
 
-   DrawTextTrans(pixels + cpplen + cplen + 8, surface->pitch, rect->w, (UTF8*)ggddstr, MK_COLOR_A(0xFF, 0x80, 0x80, 0xFF), 0, 1);
+   DrawTextTrans(pixels + cpplen + cplen + 8, surface->pitchinpix << 2, rect->w, (UTF8*)ggddstr, MK_COLOR_A(0xFF, 0x80, 0x80, 0xFF), 0, 1);
   }
   pixels += 5 + 10 * pitch32;
 
   tmpval = zebytes[0];
   trio_snprintf(cpstr, 32, "%02x(%u, %d)", tmpval, (uint8)tmpval, (int8)tmpval);
-  cpplen = DrawTextTrans(pixels, surface->pitch, rect->w, (UTF8*)"1-byte value: ", MK_COLOR_A(0xA0, 0xA0, 0xFF, 0xFF), 0, 1);
-  DrawTextTrans(pixels + cpplen, surface->pitch, rect->w, (UTF8*)cpstr , MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF), 0, 1);
+  cpplen = DrawTextTrans(pixels, surface->pitchinpix << 2, rect->w, (UTF8*)"1-byte value: ", MK_COLOR_A(0xA0, 0xA0, 0xFF, 0xFF), 0, 1);
+  DrawTextTrans(pixels + cpplen, surface->pitchinpix << 2, rect->w, (UTF8*)cpstr , MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF), 0, 1);
 
   pixels += 10 * pitch32;
 
@@ -681,9 +659,18 @@ void MemDebugger_Draw(SDL_Surface *surface, const SDL_Rect *rect, const SDL_Rect
    uint8 waveform[wf_size];
    const int32 pcm_max = (1 << ASpace->WaveBits) - 1;
 
-   DrawTextTrans(pixels - 5, surface->pitch, rect->w, (UTF8 *)"Full waveform:", MK_COLOR_A(0xA0, 0xA0, 0xFF, 0xFF), 0, 1);
+   DrawTextTrans(pixels - 5, surface->pitchinpix << 2, rect->w, (UTF8 *)"Full waveform:", MK_COLOR_A(0xA0, 0xA0, 0xFF, 0xFF), 0, 1);
    pixels += 9 * pitch32;
-   MDFN_DrawRectangleFill(pixels, surface->pitch >> 2, 0, 0, MK_COLOR_A(0xA0,0xA0,0xA0,0xFF), MK_COLOR_A(0,0,0,0xFF), 2 + wf_size * 2 + 2, 2 + (pcm_max + 1) + 2);
+
+   // BLECK, FIXME to not be so crazy.
+   {
+    uint32 tx, ty;
+
+    tx = (pixels - surface->pixels) % surface->pitchinpix;
+    ty = (pixels - surface->pixels) / surface->pitchinpix;
+
+    MDFN_DrawFillRect(surface, tx, ty, 2 + wf_size * 2 + 2, 2 + (pcm_max + 1) + 2, MK_COLOR_A(0xA0,0xA0,0xA0,0xFF), MK_COLOR_A(0,0,0,0xFF));
+   }
 
    ASpace->GetAddressSpaceBytes(ASpace->name, 0, wf_size, waveform);
 
@@ -699,39 +686,39 @@ void MemDebugger_Draw(SDL_Surface *surface, const SDL_Rect *rect, const SDL_Rect
     delta = current - previous;
 
     for(int y = previous; y != current; y += delta / abs(delta))
-     pixels[2 + i * 2 + 0 + (2 + (pcm_max - y)) * (surface->pitch >> 2)] = MK_COLOR_A(0x00,0xA0,0x00,0xFF);
+     pixels[2 + i * 2 + 0 + (2 + (pcm_max - y)) * surface->pitchinpix] = MK_COLOR_A(0x00,0xA0,0x00,0xFF);
 
-    pixels[2 + i * 2 + 0 + (2 + (pcm_max - current)) * (surface->pitch >> 2)] = MK_COLOR_A(0x00,0xA0,0x00,0xFF);
-    pixels[2 + i * 2 + 1 + (2 + (pcm_max - current)) * (surface->pitch >> 2)] = MK_COLOR_A(0x00,0xA0,0x00,0xFF);
+    pixels[2 + i * 2 + 0 + (2 + (pcm_max - current)) * surface->pitchinpix] = MK_COLOR_A(0x00,0xA0,0x00,0xFF);
+    pixels[2 + i * 2 + 1 + (2 + (pcm_max - current)) * surface->pitchinpix] = MK_COLOR_A(0x00,0xA0,0x00,0xFF);
    }
   }
   else
   {
   tmpval = zebytes[0] | (zebytes[1] << 8);
   trio_snprintf(cpstr, 32, "%04x(%u, %d)", tmpval, (uint16)tmpval, (int16)tmpval);
-  cpplen = DrawTextTrans(pixels, surface->pitch, rect->w, (UTF8*)"2-byte value(LSB): ", MK_COLOR_A(0xA0, 0xA0, 0xFF, 0xFF), 0, 1);
-  DrawTextTrans(pixels + cpplen, surface->pitch, rect->w, (UTF8*)cpstr , MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF), 0, 1);
+  cpplen = DrawTextTrans(pixels, surface->pitchinpix << 2, rect->w, (UTF8*)"2-byte value(LSB): ", MK_COLOR_A(0xA0, 0xA0, 0xFF, 0xFF), 0, 1);
+  DrawTextTrans(pixels + cpplen, surface->pitchinpix << 2, rect->w, (UTF8*)cpstr , MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF), 0, 1);
 
   pixels += 10 * pitch32;
   tmpval = zebytes[0] | (zebytes[1] << 8) | (zebytes[2] << 16) | (zebytes[3] << 24);
   trio_snprintf(cpstr, 32, "%08x(%u, %d)", tmpval, (uint32)tmpval, (int32)tmpval);
-  cpplen = DrawTextTrans(pixels, surface->pitch, rect->w, (UTF8*)"4-byte value(LSB): ", MK_COLOR_A(0xA0, 0xA0, 0xFF, 0xFF), 0, 1);
-  DrawTextTrans(pixels + cpplen, surface->pitch, rect->w, (UTF8*)cpstr , MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF), 0, 1);
+  cpplen = DrawTextTrans(pixels, surface->pitchinpix << 2, rect->w, (UTF8*)"4-byte value(LSB): ", MK_COLOR_A(0xA0, 0xA0, 0xFF, 0xFF), 0, 1);
+  DrawTextTrans(pixels + cpplen, surface->pitchinpix << 2, rect->w, (UTF8*)cpstr , MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF), 0, 1);
 
   pixels += 10 * pitch32;
   tmpval = zebytes[1] | (zebytes[0] << 8);
   trio_snprintf(cpstr, 32, "%04x(%u, %d)", tmpval, (uint16)tmpval, (int16)tmpval);
-  cpplen = DrawTextTrans(pixels, surface->pitch, rect->w, (UTF8*)"2-byte value(MSB): ", MK_COLOR_A(0xA0, 0xA0, 0xFF, 0xFF), 0, 1);
-  DrawTextTrans(pixels + cpplen, surface->pitch, rect->w, (UTF8*)cpstr , MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF), 0, 1);
+  cpplen = DrawTextTrans(pixels, surface->pitchinpix << 2, rect->w, (UTF8*)"2-byte value(MSB): ", MK_COLOR_A(0xA0, 0xA0, 0xFF, 0xFF), 0, 1);
+  DrawTextTrans(pixels + cpplen, surface->pitchinpix << 2, rect->w, (UTF8*)cpstr , MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF), 0, 1);
 
   pixels += 10 * pitch32;
   tmpval = zebytes[3] | (zebytes[2] << 8) | (zebytes[1] << 16) | (zebytes[0] << 24);
   trio_snprintf(cpstr, 32, "%08x(%u, %d)", tmpval, (uint32)tmpval, (int32)tmpval);
-  cpplen = DrawTextTrans(pixels, surface->pitch, rect->w, (UTF8*)"4-byte value(MSB): ", MK_COLOR_A(0xA0, 0xA0, 0xFF, 0xFF), 0, 1);
-  DrawTextTrans(pixels + cpplen, surface->pitch, rect->w, (UTF8*)cpstr , MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF), 0, 1);
+  cpplen = DrawTextTrans(pixels, surface->pitchinpix << 2, rect->w, (UTF8*)"4-byte value(MSB): ", MK_COLOR_A(0xA0, 0xA0, 0xFF, 0xFF), 0, 1);
+  DrawTextTrans(pixels + cpplen, surface->pitchinpix << 2, rect->w, (UTF8*)cpstr , MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF), 0, 1);
 
   trio_snprintf(cpstr, 32, "%s text: ", GameCode);
-  cpplen = DrawTextTrans(pixels + 10 * pitch32, surface->pitch, rect->w, (UTF8*)cpstr, MK_COLOR_A(0xA0, 0xA0, 0xFF, 0xFF), 0, MDFN_FONT_5x7);
+  cpplen = DrawTextTrans(pixels + 10 * pitch32, surface->pitchinpix << 2, rect->w, (UTF8*)cpstr, MK_COLOR_A(0xA0, 0xA0, 0xFF, 0xFF), 0, MDFN_FONT_5x7);
 
   {
    char rawbuf[64];
@@ -749,13 +736,13 @@ void MemDebugger_Draw(SDL_Surface *surface, const SDL_Rect *rect, const SDL_Rect
    iconv(ict_to_utf8, (ICONV_CONST char **)&inbuf, &ibl, &outbuf, &obl);
    textbuf[obl_start - obl] = 0;
 
-   DrawTextTrans(pixels + 8 * pitch32 + cpplen, surface->pitch, rect->w - cpplen - 13, (UTF8*)textbuf, MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF), 0, MDFN_FONT_9x18_18x18);
+   DrawTextTrans(pixels + 8 * pitch32 + cpplen, surface->pitchinpix << 2, rect->w - cpplen - 13, (UTF8*)textbuf, MK_COLOR_A(0xFF, 0xFF, 0xFF, 0xFF), 0, MDFN_FONT_9x18_18x18);
   }
 
  }
 
  }
- LockGameMutex(0);
+ 
 
  if(InPrompt)
   myprompt->Draw(surface, rect);
@@ -774,7 +761,7 @@ void MemDebugger_Draw(SDL_Surface *surface, const SDL_Rect *rect, const SDL_Rect
   }
   else
   {
-   DrawTextTrans((uint32*)surface->pixels + (rect->h - 7) * pitch32, surface->pitch, rect->w, (UTF8*)error_string, MK_COLOR_A(0xFF, 0x00, 0x00, 0xFF), 1, 1);
+   DrawTextTrans((uint32*)surface->pixels + (rect->h - 7) * pitch32, surface->pitchinpix << 2, rect->w, (UTF8*)error_string, MK_COLOR_A(0xFF, 0x00, 0x00, 0xFF), 1, 1);
   }
  }
 }
@@ -797,7 +784,7 @@ static void ChangePos(int64 delta)
 
 static void DoCrazy(void)
 {
- LockGameMutex(1);
+ 
 
  uint32 start = ASpacePos[CurASpace];
  uint32 A = ASpacePos[CurASpace];
@@ -861,10 +848,10 @@ static void DoCrazy(void)
  fprintf(fp, "%08x %08x\n", start, ASpacePos[CurASpace]);
  fclose(fp);
 
- LockGameMutex(0);
+ 
 }
 
-// Call this from the main thread
+// Call this from the game thread
 int MemDebugger_Event(const SDL_Event *event)
 {
  if(!InPrompt && myprompt)
@@ -902,11 +889,11 @@ int MemDebugger_Event(const SDL_Event *event)
 	 size_t result = iconv(ict_utf16_to_game, (ICONV_CONST char **)&inbuf, &ibl, &outbuf, &obl);
 	 if(result != (size_t)-1)
 	 {
-          to_write_len = 16 - obl;
+          to_write_len = obl_start - obl;
 
-	  LockGameMutex(1);
+	  
 	  ASpace->PutAddressSpaceBytes(ASpace->name, ASpacePos[CurASpace], to_write_len, 1, TRUE, to_write);
-	  LockGameMutex(0);
+	  
 
 	  LowNib = 0;
 	  ChangePos(to_write_len);
@@ -923,12 +910,12 @@ int MemDebugger_Event(const SDL_Event *event)
          else if(event->key.keysym.sym >= SDLK_a && event->key.keysym.sym <= SDLK_f)
           tc = 0xA + event->key.keysym.sym - SDLK_a;
 
-	 LockGameMutex(1);
+	 
          ASpace->GetAddressSpaceBytes(ASpace->name, ASpacePos[CurASpace], 1, &meowbyte);
          meowbyte &= 0xF << ((LowNib) * 4);
          meowbyte |= tc << ((!LowNib) * 4);
          ASpace->PutAddressSpaceBytes(ASpace->name, ASpacePos[CurASpace], 1, 1, TRUE, &meowbyte);
-	 LockGameMutex(0);
+	 
 
          LowNib = !LowNib;
          if(!LowNib)
@@ -938,9 +925,9 @@ int MemDebugger_Event(const SDL_Event *event)
 	{
 	 default: break;
 
-         case SDLK_MINUS: Debugger_ModOpacity(-8);
+         case SDLK_MINUS: Debugger_GT_ModOpacity(-8);
                           break;
-         case SDLK_EQUALS: Debugger_ModOpacity(8);
+         case SDLK_EQUALS: Debugger_GT_ModOpacity(8);
                            break;
 
 	 case SDLK_TAB:
@@ -1050,19 +1037,7 @@ int MemDebugger_Event(const SDL_Event *event)
 			LowNib = FALSE;
 			break;
 
-	 case SDLK_b: //InMarkMode = !InMarkMode;
-		      //if(InMarkMode)
- 		      // MarkModeBegin = ASpacePos[CurASpace];
-		      DoCrazy();
-		      break;
-
-	 case SDLK_e: if(InMarkMode)
-		      {
-		       FILE *fp = fopen("markers.txt", "ab");
-		       fprintf(fp, "%08x %08x\n", MarkModeBegin, ASpacePos[CurASpace]);
-		       fclose(fp);
-		       InMarkMode = FALSE;
-		      }
+	 case SDLK_b: DoCrazy();
 		      break;
 	}
 	break;
