@@ -19,93 +19,29 @@
 
 #include <string.h>
 #include <trio/trio.h>
+#include <math.h>
 
 #include "video.h"
 #include "opengl.h"
 #include "shader.h"
 
-glGetError_Func p_glGetError;
-glBindTexture_Func p_glBindTexture;
-glColorTableEXT_Func p_glColorTableEXT;
-glTexImage2D_Func p_glTexImage2D;
-glBegin_Func p_glBegin;
-glVertex2f_Func p_glVertex2f;
-glTexCoord2f_Func p_glTexCoord2f;
-glEnd_Func p_glEnd;
-glEnable_Func p_glEnable;
-glBlendFunc_Func p_glBlendFunc;
-glGetString_Func p_glGetString;
-glViewport_Func p_glViewport;
-glGenTextures_Func p_glGenTextures;
-glDeleteTextures_Func p_glDeleteTextures;
-glTexParameteri_Func p_glTexParameteri;
-glClearColor_Func p_glClearColor;
-glLoadIdentity_Func p_glLoadIdentity;
-glClear_Func p_glClear;
-glMatrixMode_Func p_glMatrixMode;
-glDisable_Func p_glDisable;
-glPixelStorei_Func p_glPixelStorei;
-glTexSubImage2D_Func p_glTexSubImage2D;
-glFinish_Func p_glFinish;
-glOrtho_Func p_glOrtho;
-glPixelTransferf_Func p_glPixelTransferf;
-glColorMask_Func p_glColorMask;
-glTexEnvf_Func p_glTexEnvf;
-glGetIntegerv_Func p_glGetIntegerv;
-glTexGend_Func p_glTexGend;
-glDrawPixels_Func p_glDrawPixels;
-glRasterPos2i_Func p_glRasterPos2i;
-glPixelZoom_Func p_glPixelZoom;
-glGetTexLevelParameteriv_Func p_glGetTexLevelParameteriv;
-glAccum_Func p_glAccum;
-glClearAccum_Func p_glClearAccum;
-glPushMatrix_Func p_glPushMatrix;
-glPopMatrix_Func p_glPopMatrix;
-glRotated_Func p_glRotated;
+void OpenGL_Blitter::ReadPixels(MDFN_Surface *surface, const MDFN_Rect *rect)
+{
+ p_glPixelStorei(GL_UNPACK_ROW_LENGTH, surface->pitchinpix);
+ p_glReadPixels(rect->x, gl_screen_h - rect->h - rect->y, rect->w, rect->h, PixelFormat, PixelType, surface->pixels);
 
-#if MDFN_WANT_OPENGL_SHADERS
-glCreateShaderObjectARB_Func p_glCreateShaderObjectARB;
-glShaderSourceARB_Func p_glShaderSourceARB;
-glCompileShaderARB_Func p_glCompileShaderARB;
-glCreateProgramObjectARB_Func p_glCreateProgramObjectARB;
-glAttachObjectARB_Func p_glAttachObjectARB;
-glLinkProgramARB_Func p_glLinkProgramARB;
-glUseProgramObjectARB_Func p_glUseProgramObjectARB;
-glUniform1fARB_Func p_glUniform1fARB;
-glUniform2fARB_Func p_glUniform2fARB;
-glUniform3fARB_Func p_glUniform3fARB;
-glUniform1iARB_Func p_glUniform1iARB;
-glUniform2iARB_Func p_glUniform2iARB;
-glUniform3iARB_Func p_glUniform3iARB;
-glActiveTextureARB_Func p_glActiveTextureARB;
-glGetInfoLogARB_Func p_glGetInfoLogARB;
-glGetUniformLocationARB_Func p_glGetUniformLocationARB;
-glDeleteObjectARB_Func p_glDeleteObjectARB;
-glDetachObjectARB_Func p_glDetachObjectARB;
-glGetObjectParameterivARB_Func p_glGetObjectParameterivARB;
-#endif
+ for(int y = 0; y < surface->h / 2; y++)
+ {
+  uint32 tmp_buffer[surface->w];
 
-static uint32 MaxTextureSize; // Maximum power-of-2 texture width/height(we assume they're the same, and if they're not, this is set to the lower value of the two)
-static bool SupportNPOT; 		// True if the OpenGL implementation supports non-power-of-2-sized textures
-static GLenum PixelFormat;		// For glTexSubImage2D()
-static GLenum PixelType;		// For glTexSubImage2D()
+  memcpy(tmp_buffer, &surface->pixels[y * surface->pitchinpix], surface->pitchinpix * sizeof(uint32));
+  memcpy(&surface->pixels[y * surface->pitchinpix], &surface->pixels[(surface->h - 1 - y) * surface->pitchinpix], surface->pitchinpix * sizeof(uint32));
+  memcpy(&surface->pixels[(surface->h - 1 - y) * surface->pitchinpix], tmp_buffer, surface->pitchinpix * sizeof(uint32));
+ }
+}
 
-static SDL_Surface *gl_screen = NULL;
-static GLuint textures[4] = {0, 0, 0, 0}; // emulated fb, scanlines, osd, raw(netplay)
-static GLuint rgb_mask = 0; // TODO:  RGB mask texture for LCD RGB triad simulation
 
-static bool using_scanlines = 0;
-static unsigned int last_w, last_h;
-
-static uint32 OSDLastWidth, OSDLastHeight;
-
-static bool UsingShader = FALSE; // TRUE if we're using a pixel shader.
-static int UsingIP;	// See VIDEOIP_* enums defined in video.h
-
-static uint32 *DummyBlack = NULL; // Black/Zeroed image data for cleaning textures
-static uint32 DummyBlackSize;
-
-void BlitOpenGLRaw(SDL_Surface *surface, const SDL_Rect *rect, const SDL_Rect *dest_rect)
+void OpenGL_Blitter::BlitRaw(MDFN_Surface *surface, const MDFN_Rect *rect, const MDFN_Rect *dest_rect, const bool source_alpha)
 {
  unsigned int tmpwidth;
  unsigned int tmpheight;
@@ -123,42 +59,47 @@ void BlitOpenGLRaw(SDL_Surface *surface, const SDL_Rect *rect, const SDL_Rect *d
 
  if(tmpwidth > MaxTextureSize || tmpheight > MaxTextureSize)
  {
-  SDL_Rect neo_rect;
-  SDL_Rect neo_dest_rect;
+  MDFN_Rect neo_rect;
+  MDFN_Rect neo_dest_rect;
 
-  for(uint32 xseg = 0; xseg < rect->w; xseg += MaxTextureSize)
+  for(int32 xseg = 0; xseg < rect->w; xseg += MaxTextureSize)
   {
-   for(uint32 yseg = 0; yseg < rect->h; yseg += MaxTextureSize)
+   for(int32 yseg = 0; yseg < rect->h; yseg += MaxTextureSize)
    {
     neo_rect.x = rect->x + xseg;
     neo_rect.w = rect->w - xseg;
+
     if(neo_rect.w > MaxTextureSize)
      neo_rect.w = MaxTextureSize;
+
     neo_rect.y = rect->y + yseg;
     neo_rect.h = rect->h - yseg;
+
     if(neo_rect.h > MaxTextureSize)
      neo_rect.h = MaxTextureSize;
 
-
     neo_dest_rect.x = dest_rect->x + xseg * dest_rect->w / rect->w;
     neo_dest_rect.y = dest_rect->y + yseg * dest_rect->h / rect->h;
     neo_dest_rect.w = neo_rect.w * dest_rect->w / rect->w;
     neo_dest_rect.h = neo_rect.h * dest_rect->h / rect->h;
-    BlitOpenGLRaw(surface, &neo_rect, &neo_dest_rect);
+    BlitRaw(surface, &neo_rect, &neo_dest_rect, source_alpha);
    }
   }
-
  }
  else
  {
-  p_glEnable(GL_BLEND);
-  p_glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+  // Don't move the source_alpha stuff out of this else { }, otherwise it will break the recursion necessary to work around maximum texture size limits.
+  if(source_alpha)
+  {
+   p_glEnable(GL_BLEND);
+   p_glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+  }
 
   p_glBindTexture(GL_TEXTURE_2D, textures[3]);
-  p_glPixelStorei(GL_UNPACK_ROW_LENGTH, surface->pitch >> 2);
+  p_glPixelStorei(GL_UNPACK_ROW_LENGTH, surface->pitchinpix);
 
   p_glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, tmpwidth, tmpheight, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
-  p_glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, rect->w, rect->h, PixelFormat, PixelType, (uint32 *)surface->pixels + rect->x + rect->y * (surface->pitch >> 2));
+  p_glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, rect->w, rect->h, PixelFormat, PixelType, surface->pixels + rect->x + rect->y * surface->pitchinpix);
 
   p_glBegin(GL_QUADS);
 
@@ -176,11 +117,14 @@ void BlitOpenGLRaw(SDL_Surface *surface, const SDL_Rect *rect, const SDL_Rect *d
 
   p_glEnd();
 
-  p_glDisable(GL_BLEND);
+  if(source_alpha)
+  {
+   p_glDisable(GL_BLEND);
+  }
  }
 }
 
-static INLINE void MakeSourceCoords(const SDL_Rect *src_rect, float sc[4][2], const int32 tmpwidth, const int32 tmpheight)
+static INLINE void MakeSourceCoords(const MDFN_Rect *src_rect, float sc[4][2], const int32 tmpwidth, const int32 tmpheight)
 {
  // Upper left
  sc[0][0] = (float)src_rect->x / tmpwidth;		// X
@@ -199,15 +143,15 @@ static INLINE void MakeSourceCoords(const SDL_Rect *src_rect, float sc[4][2], co
  sc[3][1] = (float)(src_rect->y + src_rect->h) / tmpheight;	// Y
 }
 
-static INLINE void MakeDestCoords(const SDL_Rect *dest_rect, int dest_coords[4][2])
+static INLINE void MakeDestCoords(const MDFN_Rect *dest_rect, int dest_coords[4][2], const unsigned rotated)
 {
   signed dco = 0;
 
-  if(CurGame->rotated == MDFN_ROTATE90)
+  if(rotated == MDFN_ROTATE90)
    dco = 1;
-  else if(CurGame->rotated == MDFN_ROTATE270)
+  else if(rotated == MDFN_ROTATE270)
    dco = 3;
-  else if(CurGame->rotated == MDFN_ROTATE180)
+  else if(rotated == MDFN_ROTATE180)
    dco = 2;
 
   // Upper left
@@ -234,7 +178,45 @@ static INLINE void MakeDestCoords(const SDL_Rect *dest_rect, int dest_coords[4][
   //	dest_coords[2][1], dest_coords[3][0], dest_coords[3][1]);
 }
 
-static INLINE void DrawQuad(float src_coords[4][2], int dest_coords[4][2])
+//#define MDFN_TRIANGLE_STRIP_TEST
+
+#ifdef MDFN_TRIANGLE_STRIP_TEST
+INLINE void OpenGL_Blitter::DrawQuad(float src_coords[4][2], int dest_coords[4][2])
+{
+ puts("TRIANGLESSSS");
+#if 0
+  // Lower left
+  p_glTexCoord2f(src_coords[3][0], src_coords[3][1]);
+   p_glVertex2f(dest_coords[3][0], dest_coords[3][1]);
+
+  // Upper left
+  p_glTexCoord2f(src_coords[0][0], src_coords[0][1]);
+   p_glVertex2f(dest_coords[0][0], dest_coords[0][1]);
+
+  // Lower right
+  p_glTexCoord2f(src_coords[2][0], src_coords[2][1]);
+   p_glVertex2f(dest_coords[2][0], dest_coords[2][1]);
+
+#endif
+
+  // Upper right
+  p_glTexCoord2f(src_coords[1][0], src_coords[1][1]);
+   p_glVertex2f(dest_coords[1][0], dest_coords[1][1]);
+
+  // Upper left
+  p_glTexCoord2f(src_coords[0][0], src_coords[0][1]);
+   p_glVertex2f(dest_coords[0][0], dest_coords[0][1]);
+
+  // Lower right
+  p_glTexCoord2f(src_coords[2][0], src_coords[2][1]);
+   p_glVertex2f(dest_coords[2][0], dest_coords[2][1]);
+
+  // Lower left
+  p_glTexCoord2f(src_coords[3][0], src_coords[3][1]);
+   p_glVertex2f(dest_coords[3][0], dest_coords[3][1]);
+}
+#else
+INLINE void OpenGL_Blitter::DrawQuad(float src_coords[4][2], int dest_coords[4][2])
 {
   // Lower left
   p_glTexCoord2f(src_coords[3][0], src_coords[3][1]);
@@ -252,11 +234,12 @@ static INLINE void DrawQuad(float src_coords[4][2], int dest_coords[4][2])
   p_glTexCoord2f(src_coords[0][0], src_coords[0][1]);
    p_glVertex2f(dest_coords[0][0], dest_coords[0][1]);
 }
+#endif
 
-void DrawLinearIP(const unsigned UsingIP, const unsigned rotated, const SDL_Rect *tex_src_rect, const SDL_Rect *dest_rect, const uint32 tmpwidth, const uint32 tmpheight)
+void OpenGL_Blitter::DrawLinearIP(const unsigned UsingIP, const unsigned rotated, const MDFN_Rect *tex_src_rect, const MDFN_Rect *dest_rect, const uint32 tmpwidth, const uint32 tmpheight)
 {
- SDL_Rect tmp_sr = *tex_src_rect;
- SDL_Rect tmp_dr = *dest_rect;
+ MDFN_Rect tmp_sr = *tex_src_rect;
+ MDFN_Rect tmp_dr = *dest_rect;
  float tmp_sc[4][2];
  int tmp_dc[4][2];
 
@@ -314,28 +297,45 @@ void DrawLinearIP(const unsigned UsingIP, const unsigned rotated, const SDL_Rect
   }
 
   MakeSourceCoords(&tmp_sr, tmp_sc, tmpwidth, tmpheight);
-  MakeDestCoords(&tmp_dr, tmp_dc);
+  MakeDestCoords(&tmp_dr, tmp_dc, rotated);
 
   DrawQuad(tmp_sc, tmp_dc);
  }
 }
 
-void BlitOpenGL(SDL_Surface *src_surface, const SDL_Rect *src_rect, const SDL_Rect *dest_rect, const SDL_Rect *original_src_rect, bool alpha_blend)
+void OpenGL_Blitter::Blit(MDFN_Surface *src_surface, const MDFN_Rect *src_rect, const MDFN_Rect *dest_rect, const MDFN_Rect *original_src_rect, int UsingIP, int rotated)
 {
- SDL_Rect tex_src_rect = *src_rect;
+ MDFN_Rect tex_src_rect = *src_rect;
  float src_coords[4][2];
  int dest_coords[4][2];
  unsigned int tmpwidth;
  unsigned int tmpheight;
  uint32 *src_pixies;
 
- src_pixies = (uint32 *)src_surface->pixels + tex_src_rect.x + tex_src_rect.y * (src_surface->pitch >> 2);
+ if(shader)
+ {
+  if(shader->ShaderNeedsBTIP())
+   UsingIP = VIDEOIP_BILINEAR;
+  else
+   UsingIP = VIDEOIP_OFF;
+ }
+
+ if(src_rect->w == 0 || src_rect->h == 0 || dest_rect->w == 0 || dest_rect->h == 0 || original_src_rect->w == 0 || original_src_rect->h == 0)
+ {
+  printf("[BUG] OpenGL blitting nothing? --- %d:%d %d:%d %d:%d\n", src_rect->w, src_rect->h, dest_rect->w, dest_rect->h, original_src_rect->w, original_src_rect->h);
+  return;
+ }
+
+
+ src_pixies = src_surface->pixels + tex_src_rect.x + tex_src_rect.y * src_surface->pitchinpix;
  tex_src_rect.x = 0;
  tex_src_rect.y = 0;
 
- MakeDestCoords(dest_rect, dest_coords);
+ MakeDestCoords(dest_rect, dest_coords, rotated);
 
  p_glBindTexture(GL_TEXTURE_2D, textures[0]);
+ p_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, UsingIP ? GL_LINEAR : GL_NEAREST);
+ p_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, UsingIP ? GL_LINEAR : GL_NEAREST);
 
  if(SupportNPOT)
  {
@@ -368,7 +368,7 @@ void BlitOpenGL(SDL_Surface *src_surface, const SDL_Rect *src_rect, const SDL_Re
    ImageSizeChange = TRUE;
 
   // Only clean up if we're using pixel shaders and/or bilinear interpolation
-  if(ImageSizeChange && (UsingShader || UsingIP))
+  if(ImageSizeChange && (shader || UsingIP))
   {
    uint32 neo_dbs = DummyBlackSize;
 
@@ -416,33 +416,25 @@ void BlitOpenGL(SDL_Surface *src_surface, const SDL_Rect *src_rect, const SDL_Re
 
  MakeSourceCoords(&tex_src_rect, src_coords, tmpwidth, tmpheight);
 
- #if MDFN_WANT_OPENGL_SHADERS
- if(UsingShader)
-  ShaderBegin(src_surface, src_rect, dest_rect, tmpwidth, tmpheight);
- #endif
+ if(shader)
+  shader->ShaderBegin(gl_screen_w, gl_screen_h, src_rect, dest_rect, tmpwidth, tmpheight, round((double)tmpwidth * original_src_rect->w / src_rect->w), round((double)tmpheight * original_src_rect->h / src_rect->h), rotated);
 
- if(alpha_blend)
- {
-  p_glEnable(GL_BLEND);
-  p_glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
- }
-
- p_glPixelStorei(GL_UNPACK_ROW_LENGTH, src_surface->pitch >> 2);
+ p_glPixelStorei(GL_UNPACK_ROW_LENGTH, src_surface->pitchinpix);
 
  p_glTexSubImage2D(GL_TEXTURE_2D, 0, tex_src_rect.x, tex_src_rect.y, tex_src_rect.w, tex_src_rect.h, PixelFormat, PixelType, src_pixies);
 
- bool DoneAgain = 0;
-
- DoAgain:
-
  //
  // Draw texture
  //
+#ifdef MDFN_TRIANGLE_STRIP_TEST
+ p_glBegin(GL_TRIANGLE_STRIP);
+#else
  p_glBegin(GL_QUADS);
+#endif
 
  if(UsingIP == VIDEOIP_LINEAR_X || UsingIP == VIDEOIP_LINEAR_Y)	// Linear interpolation, on one axis
  {
-  DrawLinearIP(UsingIP, CurGame->rotated, &tex_src_rect, dest_rect, tmpwidth, tmpheight);
+  DrawLinearIP(UsingIP, rotated, &tex_src_rect, dest_rect, tmpwidth, tmpheight);
  }
  else	// Regular bilinear or no interpolation.
  {
@@ -451,25 +443,8 @@ void BlitOpenGL(SDL_Surface *src_surface, const SDL_Rect *src_rect, const SDL_Re
 
  p_glEnd();
 
- if(alpha_blend)
- {
-  if(!DoneAgain)
-  {
-   DoneAgain = TRUE;
-   p_glBlendFunc(GL_ONE_MINUS_SRC_ALPHA, GL_ONE);
-
-   goto DoAgain;
-  }
-  else
-  {
-   p_glDisable(GL_BLEND);
-  }
- }
-
- #if MDFN_WANT_OPENGL_SHADERS
- if(UsingShader)
-  ShaderEnd();
- #endif
+ if(shader)
+  shader->ShaderEnd();
 
  if(using_scanlines)
  {
@@ -504,13 +479,33 @@ void BlitOpenGL(SDL_Surface *src_surface, const SDL_Rect *src_rect, const SDL_Re
  //}
 }
 
-void FlipOpenGL(void)
-{
- PumpWrap();
- SDL_GL_SwapBuffers();
-}
 
-void KillOpenGL(void)
+#if 0
+void OpenGL_Blitter::HardSync(uint64 timeout)
+{
+ GLsync s;
+
+ if(SupportARBSync)
+ {
+  if((s = p_glFenceSync(GL_SYNC_GPU_COMMANDS_COMPLETE, 0)) != NULL)
+  {
+   p_glBegin(GL_POINTS);
+   p_glVertex2f(0.0, 0.0);
+   p_glEnd();
+
+   uint32 before = MDFND_GetTime();
+
+   p_glClientWaitSync(s, 0, timeout); //50ULL * 1000 * 1000);	// 50 milliseconds.
+
+   printf("Waited: %u\n", MDFND_GetTime() - before);
+
+   p_glDeleteSync(s);
+  }
+ }
+}
+#endif
+
+void OpenGL_Blitter::Cleanup(void)
 {
  if(textures[0])
   p_glDeleteTextures(4, &textures[0]);
@@ -530,13 +525,16 @@ void KillOpenGL(void)
  }
  DummyBlackSize = 0;
 
- #if MDFN_WANT_OPENGL_SHADERS
- if(UsingShader)
+ if(shader)
  {
-  KillShader();
-  UsingShader = FALSE;
+  delete shader;
+  shader = NULL;
  }
- #endif
+}
+
+OpenGL_Blitter::~OpenGL_Blitter()
+{
+ Cleanup();
 }
 
 static bool CheckExtension(const char *extensions, const char *testval)
@@ -570,15 +568,41 @@ static bool CheckAlternateFormat(const uint32 version_h)
 }
 
 /* Rectangle, left, right(not inclusive), top, bottom(not inclusive). */
-int InitOpenGL(int ipolate, int scanlines, ShaderType pixshader, SDL_Surface *screen, int *rs, int *gs, int *bs, int *as)
+OpenGL_Blitter::OpenGL_Blitter(int scanlines, ShaderType pixshader, const int screen_w, const int screen_h, int *rs, int *gs, int *bs, int *as) : gl_screen_w(screen_w), gl_screen_h(screen_h)
 {
+ try
+ {
  const char *extensions;
  const char *vendor;
  const char *renderer;
  const char *version;
  uint32 version_h;
 
- #define LFG(x) if(!(p_##x = (x##_Func) SDL_GL_GetProcAddress(#x))) { MDFN_PrintError(_("Error getting proc address for: %s\n"), #x); return(0); }
+ MaxTextureSize = 0;
+ SupportNPOT = false;
+ SupportARBSync = false;
+ PixelFormat = 0;
+ PixelType = 0;
+
+ for(unsigned i = 0; i < 4; i++)
+  textures[i] = 0;
+
+ rgb_mask = 0;
+
+ using_scanlines = 0;
+ last_w = 0;
+ last_h = 0;
+
+ OSDLastWidth = 0;
+ OSDLastHeight = 0;
+
+ shader = NULL;
+
+ DummyBlack = NULL;
+ DummyBlackSize = 0;
+
+
+ #define LFG(x) if(!(p_##x = (x##_Func) SDL_GL_GetProcAddress(#x))) { throw MDFN_Error(0, _("Error getting proc address for: %s\n"), #x); }
  #define LFGN(x) p_##x = (x##_Func) SDL_GL_GetProcAddress(#x)
 
  LFG(glGetError);
@@ -619,8 +643,8 @@ int InitOpenGL(int ipolate, int scanlines, ShaderType pixshader, SDL_Surface *sc
  LFG(glPushMatrix);
  LFG(glPopMatrix);
  LFG(glRotated);
-
- gl_screen = screen;
+ LFG(glScalef);
+ LFG(glReadPixels);
 
  vendor = (const char *)p_glGetString(GL_VENDOR);
  renderer = (const char *)p_glGetString(GL_RENDERER);
@@ -644,23 +668,38 @@ int InitOpenGL(int ipolate, int scanlines, ShaderType pixshader, SDL_Surface *sc
  MDFN_indent(1);
 
  SupportNPOT = FALSE;
+ SupportARBSync = false;
 
  if(CheckExtension(extensions, "GL_ARB_texture_non_power_of_two"))
  {
   MDFN_printf(_("GL_ARB_texture_non_power_of_two found.\n"));
   SupportNPOT = TRUE;
  }
+
+ if(CheckExtension(extensions, "GL_ARB_sync"))
+ {
+  MDFN_printf(_("GL_ARB_sync found.\n"));
+  LFG(glFenceSync);
+  LFG(glIsSync);
+  LFG(glDeleteSync);
+  LFG(glClientWaitSync);
+  LFG(glWaitSync);
+  LFG(glGetInteger64v);
+  LFG(glGetSynciv);
+  SupportARBSync = true;
+ }
+
  MDFN_indent(-1);
 
- p_glViewport(0, 0, screen->w, screen->h);
+ // x,y specify LOWER left corner of the viewport.
+ p_glViewport(0, 0, gl_screen_w, gl_screen_h);
 
  p_glGenTextures(4, &textures[0]);
  p_glGenTextures(1, &rgb_mask);
  using_scanlines = 0;
 
- UsingShader = FALSE;
+ shader = NULL;
 
- #if MDFN_WANT_OPENGL_SHADERS
  if(pixshader != SHADER_NONE)
  {
   LFG(glCreateShaderObjectARB);
@@ -676,6 +715,7 @@ int InitOpenGL(int ipolate, int scanlines, ShaderType pixshader, SDL_Surface *sc
   LFG(glUniform1iARB);
   LFG(glUniform2iARB);
   LFG(glUniform3iARB);
+  LFG(glUniformMatrix2fvARB);
   LFG(glActiveTextureARB);
   LFG(glGetInfoLogARB);
   LFG(glGetUniformLocationARB);
@@ -684,16 +724,10 @@ int InitOpenGL(int ipolate, int scanlines, ShaderType pixshader, SDL_Surface *sc
 
   LFG(glGetObjectParameterivARB);
 
-  if(!InitShader(pixshader))
-  {
-   return(0);
-  }
-  UsingShader = TRUE;
-  ipolate = VIDEOIP_OFF; // Disable texture interpolation, otherwise our pixel shaders won't work right.
-  SupportNPOT = 0; 	 // Our pixel shaders don't work right with NPOT textures:  FIXME
+  shader = new OpenGL_Blitter_Shader(this, pixshader);
+  SupportNPOT = 0; 	 // Our pixel shaders don't work right with NPOT textures(for them to do so would probably necessitate rewriting them to use texelFetch)
   p_glActiveTextureARB(GL_TEXTURE0_ARB);
  }
- #endif
 
  // printf here because pixel shader code will set SupportNPOT to 0
 
@@ -747,12 +781,8 @@ int InitOpenGL(int ipolate, int scanlines, ShaderType pixshader, SDL_Surface *sc
 
  p_glBindTexture(GL_TEXTURE_2D, textures[0]);
      
- UsingIP = ipolate;
-
- p_glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
- p_glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
- p_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, UsingIP ? GL_LINEAR : GL_NEAREST);
- p_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, UsingIP ? GL_LINEAR : GL_NEAREST);
+ p_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
+ p_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
 
  p_glBindTexture(GL_TEXTURE_2D, textures[2]);
 
@@ -790,7 +820,7 @@ int InitOpenGL(int ipolate, int scanlines, ShaderType pixshader, SDL_Surface *sc
 
  p_glPixelTransferf(GL_MAP_COLOR, GL_FALSE);
 
- p_glOrtho(0.0, screen->w, screen->h, 0, -1.0, 1.0);
+ p_glOrtho(0, gl_screen_w, gl_screen_h, 0, -1.0, 1.0);
 
  last_w = 0;
  last_h = 0;
@@ -856,11 +886,15 @@ int InitOpenGL(int ipolate, int scanlines, ShaderType pixshader, SDL_Surface *sc
   PixelType = GL_UNSIGNED_INT_8_8_8_8_REV;
   MDFN_printf(_("Using GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV for texture source data.\n"));
  }
-
- return(1);
+ }
+ catch(...)
+ {
+  Cleanup();
+  throw;
+ }
 }
 
-void ClearBackBufferOpenGL(SDL_Surface *screen)
+void OpenGL_Blitter::ClearBackBuffer(void)
 {
  //if(1)
  //{
