@@ -276,6 +276,7 @@ bool MDFNFILE::MakeMemWrapAndClose(void *tz, int type)
     MDFN_PrintError(_("Error reading file: %s"), ene.StrError());
 
     free(f_data);
+    f_data = NULL;
     goto doret;
    }
   #ifdef HAVE_MMAP
@@ -293,7 +294,7 @@ bool MDFNFILE::MakeMemWrapAndClose(void *tz, int type)
    goto doret;
   }
 
-  while((howmany = gzread(tz, f_data + cur_size, cur_alloced - cur_size)) > 0)
+  while((howmany = gzread((gzFile)tz, f_data + cur_size, cur_alloced - cur_size)) > 0)
   {
    cur_size += howmany;
    cur_alloced <<= 1;
@@ -320,7 +321,7 @@ bool MDFNFILE::MakeMemWrapAndClose(void *tz, int type)
   {
    int gzerrnum = 0;
    const char *gzerrstring;
-   if((gzerrstring = gzerror(tz, &gzerrnum)) && gzerrnum != Z_OK && gzerrnum != Z_STREAM_END)
+   if((gzerrstring = gzerror((gzFile)tz, &gzerrnum)) && gzerrnum != Z_OK && gzerrnum != Z_STREAM_END)
    {
     if(gzerrnum != Z_ERRNO)
     {
@@ -337,8 +338,8 @@ bool MDFNFILE::MakeMemWrapAndClose(void *tz, int type)
  }
  else if(type == MDFN_FILETYPE_ZIP)
  {
-  unz_file_info ufo; 
-  unzGetCurrentFileInfo(tz, &ufo, 0, 0, 0, 0, 0, 0);
+  unz_file_info ufo;
+  unzGetCurrentFileInfo((unzFile)tz, &ufo, 0, 0, 0, 0, 0, 0);
 
   f_size = ufo.uncompressed_size;
 
@@ -353,7 +354,7 @@ bool MDFNFILE::MakeMemWrapAndClose(void *tz, int type)
    goto doret;
   }
 
-  unzReadCurrentFile(tz, f_data, ufo.uncompressed_size);
+  unzReadCurrentFile((unzFile)tz, f_data, ufo.uncompressed_size);
  }
 
  ret = TRUE;
@@ -365,12 +366,12 @@ bool MDFNFILE::MakeMemWrapAndClose(void *tz, int type)
  }
  else if(type == MDFN_FILETYPE_GZIP)
  {
-  gzclose(tz);
+  gzclose((gzFile)tz);
  }
  else if(type == MDFN_FILETYPE_ZIP)
  {
-  unzCloseCurrentFile(tz);
-  unzClose(tz);
+  unzCloseCurrentFile((unzFile)tz);
+  unzClose((unzFile)tz);
  }
 
  return(ret);
