@@ -180,7 +180,7 @@ mapperMBC3 gbDataMBC3 = {
   0, // timer latched hours
   0, // timer latched days
   0, // timer latched control
-  (time_t)-1 // last time
+  ~0ULL // last time
 };
 
 void memoryUpdateMBC3Clock()
