@@ -41,11 +41,16 @@ static string FileBase;
 static string FileExt;	/* Includes the . character, as in ".nes" */
 static string FileBaseDirectory;
 
-void MDFNI_SetBaseDirectory(const char *dir)
+void MDFN_SetBaseDirectory(const char *dir)
 {
  BaseDirectory = string(dir);
 }
 
+std::string MDFN_GetBaseDirectory(void)
+{
+ return BaseDirectory;
+}
+
 // Really dumb, maybe we should use boost?
 static bool IsAbsolutePath(const char *path)
 {
@@ -64,13 +69,14 @@ static bool IsAbsolutePath(const char *path)
   return(TRUE);
  }
 
- // FIXME if we add DOS support(HAHAHAHA).
- #if defined(WIN32)
+ #if defined(WIN32) || defined(DOS)
  if((path[0] >= 'a' && path[0] <= 'z') || (path[0] >= 'A' && path[0] <= 'Z'))
+ {
   if(path[1] == ':')
   {
    return(TRUE);
   }
+ }
  #endif
 
  return(FALSE);
@@ -102,6 +108,24 @@ bool MDFN_IsFIROPSafe(const std::string &path)
  if(path.find('/') != string::npos)
   return(false);
 
+#if defined(DOS) || defined(WIN32)
+ //
+ // http://support.microsoft.com/kb/74496
+ //
+ {
+  static const char* dev_names[] = 
+  {
+   "CON", "PRN", "AUX", "CLOCK$", "NUL", "COM1", "COM2", "COM3", "COM4", "LPT1", "LPT2", "LPT3", NULL
+  };
+
+  for(const char** ls = dev_names; *ls != NULL; ls++)
+  {
+   if(!strcasecmp(*ls, path.c_str()))
+    return(false);
+  }
+ }
+#endif
+
  return(true);
 }
 
@@ -326,7 +350,7 @@ std::string MDFN_MakeFName(MakeFName_Type type, int id1, const char *cd1)
 
 		      if(type != MDFNMKF_SAV)
 		      {
-                       snprintf(numtmp, sizeof(numtmp), "%d", id1);
+                       trio_snprintf(numtmp, sizeof(numtmp), "%d", id1);
                        fmap['p'] = std::string(numtmp);
 	              }
 
@@ -361,7 +385,7 @@ std::string MDFN_MakeFName(MakeFName_Type type, int id1, const char *cd1)
 		     std::string fstring = MDFN_GetSettingS("filesys.fname_snap");
 		     std::string fpath;
 
-		     snprintf(numtmp, sizeof(numtmp), "%04d", id1);
+		     trio_snprintf(numtmp, sizeof(numtmp), "%04d", id1);
 
 		     fmap['p'] = std::string(numtmp);
 
@@ -391,19 +415,19 @@ std::string MDFN_MakeFName(MakeFName_Type type, int id1, const char *cd1)
 		     std::string overpath = MDFN_GetSettingS("filesys.path_cheat");
 
 		     if(IsAbsolutePath(overpath))
-                      trio_snprintf(tmp_path, 4096, "%s"PSS"%s.%scht",overpath.c_str(), MDFNGameInfo->shortname, (type == MDFNMKF_CHEAT_TMP) ? "tmp" : "");
+                      trio_snprintf(tmp_path, 4096, "%s" PSS "%s.%scht",overpath.c_str(), MDFNGameInfo->shortname, (type == MDFNMKF_CHEAT_TMP) ? "tmp" : "");
                      else
-                      trio_snprintf(tmp_path, 4096, "%s"PSS"%s"PSS"%s.%scht", BaseDirectory.c_str(), overpath.c_str(), MDFNGameInfo->shortname, (type == MDFNMKF_CHEAT_TMP) ? "tmp" : "");
+                      trio_snprintf(tmp_path, 4096, "%s" PSS "%s" PSS "%s.%scht", BaseDirectory.c_str(), overpath.c_str(), MDFNGameInfo->shortname, (type == MDFNMKF_CHEAT_TMP) ? "tmp" : "");
 		    }
                     break;
 
   case MDFNMKF_AUX: if(IsAbsolutePath(cd1))
 		     trio_snprintf(tmp_path, 4096, "%s", (char *)cd1);
 		    else
-		     trio_snprintf(tmp_path, 4096, "%s"PSS"%s", FileBaseDirectory.c_str(), (char *)cd1);
+		     trio_snprintf(tmp_path, 4096, "%s" PSS "%s", FileBaseDirectory.c_str(), (char *)cd1);
 		    break;
 
-  case MDFNMKF_IPS:  trio_snprintf(tmp_path, 4096, "%s"PSS"%s%s.ips", FileBaseDirectory.c_str(), FileBase.c_str(), FileExt.c_str());
+  case MDFNMKF_IPS:  trio_snprintf(tmp_path, 4096, "%s" PSS "%s%s.ips", FileBaseDirectory.c_str(), FileBase.c_str(), FileExt.c_str());
                      break;
 
   case MDFNMKF_FIRMWARE:
@@ -417,14 +441,14 @@ std::string MDFN_MakeFName(MakeFName_Type type, int id1, const char *cd1)
 		     else
 		     {
 		      if(IsAbsolutePath(overpath))
-                       trio_snprintf(tmp_path, 4096, "%s"PSS"%s",overpath.c_str(), cd1);
+                       trio_snprintf(tmp_path, 4096, "%s" PSS "%s",overpath.c_str(), cd1);
                       else
 		      {
-                       trio_snprintf(tmp_path, 4096, "%s"PSS"%s"PSS"%s", BaseDirectory.c_str(), overpath.c_str(), cd1);
+                       trio_snprintf(tmp_path, 4096, "%s" PSS "%s" PSS "%s", BaseDirectory.c_str(), overpath.c_str(), cd1);
 
 		       // For backwards-compatibility with < 0.9.0
 		       if(stat(tmp_path,&tmpstat) == -1)
-                        trio_snprintf(tmp_path, 4096, "%s"PSS"%s", BaseDirectory.c_str(), cd1);
+                        trio_snprintf(tmp_path, 4096, "%s" PSS "%s", BaseDirectory.c_str(), cd1);
 		      }
 		     }
 		    }
@@ -439,14 +463,14 @@ std::string MDFN_MakeFName(MakeFName_Type type, int id1, const char *cd1)
 		       else
 			eff_dir = std::string(BaseDirectory) + std::string(PSS) + overpath;
 
-                       trio_snprintf(tmp_path, 4096, "%s"PSS"%s.pal", eff_dir.c_str(), FileBase.c_str());
+                       trio_snprintf(tmp_path, 4096, "%s" PSS "%s.pal", eff_dir.c_str(), FileBase.c_str());
 
                        if(stat(tmp_path,&tmpstat) == -1 && errno == ENOENT)
 		       {
-                        trio_snprintf(tmp_path, 4096, "%s"PSS"%s.%s.pal", eff_dir.c_str(), FileBase.c_str(), md5_context::asciistr(MDFNGameInfo->MD5, 0).c_str());
+                        trio_snprintf(tmp_path, 4096, "%s" PSS "%s.%s.pal", eff_dir.c_str(), FileBase.c_str(), md5_context::asciistr(MDFNGameInfo->MD5, 0).c_str());
 
 		        if(stat(tmp_path, &tmpstat) == -1 && errno == ENOENT)
-			 trio_snprintf(tmp_path, 4096, "%s"PSS"%s.pal", eff_dir.c_str(), cd1 ? cd1 : MDFNGameInfo->shortname);
+			 trio_snprintf(tmp_path, 4096, "%s" PSS "%s.pal", eff_dir.c_str(), cd1 ? cd1 : MDFNGameInfo->shortname);
 		       }
 		      }
                       break;
@@ -526,63 +550,3 @@ void GetFileBase(const char *f)
      }
 }
 
-char *MDFN_RemoveControlChars(char *str)
-{
- char *orig = str;
- if(str)
-  while(*str)
-  {
-   if(*str < 0x20) *str = 0x20;
-   str++;
-  }
- return(orig);
-}
-
-// Remove whitespace from beginning of string
-void MDFN_ltrim(char *string)
-{
- int32 di, si;
- bool InWhitespace = TRUE;
-
- di = si = 0;
-
- while(string[si])
- {
-  if(InWhitespace && (string[si] == ' ' || string[si] == '\r' || string[si] == '\n' || string[si] == '\t' || string[si] == 0x0b))
-  {
-
-  }
-  else
-  {
-   InWhitespace = FALSE;
-   string[di] = string[si];
-   di++;
-  }
-  si++;
- }
- string[di] = 0;
-}
-
-// Remove whitespace from end of string
-void MDFN_rtrim(char *string)
-{
- int32 len = strlen(string);
-
- if(len)
- {
-  for(int32 x = len - 1; x >= 0; x--)
-  {
-   if(string[x] == ' ' || string[x] == '\r' || string[x] == '\n' || string[x] == '\t' || string[x] == 0x0b)
-    string[x] = 0;
-   else
-    break;
-  }
- }
-
-}
-
-void MDFN_trim(char *string)
-{
- MDFN_rtrim(string);
- MDFN_ltrim(string);
-}
