@@ -4,7 +4,6 @@
 #include <string>
 
 #include "video.h"
-#include "sound.h"
 
 typedef struct
 {
@@ -56,22 +55,21 @@ typedef enum
 {
  IDIT_BUTTON,		// 1-bit
  IDIT_BUTTON_CAN_RAPID, // 1-bit
- IDIT_BUTTON_BYTE, // 8-bits, Button as a byte instead of a bit.
- IDIT_X_AXIS,	   // 32-bits
- IDIT_Y_AXIS,	   // 32-bits
- IDIT_X_AXIS_REL,  // 32-bits, signed
- IDIT_Y_AXIS_REL,  // 32-bits, signed
+ IDIT_X_AXIS,	   // (mouse) 32-bits, signed, fixed-point: 1.15.16 - in-screen/window range: [0.0, nominal_width)
+ IDIT_Y_AXIS,	   // (mouse) 32-bits, signed, fixed-point: 1.15.16 - in-screen/window range: [0.0, nominal_height)
+ IDIT_X_AXIS_REL,  // (mouse) 32-bits, signed
+ IDIT_Y_AXIS_REL,  // (mouse) 32-bits, signed
  IDIT_BYTE_SPECIAL,
  IDIT_BUTTON_ANALOG, // 32-bits, 0 - 32767
+ IDIT_RUMBLE,	// 32-bits, lower 8 bits are weak rumble(0-255), next 8 bits are strong rumble(0-255), 0=no rumble, 255=max rumble.  Somewhat subjective, too...
+		// May extend to 16-bit each in the future.
+		// It's also rather a special case of game module->driver code communication.
 } InputDeviceInputType;
 
-#include "git-virtb.h"
-
 typedef struct
 {
 	const char *SettingName;	// No spaces, shouldbe all a-z0-9 and _. Definitely no ~!
 	const char *Name;
-	/*const InputDeviceInputVB VirtButton;*/
         const int ConfigOrder;          // Configuration order during in-game config process, -1 for no config.
 	const InputDeviceInputType Type;
 	const char *ExcludeName;	// SettingName of a button that can't be pressed at the same time as this button
@@ -86,9 +84,10 @@ typedef struct
 {
  const char *ShortName;
  const char *FullName;
+ const char *Description;
 
  //struct InputPortInfoStruct *PortExpanderDeviceInfo;
- const void *PortExpanderDeviceInfo;
+ const void *PortExpanderDeviceInfo;	// DON'T USE, IT'S NOT IMPLEMENTED PROPERLY CURRENTLY.
  int NumInputs; // Usually just the number of buttons....OR if PortExpanderDeviceInfo is non-NULL, it's the number of input
 		// ports this port expander device provides.
  const InputDeviceInputInfoStruct *IDII;
@@ -96,7 +95,6 @@ typedef struct
 
 typedef struct
 {
- const int pid_offset;
  const char *ShortName;
  const char *FullName;
  int NumTypes; // Number of unique input devices available for this input port
@@ -110,6 +108,27 @@ typedef struct
  const InputPortInfoStruct *Types;
 } InputInfoStruct;
 
+struct MemoryPatch;
+
+struct CheatFormatStruct
+{
+ const char *FullName;		//"Game Genie", "GameShark", "Pro Action Catplay", etc.
+ const char *Description;	// Whatever?
+
+ bool (*DecodeCheat)(const std::string& cheat_string, MemoryPatch* patch);	// *patch should be left as initialized by MemoryPatch::MemoryPatch(), unless this is the
+										// second(or third or whatever) part of a multipart cheat.
+										//
+										// Will throw an std::exception(or derivative) on format error.
+										//
+										// Will return true if this is part of a multipart cheat.
+};
+
+struct CheatFormatInfoStruct
+{
+ unsigned NumFormats;
+
+ CheatFormatStruct *Formats;
+};
 
 // Miscellaneous system/simple commands(power, reset, dip switch toggles, coin insert, etc.)
 // (for DoSimpleCommand() )
@@ -169,8 +188,8 @@ enum
 
 typedef struct
 {
-	// Pitch(32-bit) must be equal to width and >= the pitch specified in the MDFNGI struct for the emulated system.
-	// Height must be >= to the fb_height specified in the MDFNGI struct for the emulated system.
+	// Pitch(32-bit) must be equal to width and >= the "fb_width" specified in the MDFNGI struct for the emulated system.
+	// Height must be >= to the "fb_height" specified in the MDFNGI struct for the emulated system.
 	// The framebuffer pointed to by surface->pixels is written to by the system emulation code.
 	MDFN_Surface *surface;
 
@@ -301,12 +320,26 @@ typedef struct
  bool (*TestMagicCD)(std::vector<CDIF *> *CDInterfaces);
 
  void (*CloseGame)(void);
- bool (*ToggleLayer)(int which);
+
+ void (*SetLayerEnableMask)(uint64 mask);	// Video
  const char *LayerNames;
- void (*InstallReadPatch)(uint32 address);
+
+ void (*SetChanEnableMask)(uint64 mask);	// Audio(TODO, placeholder)
+ const char *ChanNames;
+
+ //
+ // InstallReadPatch and RemoveReadPatches should be non-NULL(even if only pointing to dummy functions) if the emulator module supports
+ // read-substitution and read-substitution-with-compare style(IE Game Genie-style) cheats.
+ //
+ // See also "SubCheats" global stuff in mempatcher.h.
+ //
+ void (*InstallReadPatch)(uint32 address, uint8 value, int compare); // Compare is >= 0 when utilized.
  void (*RemoveReadPatches)(void);
  uint8 (*MemRead)(uint32 addr);
 
+ CheatFormatInfoStruct *CheatFormatInfo;
+
+ bool SaveStateAltersState;	// true for bsnes and some libco-style emulators, false otherwise.
  // Main save state routine, called by the save state code in state.cpp.
  // When saving, load is set to 0.  When loading, load is set to the version field of the save state being loaded.
  // data_only is true when the save state data is temporary, such as being saved into memory for state rewinding.
@@ -320,10 +353,15 @@ typedef struct
  const MDFNSetting *Settings;
 
  // Time base for EmulateSpecStruct::MasterCycles
- #define MDFN_MASTERCLOCK_FIXED(n)	((n) * (1LL << 32))
+ // MasterClock must be >= MDFN_MASTERCLOCK_FIXED(1.0)
+ // All or part of the fractional component may be ignored in some timekeeping operations in the emulator to prevent integer overflow,
+ // so it is unwise to have a fractional component when the integral component is very small(less than say, 10000).
+ #define MDFN_MASTERCLOCK_FIXED(n)	((int64)((double)(n) * (1LL << 32)))
  int64 MasterClock;
 
- uint32 fps; // frames per second * 65536 * 256, truncated
+ // Nominal frames per second * 65536 * 256, truncated.
+ // May be deprecated in the future due to many systems having slight frame rate programmability.
+ uint32 fps;
 
  // multires is a hint that, if set, indicates that the system has fairly programmable video modes(particularly, the ability
  // to display multiple horizontal resolutions, such as the PCE, PC-FX, or Genesis).  In practice, it will cause the driver
@@ -350,7 +388,7 @@ typedef struct
  int fb_width;		// Width of the framebuffer(not necessarily width of the image).  MDFN_Surface width should be >= this.
  int fb_height;		// Height of the framebuffer passed to the Emulate() function(not necessarily height of the image)
 
- int soundchan; 	// Number of output sound channels.
+ int soundchan; 	// Number of output sound channels.  Only values of 1 and 2 are currently supported.
 
 
  int rotated;
