@@ -22,11 +22,37 @@
 #include <string.h>
 #include <trio/trio.h>
 
+// Frequency cache cutoff optimization threshold (<= FREQC7M_COT)
+#define FREQC7M_COT	0x7 //0xA
+
 void PCE_PSG::SetVolume(double new_volume)
 {
- OutputVolume = new_volume;
+        for(int vl = 0; vl < 32; vl++)
+        {
+         double flub = 1.0 * new_volume * 8 / 6;
 
- Synth.volume(OutputVolume / 6);
+         if(vl)
+          flub /= pow(2, (double)1 / 4 * vl);                  // ~1.5dB reduction per increment of vl 
+
+	 if(vl == 0x1F)
+	  flub = 0;
+
+         for(int samp = 0; samp < 32; samp++)
+         {
+	  int eff_samp;
+
+	  if(revision == REVISION_HUC6280)
+	   eff_samp = samp * 2;
+	  else
+	   eff_samp = samp * 2 - 0x1F;
+
+	  dbtable[vl][samp] = (int32)(flub * eff_samp * 128); // * 256);
+	  dbtable_volonly[vl] = (int32)(flub * 65536);
+
+	  // dbtable[vl][samp] = (int32)(flub * eff_samp * 128);
+	  // dbtable_volonly[vl] = (int32)(flub * 65536);
+	 }
+	}
 }
 
 // Note: Changing the 0x1F(not that there should be) would require changing the channel pseudo-off volume check logic later on.
@@ -38,64 +64,90 @@ static const int scale_tab[] =
 
 #define CLOCK_LFSR(lfsr) { unsigned int newbit = ((lfsr >> 0) ^ (lfsr >> 1) ^ (lfsr >> 11) ^ (lfsr >> 12) ^ (lfsr >> 17)) & 1; lfsr = (lfsr >> 1) | (newbit << 17); }
 
+static const int16 Phase_Filter[2][7] =
+{
+ /*   0 */ {    35,   250,   579,   641,   425,   112,     6 }, //  2048
+ /*   1 */ {     6,   112,   425,   641,   579,   250,    35 }, //  2048
+};
+
+INLINE void PCE_PSG::UpdateOutputSub(const int32 timestamp, psg_channel *ch, const int32 samp0, const int32 samp1)
+{
+ int32 delta[2];
+
+ delta[0] = samp0 - ch->blip_prev_samp[0];
+ delta[1] = samp1 - ch->blip_prev_samp[1];
+
+ const int16* c = Phase_Filter[(timestamp >> 1) & 1];
+ const int32 l = (timestamp >> 2) & 0xFFFF;
+
+ HRBufs[0][l + 0] += delta[0] * c[0];
+ HRBufs[0][l + 1] += delta[0] * c[1];
+ HRBufs[0][l + 2] += delta[0] * c[2];
+ HRBufs[0][l + 3] += delta[0] * c[3];
+ HRBufs[0][l + 4] += delta[0] * c[4];
+ HRBufs[0][l + 5] += delta[0] * c[5];
+ HRBufs[0][l + 6] += delta[0] * c[6];
+
+ HRBufs[1][l + 0] += delta[1] * c[0];
+ HRBufs[1][l + 1] += delta[1] * c[1];
+ HRBufs[1][l + 2] += delta[1] * c[2];
+ HRBufs[1][l + 3] += delta[1] * c[3];
+ HRBufs[1][l + 4] += delta[1] * c[4];
+ HRBufs[1][l + 5] += delta[1] * c[5];
+ HRBufs[1][l + 6] += delta[1] * c[6];
+
+ ch->blip_prev_samp[0] = samp0;
+ ch->blip_prev_samp[1] = samp1;
+}
+
 void PCE_PSG::UpdateOutput_Norm(const int32 timestamp, psg_channel *ch)
 {
- int32 samp[2];
  int sv = ch->dda;
 
- samp[0] = dbtable[ch->vl[0]][sv];
- samp[1] = dbtable[ch->vl[1]][sv];
-
- Synth.offset(timestamp, samp[0] - ch->blip_prev_samp[0], sbuf[0]);
- Synth.offset(timestamp, samp[1] - ch->blip_prev_samp[1], sbuf[1]);
-
- ch->blip_prev_samp[0] = samp[0];
- ch->blip_prev_samp[1] = samp[1];
+ UpdateOutputSub(timestamp, ch, dbtable[ch->vl[0]][sv],
+ 				dbtable[ch->vl[1]][sv]);
 }
 
 void PCE_PSG::UpdateOutput_Noise(const int32 timestamp, psg_channel *ch)
 {
- int32 samp[2];
  int sv = ((ch->lfsr & 1) << 5) - (ch->lfsr & 1); //(ch->lfsr & 0x1) ? 0x1F : 0;
 
- samp[0] = dbtable[ch->vl[0]][sv];
- samp[1] = dbtable[ch->vl[1]][sv];
-
- Synth.offset(timestamp, samp[0] - ch->blip_prev_samp[0], sbuf[0]);
- Synth.offset(timestamp, samp[1] - ch->blip_prev_samp[1], sbuf[1]);
-
- ch->blip_prev_samp[0] = samp[0];
- ch->blip_prev_samp[1] = samp[1];
+ UpdateOutputSub(timestamp, ch, dbtable[ch->vl[0]][sv],
+				dbtable[ch->vl[1]][sv]);
 }
 
 void PCE_PSG::UpdateOutput_Off(const int32 timestamp, psg_channel *ch)
 {
- int32 samp[2];
-
- samp[0] = samp[1] = 0;
-
- Synth.offset_inline(timestamp, samp[0] - ch->blip_prev_samp[0], sbuf[0]);
- Synth.offset_inline(timestamp, samp[1] - ch->blip_prev_samp[1], sbuf[1]);
-
- ch->blip_prev_samp[0] = samp[0];
- ch->blip_prev_samp[1] = samp[1];
+ UpdateOutputSub(timestamp, ch, 0, 0);
 }
 
-
-void PCE_PSG::UpdateOutput_Accum(const int32 timestamp, psg_channel *ch)
+void PCE_PSG::UpdateOutput_Accum_HuC6280A(const int32 timestamp, psg_channel *ch)
 {
  int32 samp[2];
 
+ // 31(5-bit max) * 32 samples = 992
+ // 992 / 2 = 496
+ // 
+ // 8 + 5 = 13
+ // 13 - 12 = 1
+
  samp[0] = ((int32)dbtable_volonly[ch->vl[0]] * ((int32)ch->samp_accum - 496)) >> (8 + 5);
  samp[1] = ((int32)dbtable_volonly[ch->vl[1]] * ((int32)ch->samp_accum - 496)) >> (8 + 5);
 
- Synth.offset_inline(timestamp, samp[0] - ch->blip_prev_samp[0], sbuf[0]);
- Synth.offset_inline(timestamp, samp[1] - ch->blip_prev_samp[1], sbuf[1]);
-
- ch->blip_prev_samp[0] = samp[0];
- ch->blip_prev_samp[1] = samp[1];
+ UpdateOutputSub(timestamp, ch, samp[0], samp[1]);
 }
 
+void PCE_PSG::UpdateOutput_Accum_HuC6280(const int32 timestamp, psg_channel *ch)
+{
+ int32 samp[2];
+
+ samp[0] = ((int32)dbtable_volonly[ch->vl[0]] * (int32)ch->samp_accum) >> (8 + 5);
+ samp[1] = ((int32)dbtable_volonly[ch->vl[1]] * (int32)ch->samp_accum) >> (8 + 5);
+
+ UpdateOutputSub(timestamp, ch, samp[0], samp[1]);
+}
+
+
 // This function should always be called after RecalcFreqCache() (it's not called from RecalcFreqCache to avoid redundant code)
 void PCE_PSG::RecalcUOFunc(int chnum)
 {
@@ -109,8 +161,8 @@ void PCE_PSG::RecalcUOFunc(int chnum)
   ch->UpdateOutput = &PCE_PSG::UpdateOutput_Noise;
  // If the control for the channel is in waveform play mode, and the (real) playback frequency is too high, and the channel is either not the LFO modulator channel or
  // if the LFO trigger bit(which halts the LFO modulator channel's waveform incrementing when set) is clear
- else if((ch->control & 0xC0) == 0x80 && ch->freq_cache <= 0xA && (chnum != 1 || !(lfoctrl & 0x80)) )
-  ch->UpdateOutput = &PCE_PSG::UpdateOutput_Accum;
+ else if((ch->control & 0xC0) == 0x80 && ch->freq_cache <= FREQC7M_COT && (chnum != 1 || !(lfoctrl & 0x80)) )
+  ch->UpdateOutput = UpdateOutput_Accum;
  else
   ch->UpdateOutput = &PCE_PSG::UpdateOutput_Norm;
 }
@@ -349,14 +401,27 @@ void PSG_SetRegister(const unsigned int id, const uint32 value)
 }
 #endif
 
-PCE_PSG::PCE_PSG(Blip_Buffer *bb_l, Blip_Buffer *bb_r, int want_revision)
+PCE_PSG::PCE_PSG(int32* hr_l, int32* hr_r, int want_revision)
 {
+	//printf("Test: %u, %u\n", sizeof(psg_channel), (uint8*)&channel[0].balance - (uint8*)&channel[0].waveform[0]);
+
 	revision = want_revision;
+	switch(revision)
+	{
+	 default:
+		abort();
+		break;
 
-	sbuf[0] = bb_l;
-	sbuf[1] = bb_r;
+	 case REVISION_HUC6280:
+		UpdateOutput_Accum = &PCE_PSG::UpdateOutput_Accum_HuC6280;
+		break;
 
-	SoundEnabled = (sbuf[0] && sbuf[1]);
+	 case REVISION_HUC6280A:
+		UpdateOutput_Accum = &PCE_PSG::UpdateOutput_Accum_HuC6280A;
+		break;
+	}
+	HRBufs[0] = hr_l;
+	HRBufs[1] = hr_r;
 
 	lastts = 0;
 	for(int ch = 0; ch < 6; ch++)
@@ -366,32 +431,7 @@ PCE_PSG::PCE_PSG(Blip_Buffer *bb_l, Blip_Buffer *bb_r, int want_revision)
 	 channel[ch].lastts = 0;
 	}
 
-	SetVolume(1.0);
-
-        for(int vl = 0; vl < 32; vl++)
-        {
-         double flub = 1;
-
-         if(vl)
-          flub /= pow(2, (double)1 / 4 * vl);                  // ~1.5dB reduction per increment of vl 
-
-	 if(vl == 0x1F)
-	  flub = 0;
-
-         for(int samp = 0; samp < 32; samp++)
-         {
-	  int eff_samp;
-
-	  if(revision == REVISION_HUC6280)
-	   eff_samp = samp * 2;
-	  else
-	   eff_samp = samp * 2 - 0x1F;
-
-	  dbtable[vl][samp] = (int32)(flub * eff_samp * 128);
-	  dbtable_volonly[vl] = (int32)(flub * 65536);
-	 }
-	}
-
+	SetVolume(1.0);	// Will build dbtable in the process.
 	Power(0);
 }
 
@@ -441,15 +481,7 @@ void PCE_PSG::Write(int32 timestamp, uint8 A, uint8 V)
 
         case 0x01: /* Global sound balance */
             globalbalance = V;
-
-	    if(REVISION_ENHANCED == revision)
-	    {
-	     for(int cht = 0; cht < 6; cht++)
-	      for(int lr = 0; lr < 2; lr++)
-	       channel[cht].vl[lr] = GetVL(cht, lr);	
-	    }
-	    else
-	     vol_pending = true;
+	    vol_pending = true;
             break;
 
         case 0x02: /* Channel frequency (LSB) */
@@ -491,28 +523,14 @@ void PCE_PSG::Write(int32 timestamp, uint8 A, uint8 V)
 	    RecalcFreqCache(select);
 	    RecalcUOFunc(select);
 
-            if(REVISION_ENHANCED == revision)
-            {
-	     ch->vl[0] = GetVL(select, 0);
-	     ch->vl[1] = GetVL(select, 1);
-	    }
-	    else
-	     vol_pending = true;
-
+	    vol_pending = true;
             break;
 
         case 0x05: /* Channel balance */
 	    if(select > 5) return; // no more than 6 channels, silly game.
             ch->balance = V;
 
-            if(REVISION_ENHANCED == revision)
-            {
-	     ch->vl[0] = GetVL(select, 0);
-	     ch->vl[1] = GetVL(select, 1);
-	    }
-	    else
-	     vol_pending = true;
-
+	    vol_pending = true;
             break;
 
         case 0x06: /* Channel waveform data */
@@ -534,12 +552,6 @@ void PCE_PSG::Write(int32 timestamp, uint8 A, uint8 V)
 	     // According to my tests(on SuperGrafx), writing to this channel
 	     // will update the waveform value cache/latch regardless of DDA mode being enabled.
              ch->dda = V;
-
-	     if(REVISION_ENHANCED == revision)
-	     {
-	      if(&PCE_PSG::UpdateOutput_Norm == ch->UpdateOutput)
- 	       UpdateOutput_Norm(timestamp, ch);
-	     }
 	    }
             break;
 
@@ -577,7 +589,7 @@ void PCE_PSG::Write(int32 timestamp, uint8 A, uint8 V)
 
 // Don't use INLINE, which has always_inline in it, due to gcc's inability to cope with the type of recursion
 // used in this function.
-inline void PCE_PSG::RunChannel(int chc, int32 timestamp, const bool LFO_On)
+void PCE_PSG::RunChannel(int chc, int32 timestamp, const bool LFO_On)
 {
  psg_channel *ch = &channel[chc];
  int32 running_timestamp = ch->lastts;
@@ -588,11 +600,7 @@ inline void PCE_PSG::RunChannel(int chc, int32 timestamp, const bool LFO_On)
  if(!run_time)
   return;
 
- //if(chc != 5)
- // return;
-
- if(REVISION_ENHANCED != revision)
-  (this->*ch->UpdateOutput)(running_timestamp, ch);
+ (this->*ch->UpdateOutput)(running_timestamp, ch);
 
  if(chc >= 4)
  {
@@ -625,7 +633,7 @@ inline void PCE_PSG::RunChannel(int chc, int32 timestamp, const bool LFO_On)
 
  ch->counter -= run_time;
 
- if(!LFO_On && ch->freq_cache <= 0xA)
+ if(!LFO_On && ch->freq_cache <= FREQC7M_COT)
  {
   if(ch->counter <= 0)
   {
@@ -651,7 +659,7 @@ inline void PCE_PSG::RunChannel(int chc, int32 timestamp, const bool LFO_On)
    RecalcFreqCache(0);
    RecalcUOFunc(0);
 
-   ch->counter += (ch->freq_cache <= 0xA) ? 0xA : ch->freq_cache;	// Not particularly accurate, but faster.
+   ch->counter += (ch->freq_cache <= FREQC7M_COT) ? FREQC7M_COT : ch->freq_cache;	// Not particularly accurate, but faster.
   }
   else
    ch->counter += ch->freq_cache;
@@ -670,23 +678,14 @@ void PCE_PSG::UpdateSubNonLFO(int32 timestamp)
   RunChannel(chc, timestamp, false);
 }
 
-//static int32 last_read;
-//static int32 last_apply;
-
 void PCE_PSG::Update(int32 timestamp)
 {
  int32 run_time = timestamp - lastts;
 
- if(!SoundEnabled)
-  return;
-
- if(REVISION_ENHANCED != revision)
+ if(vol_pending && !vol_update_counter && !vol_update_which)
  {
-  if(vol_pending && !vol_update_counter && !vol_update_which)
-  {
-   vol_update_counter = 1;
-   vol_pending = false;
-  }
+  vol_update_counter = 1;
+  vol_pending = false;
  }
 
  bool lfo_on = (bool)(lfoctrl & 0x03);
@@ -708,11 +707,8 @@ void PCE_PSG::Update(int32 timestamp)
  {
   int32 chunk_clocks = clocks;
 
-  if(REVISION_ENHANCED != revision)
-  {
-   if(vol_update_counter > 0 && chunk_clocks > vol_update_counter)
-    chunk_clocks = vol_update_counter;
-  }
+  if(vol_update_counter > 0 && chunk_clocks > vol_update_counter)
+   chunk_clocks = vol_update_counter;
 
   running_timestamp += chunk_clocks;
   clocks -= chunk_clocks;
@@ -722,7 +718,7 @@ void PCE_PSG::Update(int32 timestamp)
   else
    UpdateSubNonLFO(running_timestamp);
 
-  if(REVISION_ENHANCED != revision && vol_update_counter > 0)
+  if(vol_update_counter > 0)
   {
    vol_update_counter -= chunk_clocks;
    if(!vol_update_counter)
@@ -766,12 +762,12 @@ void PCE_PSG::Update(int32 timestamp)
  }
 }
 
-void PCE_PSG::EndFrame(int32 timestamp)
+void PCE_PSG::ResetTS(int32 ts_base)
 {
- Update(timestamp);
- lastts = 0;
+ lastts = ts_base;
+
  for(int chc = 0; chc < 6; chc++)
-  channel[chc].lastts = 0;
+  channel[chc].lastts = ts_base;
 }
 
 void PCE_PSG::Power(const int32 timestamp)
@@ -780,7 +776,7 @@ void PCE_PSG::Power(const int32 timestamp)
  if(timestamp != lastts)
   Update(timestamp);
 
- memset(&channel, 0, sizeof(channel));
+ // Don't memset channel to 0, there's stuff like lastts and blip_prev_samp that shouldn't be altered on Power().
 
  select = 0;
  globalbalance = 0;
