@@ -284,10 +284,9 @@ void VDC::SetRegister(const unsigned int id, const uint32 value)
  }
 }
 
-bool VDC::ToggleLayer(int which)
+void VDC::SetLayerEnableMask(uint64 mask)
 {
- userle ^= 1 << which;
- return((userle >> which) & 1);
+ userle = mask;
 }
 
 void VDC::RunSATDMA(int32 cycles, bool force_completion)
@@ -987,7 +986,7 @@ void VDC::DrawBG(uint16 *target, int enabled)
 #define SPRF_SPRITE0	0x10000
 
 static const unsigned int sprite_height_tab[4] = { 16, 32, 64, 64 };
-static const unsigned int sprite_height_no_mask[4] = { ~0, ~2, ~6, ~6 };
+static const unsigned int sprite_height_no_mask[4] = { ~0U, ~2U, ~6U, ~6U };
 static const unsigned int sprite_width_tab[2] = { 16, 32 };
 
 void VDC::FetchSpriteData(void)
