@@ -69,17 +69,17 @@ static uint16 COMM_FLAGS;
 static uint8 COMM_CMD[8 * 2];
 static uint8 COMM_STATUS[8 * 2];
 
-#define DEBUGMAIN(format, ...) { printf("MAIN "format, ## __VA_ARGS__); }
-#define DEBUGSUB(format, ...) { printf("SUB "format, ## __VA_ARGS__); }
+#define DEBUGMAIN(format, ...) { printf("MAIN " format, ## __VA_ARGS__); }
+#define DEBUGSUB(format, ...) { printf("SUB " format, ## __VA_ARGS__); }
 
-#define UNDEFMAIN(format, ...) { printf("MAIN Undefined "format, ## __VA_ARGS__); }
-#define UNDEFSUB(format, ...) { printf("SUB Undefined "format, ## __VA_ARGS__); }
+#define UNDEFMAIN(format, ...) { printf("MAIN Undefined " format, ## __VA_ARGS__); }
+#define UNDEFSUB(format, ...) { printf("SUB Undefined " format, ## __VA_ARGS__); }
 
 //#if 0
-#define DEBUGMAIN(format, ...) { }
+//#define DEBUGMAIN(format, ...) { }
 //#define DEBUGSUB(format, ...)
 
-#define UNDEFMAIN(format, ...) { }
+//#define UNDEFMAIN(format, ...) { }
 //#define UNDEFSUB(format, ...)
 //#endif
 
@@ -837,7 +837,9 @@ void MDCD_Run(int32 md_master_cycles)
   else
   {
    //printf("Yay: %08x\n", C68k_Get_PC(&Sub68K));
-   temp_cycles = C68k_Exec(&Sub68K);
+   Sub68K.timestamp = 0;
+   C68k_Exec(&Sub68K);
+   temp_cycles = Sub68K.timestamp;
   }
   MDCD_Timer_Run(temp_cycles);
   MDCD_PCM_Run(temp_cycles);
@@ -995,10 +997,13 @@ bool MDCD_Load(std::vector<CDIF *> *CDInterfaces, md_game_info *ginfo)
 }
 
  
-void MDCD_Reset(void)
+void MDCD_Reset(bool poweron)
 {
- memset(PRAM, 0, 0x80000);
- memset(WordRAM, 0, 0x40000);
+ if(poweron)
+ {
+  memset(PRAM, 0, 0x80000);
+  memset(WordRAM, 0, 0x40000);
+ }
 
  C68k_Reset(&Sub68K);
  MDCD_InterruptReset();
