@@ -23,11 +23,10 @@ namespace MDFN_IEN_MD
 {
 bool MD_DebugMode = FALSE;
 
-static void (*DriverCPUHook)(uint32);
-
+static void (*DriverCPUHook)(uint32, bool) = NULL;
+static bool DriverCPUHookContinuous = false;
 
 static c68k_struc Main68K_BP;
-static void (*BPCallB)(uint32 PC) = NULL;
 static bool BPActive = FALSE; // Any breakpoints on?
 static bool BPNonPCActive = FALSE;	// Any breakpoints other than PC on?
 static bool FoundBPoint;
@@ -81,7 +80,10 @@ static uint32 M68K_GetRegister(const unsigned int id, char *special, const uint3
 
 void M68K_SetRegister(const unsigned int id, uint32 value)
 {
-
+ if(id == C68K_GSREG_PC)
+ {
+  C68k_Set_PC(&Main68K, value);
+ } 
 }
 
 
@@ -215,11 +217,10 @@ void MDDBG_CPUHook(void)	//uint32 PC, uint16 op)
   MD_HackyHackyMode--;
  }
 
- if(FoundBPoint)
-  BPCallB(PC);
+ DriverCPUHookContinuous |= FoundBPoint;
 
- if(DriverCPUHook)
-  DriverCPUHook(PC);
+ if(DriverCPUHookContinuous && DriverCPUHook)
+  DriverCPUHook(PC, FoundBPoint);
 }
 
 static void RedoCPUHook(void)
@@ -273,15 +274,16 @@ void FlushBreakPoints(int type)
  RedoCPUHook();
 }
 
-void SetCPUCallback(void (*callb)(uint32 PC))
+void SetCPUCallback(void (*callb)(uint32 PC, bool bpoint), bool continuous)
 {
  DriverCPUHook = callb;
+ DriverCPUHookContinuous = continuous;
  RedoCPUHook();
 }
 
-void SetBPCallback(void (*callb)(uint32 PC))
+static void EnableBranchTrace(bool enable)
 {
- BPCallB = callb;
+
 }
 
 std::vector<BranchTraceResult> GetBranchTrace(void)
@@ -327,6 +329,15 @@ static void PutAddressSpaceBytes(const char *name, uint32 Address, uint32 Length
    Address &= 0xFFFFFF;
   }
  }
+ else if(!strcmp(name, "ram"))
+ {
+  while(Length--)
+  {
+   MD_WriteMemory8((Address & 0xFFFF) | 0xFF0000, *Buffer);
+   Address++;
+   Buffer++;
+  }
+ }
 }
 
 static int dbg_int_ack_callback(int int_level)
@@ -443,7 +454,7 @@ DebuggerInfoStruct DBGInfo =
  24,		// Logical address bits
  24,		// Physical address bits
  0xFF0000,	// Default watch address
- ~0,		// ZP
+ ~0U,		// ZP
 
  MemPeek,
  Disassemble,
@@ -453,7 +464,7 @@ DebuggerInfoStruct DBGInfo =
  FlushBreakPoints,
  AddBreakPoint,
  SetCPUCallback,
- SetBPCallback,
+ EnableBranchTrace,
  GetBranchTrace,
  NULL, //SetGraphicsDecode,
  NULL, //GetGraphicsDecodeBuffer,
