@@ -27,7 +27,7 @@ extern MDVDP MainVDP;
 
 /* Function prototypes */
 void gen_init(void);
-void gen_reset(void);
+void gen_reset(bool poweron);
 void gen_shutdown(void);
 int gen_busack_r(void);
 void gen_busreq_w(int state);
