@@ -40,13 +40,12 @@ enum
 int MD_HackyHackyMode = 0;
 bool MD_IsCD;
 static int32 z80_cycle_counter;
+static int32 z80_last_ts;
 int32 md_timestamp;
 static bool suspend68k = FALSE;
 
 static bool run_cpu;
 
-bool MD_OverClock = 0;
-
 void MD_Suspend68K(bool state)
 {
  suspend68k = state;
@@ -77,21 +76,20 @@ static void system_init(bool overseas, bool PAL, bool overseas_reported, bool PA
 
 }
 
-static void system_reset(void)
+static void system_reset(bool poweron)
 {
  z80_cycle_counter = 0;
+ z80_last_ts = 0;
 
  if(MD_IsCD)
-  MDCD_Reset();
+  MDCD_Reset(poweron);
  else
   MDCart_Reset();
 
- gen_reset();
- MainVDP.Reset();
+ gen_reset(poweron);
+ if(poweron)
+  MainVDP.Reset();
  MDSound_Power();
-
- if(MD_IsCD)
-  MDCD_Reset(); 
 }
 
 static void system_shutdown(void)
@@ -104,18 +102,49 @@ void MD_ExitCPULoop(void)
  run_cpu = FALSE;
 }
 
+void MD_UpdateSubStuff(void)
+{
+ int32 max_md_timestamp;
+
+ max_md_timestamp = Main68K.timestamp * 7;
+
+ if(zreset == 1 && zbusreq == 0)
+ {
+  z80_cycle_counter += max_md_timestamp - z80_last_ts;
+
+  while(z80_cycle_counter > 0)
+  {
+   int32 z80_temp = z80_do_opcode() * 15;
+
+   z80_cycle_counter -= z80_temp;
+   md_timestamp += z80_temp;
+
+   if(md_timestamp > max_md_timestamp)
+   {
+    //printf("Meow: %d\n", md_timestamp - max_md_timestamp);
+    md_timestamp = max_md_timestamp;
+   }
+   MainVDP.Run();
+  }
+ }
+ z80_last_ts = max_md_timestamp;
+
+ md_timestamp = max_md_timestamp;
+ MainVDP.Run();
+
+ //if(MD_IsCD)
+ // MDCD_Run(master_cycles);
+}
+
 static int system_frame(int do_skip)
 {
  run_cpu = TRUE;
 
  while(run_cpu > 0)
  {
-  int32 temp_master;
-  int32 max_md_timestamp;
-
   if(suspend68k)
   {
-   temp_master = 8 * 7;
+   Main68K.timestamp += 4;
   }
   else
   {
@@ -123,34 +152,11 @@ static int system_frame(int do_skip)
    if(MD_DebugMode)
     MDDBG_CPUHook();
    #endif
-   temp_master = C68k_Exec(&Main68K) * 7;
+
+   C68k_Exec(&Main68K);
   }
-  max_md_timestamp = md_timestamp + temp_master;
 
-  if(zreset == 1 && zbusreq == 0)
-  {
-   z80_cycle_counter += temp_master;
-
-   while(z80_cycle_counter > 0)
-   {
-    int32 z80_temp = z80_do_opcode() * 15;
-
-    z80_cycle_counter -= z80_temp;
-    md_timestamp += z80_temp;
-
-    if(md_timestamp > max_md_timestamp)
-    {
-     //printf("Meow: %d\n", md_timestamp - max_md_timestamp);
-     md_timestamp = max_md_timestamp;
-    }
-    MainVDP.Run();
-   }
-  }
-  md_timestamp = max_md_timestamp;
-  MainVDP.Run();
-
-  if(MD_IsCD)
-   MDCD_Run(temp_master);
+  MD_UpdateSubStuff();
  }
  return gen_running;
 }
@@ -191,6 +197,8 @@ static void Emulate(EmulateSpecStruct *espec)
  MDIO_EndTimePeriod(md_timestamp);
 
  md_timestamp = 0;
+ z80_last_ts = 0;
+ Main68K.timestamp = 0;
  MainVDP.ResetTS();
 
  //MainVDP.SetSurface(NULL);
@@ -291,16 +299,16 @@ static int LoadCommonPost(const md_game_info &ginfo)
    game_overseas = TRUE;
    game_pal = FALSE;
   }
-  else if(ginfo.region_support & REGIONMASK_OVERSEAS_PAL)
-  {
-   game_overseas = TRUE;
-   game_pal = TRUE;
-  }
   else if(ginfo.region_support & REGIONMASK_JAPAN_NTSC)
   {
    game_overseas = FALSE;
    game_pal = FALSE;
   }
+  else if(ginfo.region_support & REGIONMASK_OVERSEAS_PAL)
+  {
+   game_overseas = TRUE;
+   game_pal = TRUE;
+  }
   else if(ginfo.region_support & REGIONMASK_JAPAN_PAL) // WTF?
   {
    game_overseas = FALSE;
@@ -355,7 +363,7 @@ static int LoadCommonPost(const md_game_info &ginfo)
 
  if(MDFN_GetSettingB("md.correct_aspect"))
  {
-  MDFNGameInfo->nominal_width = 288;
+  MDFNGameInfo->nominal_width = 292;
   MDFNGameInfo->lcm_width = 1280;
  }
  else
@@ -368,7 +376,7 @@ static int LoadCommonPost(const md_game_info &ginfo)
 
  MDFNGameInfo->LayerNames = "BG0\0BG1\0OBJ\0";
 
- system_reset();
+ system_reset(true);
 
  return(1);
 }
@@ -431,8 +439,8 @@ static void DoSimpleCommand(int cmd)
 {
  switch(cmd)
  {
-  case MDFN_MSC_POWER:
-  case MDFN_MSC_RESET: system_reset(); break;
+  case MDFN_MSC_POWER: system_reset(true); break;
+  case MDFN_MSC_RESET: system_reset(false); break;
  }
 }
 
@@ -530,9 +538,9 @@ static const FileExtensionSpecStruct KnownExtensions[] =
  { NULL, NULL }
 };
 
-bool ToggleLayer(int which)
+void SetLayerEnableMask(uint64 mask)
 {
- return(MainVDP.ToggleLayer(which));
+ MainVDP.SetLayerEnableMask(mask);
 }
 
 }
@@ -555,11 +563,15 @@ MDFNGI EmulatedMD =
  LoadCD,
  TestMagicCD,
  CloseGame,
- ToggleLayer,
+ SetLayerEnableMask,
+ NULL,
+ NULL,
  NULL,
  NULL, //InstallReadPatch,
  NULL, //RemoveReadPatches,
  NULL, //MemRead,
+ NULL,
+ false,
  StateAction,
  Emulate,
  MDINPUT_SetInput,
@@ -576,7 +588,7 @@ MDFNGI EmulatedMD =
 
  // We want maximum values for nominal width and height here so the automatic fullscreen setting generation code will have
  // selected a setting suitable if aspect ratio correction is turned off.
- 320,   // Nominal width(adjusted in game loading code, with aspect ratio correction enabled, it's 288, otherwise 320)
+ 320,   // Nominal width(adjusted in game loading code, with aspect ratio correction enabled, it's 292, otherwise 320)
  240,   // Nominal height(adjusted in game loading code to 224 for NTSC, and 240 for PAL)
  1024,	// Framebuffer width
  512,	// Framebuffer height
