@@ -25,7 +25,6 @@
 
 #include "shared.h"
 #include "vcnt.h"
-#include "hcnt.h"
 #include "hvc.h"
 
 namespace MDFN_IEN_MD
@@ -129,6 +128,11 @@ void MDVDP::Reset(void)
     memset(vram, 0, sizeof(vram));
     memset(cram, 0, sizeof(cram));
     memset(vsram, 0, sizeof(vsram));
+
+    memset(bg_name_dirty, 0, sizeof(bg_name_dirty));
+    memset(bg_name_list, 0, sizeof(bg_name_list));
+    bg_list_index = 0;
+    memset(bg_pattern_cache, 0, sizeof(bg_pattern_cache));
     memset(reg, 0, sizeof(reg));
 
     addr = addr_latch = code = pending = buffer = status = 0;
@@ -139,11 +143,6 @@ void MDVDP::Reset(void)
     /* Mark all colors as dirty to force a palette update */
     border = 0x00;
 
-    memset(bg_name_dirty, 0, sizeof(bg_name_dirty));
-    memset(bg_name_list, 0, sizeof(bg_name_list));
-    bg_list_index = 0;
-    memset(bg_pattern_cache, 0, sizeof(bg_pattern_cache));
-
     playfield_shift = 6;
     playfield_col_mask = 0x1F;
     playfield_row_mask = 0x0FF;
@@ -154,15 +153,16 @@ void MDVDP::Reset(void)
     visible_frame_end = 0xE0;
     v_counter = v_update = 0;
 
+    fifo_simu_count = 0;
     dma_fill_latch = 0;
     DMASource = 0;
     DMALength = 0;
     dma_fill = 0;
     im2_flag = 0;
 
-    vdp_cycle_counter = 4 * 10;
     vdp_last_ts = 0;
-    vdp_line_phase = VDPLP_VISIBLE_0;
+    vdp_cycle_counter = 102;
+    vdp_line_phase = 5;
     vdp_hcounter_start_ts = 0;
     scanline = visible_frame_end;
 
@@ -222,7 +222,7 @@ void MDVDP::vdp_ctrl_w(uint16 data)
 		    if((code & 0x0F) != 0x1 && (code & 0x0F) != 0x3 && (code & 0x0F) != 0x5)
                      printf("Invalid code for V bus to VDP dma: %02x\n", code);
 		    status |= 0x2;
-		    MD_Suspend68K(TRUE);
+		    Recalc68KSuspend();
                     break;
 
                 case 0x80: /* VRAM fill */
@@ -254,7 +254,12 @@ uint16 MDVDP::vdp_ctrl_r(void)
     status &= ~0x0001;
     status |= report_pal ? 0x0001 : 0x0000;
 
-    status |= 0x0200;  	// FIFO empty
+    if(fifo_simu_count >= 4)
+     temp |= 0x0100;
+
+    if(fifo_simu_count == 0)
+     temp |= 0x0200;  	// FIFO empty
+
     temp |= status & 0x03FF;
 
     if(!(reg[1] & 0x40))
@@ -373,11 +378,19 @@ INLINE void MDVDP::MemoryWrite16(uint16 data)
  addr += reg[15];
 }
 
+INLINE void MDVDP::Recalc68KSuspend(void)
+{
+ MD_Suspend68K((fifo_simu_count > 4) || ((status & 0x2) && (reg[23] & 0xC0) < 0x80));
+}
+
 void MDVDP::vdp_data_w(uint16 data)
 {
     /* Clear pending flag */
     pending = 0;
 
+    if(fifo_simu_count < 16)
+     fifo_simu_count++;
+
     //printf("%04x, %d\n", data, scanline);
     MemoryWrite16(data);
 
@@ -388,6 +401,8 @@ void MDVDP::vdp_data_w(uint16 data)
      //printf("DMA Fill: %d, %02x, %04x\n", DMALength, code & 0x0F, data);
      dma_fill = 0;
     }
+
+    Recalc68KSuspend();
 }
 
 uint16 MDVDP::vdp_data_r(void)
@@ -548,17 +563,11 @@ void MDVDP::vdp_reg_w(uint8 r, uint8 d)
 
 uint16 MDVDP::vdp_hvc_r(void)
 {
- int32 cycle = (md_timestamp - vdp_hcounter_start_ts) / ((reg[0x0C] & 1) ? 8 : 10);
+ int32 cycle = (md_timestamp - vdp_hcounter_start_ts) % 3420;
  int hc;
  uint8 vc;
 
- cycle >>= 1;
-
- //if(cycle / ((reg[0xC] & 1) ? 211 : 171))
- //printf("MOO: %d\n", cycle); //cycle / ((reg[0xC] & 1) ? 211 : 171));
- cycle %= (reg[0xC] & 1) ? 211 : 171;
-
- hc = (reg[0xC] & 1) ? hc_320[cycle] : hc_256[cycle];
+ hc = (reg[0xC] & 1) ? cycle2hc40[cycle] : cycle2hc32[cycle];
 
  if(is_pal)
   vc = (reg[0x1] & 0x8) ? vc30_pal[v_counter] : vc28_pal[v_counter];
@@ -588,9 +597,10 @@ INLINE void MDVDP::CheckDMA(void)
  };
  static const int fill_cc[2][2] =
  {
-  { 15, 166 },
-  { 17, 204 },
+  { 16, 167 },
+  { 18, 205 },
  };
+ const int vb_index = ((status & 0x8) >> 3) | (((reg[1] & 0x40) ^ 0x40) >> 6);
 
  if(!(status & 0x2))
   return;
@@ -600,8 +610,6 @@ INLINE void MDVDP::CheckDMA(void)
   case 0x00: /* V bus to VDP DMA */
   case 0x40: /* V bus to VDP DMA */
 	   {
-	    int vb_index = ((status & 0x8) >> 3) | (((reg[1] & 0x40) ^ 0x40) >> 6);
-
 	    int32 runcount = (vbus_cc[reg[0x0C] & 1][vb_index] + 1) / 2;
 
 	    //printf("%08x, %d, %02x\n", DMASource, DMALength, reg[0xC] & 0x81);
@@ -617,13 +625,13 @@ INLINE void MDVDP::CheckDMA(void)
 	    if(!DMALength)
 	    {
              status &= ~0x2;
-             MD_Suspend68K(FALSE);
+             Recalc68KSuspend();
 	    }
 	    break;
 	   }
   case 0x80: // DMA Fill
 	   {
-	    int32 runcount = fill_cc[reg[0x0C] & 1][(status & 0x8) >> 3];
+	    int32 runcount = fill_cc[reg[0x0C] & 1][vb_index];
 
 	    do
 	    {
@@ -640,13 +648,17 @@ INLINE void MDVDP::CheckDMA(void)
 
   case 0xC0: // VRAM copy
 	   {
-            int32 runcount = copy_cc[reg[0x0C] & 1][(status & 0x8) >> 3];
+            int32 runcount = copy_cc[reg[0x0C] & 1][vb_index];
 
 	    do
 	    {
   	     uint8 temp = READ_BYTE_LSB(vram, DMASource & 0xFFFF);
              WRITE_BYTE_LSB(vram, addr, temp);
              MARK_BG_DIRTY(addr);
+             if((addr & sat_base_mask) == satb)
+             {
+                sat[addr & sat_addr_mask] = temp;
+             }
              DMASource = (DMASource & 0xFF0000) | ((DMASource + 1) & 0xFFFF);
              addr = (addr + reg[15]) & 0xFFFF;
              DMALength--;
@@ -668,13 +680,10 @@ void MDVDP::vdp_test_w(uint16 value)
 {
 }
 
-#if 0
-#else
 void MDVDP::Run(void)
 {
  #include "vdp_run.inc"
 }
-#endif
 
 void MDVDP::ResetTS(void)
 {
@@ -730,6 +739,8 @@ int MDVDP::StateAction(StateMem *sm, int load, int data_only)
   SFVAR(vdp_hcounter_start_ts),
   SFVAR(scanline),
 
+  SFVAR(fifo_simu_count),
+
   //SFVAR(is_pal), SFVAR(report_pal),
   SFEND
  };
@@ -738,6 +749,10 @@ int MDVDP::StateAction(StateMem *sm, int load, int data_only)
 
  if(load)
  {
+  vdp_line_phase %= 8; // tied to table in vdp_run.inc !!!
+
+  status &= ~(0x200 | 0x100);
+
   for(int i = 0; i < 0x800; i++)
   {
    bg_name_list[i] = i;
@@ -842,10 +857,9 @@ void MDVDP::SetSurface(EmulateSpecStruct *espec_arg)	//MDFN_Surface *new_surface
 }
 
 
-bool MDVDP::ToggleLayer(int which)
+void MDVDP::SetLayerEnableMask(uint64 mask)
 {
- UserLE ^= 1 << which;
- return((UserLE >> which) & 1);
+ UserLE = mask;
 }
 
 
@@ -901,13 +915,6 @@ void MDVDP::render_line(int line)
     const int32 vp_w = ((reg[0xC] & 0x1) ? 320 : 256);
 
     // Our display output window is nominally XXX*224 with NTSC, XXX*240 with PAL.
-    //if(line == 0)
-    {
-     if(!(reg[0x1] & 0x8) && is_pal)
-     {
-      //puts("FIXME, lines clear");
-     }
-    }
 
     if(line == 120)
     {
@@ -915,7 +922,7 @@ void MDVDP::render_line(int line)
      rect->w = WantAutoAspect ? vp_w : 320;
     }
 
-    if((reg[1] & 0x40) == 0x00)
+    if((reg[1] & 0x40) == 0x00 || line >= visible_frame_end)
     {
         /* Use the overscan color to clear the screen */
         memset(&lb[vp_x], 0x40 | border, vp_w);
@@ -983,44 +990,28 @@ void MDVDP::render_line(int line)
           if(!(UserLE & 0x4))
            merge(&nta_buf[0x20], &ntb_buf[0x20], &lb[0x20], lut[0], (reg[12] & 1) ? 320 : 256);
         }
+
+	if(reg[0] & 0x20)
+    	{
+        	memset(&lb[vp_x], 0x40 | border, 0x08);
+    	}
     }
 
-    if(reg[0] & 0x20)
+    const unsigned int lines_per_frame = is_pal ? 313 : 262;	// FIXME: have this as an inline function(duplicated in vdp_run.inc)
+    unsigned int cvp_line = (line + ((240 - visible_frame_end) >> 1)) % lines_per_frame;
+
+    if(cvp_line < 240)
     {
-        memset(&lb[vp_x], 0x40 | border, 0x08);
-    }
-
-    {
-     int width = (reg[12] & 1) ? 320 : 256;
-     uint32 *out = &surface->pixels[((line + ((240 - visible_frame_end) >> 1)) * (espec->InterlaceOn ? 2 : 1) + espec->InterlaceField) * surface->pitch32];
-
-	//printf("%d, %d %d\n", line, espec->InterlaceOn, espec->InterlaceField);
-
-     if(!WantAutoAspect)
+     //printf("ION: %d, IFIELD: %d --- %d, %d, im2f=%d\n", espec->InterlaceOn, espec->InterlaceField, line, ((line + ((240 - visible_frame_end) >> 1)) * (espec->InterlaceOn ? 2 : 1) + espec->InterlaceField), im2_flag);
+     //printf("%d, %d %d\n", line, espec->InterlaceOn, espec->InterlaceField);
+     switch(surface->format.bpp)
      {
-      int half_diff = (320 - vp_w) >> 1;
-      const uint32 cb = pixel_32[0x40 | border]; //surface->MakeColor(0,0,0);
+	case 16:CopyLineSurface<uint16>(&lb[vp_x], cvp_line, vp_w);
+		break;
 
-      if(im2_flag)
-      {
-       for(int i = 0; i < half_diff; i++)
-       {
-        out[i] = out[surface->pitch32 + i] = cb;
-        out[half_diff + vp_w + i] = out[surface->pitch32 + half_diff + vp_w + i]= cb;
-       }
-      }
-      else
-      {
-       for(int i = 0; i < half_diff; i++)
-       {
-        out[i] = cb;
-        out[half_diff + vp_w + i] = cb;
-       }
-      }
-      out += (320 - vp_w) >> 1;
+	case 32:CopyLineSurface<uint32>(&lb[vp_x], cvp_line, vp_w);
+		break;
      }
-
-     remap_32(lb + 0x20, out, pixel_32, width);
     }
 }
 /*--------------------------------------------------------------------------*/
@@ -1641,14 +1632,26 @@ int MDVDP::make_lut_bgobj_ste(int bx, int sx)
 /*--------------------------------------------------------------------------*/
 /* Remap functions                                                          */
 /*--------------------------------------------------------------------------*/
-
-void MDVDP::remap_32(uint8 *src, uint32 *dst, uint32 *table, int length)
+template<typename T>
+void MDVDP::CopyLineSurface(const uint8 *src, const unsigned cvp_line, const unsigned width)
 {
-    int count;
-    for(count = 0; count < length; count += 1)
-    {
-        *dst++ = table[*src++];
-    }
+     T *out = &surface->pix<T>()[(cvp_line * (espec->InterlaceOn ? 2 : 1) + espec->InterlaceField) * surface->pitchinpix];
+
+     if(!WantAutoAspect)
+     {
+      int half_diff = (320 - width) >> 1;
+      const uint32 cb = pixel_32[0x40 | border]; //surface->MakeColor(0,0,0);
+
+      for(int i = 0; i < half_diff; i++)
+      {
+       out[i] = cb;
+       out[half_diff + width + i] = cb;
+      }
+      out += (320 - width) >> 1;
+     }
+
+     for(unsigned i = 0; i < width; i++)
+      out[i] = pixel_32[src[i]];
 }
 
 /*--------------------------------------------------------------------------*/
