@@ -17,6 +17,7 @@
 
 #include        "mednafen.h"
 
+#include	<math.h>
 #include        <string.h>
 #include	<stdarg.h>
 #include	<errno.h>
@@ -30,6 +31,7 @@
 #include	"netplay.h"
 #include	"netplay-driver.h"
 #include	"general.h"
+#include	"string/trim.h"
 
 #include	"state.h"
 #include	"movie.h"
@@ -85,11 +87,18 @@ static const char *fname_extra = gettext_noop("See fname_format.txt for more inf
 
 static MDFNSetting MednafenSettings[] =
 {
+  { "netplay.password", MDFNSF_NOFLAGS, gettext_noop("Server password."), gettext_noop("Password to connect to the netplay server."), MDFNST_STRING, "" },
+  { "netplay.localplayers", MDFNSF_NOFLAGS, gettext_noop("Local player count."), gettext_noop("Number of local players for network play.  This number is advisory to the server, and the server may assign fewer players if the number of players requested is higher than the number of controllers currently available."), MDFNST_UINT, "1", "0", "16" },
+  { "netplay.nick", MDFNSF_NOFLAGS, gettext_noop("Nickname."), gettext_noop("Nickname to use for network play chat."), MDFNST_STRING, "" },
+  { "netplay.gamekey", MDFNSF_NOFLAGS, gettext_noop("Key to hash with the MD5 hash of the game."), NULL, MDFNST_STRING, "" },
+
   { "srwcompressor", MDFNSF_NOFLAGS, gettext_noop("Compressor to use with state rewinding"), NULL, MDFNST_ENUM, "quicklz", NULL, NULL, NULL, NULL, CompressorList },
 
   { "srwframes", MDFNSF_NOFLAGS, gettext_noop("Number of frames to keep states for when state rewinding is enabled."), 
 	gettext_noop("WARNING: Setting this to a large value may cause excessive RAM usage in some circumstances, such as with games that stream large volumes of data off of CDs."), MDFNST_UINT, "600", "10", "99999" },
 
+  { "cd.image_memcache", MDFNSF_NOFLAGS, gettext_noop("Cache entire CD images in memory."), gettext_noop("Reads the entire CD image(s) into memory at startup(which will cause a small delay).  Can help obviate emulation hiccups due to emulated CD access.  May cause more harm than good on low memory systems, systems with swap enabled, and/or when the disc images in question are on a fast SSD."), MDFNST_BOOL, "0" },
+
   { "filesys.untrusted_fip_check", MDFNSF_NOFLAGS, gettext_noop("Enable untrusted file-inclusion path security check."),
 	gettext_noop("When this setting is set to \"1\", the default, paths to files referenced from files like CUE sheets and PSF rips are checked for certain characters that can be used in directory traversal, and if found, loading is aborted.  Set it to \"0\" if you want to allow constructs like absolute paths in CUE sheets, but only if you understand the security implications of doing so(see \"Security Issues\" section in the documentation)."), MDFNST_BOOL, "1" },
 
@@ -138,7 +147,6 @@ static MDFNSetting RenamedSettings[] =
  { "netlocalplayers", MDFNSF_NOFLAGS, NULL, NULL, MDFNST_ALIAS , "netplay.localplayers" },
  { "netnick", MDFNSF_NOFLAGS, NULL, NULL, MDFNST_ALIAS         , "netplay.nick"   },
  { "netgamekey", MDFNSF_NOFLAGS, NULL, NULL, MDFNST_ALIAS      , "netplay.gamekey"        },
- { "netmerge", MDFNSF_NOFLAGS, NULL, NULL, MDFNST_ALIAS        , "netplay.merge"  },
  { "netsmallfont", MDFNSF_NOFLAGS, NULL, NULL, MDFNST_ALIAS    , "netplay.smallfont" },
 
  { "frameskip", MDFNSF_NOFLAGS, NULL, NULL, MDFNST_ALIAS       , "video.frameskip" },
@@ -146,6 +154,14 @@ static MDFNSetting RenamedSettings[] =
  { "glvsync", MDFNSF_NOFLAGS, NULL, NULL, MDFNST_ALIAS         , "video.glvsync" },
  { "fs", MDFNSF_NOFLAGS, NULL, NULL, MDFNST_ALIAS              , "video.fs" },
 
+ { "autofirefreq", MDFNSF_NOFLAGS, NULL, NULL, MDFNST_ALIAS    , "input.autofirefreq" },
+ { "analogthreshold", MDFNSF_NOFLAGS, NULL, NULL, MDFNST_ALIAS , "input.joystick.axis_threshold" },
+ { "ckdelay", MDFNSF_NOFLAGS, NULL, NULL, MDFNST_ALIAS         , "input.ckdelay" },
+
+
+ { "psx.input.port1.multitap", MDFNSF_NOFLAGS, NULL, NULL, MDFNST_ALIAS         , "psx.input.pport1.multitap" },
+ { "psx.input.port2.multitap", MDFNSF_NOFLAGS, NULL, NULL, MDFNST_ALIAS         , "psx.input.pport2.multitap" },
+
  { NULL }
 };
 
@@ -198,6 +214,7 @@ bool MDFNI_StartAVRecord(const char *path, double SoundRate)
   spec.VideoWidth = MDFNGameInfo->lcm_width;
   spec.VideoHeight = MDFNGameInfo->lcm_height;
   spec.VideoCodec = MDFN_GetSettingI("qtrecord.vcodec");
+  spec.MasterClock = MDFNGameInfo->MasterClock;
 
   if(spec.VideoWidth < MDFN_GetSettingUI("qtrecord.w_double_threshold"))
    spec.VideoWidth *= 2;
@@ -215,8 +232,15 @@ bool MDFNI_StartAVRecord(const char *path, double SoundRate)
   MDFN_printf(_("Video width: %u\n"), spec.VideoWidth);
   MDFN_printf(_("Video height: %u\n"), spec.VideoHeight);
   MDFN_printf(_("Video codec: %s\n"), MDFN_GetSettingS("qtrecord.vcodec").c_str());
-  MDFN_printf(_("Sound rate: %u\n"), spec.SoundRate);
-  MDFN_printf(_("Sound channels: %u\n"), spec.SoundChan);
+
+  if(spec.SoundRate && spec.SoundChan)
+  {
+   MDFN_printf(_("Sound rate: %u\n"), std::min<uint32>(spec.SoundRate, 64000));
+   MDFN_printf(_("Sound channels: %u\n"), spec.SoundChan);
+  }
+  else
+   MDFN_printf(_("Sound: Disabled\n"));
+
   MDFN_indent(-1);
   MDFN_printf("\n");
 
@@ -264,7 +288,7 @@ void MDFNI_CloseGame(void)
   if(MDFNGameInfo->name)
   {
    free(MDFNGameInfo->name);
-   MDFNGameInfo->name=0;
+   MDFNGameInfo->name = NULL;
   }
   MDFNMP_Kill();
 
@@ -295,9 +319,9 @@ void MDFNI_CloseGame(void)
  memset(PortDeviceCache, 0, sizeof(PortDeviceCache));
 }
 
-int MDFNI_NetplayStart(uint32 local_players, uint32 netmerge, const std::string &nickname, const std::string &game_key, const std::string &connect_password)
+int MDFNI_NetplayStart(void)
 {
- return(NetplayStart((const char**)PortDeviceCache, PortDataLenCache, local_players, netmerge, nickname, game_key, connect_password));
+ return(NetplayStart((const char**)PortDeviceCache, PortDataLenCache));
 }
 
 
@@ -309,6 +333,10 @@ extern MDFNGI EmulatedNES;
 extern MDFNGI EmulatedSNES;
 #endif
 
+#ifdef WANT_SNES_PERF_EMU
+extern MDFNGI EmulatedSNES_Perf;
+#endif
+
 #ifdef WANT_GBA_EMU
 extern MDFNGI EmulatedGBA;
 #endif
@@ -329,6 +357,10 @@ extern MDFNGI EmulatedMD;
 extern MDFNGI EmulatedNGP;
 #endif
 
+#ifdef WANT_PC_EMU
+extern MDFNGI EmulatedPC;
+#endif
+
 #ifdef WANT_PCE_EMU
 extern MDFNGI EmulatedPCE;
 #endif
@@ -428,9 +460,35 @@ static void ReadM3U(std::vector<std::string> &file_list, std::string path, unsig
  }
 }
 
+static void MakeGIName(MDFNGI* gi, const char* path)
+{
+ char* ns = NULL;
+ char* tmp;
+
+ if((ns = strdup(GetFNComponent(path))))
+ {
+  unsigned nslen = strlen(ns);
+
+  for(unsigned x = 0; x < nslen; x++)
+  {
+   if(ns[x] == '_')
+    ns[x] = ' ';
+   else if(ns[x] < 0x20)
+    ns[x] = ' ';
+  }
+
+  if((tmp = strrchr(ns, '.')))
+   *tmp = 0;
+
+  MDFN_trim(ns);
+
+  gi->name = (UTF8*)ns;
+ }
+}
+
 // TODO: LoadCommon()
 
-MDFNGI *MDFNI_LoadCD(const char *force_module, const char *devicename)
+MDFNGI *MDFNI_LoadCD(const char *force_module, const char *devicename, const bool is_device)
 {
  uint8 LayoutMD5[16];
 
@@ -438,38 +496,51 @@ MDFNGI *MDFNI_LoadCD(const char *force_module, const char *devicename)
 
  LastSoundMultiplier = 1;
 
- MDFN_printf(_("Loading %s...\n\n"), devicename ? devicename : _("PHYSICAL CD"));
+ if(is_device)
+ {
+  if(devicename)
+   MDFN_printf(_("Loading from CD drive device %s...\n\n"), devicename);
+  else
+   MDFN_printf(_("Load from default CD drive device...\n\n"));
+ }
+ else
+ {
+  assert(devicename != NULL);
+  MDFN_printf(_("Loading %s...\n\n"), devicename);
+ }
 
  try
  {
-  if(devicename && strlen(devicename) > 4 && !strcasecmp(devicename + strlen(devicename) - 4, ".m3u"))
+  const bool image_memcache = MDFN_GetSettingB("cd.image_memcache");
+
+  if(is_device)
   {
-   std::vector<std::string> file_list;
-
-   ReadM3U(file_list, devicename);
-
-   for(unsigned i = 0; i < file_list.size(); i++)
-   {
-    CDInterfaces.push_back(new CDIF(file_list[i].c_str()));
-   }
-
-   GetFileBase(devicename);
+   CDInterfaces.push_back(CDIF_Open(devicename, true, image_memcache));
+   GetFileBase("cdrom");
   }
   else
   {
-   CDInterfaces.push_back(new CDIF(devicename));
-   if(CDInterfaces[0]->IsPhysical())
+   if(devicename && strlen(devicename) > 4 && !strcasecmp(devicename + strlen(devicename) - 4, ".m3u"))
    {
-    GetFileBase("cdrom");
+    std::vector<std::string> file_list;
+
+    ReadM3U(file_list, devicename);
+
+    for(unsigned i = 0; i < file_list.size(); i++)
+    {
+     CDInterfaces.push_back(CDIF_Open(file_list[i].c_str(), false, image_memcache));
+    }
    }
    else
-    GetFileBase(devicename);
+   {
+    CDInterfaces.push_back(CDIF_Open(devicename, false, image_memcache));
+   }
+   GetFileBase(devicename);
   }
  }
  catch(std::exception &e)
  {
-  MDFND_PrintError(e.what());
-  MDFN_PrintError(_("Error opening CD."));
+  MDFN_PrintError(_("Error opening CD: %s"), e.what());
   return(0);
  }
 
@@ -486,12 +557,19 @@ MDFNGI *MDFNI_LoadCD(const char *force_module, const char *devicename)
   MDFN_printf(_("CD %d Layout:\n"), i + 1);
   MDFN_indent(1);
 
+  MDFN_printf(_("Disc Type: 0x%02x\n"), toc.disc_type);
+
   for(int32 track = toc.first_track; track <= toc.last_track; track++)
   {
    MDFN_printf(_("Track %2d, LBA: %6d  %s\n"), track, toc.tracks[track].lba, (toc.tracks[track].control & 0x4) ? "DATA" : "AUDIO");
   }
 
-  MDFN_printf("Leadout: %6d\n", toc.tracks[100].lba);
+  MDFN_printf(_("Leadout: %6d  %s\n"), toc.tracks[100].lba, (toc.tracks[100].control & 0x4) ? "DATA" : "AUDIO");
+
+  if((toc.tracks[toc.last_track].control & 0x4) != (toc.tracks[100].control & 0x4))
+   MDFN_printf(_("WARNING:  DATA/AUDIO TYPE MISMATCH BETWEEN LAST TRACK AND LEADOUT AREA."));
+
+
   MDFN_indent(-1);
   MDFN_printf("\n");
  }
@@ -582,10 +660,21 @@ MDFNGI *MDFNI_LoadCD(const char *force_module, const char *devicename)
 
         MDFN_printf(_("Using module: %s(%s)\n\n"), MDFNGameInfo->shortname, MDFNGameInfo->fullname);
 
-
  // TODO: include module name in hash
  memcpy(MDFNGameInfo->MD5, LayoutMD5, 16);
 
+ std::string modoverride_settings_file_path = MDFN_GetBaseDirectory() + std::string(PSS) + std::string(MDFNGameInfo->shortname) + std::string(".cfg");
+
+ if(!MDFN_LoadSettings(modoverride_settings_file_path.c_str(), true))
+ {
+  for(unsigned i = 0; i < CDInterfaces.size(); i++)
+   delete CDInterfaces[i];
+  CDInterfaces.clear();
+
+  MDFNGameInfo = NULL;
+  return(0);
+ }
+
  if(!(MDFNGameInfo->LoadCD(&CDInterfaces)))
  {
   for(unsigned i = 0; i < CDInterfaces.size(); i++)
@@ -596,6 +685,11 @@ MDFNGI *MDFNI_LoadCD(const char *force_module, const char *devicename)
   return(0);
  }
 
+ if(!MDFNGameInfo->name && !is_device)
+  MakeGIName(MDFNGameInfo, devicename);
+
+ MDFNI_SetLayerEnableMask(~0ULL);
+
  #ifdef WANT_DEBUGGER
  MDFNDBG_PostGameLoad(); 
  #endif
@@ -616,8 +710,8 @@ MDFNGI *MDFNI_LoadCD(const char *force_module, const char *devicename)
   MDFNMP_InstallReadPatches();
  }
 
-  last_sound_rate = -1;
-  memset(&last_pixel_format, 0, sizeof(MDFN_PixelFormat));
+ last_sound_rate = -1;
+ memset(&last_pixel_format, 0, sizeof(MDFN_PixelFormat));
 
  return(MDFNGameInfo);
 }
@@ -635,7 +729,9 @@ static bool LoadIPS(MDFNFILE &GameFile, const char *path)
  {
   ErrnoHolder ene(errno);
 
+  MDFN_indent(1);
   MDFN_printf(_("Failed: %s\n"), ene.StrError());
+  MDFN_indent(-1);
 
   if(ene.Errno() == ENOENT)
    return(1);
@@ -659,17 +755,11 @@ static bool LoadIPS(MDFNFILE &GameFile, const char *path)
 MDFNGI *MDFNI_LoadGame(const char *force_module, const char *name)
 {
         MDFNFILE GameFile;
-	struct stat stat_buf;
 	std::vector<FileExtensionSpecStruct> valid_iae;
 
-	if(strlen(name) > 4 && (!strcasecmp(name + strlen(name) - 4, ".cue") || !strcasecmp(name + strlen(name) - 4, ".toc") || !strcasecmp(name + strlen(name) - 4, ".m3u")))
+	if(strlen(name) > 4 && (!strcasecmp(name + strlen(name) - 4, ".cue") || !strcasecmp(name + strlen(name) - 4, ".toc") || !strcasecmp(name + strlen(name) - 4, ".ccd") || !strcasecmp(name + strlen(name) - 4, ".m3u")))
 	{
-	 return(MDFNI_LoadCD(force_module, name));
-	}
-	
-	if(!stat(name, &stat_buf) && !S_ISREG(stat_buf.st_mode))
-	{
-	 return(MDFNI_LoadCD(force_module, name));
+	 return(MDFNI_LoadCD(force_module, name, false));
 	}
 
 	MDFNI_CloseGame();
@@ -678,7 +768,7 @@ MDFNGI *MDFNI_LoadGame(const char *force_module, const char *name)
 
 	MDFNGameInfo = NULL;
 
-	MDFN_printf(_("Loading %s...\n"),name);
+	MDFN_printf(_("Loading %s...\n"), name);
 
 	MDFN_indent(1);
 
@@ -769,7 +859,7 @@ MDFNGI *MDFNI_LoadGame(const char *force_module, const char *name)
 	 if(force_module)
           MDFN_PrintError(_("Unrecognized system \"%s\"!"), force_module);
 	 else
-          MDFN_PrintError(_("Unrecognized file format.  Sorry."));
+          MDFN_PrintError(_("Unrecognized file format."));
 
          MDFN_indent(-1);
          MDFNGameInfo = NULL;
@@ -785,6 +875,17 @@ MDFNGI *MDFNI_LoadGame(const char *force_module, const char *name)
         MDFNGameInfo->name = NULL;
         MDFNGameInfo->rotated = 0;
 
+	std::string modoverride_settings_file_path = MDFN_GetBaseDirectory() + std::string(PSS) + std::string(MDFNGameInfo->shortname) + std::string(".cfg");
+
+	if(!MDFN_LoadSettings(modoverride_settings_file_path.c_str(), true))
+	{
+	 GameFile.Close();
+
+	 MDFN_indent(-2);
+	 MDFNGameInfo = NULL;
+	 return(0);
+	}
+
         if(MDFNGameInfo->Load(name, &GameFile) <= 0)
 	{
          GameFile.Close();
@@ -799,6 +900,8 @@ MDFNGI *MDFNI_LoadGame(const char *force_module, const char *name)
 	 MDFNMP_InstallReadPatches();
 	}
 
+	MDFNI_SetLayerEnableMask(~0ULL);
+
 	#ifdef WANT_DEBUGGER
 	MDFNDBG_PostGameLoad();
 	#endif
@@ -811,20 +914,7 @@ MDFNGI *MDFNI_LoadGame(const char *force_module, const char *name)
 	MDFN_indent(-2);
 
 	if(!MDFNGameInfo->name)
-        {
-         unsigned int x;
-         char *tmp;
-
-         MDFNGameInfo->name = (UTF8 *)strdup(GetFNComponent(name));
-
-         for(x=0;x<strlen((char *)MDFNGameInfo->name);x++)
-         {
-          if(MDFNGameInfo->name[x] == '_')
-           MDFNGameInfo->name[x] = ' ';
-         }
-         if((tmp = strrchr((char *)MDFNGameInfo->name, '.')))
-          *tmp = 0;
-        }
+	 MakeGIName(MDFNGameInfo, name); 
 
 	PrevInterlaced = false;
 	deint.ClearState();
@@ -948,6 +1038,10 @@ bool MDFNI_InitializeModules(const std::vector<MDFNGI *> &ExternalSystems)
   &EmulatedSNES,
   #endif
 
+  #ifdef WANT_SNES_PERF_EMU
+  &EmulatedSNES_Perf,
+  #endif
+
   #ifdef WANT_GB_EMU
   &EmulatedGB,
   #endif
@@ -956,6 +1050,10 @@ bool MDFNI_InitializeModules(const std::vector<MDFNGI *> &ExternalSystems)
   &EmulatedGBA,
   #endif
 
+  #ifdef WANT_PC_EMU
+  &EmulatedPC,
+  #endif
+
   #ifdef WANT_PCE_EMU
   &EmulatedPCE,
   #endif
@@ -1001,6 +1099,8 @@ bool MDFNI_InitializeModules(const std::vector<MDFNGI *> &ExternalSystems)
  };
  std::string i_modules_string, e_modules_string;
 
+ assert(MEDNAFEN_VERSION_NUMERIC >= 0x0927);
+
  for(unsigned int i = 0; i < sizeof(InternalSystems) / sizeof(MDFNGI *); i++)
  {
   AddSystem(InternalSystems[i]);
@@ -1026,38 +1126,30 @@ bool MDFNI_InitializeModules(const std::vector<MDFNGI *> &ExternalSystems)
 
  MDFNSystemsPrio.sort(MDFNSystemsPrio_CompareFunc);
 
- #if 0
- std::string a_modules;
-
- std::list<MDFNGI *>:iterator it;
-
- for(it = MDFNSystemsPrio.
- f
- #endif
-
  CDUtility::CDUtility_Init();
 
  return(1);
 }
 
+static std::string settings_file_path;
 int MDFNI_Initialize(const char *basedir, const std::vector<MDFNSetting> &DriverSettings)
 {
 	// FIXME static
 	static std::vector<MDFNSetting> dynamic_settings;
 
+	// DO NOT REMOVE/DISABLE THESE MATH AND COMPILER SANITY TESTS.  THEY EXIST FOR A REASON.
 	if(!MDFN_RunMathTests())
 	{
 	 return(0);
 	}
 
-
 	memset(PortDataCache, 0, sizeof(PortDataCache));
 	memset(PortDataLenCache, 0, sizeof(PortDataLenCache));
 	memset(PortDeviceCache, 0, sizeof(PortDeviceCache));
 
 	lzo_init();
 
-	MDFNI_SetBaseDirectory(basedir);
+	MDFN_SetBaseDirectory(basedir);
 
 	MDFN_InitFontData();
 
@@ -1066,9 +1158,9 @@ int MDFNI_Initialize(const char *basedir, const std::vector<MDFNSetting> &Driver
 	{
 	 MDFNSetting setting;
 	 const char *sysname;
-	
+
 	 sysname = (const char *)MDFNSystems[i]->shortname;
- 
+
 	 if(!MDFNSystems[i]->soundchan)
 	  printf("0 sound channels for %s????\n", sysname);
 
@@ -1107,7 +1199,8 @@ int MDFNI_Initialize(const char *basedir, const std::vector<MDFNSetting> &Driver
 
 	MDFN_MergeSettings(RenamedSettings);
 
-        if(!MFDN_LoadSettings(basedir))
+	settings_file_path = std::string(basedir) + std::string(PSS) + std::string("mednafen-09x.cfg");
+	if(!MDFN_LoadSettings(settings_file_path.c_str()))
 	 return(0);
 
 	#ifdef WANT_DEBUGGER
@@ -1119,7 +1212,8 @@ int MDFNI_Initialize(const char *basedir, const std::vector<MDFNSetting> &Driver
 
 void MDFNI_Kill(void)
 {
- MDFN_SaveSettings();
+ MDFN_SaveSettings(settings_file_path.c_str());
+ MDFN_KillSettings();
 }
 
 static double multiplier_save, volume_save;
@@ -1149,6 +1243,54 @@ static void ProcessAudio(EmulateSpecStruct *espec)
     SoundBufPristine[orig_size + i] = SoundBuf[i];
   }
 
+#if 0
+  //
+  // Sine wave sweep for test purposes.
+  //
+  {
+   static double phase = 0;
+   static double phase_inc = 0.000;
+   static double phase_inc_inc = 0.000003;
+   static int32 scounter = 0;
+
+   int16 *sbuf = SoundBuf;
+   int32 slen = SoundBufSize;
+
+
+   if(MDFNGameInfo->soundchan == 2)
+   {
+    for(int i = 0; i < slen; i++)
+    {
+     int16 tmp = 127 * 256 * sin(phase);
+
+     tmp = (scounter & 8) ? 127 * 256 : -127 * 256;
+
+     sbuf[i * 2 + 0] = tmp;
+     sbuf[i * 2 + 1] = tmp;
+     phase += phase_inc;
+     phase_inc += phase_inc_inc;
+     scounter++;
+    }
+   }
+   else
+   {
+    for(int i = 0; i < slen; i++)
+    {
+     int16 tmp = 127 * 256 * sin(phase);
+
+     tmp = (scounter & 8) ? 127 * 256 : -127 * 256;
+
+     sbuf[i] = tmp;
+
+     phase += phase_inc;
+     phase_inc += phase_inc_inc;
+     scounter++;
+    }
+   }
+  }
+
+#endif
+
   if(espec->NeedSoundReverse)
   {
    int16 *yaybuf = SoundBuf;
@@ -1296,11 +1438,22 @@ void MDFN_MidSync(EmulateSpecStruct *espec)
  espec->MasterCyclesALMS = espec->MasterCycles;
 }
 
+void MDFN_MidLineUpdate(EmulateSpecStruct *espec, int y)
+{
+ //MDFND_MidLineUpdate(espec, y);
+}
+
 void MDFNI_Emulate(EmulateSpecStruct *espec)
 {
  multiplier_save = 1;
  volume_save = 1;
 
+ // Initialize some espec member data to zero, to catch some types of bugs.
+ espec->DisplayRect.x = 0;
+ espec->DisplayRect.w = 0;
+ espec->DisplayRect.y = 0;
+ espec->DisplayRect.h = 0;
+
  assert((bool)(espec->SoundBuf != NULL) == (bool)espec->SoundRate && (bool)espec->SoundRate == (bool)espec->SoundBufMaxSize);
 
  espec->SoundBufSize = 0;
@@ -1354,12 +1507,12 @@ void MDFNI_Emulate(EmulateSpecStruct *espec)
   if(MDFNMOV_IsPlaying())
   {
    espec->NeedRewind = 0;
-   MDFN_DispMessage(_("Can't rewind during movie playback(yet!)."));
+   MDFN_DispMessage(_("Can't rewind during movie playback."));
   }
   else if(MDFNnetplay)
   {
    espec->NeedRewind = 0;
-   MDFN_DispMessage(_("Silly-billy, can't rewind during netplay."));
+   MDFN_DispMessage(_("Can't rewind during netplay."));
   }
   else if(MDFNGameInfo->GameType == GMT_PLAYER)
   {
@@ -1368,10 +1521,86 @@ void MDFNI_Emulate(EmulateSpecStruct *espec)
   }
  }
 
- espec->NeedSoundReverse = MDFN_StateEvil(espec->NeedRewind);
+ // Don't even save states with state rewinding if netplay is enabled, it will degrade netplay performance, and can cause
+ // desynchs with some emulation(IE SNES based on bsnes).
+
+ if(MDFNnetplay)
+  espec->NeedSoundReverse = false;
+ else
+  espec->NeedSoundReverse = MDFN_StateEvil(espec->NeedRewind);
 
  MDFNGameInfo->Emulate(espec);
 
+#if 0
+ static const unsigned SpeculativeRender = 6;
+
+ if(SpeculativeRender == 0)
+  MDFNGameInfo->Emulate(espec);
+ else
+ {
+  StateMem shoe;
+
+  for(unsigned ra = 0; ra <= SpeculativeRender; ra++)
+  {
+   EmulateSpecStruct tmp_espec = *espec;
+
+   if(ra != SpeculativeRender)
+   {
+    if(ra == 1)
+    {
+     memset(&shoe, 0, sizeof(shoe));
+     MDFNSS_SaveSM(&shoe, false, false);
+    }
+    tmp_espec.skip = true;
+    tmp_espec.NeedSoundReverse = false;
+//    tmp_espec.SoundBuf = NULL;
+//    tmp_espec.SoundBufMaxSize = 0;
+    MDFNGameInfo->Emulate(&tmp_espec);
+    espec->VideoFormatChanged = false;
+    espec->SoundFormatChanged = false;
+   }
+   else
+   {
+    MDFNGameInfo->Emulate(espec);
+
+    shoe.loc = 0;
+    MDFNSS_LoadSM(&shoe, false, false);
+
+    if(shoe.data)
+    {
+     free(shoe.data);
+     shoe.data = NULL;
+    }
+   }
+  }
+ }
+#endif
+
+ //
+ // Sanity checks
+ //
+ if(!espec->skip)
+ {
+  if(espec->DisplayRect.h == 0)
+  {
+   fprintf(stderr, "[BUG] espec->DisplayRect.h == 0\n");
+  }
+ }
+
+ if(!espec->MasterCycles)
+ {
+  fprintf(stderr, "[BUG] espec->MasterCycles == 0\n");
+ }
+
+ if(espec->MasterCycles < espec->MasterCyclesALMS)
+ {
+  fprintf(stderr, "[BUG] espec->MasterCycles < espec->MasterCyclesALMS\n");
+ }
+
+ //
+ //
+ //
+
  if(espec->InterlaceOn)
  {
   if(!PrevInterlaced)
@@ -1402,7 +1631,7 @@ void MDFNI_Emulate(EmulateSpecStruct *espec)
 
   try
   {
-   qtrecorder->WriteFrame(espec->surface, espec->DisplayRect, espec->LineWidths, espec->SoundBuf, espec->SoundBufSize);
+   qtrecorder->WriteFrame(espec->surface, espec->DisplayRect, espec->LineWidths, espec->SoundBuf, espec->SoundBufSize, espec->MasterCycles);
   }
   catch(std::exception &e)
   {
@@ -1417,7 +1646,8 @@ void MDFNI_Emulate(EmulateSpecStruct *espec)
   espec->SoundBufSize = sbs_backup;
  }
 
- TBlur_Run(espec);
+ if(TBlur_IsOn())
+  TBlur_Run(espec);
 }
 
 // This function should only be called for state rewinding.
@@ -1551,7 +1781,7 @@ void MDFN_DoSimpleCommand(int cmd)
 void MDFN_QSimpleCommand(int cmd)
 {
  if(MDFNnetplay)
-  MDFNNET_SendCommand(cmd, 0);
+  NetplaySendCommand(cmd, 0);
  else
  {
   if(!MDFNMOV_IsPlaying())
@@ -1577,9 +1807,22 @@ void MDFNI_Reset(void)
 }
 
 // Arcade-support functions
+
+
+//
+// Quick and dirty kludge until we can (re-)abstract DIP switch handling properly.
+//
+#ifdef WANT_NES_EMU
+void MDFN_VSUniToggleDIPView(void);
+#endif
 void MDFNI_ToggleDIPView(void)
 {
-
+#ifdef WANT_NES_EMU
+ if(MDFNGameInfo == &EmulatedNES)
+ {
+  MDFN_VSUniToggleDIPView();
+ }
+#endif
 }
 
 void MDFNI_ToggleDIP(int which)
@@ -1626,25 +1869,11 @@ void MDFNI_DiskEject()
  MDFN_QSimpleCommand(MDFN_MSC_EJECT_DISK);
 }
 
-
-void MDFNI_ToggleLayer(int which)
+void MDFNI_SetLayerEnableMask(uint64 mask)
 {
- if(MDFNGameInfo && MDFNGameInfo->LayerNames)
+ if(MDFNGameInfo && MDFNGameInfo->SetLayerEnableMask)
  {
-  const char *goodies = MDFNGameInfo->LayerNames;
-  int x = 0;
-  while(x != which)
-  {
-   while(*goodies)
-    goodies++;
-   goodies++;
-   if(!*goodies) return; // ack, this layer doesn't exist.
-   x++;
-  }
-  if(MDFNGameInfo->ToggleLayer(which))
-   MDFN_DispMessage(_("%s enabled."), _(goodies));
-  else
-   MDFN_DispMessage(_("%s disabled."), _(goodies));
+  MDFNGameInfo->SetLayerEnableMask(mask);
  }
 }
 
