@@ -23,27 +23,12 @@ void MDFN_DebugPrintReal(const char *file, const int line, const char *format, .
 
 #define MDFN_DebugPrint(format, ...) MDFN_DebugPrintReal(__FILE__, __LINE__, format, ## __VA_ARGS__)
 
-
-class MDFNException
-{
-	public:
-
-	MDFNException();
-	~MDFNException();
-
-	char TheMessage[1024];
-
-	void AddPre(const char *format, ...);
-	void AddPost(const char *format, ...);
-};
-
-
-void MDFN_LoadGameCheats(FILE *override);
 void MDFN_FlushGameCheats(int nosave);
 void MDFN_DoSimpleCommand(int cmd);
 void MDFN_QSimpleCommand(int cmd);
 
 void MDFN_MidSync(EmulateSpecStruct *espec);
+void MDFN_MidLineUpdate(EmulateSpecStruct *espec, int y);
 
 #include "state.h"
 int MDFN_RawInputStateAction(StateMem *sm, int load, int data_only);
