@@ -34,6 +34,10 @@
  #include <altivec.h>
 #endif
 
+#ifdef __FAST_MATH__
+ #error "filter.cpp not compatible with unsafe math optimizations!"
+#endif
+
 static void kaiser_window( double* io, int count, double beta )
 {
         int const accuracy = 16; //12;
@@ -190,20 +194,20 @@ static INLINE void DoMAC_MMX(int16 *wave, int16 *coeffs, int32 count, int32 *acc
 "pxor %%mm5, %%mm5\n\t"
 "MMX_Loop:\n\t"
 
-"movq (%%"X86_REGC"di), %%mm0\n\t"
-"pmaddwd (%%"X86_REGC"si), %%mm0\n\t"
+"movq (%%" X86_REGC "di), %%mm0\n\t"
+"pmaddwd (%%" X86_REGC "si), %%mm0\n\t"
 
-"movq 8(%%"X86_REGC"di), %%mm2\n\t"
+"movq 8(%%" X86_REGC "di), %%mm2\n\t"
 "psrad $1, %%mm0\n\t"
-"pmaddwd 8(%%"X86_REGC"si), %%mm2\n\t"
+"pmaddwd 8(%%" X86_REGC "si), %%mm2\n\t"
 
-"movq 16(%%"X86_REGC"di), %%mm6\n\t"
+"movq 16(%%" X86_REGC "di), %%mm6\n\t"
 "psrad $1, %%mm2\n\t"
-"pmaddwd 16(%%"X86_REGC"si), %%mm6\n\t"
+"pmaddwd 16(%%" X86_REGC "si), %%mm6\n\t"
 
-"movq 24(%%"X86_REGC"di), %%mm7\n\t"
+"movq 24(%%" X86_REGC "di), %%mm7\n\t"
 "psrad $1, %%mm6\n\t"
-"pmaddwd 24(%%"X86_REGC"si), %%mm7\n\t"
+"pmaddwd 24(%%" X86_REGC "si), %%mm7\n\t"
 
 "paddd %%mm0, %%mm1\n\t"
 "paddd %%mm2, %%mm3\n\t"
@@ -211,17 +215,17 @@ static INLINE void DoMAC_MMX(int16 *wave, int16 *coeffs, int32 count, int32 *acc
 "paddd %%mm6, %%mm4\n\t"
 "paddd %%mm7, %%mm5\n\t"
 
-"add"X86_REGAT" $32, %%"X86_REGC"si\n\t"
-"add"X86_REGAT" $32, %%"X86_REGC"di\n\t"
+"add" X86_REGAT " $32, %%" X86_REGC "si\n\t"
+"add" X86_REGAT " $32, %%" X86_REGC "di\n\t"
 "subl $1, %%ecx\n\t"
 "jnz MMX_Loop\n\t"
 
 //
 #if FIR_TABLE_EXTRA_BITS != 0
-"psrad $"FIR_TABLE_EXTRA_BITS_S", %%mm1\n\t"
-"psrad $"FIR_TABLE_EXTRA_BITS_S", %%mm3\n\t"
-"psrad $"FIR_TABLE_EXTRA_BITS_S", %%mm4\n\t"
-"psrad $"FIR_TABLE_EXTRA_BITS_S", %%mm5\n\t"
+"psrad $" FIR_TABLE_EXTRA_BITS_S ", %%mm1\n\t"
+"psrad $" FIR_TABLE_EXTRA_BITS_S ", %%mm3\n\t"
+"psrad $" FIR_TABLE_EXTRA_BITS_S ", %%mm4\n\t"
+"psrad $" FIR_TABLE_EXTRA_BITS_S ", %%mm5\n\t"
 #endif
 
 // Now, mm1, mm3, mm4, mm5 contain 8 32-bit sums that need to be added together.
@@ -235,7 +239,7 @@ static INLINE void DoMAC_MMX(int16 *wave, int16 *coeffs, int32 count, int32 *acc
 
 "psrad $15, %%mm1\n\t"
 //"psrad $16, %%mm1\n\t"
-"movd %%mm1, (%%"X86_REGC"dx)\n\t"
+"movd %%mm1, (%%" X86_REGC "dx)\n\t"
  : "=D" (dummy), "=S" (dummy), "=c" (dummy)
  : "D" (wave), "S" (coeffs), "c" ((count + 0xF) >> 4), "d" (accum_output)
 #ifdef __x86_64__
@@ -534,9 +538,9 @@ NES_Resampler::NES_Resampler(double input_rate, double output_rate, double rate_
 
  if(quality == -2)
  {
-  k_beta = 3.384;
-  k_d = 2.23;
-  NumCoeffs = 128;
+  k_beta = 4.538;
+  k_d = 2.93;
+  NumCoeffs = 192;
  }
  else if(quality == -1)
  {
@@ -594,10 +598,10 @@ NES_Resampler::NES_Resampler(double input_rate, double output_rate, double rate_
   {
    count++;
    findo += s_ratio;
-  } while( fabs(1.0 - ((round(findo) / count) / s_ratio)) > rate_error);
+  } while( fabs(1.0 - ((floor(0.5 + findo) / count) / s_ratio)) > rate_error);
 
-  s_ratio = round(findo) / count;
-  findo_i = (uint32) round(findo);
+  s_ratio = floor(0.5 + findo) / count;
+  findo_i = (uint32) floor(0.5 + findo);
   ratio = 1 / s_ratio;
   NumPhases = count;
 
