@@ -70,81 +70,81 @@ static const InputDeviceInputInfoStruct ArkanoidIDII[] =
 
 static const InputDeviceInputInfoStruct FKBIDII[0x48] =
 {
- { "f1", "F1", 0, IDIT_BUTTON_BYTE },
- { "f2", "F2", 1, IDIT_BUTTON_BYTE },
- { "f3", "F3", 2, IDIT_BUTTON_BYTE },
- { "f4", "F4", 3, IDIT_BUTTON_BYTE },
- { "f5", "F5", 4, IDIT_BUTTON_BYTE },
- { "f6", "F6", 5, IDIT_BUTTON_BYTE },
- { "f7", "F7", 6, IDIT_BUTTON_BYTE },
- { "f8", "F8", 7, IDIT_BUTTON_BYTE },
+ { "f1", "F1", 0, IDIT_BUTTON },
+ { "f2", "F2", 1, IDIT_BUTTON },
+ { "f3", "F3", 2, IDIT_BUTTON },
+ { "f4", "F4", 3, IDIT_BUTTON },
+ { "f5", "F5", 4, IDIT_BUTTON },
+ { "f6", "F6", 5, IDIT_BUTTON },
+ { "f7", "F7", 6, IDIT_BUTTON },
+ { "f8", "F8", 7, IDIT_BUTTON },
 
- { "1", "1", 8, IDIT_BUTTON_BYTE },
- { "2", "2", 9, IDIT_BUTTON_BYTE },
- { "3", "3", 10, IDIT_BUTTON_BYTE },
- { "4", "4", 11, IDIT_BUTTON_BYTE },
- { "5", "5", 12, IDIT_BUTTON_BYTE },
- { "6", "6", 13, IDIT_BUTTON_BYTE },
- { "7", "7", 14, IDIT_BUTTON_BYTE },
- { "8", "8", 15, IDIT_BUTTON_BYTE },
- { "9", "9", 16, IDIT_BUTTON_BYTE },
- { "0", "0", 17, IDIT_BUTTON_BYTE },
- { "minus", "-", 18, IDIT_BUTTON_BYTE },
- { "caret", "^", 19, IDIT_BUTTON_BYTE },
- { "backslash", "\\", 20, IDIT_BUTTON_BYTE },
- { "stop", "STOP", 21, IDIT_BUTTON_BYTE },
+ { "1", "1", 8, IDIT_BUTTON },
+ { "2", "2", 9, IDIT_BUTTON },
+ { "3", "3", 10, IDIT_BUTTON },
+ { "4", "4", 11, IDIT_BUTTON },
+ { "5", "5", 12, IDIT_BUTTON },
+ { "6", "6", 13, IDIT_BUTTON },
+ { "7", "7", 14, IDIT_BUTTON },
+ { "8", "8", 15, IDIT_BUTTON },
+ { "9", "9", 16, IDIT_BUTTON },
+ { "0", "0", 17, IDIT_BUTTON },
+ { "minus", "-", 18, IDIT_BUTTON },
+ { "caret", "^", 19, IDIT_BUTTON },
+ { "backslash", "\\", 20, IDIT_BUTTON },
+ { "stop", "STOP", 21, IDIT_BUTTON },
 
- { "escape", "ESC", 22, IDIT_BUTTON_BYTE },
- { "q", "Q", 23, IDIT_BUTTON_BYTE },
- { "w", "W", 24, IDIT_BUTTON_BYTE },
- { "e", "E", 25, IDIT_BUTTON_BYTE },
- { "r", "R", 26, IDIT_BUTTON_BYTE },
- { "t", "T", 27, IDIT_BUTTON_BYTE },
- { "y", "Y", 28, IDIT_BUTTON_BYTE },
- { "u", "U", 29, IDIT_BUTTON_BYTE },
- { "i", "I", 30, IDIT_BUTTON_BYTE },
- { "o", "O", 31, IDIT_BUTTON_BYTE },
- { "p", "P", 32, IDIT_BUTTON_BYTE },
- { "at", "@", 33, IDIT_BUTTON_BYTE },
- { "left_bracket", "[", 34, IDIT_BUTTON_BYTE },
- { "return", "RETURN", 35, IDIT_BUTTON_BYTE },
- { "ctrl", "CTR", 36, IDIT_BUTTON_BYTE },
- { "a", "A", 37, IDIT_BUTTON_BYTE },
- { "s", "S", 38, IDIT_BUTTON_BYTE },
- { "d", "D", 39, IDIT_BUTTON_BYTE },
- { "f", "F", 40, IDIT_BUTTON_BYTE },
- { "g", "G", 41, IDIT_BUTTON_BYTE },
- { "h", "H", 42, IDIT_BUTTON_BYTE },
- { "j", "J", 43, IDIT_BUTTON_BYTE },
- { "k", "K", 44, IDIT_BUTTON_BYTE },
- { "l", "L", 45, IDIT_BUTTON_BYTE },
- { "semicolon", ";", 46, IDIT_BUTTON_BYTE },
- { "colon", ":", 47, IDIT_BUTTON_BYTE },
- { "right_bracket", "]", 48, IDIT_BUTTON_BYTE },
- { "kana", "カナ", 49, IDIT_BUTTON_BYTE },
- { "left_shift", "Left SHIFT", 50, IDIT_BUTTON_BYTE },
- { "z", "Z", 51, IDIT_BUTTON_BYTE },
- { "x", "X", 52, IDIT_BUTTON_BYTE },
- { "c", "C", 53, IDIT_BUTTON_BYTE },
- { "v", "V", 54, IDIT_BUTTON_BYTE },
- { "b", "B", 55, IDIT_BUTTON_BYTE },
- { "n", "N", 56, IDIT_BUTTON_BYTE },
- { "m", "M", 57, IDIT_BUTTON_BYTE },
- { "comma", ",", 58, IDIT_BUTTON_BYTE },
- { "period", ".", 59, IDIT_BUTTON_BYTE },
- { "slash", "/", 60, IDIT_BUTTON_BYTE },
- { "empty", "Empty", 61, IDIT_BUTTON_BYTE },
- { "right_shift", "Right SHIFT", 62, IDIT_BUTTON_BYTE },
- { "graph", "GRPH", 63, IDIT_BUTTON_BYTE },
- { "space", "SPACE", 64, IDIT_BUTTON_BYTE },
+ { "escape", "ESC", 22, IDIT_BUTTON },
+ { "q", "Q", 23, IDIT_BUTTON },
+ { "w", "W", 24, IDIT_BUTTON },
+ { "e", "E", 25, IDIT_BUTTON },
+ { "r", "R", 26, IDIT_BUTTON },
+ { "t", "T", 27, IDIT_BUTTON },
+ { "y", "Y", 28, IDIT_BUTTON },
+ { "u", "U", 29, IDIT_BUTTON },
+ { "i", "I", 30, IDIT_BUTTON },
+ { "o", "O", 31, IDIT_BUTTON },
+ { "p", "P", 32, IDIT_BUTTON },
+ { "at", "@", 33, IDIT_BUTTON },
+ { "left_bracket", "[", 34, IDIT_BUTTON },
+ { "return", "RETURN", 35, IDIT_BUTTON },
+ { "ctrl", "CTR", 36, IDIT_BUTTON },
+ { "a", "A", 37, IDIT_BUTTON },
+ { "s", "S", 38, IDIT_BUTTON },
+ { "d", "D", 39, IDIT_BUTTON },
+ { "f", "F", 40, IDIT_BUTTON },
+ { "g", "G", 41, IDIT_BUTTON },
+ { "h", "H", 42, IDIT_BUTTON },
+ { "j", "J", 43, IDIT_BUTTON },
+ { "k", "K", 44, IDIT_BUTTON },
+ { "l", "L", 45, IDIT_BUTTON },
+ { "semicolon", ";", 46, IDIT_BUTTON },
+ { "colon", ":", 47, IDIT_BUTTON },
+ { "right_bracket", "]", 48, IDIT_BUTTON },
+ { "kana", "カナ", 49, IDIT_BUTTON },
+ { "left_shift", "Left SHIFT", 50, IDIT_BUTTON },
+ { "z", "Z", 51, IDIT_BUTTON },
+ { "x", "X", 52, IDIT_BUTTON },
+ { "c", "C", 53, IDIT_BUTTON },
+ { "v", "V", 54, IDIT_BUTTON },
+ { "b", "B", 55, IDIT_BUTTON },
+ { "n", "N", 56, IDIT_BUTTON },
+ { "m", "M", 57, IDIT_BUTTON },
+ { "comma", ",", 58, IDIT_BUTTON },
+ { "period", ".", 59, IDIT_BUTTON },
+ { "slash", "/", 60, IDIT_BUTTON },
+ { "empty", "Empty", 61, IDIT_BUTTON },
+ { "right_shift", "Right SHIFT", 62, IDIT_BUTTON },
+ { "graph", "GRPH", 63, IDIT_BUTTON },
+ { "space", "SPACE", 64, IDIT_BUTTON },
 
- { "clear", "CLR", 65, IDIT_BUTTON_BYTE },
- { "insert", "INS", 66, IDIT_BUTTON_BYTE },
- { "delete", "DEL", 67, IDIT_BUTTON_BYTE },
- { "up", "UP", 68, IDIT_BUTTON_BYTE },
- { "left", "LEFT", 69, IDIT_BUTTON_BYTE },
- { "right", "RIGHT", 70, IDIT_BUTTON_BYTE },
- { "down", "DOWN", 71, IDIT_BUTTON_BYTE },
+ { "clear", "CLR", 65, IDIT_BUTTON },
+ { "insert", "INS", 66, IDIT_BUTTON },
+ { "delete", "DEL", 67, IDIT_BUTTON },
+ { "up", "UP", 68, IDIT_BUTTON },
+ { "left", "LEFT", 69, IDIT_BUTTON },
+ { "right", "RIGHT", 70, IDIT_BUTTON },
+ { "down", "DOWN", 71, IDIT_BUTTON },
 };
 
 static const InputDeviceInputInfoStruct HypershotIDII[] =
@@ -231,23 +231,6 @@ static const InputDeviceInputInfoStruct BWorldIDII[] =
  { "bd13", "Barcode Digit 13", -1, IDIT_BYTE_SPECIAL },
 };
 
-static InputDeviceInfoStruct InputDevice4Player[] = 
-{
- {
-  "gamepad",
-  "Gamepad",
-  NULL,
-  sizeof(GamepadIDII) / sizeof(InputDeviceInputInfoStruct),
-  GamepadIDII,
- }
-};
-
-static const InputPortInfoStruct PortInfo4Player[] =
-{
- { -2, "port3", "Port 3", sizeof(InputDevice4Player) / sizeof(InputDeviceInfoStruct), InputDevice4Player, "gamepad" },
- { -2, "port4", "Port 4", sizeof(InputDevice4Player) / sizeof(InputDeviceInfoStruct), InputDevice4Player, "gamepad" },
-};
-
 static InputDeviceInfoStruct InputDeviceInfoNESPort34[] =
 {
  // None
@@ -255,6 +238,7 @@ static InputDeviceInfoStruct InputDeviceInfoNESPort34[] =
   "none",
   "none",
   NULL,
+  NULL,
   0,
   NULL
  },
@@ -264,6 +248,7 @@ static InputDeviceInfoStruct InputDeviceInfoNESPort34[] =
   "gamepad",
   "Gamepad",
   NULL,
+  NULL,
   sizeof(GamepadIDII) / sizeof(InputDeviceInputInfoStruct),
   GamepadIDII,
  },
@@ -277,6 +262,7 @@ static InputDeviceInfoStruct InputDeviceInfoNESPort[] =
   "none",
   "none",
   NULL,
+  NULL,
   0,
   NULL
  },
@@ -286,6 +272,7 @@ static InputDeviceInfoStruct InputDeviceInfoNESPort[] =
   "gamepad",
   "Gamepad",
   NULL,
+  NULL,
   sizeof(GamepadIDII) / sizeof(InputDeviceInputInfoStruct),
   GamepadIDII,
  },
@@ -295,15 +282,17 @@ static InputDeviceInfoStruct InputDeviceInfoNESPort[] =
   "zapper",
   "Zapper",
   NULL,
+  NULL,
   sizeof(ZapperIDII) / sizeof(InputDeviceInputInfoStruct),
   ZapperIDII,
  },
- 
+
  // Powerpad A
  {
   "powerpada",
   "Power Pad Side A",
   NULL,
+  NULL,
   sizeof(PowerpadIDII) / sizeof(InputDeviceInputInfoStruct),
   PowerpadIDII,
  },
@@ -313,6 +302,7 @@ static InputDeviceInfoStruct InputDeviceInfoNESPort[] =
   "powerpadb",
   "Power Pad Side B",
   NULL,
+  NULL,
   sizeof(PowerpadIDII) / sizeof(InputDeviceInputInfoStruct),
   PowerpadIDII,
  },
@@ -322,6 +312,7 @@ static InputDeviceInfoStruct InputDeviceInfoNESPort[] =
   "arkanoid",
   "Arkanoid Paddle",
   NULL,
+  NULL,
   sizeof(ArkanoidIDII) / sizeof(InputDeviceInputInfoStruct),
   ArkanoidIDII,
  },
@@ -336,6 +327,7 @@ static InputDeviceInfoStruct InputDeviceInfoFamicomPort[] =
   "none",
   "none",
   NULL,
+  NULL,
   0,
   NULL
  },
@@ -345,6 +337,7 @@ static InputDeviceInfoStruct InputDeviceInfoFamicomPort[] =
   "arkanoid",
   "Arkanoid Paddle",
   NULL,
+  NULL,
   sizeof(ArkanoidIDII) / sizeof(InputDeviceInputInfoStruct),
   ArkanoidIDII,
  },
@@ -354,6 +347,7 @@ static InputDeviceInfoStruct InputDeviceInfoFamicomPort[] =
   "shadow",
   "Space Shadow Gun",
   NULL,
+  NULL,
   sizeof(ZapperIDII) / sizeof(InputDeviceInputInfoStruct),
   ZapperIDII,
  },
@@ -362,8 +356,9 @@ static InputDeviceInfoStruct InputDeviceInfoFamicomPort[] =
  {
   "4player",
   "4-player Adapter",
-  PortInfo4Player,
-  2,
+  NULL,
+  NULL,
+  0,
   NULL,
  },
  // Family Keyboard
@@ -371,6 +366,7 @@ static InputDeviceInfoStruct InputDeviceInfoFamicomPort[] =
   "fkb",
   "Family Keyboard",
   NULL,
+  NULL,
   sizeof(FKBIDII) / sizeof(InputDeviceInputInfoStruct),
   FKBIDII,
  },
@@ -380,6 +376,7 @@ static InputDeviceInfoStruct InputDeviceInfoFamicomPort[] =
   "hypershot",
   "Hypershot Paddles",
   NULL,
+  NULL,
   sizeof(HypershotIDII) / sizeof(InputDeviceInputInfoStruct),
   HypershotIDII,
  },
@@ -389,6 +386,7 @@ static InputDeviceInfoStruct InputDeviceInfoFamicomPort[] =
   "mahjong",
   "Mahjong Controller",
   NULL,
+  NULL,
   sizeof(MahjongIDII) / sizeof(InputDeviceInputInfoStruct),
   MahjongIDII,
  },
@@ -398,6 +396,7 @@ static InputDeviceInfoStruct InputDeviceInfoFamicomPort[] =
   "partytap",
   "Party Tap",
   NULL,
+  NULL,
   sizeof(PartyTapIDII) / sizeof(InputDeviceInputInfoStruct),
   PartyTapIDII,
  },
@@ -407,6 +406,7 @@ static InputDeviceInfoStruct InputDeviceInfoFamicomPort[] =
   "ftrainera",
   "Family Trainer Side A",
   NULL,
+  NULL,
   sizeof(FTrainerIDII) / sizeof(InputDeviceInputInfoStruct),
   FTrainerIDII,
  },
@@ -416,6 +416,7 @@ static InputDeviceInfoStruct InputDeviceInfoFamicomPort[] =
   "ftrainerb",
   "Family Trainer Side B",
   NULL,
+  NULL,
   sizeof(FTrainerIDII) / sizeof(InputDeviceInputInfoStruct),
   FTrainerIDII,
  },
@@ -425,6 +426,7 @@ static InputDeviceInfoStruct InputDeviceInfoFamicomPort[] =
   "oekakids",
   "Oeka Kids Tablet",
   NULL,
+  NULL,
   sizeof(OekaIDII) / sizeof(InputDeviceInputInfoStruct),
   OekaIDII,
  },
@@ -434,19 +436,22 @@ static InputDeviceInfoStruct InputDeviceInfoFamicomPort[] =
   "bworld",
   "Barcode World Scanner",
   NULL,
+  NULL,
   sizeof(BWorldIDII) / sizeof(InputDeviceInputInfoStruct),
   BWorldIDII,
  },
 
 };
 
+// The temptation is there, but don't change the "fcexp" default setting to anything other than "none", as the presence of a device
+// there by default will cause compatibility problems with games.
 static const InputPortInfoStruct PortInfo[] =
 {
- { 0, "port1", "Port 1", sizeof(InputDeviceInfoNESPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoNESPort, "gamepad" },
- { 0, "port2", "Port 2", sizeof(InputDeviceInfoNESPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoNESPort, "gamepad" },
- { 0, "port3", "Port 3", sizeof(InputDeviceInfoNESPort34) / sizeof(InputDeviceInfoStruct), InputDeviceInfoNESPort34, "gamepad" },
- { 0, "port4", "Port 4", sizeof(InputDeviceInfoNESPort34) / sizeof(InputDeviceInfoStruct), InputDeviceInfoNESPort34, "gamepad" },
- { 0, "fcexp", "Famicom Expansion Port", sizeof(InputDeviceInfoFamicomPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoFamicomPort, "none" },
+ { "port1", "Port 1", sizeof(InputDeviceInfoNESPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoNESPort, "gamepad" },
+ { "port2", "Port 2", sizeof(InputDeviceInfoNESPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoNESPort, "gamepad" },
+ { "port3", "Port 3", sizeof(InputDeviceInfoNESPort34) / sizeof(InputDeviceInfoStruct), InputDeviceInfoNESPort34, "gamepad" },
+ { "port4", "Port 4", sizeof(InputDeviceInfoNESPort34) / sizeof(InputDeviceInfoStruct), InputDeviceInfoNESPort34, "gamepad" },
+ { "fcexp", "Famicom Expansion Port", sizeof(InputDeviceInfoFamicomPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoFamicomPort, "none" },
 };
 
 InputInfoStruct NESInputInfo = 
@@ -494,7 +499,7 @@ static void *InputDataPtr[5];
 void (*InputScanlineHook)(uint8 *bg, uint8 *spr, uint32 linets, int final);
 
 
-static INPUTC DummyJPort = {0, 0, 0, 0, 0, NULL, 0, 0};
+static INPUTC DummyJPort = {0, 0, 0, 0, 0, NULL};
 static INPUTC *JPorts[4] = {&DummyJPort, &DummyJPort, &DummyJPort, &DummyJPort};
 static INPUTCFC *FCExp = NULL;
 
@@ -642,9 +647,9 @@ static void UpdateGamepad(int w, void *data)
  joy[w] = *(uint8*)data;
 }
 
-static INPUTCFC FAMI4C = { ReadFami4,0,StrobeFami4,0,0,0, StateActionGPFC, 4, 1 };
-static INPUTC GPC = {ReadGP,0,StrobeGP,UpdateGamepad,0,0, StateActionGP, 4, 1};
-static INPUTC GPCVS = {ReadGPVS,0,StrobeGP,UpdateGamepad,0,0, StateActionGP, 4, 1};
+static INPUTCFC FAMI4C = { ReadFami4,0,StrobeFami4,0,0,0, StateActionGPFC };
+static INPUTC GPC = {ReadGP,0,StrobeGP,UpdateGamepad,0,0, StateActionGP };
+static INPUTC GPCVS = {ReadGPVS,0,StrobeGP,UpdateGamepad,0,0, StateActionGP };
 
 void MDFN_DrawInput(MDFN_Surface *surface)
 {
