@@ -43,7 +43,7 @@ void MDFN_DrawGunSight(MDFN_Surface *surface, int xc, int yc)
  uint32 *buf = surface->pixels;
  int x,y;
  int c,d;
- const int ctransform[2] = { surface->MakeColor(0xff, 0xff, 0xff), surface->MakeColor(0, 0, 0) };
+ const uint32 ctransform[2] = { surface->MakeColor(0xff, 0xff, 0xff), surface->MakeColor(0, 0, 0) };
  MDFN_Rect DisplayRect;
 
  NESPPU_GetDisplayRect(&DisplayRect);
@@ -74,7 +74,7 @@ void MDFN_DrawCursor(MDFN_Surface *surface, int xc, int yc)
  uint32 *buf = surface->pixels;
  int x,y;
  int c,d;
- const int ctransform[4] = { surface->MakeColor(0, 0, 0), surface->MakeColor(0, 0, 0),
+ const uint32 ctransform[4] = { surface->MakeColor(0, 0, 0), surface->MakeColor(0, 0, 0),
 			     surface->MakeColor(0xff, 0xff, 0xff), surface->MakeColor(0, 0, 0)
 			   };
  MDFN_Rect DisplayRect;
