@@ -36,6 +36,7 @@
 #include	"vsuni.h"
 #include	"debug.h"
 
+#include	<trio/trio.h>
 
 extern MDFNGI EmulatedNES;
 
@@ -309,6 +310,30 @@ static void Emulate(EmulateSpecStruct *espec)
 {
  int ssize;
 
+#if 0
+ static bool firstcat = true;
+
+ MDFN_PixelFormat tmp_pf;
+
+ tmp_pf.Rshift = 0;
+ tmp_pf.Gshift = 0;
+ tmp_pf.Bshift = 0;
+ tmp_pf.Ashift = 8;
+
+ tmp_pf.Rprec = 0;
+ tmp_pf.Gprec = 0;
+ tmp_pf.Bprec = 0;
+ tmp_pf.Aprec = 0;
+
+ tmp_pf.bpp = 8;
+ tmp_pf.colorspace = MDFN_COLORSPACE_RGB;
+
+ espec->surface->SetFormat(tmp_pf, false);
+ espec->VideoFormatChanged = firstcat;
+ firstcat = false;
+#endif
+
+
  if(espec->VideoFormatChanged)
   MDFNNES_SetPixelFormat(espec->surface->format); //.Rshift, espec->surface->format.Gshift, espec->surface->format.Bshift);
 
@@ -322,7 +347,7 @@ static void Emulate(EmulateSpecStruct *espec)
  if(!Genie_BIOSInstalled())
   MDFNMP_ApplyPeriodicCheats();
 
- MDFNPPU_Loop(espec->surface, espec->skip);
+ MDFNPPU_Loop(espec);
 
  ssize = FlushEmulateSound(espec->NeedSoundReverse, espec->SoundBuf, espec->SoundBufMaxSize);
  espec->NeedSoundReverse = 0;
@@ -452,8 +477,8 @@ static MDFNSetting NESSettings[] =
   { "nes.no8lim", MDFNSF_NOFLAGS, gettext_noop("Remove 8-sprites-per-scanline hardware limit."), 
 	gettext_noop("WARNING: Enabling this option will cause graphical glitches in some games, including \"Solstice\"."), MDFNST_BOOL, "0", NULL, NULL, NULL, NESPPU_SettingChanged },
 
-  { "nes.soundq", MDFNSF_NOFLAGS, gettext_noop("Sound quality."), NULL, MDFNST_INT, "0", "-2", "3" },
-  { "nes.sound_rate_error", MDFNSF_NOFLAGS, gettext_noop("Output rate tolerance."), NULL, MDFNST_FLOAT, "0.00004", "0.0000001", "0.01" },
+  { "nes.soundq", MDFNSF_NOFLAGS, gettext_noop("Sound quality."), gettext_noop("Higher values correspond to better SNR and better preservation of higher frequencies(\"brightness\"), at the cost of increased computational complexity and a negligible(<0.5ms) increase in latency."), MDFNST_INT, "0", "-2", "3" },
+  { "nes.sound_rate_error", MDFNSF_NOFLAGS, gettext_noop("Output rate tolerance."), gettext_noop("Lower values correspond to better matching of the output rate of the resampler to the actual desired output rate, at the expense of increased RAM usage and poorer CPU cache utilization.  DO NOT INCREASE THIS VALUE, OR SOUND WILL LIKELY BE OFF-KEY AND THE WRONG TEMPO, AMONG OTHER PROBLEMS."), MDFNST_FLOAT, "0.00004", "0.0000001", "0.01" },
   { "nes.n106bs", MDFNSF_NOFLAGS, gettext_noop("Enable less-accurate, but better sounding, Namco 106(mapper 19) sound emulation."), NULL, MDFNST_BOOL, "0" },
   { "nes.fnscan", MDFNSF_EMU_STATE, gettext_noop("Scan filename for (U),(J),(E),etc. strings to en/dis-able PAL emulation."), 
 	gettext_noop("Warning: This option may break NES network play when enabled IF the players are using ROM images with different filenames."), MDFNST_BOOL, "1" },
@@ -463,9 +488,11 @@ static MDFNSetting NESSettings[] =
   { "nes.ggrom", MDFNSF_EMU_STATE, gettext_noop("Path to Game Genie ROM image."), NULL, MDFNST_STRING, "gg.rom" },
   { "nes.clipsides", MDFNSF_NOFLAGS, gettext_noop("Clip left+right 8 pixel columns."), NULL, MDFNST_BOOL, "0" },
   { "nes.slstart", MDFNSF_NOFLAGS, gettext_noop("First displayed scanline in NTSC mode."), NULL, MDFNST_UINT, "8", "0", "239" },
-  { "nes.slend", MDFNSF_NOFLAGS, gettext_noop("Last displayed scanlines in NTSC mode."), NULL, MDFNST_UINT, "231", "0", "239" },
+  { "nes.slend", MDFNSF_NOFLAGS, gettext_noop("Last displayed scanline in NTSC mode."), NULL, MDFNST_UINT, "231", "0", "239" },
   { "nes.slstartp", MDFNSF_NOFLAGS, gettext_noop("First displayed scanline in PAL mode."), NULL, MDFNST_UINT, "0", "0", "239" },
-  { "nes.slendp", MDFNSF_NOFLAGS, gettext_noop("Last displayedscanlines in PAL mode."), NULL, MDFNST_UINT, "239", "0", "239" },
+  { "nes.slendp", MDFNSF_NOFLAGS, gettext_noop("Last displayed scanline in PAL mode."), NULL, MDFNST_UINT, "239", "0", "239" },
+
+  { "nes.correct_aspect", MDFNSF_CAT_VIDEO, gettext_noop("Correct the aspect ratio."), NULL, MDFNST_BOOL, "0" },
   { "nes.ntscblitter", MDFNSF_NOFLAGS, gettext_noop("Enable NTSC color generation and blitter."), 
 	gettext_noop("NOTE: If your refresh rate isn't very close to 60.1Hz(+-0.1), you will need to set the nes.ntsc.mergefields setting to \"1\" to avoid excessive flickering."), MDFNST_BOOL, "0" },
 
@@ -481,12 +508,12 @@ static MDFNSetting NESSettings[] =
   { "nes.ntsc.matrix", MDFNSF_NOFLAGS, gettext_noop("Enable NTSC custom decoder matrix."), NULL, MDFNST_BOOL, "0" },
 
   /* Default custom decoder matrix(not plain default matrix) is from Sony */
-  { "nes.ntsc.matrix.0", MDFNSF_NOFLAGS, gettext_noop("NTSC custom decoder matrix element 0(red, value * V)."), NULL, MDFNST_FLOAT, "1.539", "-2", "2", NULL, NESPPU_SettingChanged },
-  { "nes.ntsc.matrix.1", MDFNSF_NOFLAGS, gettext_noop("NTSC custom decoder matrix element 1(red, value * U)."), NULL, MDFNST_FLOAT, "-0.622", "-2", "2", NULL, NESPPU_SettingChanged },
-  { "nes.ntsc.matrix.2", MDFNSF_NOFLAGS, gettext_noop("NTSC custom decoder matrix element 2(green, value * V)."), NULL, MDFNST_FLOAT, "-0.571", "-2", "2", NULL, NESPPU_SettingChanged },
-  { "nes.ntsc.matrix.3", MDFNSF_NOFLAGS, gettext_noop("NTSC custom decoder matrix element 3(green, value * U)."), NULL, MDFNST_FLOAT, "-0.185", "-2", "2", NULL, NESPPU_SettingChanged },
-  { "nes.ntsc.matrix.4", MDFNSF_NOFLAGS, gettext_noop("NTSC custom decoder matrix element 4(blue, value * V)."), NULL, MDFNST_FLOAT, "0.000", "-2", "2", NULL, NESPPU_SettingChanged },
-  { "nes.ntsc.matrix.5", MDFNSF_NOFLAGS, gettext_noop("NTSC custom decoder matrix element 5(blue, value * U."), NULL, MDFNST_FLOAT, "2.000", "-2", "2", NULL, NESPPU_SettingChanged },
+  { "nes.ntsc.matrix.0", MDFNSF_NOFLAGS, gettext_noop("NTSC custom decoder matrix element 0(red, value * V)."), NULL, MDFNST_FLOAT, "1.539", "-2.000", "2.000", NULL, NESPPU_SettingChanged },
+  { "nes.ntsc.matrix.1", MDFNSF_NOFLAGS, gettext_noop("NTSC custom decoder matrix element 1(red, value * U)."), NULL, MDFNST_FLOAT, "-0.622", "-2.000", "2.000", NULL, NESPPU_SettingChanged },
+  { "nes.ntsc.matrix.2", MDFNSF_NOFLAGS, gettext_noop("NTSC custom decoder matrix element 2(green, value * V)."), NULL, MDFNST_FLOAT, "-0.571", "-2.000", "2.000", NULL, NESPPU_SettingChanged },
+  { "nes.ntsc.matrix.3", MDFNSF_NOFLAGS, gettext_noop("NTSC custom decoder matrix element 3(green, value * U)."), NULL, MDFNST_FLOAT, "-0.185", "-2.000", "2.000", NULL, NESPPU_SettingChanged },
+  { "nes.ntsc.matrix.4", MDFNSF_NOFLAGS, gettext_noop("NTSC custom decoder matrix element 4(blue, value * V)."), NULL, MDFNST_FLOAT, "0.000", "-2.000", "2.000", NULL, NESPPU_SettingChanged },
+  { "nes.ntsc.matrix.5", MDFNSF_NOFLAGS, gettext_noop("NTSC custom decoder matrix element 5(blue, value * U."), NULL, MDFNST_FLOAT, "2.000", "-2.000", "2.000", NULL, NESPPU_SettingChanged },
   { NULL }
 };
 
@@ -517,7 +544,7 @@ static DECLFR(CheatReadFunc)
  return(retval);
 }
 
-static void InstallReadPatch(uint32 address)
+static void InstallReadPatch(uint32 address, uint8 value, int compare)
 {
  if(Genie_BIOSInstalled())
   return;
@@ -538,6 +565,193 @@ static void RemoveReadPatches(void)
  }
 }
 
+
+static int GGtobin(char c)
+{
+ static char lets[16]={'A','P','Z','L','G','I','T','Y','E','O','X','U','K','S','V','N'};
+
+ for(int x = 0; x < 16; x++)
+  if(lets[x] == toupper(c))
+   return(x);
+
+ if(c & 0x80)
+  throw MDFN_Error(0, _("Invalid character in Game Genie code."));
+ else
+  throw MDFN_Error(0, _("Invalid character in Game Genie code: %c"), c);
+}
+
+static bool DecodeGG(const std::string& cheat_string, MemoryPatch* patch)
+{
+ uint16 A;
+ uint8 V,C;
+ uint8 t;
+ const unsigned s = cheat_string.size();
+ const char *str = cheat_string.c_str();
+
+ if(s != 6 && s != 8)
+  throw(MDFN_Error(0, _("Game Genie code is of an incorrect length.")));
+
+ A = 0x8000;
+ V = 0;
+ C = 0;
+
+ t = GGtobin(*str++);
+ V |= (t&0x07);
+ V |= (t&0x08)<<4;
+
+ t = GGtobin(*str++);
+ V |= (t&0x07)<<4;
+ A |= (t&0x08)<<4;
+
+ t = GGtobin(*str++);
+ A |= (t&0x07)<<4;
+ //if(t&0x08) return(0);	/* 8-character code?! */
+
+ t = GGtobin(*str++);
+ A |= (t&0x07)<<12;
+ A |= (t&0x08);
+
+ t = GGtobin(*str++);
+ A |= (t&0x07);
+ A |= (t&0x08)<<8;
+
+ if(s == 6)
+ {
+  t = GGtobin(*str++);
+  A |= (t&0x07)<<8;
+  V |= (t&0x08);
+
+  patch->addr = A;
+  patch->val = V;
+  patch->compare = 0;
+  patch->type = 'S';
+  patch->length = 1;
+ }
+ else
+ {
+  t = GGtobin(*str++);
+  A |= (t&0x07)<<8;
+  C |= (t&0x08);
+
+  t = GGtobin(*str++);
+  C |= (t&0x07);
+  C |= (t&0x08)<<4;
+  
+  t = GGtobin(*str++);
+  C |= (t&0x07)<<4;
+  V |= (t&0x08);
+
+  patch->addr = A;
+  patch->val = V;
+  patch->compare = C;
+  patch->type = 'C';
+  patch->length = 1;
+ }
+ return(false);
+}
+
+
+static bool DecodePAR(const std::string& cheat_string, MemoryPatch* patch)
+{
+ int boo[4];
+
+ if(cheat_string.size() != 8)
+  throw MDFN_Error(0, _("Pro Action Replay code is of an incorrect length."));
+
+ if(trio_sscanf(cheat_string.c_str(), "%02x%02x%02x%02x", boo, boo + 1, boo + 2, boo + 3) != 4)
+  throw MDFN_Error(0, _("Malformed Pro Action Replay code."));
+
+ if(boo[0])
+ {
+  patch->addr = (boo[3] << 8) | ((boo[2] + 0x7F) & 0xFF);
+  patch->val = boo[1];
+  patch->compare = 0;
+  patch->type = 'S';
+ }
+ else
+ {
+  patch->addr = ((boo[1] & 0x07) << 8) | (boo[2] << 0);
+  patch->val = boo[3];
+  patch->compare = 0;
+  patch->type = 'R';
+ }
+
+ patch->length = 1;
+
+ return(false);
+}
+
+static bool DecodeRocky(const std::string& cheat_string, MemoryPatch* patch)
+{
+ if(cheat_string.size() != 8)
+  throw MDFN_Error(0, _("Pro Action Rocky code is of an incorrect length."));
+
+ uint32 ev = 0;
+
+ for(unsigned i = 0; i < 8; i++)
+ {
+  int c = cheat_string[i];
+
+  ev <<= 4;
+
+  if(c >= '0' && c <= '9')
+   ev |= c - '0';
+  else if(c >= 'a' && c <= 'f')
+   ev |= c - 'a' + 0xA;
+  else if(c >= 'A' && c <= 'F')
+   ev |= c - 'A' + 0xA;
+  else
+  {
+   if(c & 0x80)
+    throw MDFN_Error(0, _("Invalid character in Pro Action Rocky code."));
+   else
+    throw MDFN_Error(0, _("Invalid character in Pro Action Rocky code: %c"), c);
+  }
+ }
+
+ uint32 accum = 0xfcbdd275;
+ uint32 result = 0;
+
+ for(signed int b = 30; b >= 0; b--)
+ {
+  const unsigned tmp = (accum ^ ev) >> 31;
+  static const uint8 lut[31] =
+  {
+   3, 13, 14, 1, 6, 9, 5, 0, 12, 7, 2, 8, 10, 11, 4,
+   19, 21, 23, 22, 20, 17, 16, 18,
+   29, 31, 24, 26, 25, 30, 27, 28
+  };
+  result |= tmp << lut[b];
+
+  if(tmp)
+   accum ^= 0xb8309722;
+  accum <<= 1;
+  ev <<= 1;
+ }
+
+ patch->addr = (result & 0x7FFF) | 0x8000;
+ patch->val = (result >> 24) & 0xFF;
+ patch->compare = (result >> 16) & 0xFF;
+ patch->length = 1;
+ patch->type = 'C';
+
+ return(false);
+}
+
+
+static CheatFormatStruct CheatFormats[] =
+{
+ { "Game Genie", gettext_noop("Genies will eat your cheeses."), DecodeGG },
+ { "Pro Action Replay (Incomplete)", gettext_noop("Prooooooooooooooooocom."), DecodePAR },
+ { "Pro Action Rocky", gettext_noop("No pie."), DecodeRocky },
+};
+
+static CheatFormatInfoStruct CheatFormatInfo =
+{
+ 3,
+ CheatFormats
+};
+
 static const FileExtensionSpecStruct KnownExtensions[] =
 {
  { ".nes", "iNES Format ROM Image" },
@@ -567,11 +781,15 @@ MDFNGI EmulatedNES =
  NULL,
  NULL,
  CloseGame,
- MDFNNES_ToggleLayer,
+ MDFNNES_SetLayerEnableMask,
  "Background\0Sprites\0",
+ NULL,
+ NULL,
  InstallReadPatch,
  RemoveReadPatches,
  MemRead,
+ &CheatFormatInfo,
+ false,
  StateAction,
  Emulate,
  MDFNNES_SetInput,
