@@ -19,17 +19,17 @@ extern MDFN_ALIGN(16) int16 WaveHiEx[40000];
 extern uint32 soundtsoffs;
 #define SOUNDTS (timestamp + soundtsoffs)
 
-int MDFNSND_Init(bool IsPAL);
-void MDFNSND_Close(void);
-void MDFNSND_Power(void);
-void MDFNSND_Reset(void);
+int MDFNSND_Init(bool IsPAL) MDFN_COLD;
+void MDFNSND_Close(void) MDFN_COLD;
+void MDFNSND_Power(void) MDFN_COLD;
+void MDFNSND_Reset(void) MDFN_COLD;
 void MDFNSND_SaveState(void);
 void MDFNSND_LoadState(int version);
 
 void MDFN_SoundCPUHook(int);
 int MDFNSND_StateAction(StateMem *sm, int load, int data_only);
-void MDFNNES_SetSoundVolume(uint32 volume);
-void MDFNNES_SetSoundMultiplier(double multiplier);
-bool MDFNNES_SetSoundRate(double Rate);
+void MDFNNES_SetSoundVolume(uint32 volume) MDFN_COLD;
+void MDFNNES_SetSoundMultiplier(double multiplier) MDFN_COLD;
+bool MDFNNES_SetSoundRate(double Rate) MDFN_COLD;
 
 #endif
