@@ -251,18 +251,18 @@ uint16 loadW(uint32 address)
 	 return(loadB(address) | (loadB(address + 1) << 8));
 
 	if(FastReadMap[address >> 16])
-	 return(le16toh(*(uint16*)&FastReadMap[address >> 16][address]));
+	 return(LoadU16_LE((uint16*)&FastReadMap[address >> 16][address]));
 
         uint16* ptr = (uint16*)translate_address_read(address);
 	if(ptr)
-                return le16toh(*ptr);
+                return LoadU16_LE(ptr);
 
         if(address >= 0x8000 && address <= 0xbfff)
          return(NGPGfx->read16(address));
 
         if(address >= 0x4000 && address <= 0x7fff)
 	{
-         return(le16toh(*(uint16 *)(CPUExRAM + address - 0x4000)));
+         return(LoadU16_LE((uint16 *)(CPUExRAM + address - 0x4000)));
 	}
 	if(address == 0x50)
 	 return(SC0BUF);
@@ -429,7 +429,7 @@ void storeW(uint32 address, uint16 data)
         }
         if(address >= 0x4000 && address <= 0x7fff)
         {
-         *(uint16 *)(CPUExRAM + address - 0x4000) = htole16(data);
+         StoreU16_LE((uint16 *)(CPUExRAM + address - 0x4000), data);
          return;
         }
         if(address >= 0x70 && address <= 0x7F)
@@ -500,7 +500,7 @@ void storeW(uint32 address, uint16 data)
 	//Write
 	if (ptr)
 	{
-		*ptr = htole16(data);
+		StoreU16_LE(ptr, data);
 	}
         //else
         //        printf("ACK16: %08x %04x\n", address, data);
