@@ -18,7 +18,7 @@
 #include "pce.h"
 #include "hes.h"
 #include "huc.h"
-#include "../cdrom/pcecd.h"
+#include "pcecd.h"
 #include "../player.h"
 
 namespace MDFN_IEN_PCE
@@ -163,6 +163,17 @@ int PCE_HESLoad(const uint8 *buf, uint32 size)
 
  memcpy(rom_backup, rom, 0x88 * 8192);
 
+ //
+ // Try to detect SuperGrafx rips in the future?
+ //
+ //for(unsigned i = 0; i < 0x80; i++)
+ //{
+ // printf("0x%02x: 0x%08x\n", i, (uint32)crc32(0, &rom[i * 8192], 8192));
+ //}
+ //
+ //
+ //
+
  CurrentSong = StartingSong;
  TotalSongs = 256;
  uint8 *IBP_WR = IBP;
@@ -177,22 +188,51 @@ int PCE_HESLoad(const uint8 *buf, uint32 size)
 
  // Initialize VDC registers.
  {
-  //  uint8 vdc_init_rom[] = { 0xA2, 0x0A, 0xA9, 0xFF, 0x8E, 0x00, 0x00, 0x8D, 0x02, 0x00, 0x8D, 0x03, 0x00, 0xE8, 0xE0, 0x0F, 0xD0, 0xF2 };
-  //  uint8 vdc_init_rom[] = { 0xEA, 0xEA, 0xEA, 0xEA, 0xEA, 0xEA, 0xEA, 0xEA, 0xEA, 0xEA, 0xEA, 0xEA, 0xEA, 0xEA, 0xEA, 0xEA, 0xEa, 0xEA };
+  const uint8 vdc_init_rom[] = {
+	0xA0, 0x01,
 
-  static const uint8 vdc_init_rom[] = {
-	0xA2, 0x1F, 0x8E, 0x00,
-	0x00, 0x9C, 0x02, 0x00,
-	0x9C, 0x03, 0x00, 0xCA,
-	0x10, 0xF4, 0x03, 0x0A,
-	0x13, 0x02, 0x23, 0x02,
-	0x03, 0x0B, 0x13, 0x1F,
-	0x23, 0x04, 0x03, 0x0C,
-	0x13, 0x02, 0x23, 0x0D,
-	0x03, 0x0D, 0x13, 0xEF,
-	0x23, 0x00, 0x03, 0x0E,
-	0x13, 0x04, 0xA9, 0x04,
-	0x8D, 0x00, 0x04
+	0x8C, 0x0E, 0x00,	// SGFX ST_mode
+
+	//
+	//
+	//
+	0xA2, 0x1F,
+
+	0x8E, 0x00, 0x00,
+	0x8E, 0x10, 0x00,
+
+	0x13, 0x00,
+	0x23, 0x00,
+	0xCA,
+	0x10, 0xF3,
+	//
+	//
+	//
+	0x03, 0x0A,
+	0x13, 0x02,
+	0x23, 0x02,
+
+	0x03, 0x0B,
+	0x13, 0x1F,
+	0x23, 0x04,
+
+	0x03, 0x0C,
+	0x13, 0x02,
+	0x23, 0x0D,
+
+	0x03, 0x0D,
+	0x13, 0xEF,
+	0x23, 0x00,
+
+	0x03, 0x0E,
+	0x13, 0x04,
+
+	0x88,
+	0x10, 0xCF,
+
+	// VCE init
+	0xA9, 0x04,
+	0x8D, 0x00, 0x04,
        };
 
   memcpy(IBP_WR, vdc_init_rom, sizeof(vdc_init_rom));
