@@ -17,7 +17,7 @@
 
 #include "pce.h"
 #include "huc.h"
-#include "../cdrom/pcecd.h"
+#include "pcecd.h"
 #include "arcade_card/arcade_card.h"
 #include "../md5.h"
 #include "../file.h"
@@ -31,12 +31,11 @@
 namespace MDFN_IEN_PCE
 {
 
-static const char BRAM_Init_String[8] = { 'H', 'U', 'B', 'M', 0x00, 0x88, 0x10, 0x80 }; //"HUBM\x00\x88\x10\x80";
+static const uint8 BRAM_Init_String[8] = { 'H', 'U', 'B', 'M', 0x00, 0x88, 0x10, 0x80 }; //"HUBM\x00\x88\x10\x80";
 static bool BRAM_Disabled;       // Cached at game load, don't remove this caching behavior or save game loss may result(if we ever get a GUI).
 
 ArcadeCard *arcade_card = NULL;
 
-extern Blip_Buffer huc_sbuf;
 static MCGenjin *mcg = NULL;
 static uint8 *HuCROM = NULL;
 static uint8 *ROMMap[0x100] = { NULL };
@@ -251,7 +250,7 @@ int HuCLoad(const uint8 *data, uint32 len, uint32 crc32, bool DisableBRAM, SysCa
 
  if(mcg_mapper)
  {
-  mcg = new MCGenjin(&huc_sbuf, data, len);
+  mcg = new MCGenjin(data, len);
 
   for(unsigned i = 0; i < 128; i++)
   {
@@ -466,7 +465,7 @@ int HuCLoad(const uint8 *data, uint32 len, uint32 crc32, bool DisableBRAM, SysCa
 
  return(1);
 }
- 
+
 bool IsBRAMUsed(void)
 {
  if(BRAM_Disabled)
@@ -557,16 +556,22 @@ void HuCClose(void)
  }
  else if(!BRAM_Disabled && IsBRAMUsed())
  {
-  MDFN_DumpToFile(MDFN_MakeFName(MDFNMKF_SAV, 0, "sav").c_str(), 6, SaveRAM, 2048);
+  MDFN_DumpToFile(MDFN_MakeFName(MDFNMKF_SAV, 0, "sav").c_str(), 0, SaveRAM, 2048);
  }
 
  Cleanup();
 }
 
-void HuC_EndFrame(int32 timestamp)
+void HuC_Update(int32 timestamp)
 {
  if(mcg)
-  mcg->EndFrame(timestamp);
+  mcg->Update(timestamp);
+}
+
+void HuC_ResetTS(int32 ts_base)
+{
+ if(mcg)
+  mcg->ResetTS(ts_base);
 }
 
 
@@ -588,5 +593,33 @@ void HuC_Power(void)
 }
 
 
+bool HuC_IsBRAMAvailable(void)
+{
+ if(IsPopulous)
+  return(false);
+
+ if(IsTsushin)
+  return(false);
+
+ if(BRAM_Disabled)
+  return(false);
+
+ return(true);
+}
+
+uint8 HuC_PeekBRAM(uint32 A)
+{
+ assert(HuC_IsBRAMAvailable());
+
+ return(SaveRAM[A & 2047]);
+}
+
+void HuC_PokeBRAM(uint32 A, uint8 V)
+{
+ assert(HuC_IsBRAMAvailable());
+
+ SaveRAM[A & 2047] = V;
+}
+
 
 };
