@@ -15,7 +15,8 @@ typedef enum
 
 int HuCLoad(const uint8 *data, uint32 len, uint32 crc32, bool DisableBRAM = false, SysCardType syscard = SYSCARD_NONE);
 
-void HuC_EndFrame(int32 timestamp);
+void HuC_Update(int32 timestamp);
+void HuC_ResetTS(int32 ts_base);
 
 void HuCClose(void);
 int HuC_StateAction(StateMem *sm, int load, int data_only);
@@ -28,6 +29,10 @@ DECLFW(PCE_ACWrite);
 extern bool PCE_IsCD;
 extern bool IsTsushin;
 
+// Debugger support functions.
+bool HuC_IsBRAMAvailable(void);
+uint8 HuC_PeekBRAM(uint32 A);
+void HuC_PokeBRAM(uint32 A, uint8 V);
 };
 
 #endif
