@@ -26,6 +26,9 @@ void PCE_Power(void);
 
 void PCE_MidSync(void);
 
+uint8 PCE_PeekMainRAM(uint32 A);
+void PCE_PokeMainRAM(uint32 A, uint8 V);
+
 };
 
 #define _PCE_H
