@@ -13,6 +13,7 @@ struct CD_Audio_Buffer
  int16 Samples[2][0x1000];	// [0][...] = l, [1][...] = r
  int32 Size;
  uint32 Freq;
+ int32 ReadPos;
 };
 
 class PS_CDC
@@ -25,6 +26,7 @@ class PS_CDC
  void SetDisc(bool tray_open, CDIF *cdif, const char disc_id[4]);
 
  void Power(void);
+ int StateAction(StateMem *sm, int load, int data_only);
  void ResetTS(void);
 
  int32 CalcNextEvent(void);	// Returns in master cycles to next event.
@@ -36,78 +38,42 @@ class PS_CDC
 
  bool DMACanRead(void);
  uint32 DMARead(void);
-
- INLINE uint32 GetCDAudioFreq(void)
- {
-  if(AudioBuffer_UsedCount)
-  {
-   const unsigned wb = AudioBuffer_ReadPos >> 12;
-   return AudioBuffer[wb].Freq;
-  }
-  return 0;
- }
-
- private:
- inline void ApplyVolume(int32 samples[2])
- {
-  int32 left_source = samples[0]; //(int16)MDFN_de16lsb(&buf[i * sizeof(int16) * 2 + 0]);
-  int32 right_source = samples[1]; //(int16)MDFN_de16lsb(&buf[i * sizeof(int16) * 2 + 2]);
-  int32 left_out = ((left_source * DecodeVolume[0][0]) >> 7) + ((right_source * DecodeVolume[1][0]) >> 7);
-  int32 right_out = ((left_source * DecodeVolume[0][1]) >> 7) + ((right_source * DecodeVolume[1][1]) >> 7);
-
-  clamp(&left_out, -32768, 32767);
-  clamp(&right_out, -32768, 32767);
-
-  samples[0] = left_out;
-  samples[1] = right_out;
- }
- public:
-
- INLINE void GetCDAudio(int32 &l, int32 &r)
- {
-  if(AudioBuffer_UsedCount)
-  {
-   const unsigned wb = AudioBuffer_ReadPos >> 12;
-   int32 samples[2] = { AudioBuffer[wb].Samples[0][AudioBuffer_ReadPos & 0xFFF], AudioBuffer[wb].Samples[1][AudioBuffer_ReadPos & 0xFFF] };
-
-   ApplyVolume(samples);
-
-   l = samples[0];
-   r = samples[1];
-
-   AudioBuffer_ReadPos = ((AudioBuffer_ReadPos + 1) & 0xFFF) | (AudioBuffer_ReadPos & ~0xFFF);
-
-   if((AudioBuffer_ReadPos & 0xFFF) == (AudioBuffer[wb].Size & 0xFFF))
-   {
-    AudioBuffer_ReadPos = ((((AudioBuffer_ReadPos >> 12) + 1) % AudioBuffer_Count) << 12);
-    AudioBuffer_UsedCount--;
-   }
-  }
- }
-
- private:
-
  void SoftReset(void);
 
+ void GetCDAudio(int32 samples[2]);
+
+ private:
  CDIF *Cur_CDIF;
  bool DiscChanged;
  int32 DiscStartupDelay;
 
-
- enum { AudioBuffer_Count = 2 };
-
- CD_Audio_Buffer AudioBuffer[AudioBuffer_Count];
- uint32 AudioBuffer_ReadPos;
- uint32 AudioBuffer_WritePos;
- uint32 AudioBuffer_UsedCount;
+ CD_Audio_Buffer AudioBuffer;
 
  uint8 Pending_DecodeVolume[2][2], DecodeVolume[2][2];		// [data_source][output_port]
 
+ int16 ADPCM_ResampBuf[2][32 * 2];
+ uint8 ADPCM_ResampCurPos;
+ uint8 ADPCM_ResampCurPhase;
+
+ void ApplyVolume(int32 samples[2]);
+ void ReadAudioBuffer(int32 samples[2]);
+
  void ClearAudioBuffers(void);
 
+
+ //
+ //
+ //
+
+
  uint8 RegSelector;
  uint8 ArgsBuf[16];
- uint32 ArgsIn;		// 5-bit(0 ... 31)
+ uint8 ArgsWP;		// 5-bit(0 ... 31)
+ uint8 ArgsRP;		// 5-bit(0 ... 31)
+
+ uint8 ArgsReceiveLatch;
+ uint8 ArgsReceiveBuf[32];
+ uint8 ArgsReceiveIn;
 
  uint8 ResultsBuffer[16];
  uint8 ResultsIn;	// 5-bit(0 ... 31)
@@ -118,11 +84,16 @@ class PS_CDC
  uint8 SB[2340];
  uint32 SB_In;
 
+ enum { SectorPipe_Count = 2 };
+ uint8 SectorPipe[SectorPipe_Count][2352];
+ uint8 SectorPipe_Pos;
+ uint8 SectorPipe_In;
 
  uint8 SubQBuf[0xC];
  uint8 SubQBuf_Safe[0xC];
  bool SubQChecksumOK;
 
+ bool HeaderBufValid;
  uint8 HeaderBuf[12];
 
  void RecalcIRQ(void);
@@ -163,7 +134,7 @@ class PS_CDC
 
 
  uint8 PendingCommand;
- bool PendingCommandPhase;
+ int PendingCommandPhase;
  int32 PendingCommandCounter;
 
  int32 SPUCounter;
@@ -180,6 +151,7 @@ class PS_CDC
 
  enum
  {
+  DS_STANDBY = -2,
   DS_PAUSED = -1,
   DS_STOPPED = 0,
   DS_SEEKING,
@@ -205,6 +177,8 @@ class PS_CDC
  uint8 AsyncResultsPending[16];
  uint8 AsyncResultsPendingCount;
 
+ int32 CalcSeekTime(int32 initial, int32 target, bool motor_on, bool paused);
+
  void ClearAIP(void);
  void CheckAIP(void);
  void SetAIP(unsigned irq, unsigned result_count, uint8 *r);
@@ -226,9 +200,15 @@ class PS_CDC
  bool DecodeSubQ(uint8 *subpw);
  bool CommandCheckDiscPresent(void);
 
+ void EnbufferizeCDDASector(const uint8 *buf);
  bool XA_Test(const uint8 *sdata);
  void XA_ProcessSector(const uint8 *sdata, CD_Audio_Buffer *ab);
  int16 xa_previous[2][2];
+ bool xa_cur_set;
+ uint8 xa_cur_file;
+ uint8 xa_cur_chan;
+
+ void HandlePlayRead(void);
 
  struct CDC_CTEntry
  {
@@ -240,7 +220,7 @@ class PS_CDC
  };
 
  void BeginSeek(uint32 target);
- void PreSeekHack(uint32 target);
+ void PreSeekHack(bool logical, uint32 target);
  void ReadBase(void);
 
  static CDC_CTEntry Commands[0x20];
@@ -253,6 +233,7 @@ class PS_CDC
  int32 Command_Backward(const int arg_count, const uint8 *args);
  int32 Command_ReadN(const int arg_count, const uint8 *args);
  int32 Command_Standby(const int arg_count, const uint8 *args);
+ int32 Command_Standby_Part2(void);
  int32 Command_Stop(const int arg_count, const uint8 *args);
  int32 Command_Stop_Part2(void); 
  int32 Command_Pause(const int arg_count, const uint8 *args);
