@@ -1,11 +1,50 @@
 #ifndef __MDFN_PSX_CPU_H
 #define __MDFN_PSX_CPU_H
 
+/*
+ Load delay notes:
+
+	// Takes 1 less
+	".set noreorder\n\t"
+	".set nomacro\n\t"
+	"lw %0, 0(%2)\n\t"
+	"nop\n\t"
+	"nop\n\t"
+	"or %0, %1, %1\n\t"
+
+	// cycle than this:
+	".set noreorder\n\t"
+	".set nomacro\n\t"
+	"lw %0, 0(%2)\n\t"
+	"nop\n\t"
+	"or %0, %1, %1\n\t"
+	"nop\n\t"
+
+
+	// Both of these
+	".set noreorder\n\t"
+	".set nomacro\n\t"
+	"lw %0, 0(%2)\n\t"
+	"nop\n\t"
+	"nop\n\t"
+	"or %1, %0, %0\n\t"
+
+	// take same...(which is kind of odd).
+	".set noreorder\n\t"
+	".set nomacro\n\t"
+	"lw %0, 0(%2)\n\t"
+	"nop\n\t"
+	"or %1, %0, %0\n\t"
+	"nop\n\t"
+*/
+
 #include "gte.h"
 
 namespace MDFN_IEN_PSX
 {
 
+#define PS_CPU_EMULATE_ICACHE 1
+
 class PS_CPU
 {
  public:
@@ -17,35 +56,16 @@ class PS_CPU
  // will always be multiples of 4.
  enum { FAST_MAP_SHIFT = 16 };
  enum { FAST_MAP_PSIZE = 1 << FAST_MAP_SHIFT };
- enum { FAST_MAP_TRAMPOLINE_SIZE = 131072 };	// Trampoline exists at both beginning and end.
-
-
- inline uint32 GetFastMapAllocSize(uint32 region_size)	// In bytes
- {
-  return(FAST_MAP_TRAMPOLINE_SIZE + region_size + FAST_MAP_TRAMPOLINE_SIZE);
- }
-
- uint32 SetupFastMapTrampoline(void *alloced_mem, uint32 region_size);	// In bytes, returns offset in bytes where the
-									// initial trampoline ends.
 
  void SetFastMap(void *region_mem, uint32 region_address, uint32 region_size);
-								// Pointer should be to start of actual region data,
-								// AKA alloced pointer + return value of SetupFastMapTrampoline()
 
  INLINE void SetEventNT(const pscpu_timestamp_t next_event_ts_arg)
  {
   next_event_ts = next_event_ts_arg;
  }
 
- INLINE pscpu_timestamp_t GetEventNT(void)
- {
-  return(next_event_ts);
- }
+ pscpu_timestamp_t Run(pscpu_timestamp_t timestamp_in, bool ILHMode);
 
-
- pscpu_timestamp_t Run(pscpu_timestamp_t timestamp_in);
-
- void Exit(void);
  void Power(void);
 
  // which ranges 0-5, inclusive
@@ -53,12 +73,18 @@ class PS_CPU
 
  void SetHalt(bool status);
 
+ // TODO eventually: factor BIU address decoding directly in the CPU core somehow without hurting speed.
+ void SetBIU(uint32 val);
+ uint32 GetBIU(void);
+
+ int StateAction(StateMem *sm, int load, int data_only);
+
  private:
 
  struct
  {
   uint32 GPR[32];
-  uint32 GPR_dummy;	// Used in load delay simulation
+  uint32 GPR_dummy;	// Used in load delay simulation(indexing past the end of GPR)
  };
  uint32 LO;
  uint32 HI;
@@ -72,21 +98,21 @@ class PS_CPU
  void RecalcIPCache(void);
  bool Halted;
 
- uint32 LDWhich;
- uint32 LDValue;
- uint32 LDPenalty;
-
- uint32 ReadPenaltyCounter;
- uint32 ReadPenaltyMask;
-
- bool InBDSlot;
+ uint32 BACKED_LDWhich;
+ uint32 BACKED_LDValue;
+ uint32 LDAbsorb;
 
  pscpu_timestamp_t next_event_ts;
  pscpu_timestamp_t gte_ts_done;
- bool Running;
+ pscpu_timestamp_t muldiv_ts_done;
 
- uint8 *FastMap[1 << (32 - FAST_MAP_SHIFT)];
- uint8 DummyPage[(FAST_MAP_TRAMPOLINE_SIZE + FAST_MAP_PSIZE + FAST_MAP_TRAMPOLINE_SIZE)];
+ uint32 BIU;
+
+ struct __ICache
+ {
+  uint32 TV;
+  uint32 Data;
+ } ICache[1024];
 
  enum
  {
@@ -131,8 +157,41 @@ class PS_CPU
   };
  } CP0;
 
+#if 1
+ //uint32 WrAbsorb;
+ //uint8 WrAbsorbShift;
+
+ // On read:
+ //WrAbsorb = 0;
+ //WrAbsorbShift = 0;
+
+ // On write:
+ //WrAbsorb >>= (WrAbsorbShift >> 2) & 8;
+ //WrAbsorbShift -= (WrAbsorbShift >> 2) & 8;
+
+ //WrAbsorb |= (timestamp - pre_write_timestamp) << WrAbsorbShift;
+ //WrAbsorbShift += 8;
+#endif
+
+ struct
+ {
+  uint8 ReadAbsorb[0x20];
+  uint8 ReadAbsorbDummy;
+ };
+ uint8 ReadAbsorbWhich;
+ uint8 ReadFudge;
+
+ //uint32 WriteAbsorb;
+ //uint8 WriteAbsorbCount;
+ //uint8 WriteAbsorbMonkey;
+
+ MultiAccessSizeMem<1024, uint32, false> ScratchRAM;
+
  //PS_GTE GTE;
 
+ uint8 *FastMap[1 << (32 - FAST_MAP_SHIFT)];
+ uint8 DummyPage[FAST_MAP_PSIZE];
+
  enum
  {
   EXCEPTION_INT = 0,
@@ -150,11 +209,12 @@ class PS_CPU
   EXCEPTION_OV = 12	// Arithmetic overflow
  };
 
- uint32 Exception(uint32 code, uint32 PC) MDFN_WARN_UNUSED_RESULT;
+ uint32 Exception(uint32 code, uint32 PC, const uint32 NPM) MDFN_WARN_UNUSED_RESULT;
 
- template<bool DebugMode> pscpu_timestamp_t RunReal(pscpu_timestamp_t timestamp_in);
+ template<bool DebugMode, bool ILHMode> pscpu_timestamp_t RunReal(pscpu_timestamp_t timestamp_in);
 
- template<typename T> T ReadMemory(pscpu_timestamp_t &timestamp, uint32 address, bool DS24 = false);
+ template<typename T> T PeekMemory(uint32 address) MDFN_COLD;
+ template<typename T> T ReadMemory(pscpu_timestamp_t &timestamp, uint32 address, bool DS24 = false, bool LWC_timing = false);
  template<typename T> void WriteMemory(pscpu_timestamp_t &timestamp, uint32 address, uint32 value, bool DS24 = false);
 
 
@@ -162,7 +222,7 @@ class PS_CPU
  // Mednafen debugger stuff follows:
  //
  public:
- void SetCPUHook(void (*cpuh)(uint32 pc), void (*addbt)(uint32 from, uint32 to, bool exception));
+ void SetCPUHook(void (*cpuh)(const pscpu_timestamp_t timestamp, uint32 pc), void (*addbt)(uint32 from, uint32 to, bool exception));
  void CheckBreakpoints(void (*callback)(bool write, uint32 address, unsigned int len), uint32 instr);
 
  enum
@@ -174,13 +234,18 @@ class PS_CPU
   GSREG_LO,
   GSREG_HI,
   GSREG_SR,
+  GSREG_CAUSE,
+  GSREG_EPC,
  };
 
  uint32 GetRegister(unsigned int which, char *special, const uint32 special_len);
  void SetRegister(unsigned int which, uint32 value);
-
+ bool PeekCheckICache(uint32 PC, uint32 *iw);
+ uint8 PeekMem8(uint32 A);
+ uint16 PeekMem16(uint32 A);
+ uint32 PeekMem32(uint32 A);
  private:
- void (*CPUHook)(uint32 pc);
+ void (*CPUHook)(const pscpu_timestamp_t timestamp, uint32 pc);
  void (*ADDBT)(uint32 from, uint32 to, bool exception);
 };
 
