@@ -26,35 +26,10 @@ namespace MDFN_IEN_PSX
 extern PS_GPU *GPU;
 extern PS_SPU *SPU;
 
+static void RedoCPUHook(void);
 
-
-//
-//
-//
-
-void DBG_GPUScanlineHook(unsigned scanline)
-{
-
-
-
-
-}
-
-//
-//
-//
-
-
-
-
-
-
-
-
-
-
-static void (*CPUHook)(uint32) = NULL;
-static void (*BPCallB)(uint32 PC) = NULL;
+static void (*CPUHook)(uint32, bool) = NULL;
+static bool CPUHookContinuous = false;
 
 struct PSX_BPOINT
 {
@@ -65,7 +40,8 @@ struct PSX_BPOINT
 static std::vector<PSX_BPOINT> BreakPointsPC, BreakPointsRead, BreakPointsWrite;
 static bool FoundBPoint;
 
-static int BTIndex = 0;
+static bool BTEnabled;
+static int BTIndex;
 
 struct BTEntry
 {
@@ -73,6 +49,7 @@ struct BTEntry
  uint32 to;
  uint32 branch_count;
  bool exception;
+ bool valid;
 };
 
 #define NUMBT 24
@@ -89,7 +66,7 @@ static void AddBranchTrace(uint32 from, uint32 to, bool exception)
 
  //if(BTEntries[(BTIndex - 1) & 0xF] == PC) return;
 
- if(prevbt->from == from && prevbt->to == to && prevbt->exception == exception && prevbt->branch_count < 0xFFFFFFFF)
+ if(prevbt->from == from && prevbt->to == to && prevbt->exception == exception && prevbt->branch_count < 0xFFFFFFFF && prevbt->valid)
   prevbt->branch_count++;
  else
  {
@@ -97,11 +74,23 @@ static void AddBranchTrace(uint32 from, uint32 to, bool exception)
   BTEntries[BTIndex].to = to;
   BTEntries[BTIndex].exception = exception;
   BTEntries[BTIndex].branch_count = 1;
+  BTEntries[BTIndex].valid = true;
 
   BTIndex = (BTIndex + 1) % NUMBT;
  }
 }
 
+static void EnableBranchTrace(bool enable)
+{
+ BTEnabled = enable;
+ if(!enable)
+ {
+  BTIndex = 0;
+  memset(BTEntries, 0, sizeof(BTEntries));
+ }
+ RedoCPUHook();
+}
+
 static std::vector<BranchTraceResult> GetBranchTrace(void)
 {
  BranchTraceResult tmp;
@@ -148,7 +137,7 @@ void CheckCPUBPCallB(bool write, uint32 address, unsigned int len)
  }
 }
 
-static void CPUHandler(uint32 PC)
+static void CPUHandler(const pscpu_timestamp_t timestamp, uint32 PC)
 {
  std::vector<PSX_BPOINT>::iterator bpit;
 
@@ -158,9 +147,7 @@ static void CPUHandler(uint32 PC)
   //exit(1);
   //puts((const char *)&MainRAM[CPU->GetRegister(PS_CPU::GSREG_GPR + 4, NULL, 0) & 0x1FFFFF]);
  }
-  
 
- // FIXME/TODO: Call ForceEventUpdates() somewhere
 
  for(bpit = BreakPointsPC.begin(); bpit != BreakPointsPC.end(); bpit++)
  {
@@ -171,27 +158,25 @@ static void CPUHandler(uint32 PC)
   }
  }
 
- CPU->CheckBreakpoints(CheckCPUBPCallB, PSX_MemPeek32(PC));
+ CPU->CheckBreakpoints(CheckCPUBPCallB, CPU->PeekMem32(PC));
 
- if(FoundBPoint)
+ CPUHookContinuous |= FoundBPoint;
+
+ if(CPUHookContinuous && CPUHook)
  {
-  BPCallB(PC);
-  FoundBPoint = 0;
+  ForceEventUpdates(timestamp);
+  CPUHook(PC, FoundBPoint);
  }
 
- if(CPUHook)
-  CPUHook(PC);
+ FoundBPoint = false;
 }
 
 
 static void RedoCPUHook(void)
 {
- const bool HappyTest = BreakPointsPC.size() || BreakPointsRead.size() || BreakPointsWrite.size();
- void (*cpuh)(uint32);
+ const bool HappyTest = CPUHook || BreakPointsPC.size() || BreakPointsRead.size() || BreakPointsWrite.size();
 
- cpuh = HappyTest ? CPUHandler : CPUHook;
-
- CPU->SetCPUHook(cpuh, cpuh ? AddBranchTrace : NULL);
+ CPU->SetCPUHook(HappyTest ? CPUHandler : NULL, BTEnabled ? AddBranchTrace : NULL);
 }
 
 static void FlushBreakPoints(int type)
@@ -224,18 +209,13 @@ static void AddBreakPoint(int type, unsigned int A1, unsigned int A2, bool logic
  RedoCPUHook();
 }
 
-static void SetCPUCallback(void (*callb)(uint32 PC))
+static void SetCPUCallback(void (*callb)(uint32 PC, bool bpoint), bool continuous)
 {
  CPUHook = callb;
+ CPUHookContinuous = continuous;
  RedoCPUHook();
 }
 
-static void SetBPCallback(void (*callb)(uint32 PC))
-{
- BPCallB = callb;
-}
-
-
 static void GetAddressSpaceBytes(const char *name, uint32 Address, uint32 Length, uint8 *Buffer)
 {
  if(!strcmp(name, "cpu"))
@@ -243,7 +223,7 @@ static void GetAddressSpaceBytes(const char *name, uint32 Address, uint32 Length
   while(Length--)
   {
    Address &= 0xFFFFFFFF;
-   *Buffer = PSX_MemPeek8(Address);
+   *Buffer = CPU->PeekMem8(Address);
    Address++;
    Buffer++;
   }
@@ -253,7 +233,7 @@ static void GetAddressSpaceBytes(const char *name, uint32 Address, uint32 Length
   while(Length--)
   {
    Address &= 0x1FFFFF;
-   *Buffer = PSX_MemPeek8(Address);
+   *Buffer = CPU->PeekMem8(Address);
    Address++;
    Buffer++;
   }
@@ -287,10 +267,11 @@ static void PutAddressSpaceBytes(const char *name, uint32 Address, uint32 Length
  {
   while(Length--)
   {
+   pscpu_timestamp_t dummy = 0;
    Address &= 0xFFFFFFFF;
 
    // TODO
-   PSX_MemWrite8(0, Address, *Buffer);
+   PSX_MemWrite8(dummy, Address, *Buffer);
 
    Address++;
    Buffer++;
@@ -300,22 +281,37 @@ static void PutAddressSpaceBytes(const char *name, uint32 Address, uint32 Length
  {
   while(Length--)
   {
+   Address &= 0xFFFFF;
+
    uint16 peeko = GPU->PeekRAM(Address >> 1);
 
-   Address &= 0xFFFFF;
    GPU->PokeRAM(Address >> 1, (*Buffer << ((Address & 1) * 8)) | (peeko & (0xFF00 >> ((Address & 1) * 8))) );
    Address++;
    Buffer++;
   }
  }
+ else if(!strcmp(name, "spu"))
+ {
+  while(Length--)
+  {
+   Address &= 0x7FFFF;
+
+   uint16 peeko = SPU->PeekSPURAM(Address >> 1);
+
+   SPU->PokeSPURAM(Address >> 1, (*Buffer << ((Address & 1) * 8)) | (peeko & (0xFF00 >> ((Address & 1) * 8))) );
+   Address++;
+   Buffer++;
+  }
+ }
+
 }
 
 static uint32 MemPeek(uint32 A, unsigned int bsize, bool hl, bool logical)
 {
  uint32 ret = 0;
 
- for(int i = 0; i < bsize; i++)
-  ret |= PSX_MemPeek8(A + i) << (i * 8);
+ for(unsigned int i = 0; i < bsize; i++)
+  ret |= CPU->PeekMem8(A + i) << (i * 8);
 
  return(ret);
 }
@@ -324,8 +320,9 @@ static void Disassemble(uint32 &A, uint32 SpecialA, char *TextBuf)
 {
  assert(!(A & 0x3));
 
- const uint32 instr = PSX_MemPeek32(A);
+ uint32 instr = CPU->PeekMem32(A);
 
+ CPU->PeekCheckICache(A, &instr);
 
  strncpy(TextBuf, DisassembleMIPS(A, instr).c_str(), 256);
  TextBuf[255] = 0;
@@ -335,6 +332,60 @@ static void Disassemble(uint32 &A, uint32 SpecialA, char *TextBuf)
  A += 4;
 }
 
+static MDFN_Surface *GfxDecode_Buf = NULL;
+static int GfxDecode_Line = -1;
+static int GfxDecode_Layer = 0;
+static int GfxDecode_Scroll = 0;
+static int GfxDecode_PBN = 0;
+
+static void DoGfxDecode(void)
+{
+ unsigned tp_w, tp_h;
+
+ tp_w = 256;
+ tp_h = 256;
+
+ if(GfxDecode_Buf)
+ {
+  for(int sy = 0; sy < GfxDecode_Buf->h; sy++)
+  {
+   for(int sx = 0; sx < GfxDecode_Buf->w; sx++)
+   {
+    unsigned fb_x = ((sx % GfxDecode_Buf->w) + ((sy + GfxDecode_Scroll) / GfxDecode_Buf->w * GfxDecode_Buf->w)) & 1023;
+    unsigned fb_y = (((sy + GfxDecode_Scroll) % GfxDecode_Buf->w) + ((((sx % GfxDecode_Buf->w) + ((sy + GfxDecode_Scroll) / GfxDecode_Buf->w * GfxDecode_Buf->w)) / 1024) * 256)) & 511;
+
+    uint16 pixel = GPU->PeekRAM(fb_y * 1024 + fb_x);
+
+    GfxDecode_Buf->pixels[(sy * GfxDecode_Buf->w * 3) + sx] = GfxDecode_Buf->MakeColor(((pixel >> 0) & 0x1F) * 255 / 31,
+											((pixel >> 5) & 0x1F) * 255 / 31,
+											((pixel >> 10) & 0x1F) * 255 / 31, 0xFF);
+   }
+  }
+ }
+}
+
+
+void DBG_GPUScanlineHook(unsigned scanline)
+{
+ if((int)scanline == GfxDecode_Line)
+ {
+  DoGfxDecode();
+ }
+}
+
+
+static void SetGraphicsDecode(MDFN_Surface *surface, int line, int which, int xscroll, int yscroll, int pbn)
+{
+ GfxDecode_Buf = surface;
+ GfxDecode_Line = line;
+ GfxDecode_Layer = which;
+ GfxDecode_Scroll = yscroll;
+ GfxDecode_PBN = pbn;
+
+ if(GfxDecode_Buf && GfxDecode_Line == -1)
+  DoGfxDecode();
+}
+
 DebuggerInfoStruct PSX_DBGInfo =
 {
  "shift_jis",
@@ -343,7 +394,7 @@ DebuggerInfoStruct PSX_DBGInfo =
  32,		// Logical address bits
  32,		// Physical address bits
  0x00000000,	// Default watch addr
- ~0,		// ZP addr
+ ~0U,		// ZP addr
 
  MemPeek,
  Disassemble,
@@ -353,35 +404,35 @@ DebuggerInfoStruct PSX_DBGInfo =
  FlushBreakPoints,
  AddBreakPoint,
  SetCPUCallback,
- SetBPCallback,
+ EnableBranchTrace,
  GetBranchTrace,
- NULL, //KING_SetGraphicsDecode,
+ SetGraphicsDecode,
  NULL, //PCFXDBG_SetLogFunc,
 };
 
 static RegType Regs_Misc[] =
 {
- { TIMER_GSREG_COUNTER0,	"COUNTER0", "Counter 0", 2 	},
+ { TIMER_GSREG_COUNTER0,	"COUNT0", "Counter 0", 2 	},
  { TIMER_GSREG_MODE0,		"MODE0", "Mode 0", 	2   	},
  { TIMER_GSREG_TARGET0,		"TARGET0", "Target 0",	2	},
 
  { 0, "------", "", 0xFFFF },
 
 
- { TIMER_GSREG_COUNTER1,	"COUNTER1", "Counter 1", 2 	},
+ { TIMER_GSREG_COUNTER1,	"COUNT1", "Counter 1", 2 	},
  { TIMER_GSREG_MODE1,		"MODE1", "Mode 1", 	2   	},
  { TIMER_GSREG_TARGET1,		"TARGET1", "Target 1",	2	},
 
  { 0, "------", "", 0xFFFF },
 
- { TIMER_GSREG_COUNTER2,	"COUNTER2", "Counter 2", 2 	},
+ { TIMER_GSREG_COUNTER2,	"COUNT2", "Counter 2", 2 	},
  { TIMER_GSREG_MODE2,		"MODE2", "Mode 2", 	2   	},
  { TIMER_GSREG_TARGET2,		"TARGET2", "Target 2",	2	},
 
  { 0, "------", "", 0xFFFF },
  { 0, "------", "", 0xFFFF },
 
- { 0x10000 | IRQ_GSREG_ASSERTED,		"ASSERTED",	"IRQ Asserted",	2 },
+ { 0x10000 | IRQ_GSREG_ASSERTED,	"ASSERTD",	"IRQ Asserted",	2 },
  { 0x10000 | IRQ_GSREG_STATUS,		"STATUS",	"IRQ Status", 2 },
  { 0x10000 | IRQ_GSREG_MASK,		"MASK",		"IRQ Mask", 2 },
 
@@ -456,26 +507,26 @@ static RegType Regs_SPU[] =
  { PS_SPU::GSREG_IIR_COEF, "IIR_COEF", "", 2 },
  { PS_SPU::GSREG_FB_ALPHA, "FB_ALPHA", "", 2 },
  { PS_SPU::GSREG_FB_X, "FB_X", "", 2 },
- { PS_SPU::GSREG_IIR_DEST_A0, "IIR_DEST_A0", "", 2 },
- { PS_SPU::GSREG_IIR_DEST_A1, "IIR_DEST_A1", "", 2 },
+ { PS_SPU::GSREG_IIR_DEST_A0, "IIR_DST_A0", "", 2 },
+ { PS_SPU::GSREG_IIR_DEST_A1, "IIR_DST_A1", "", 2 },
  { PS_SPU::GSREG_ACC_SRC_A0, "ACC_SRC_A0", "", 2 },
  { PS_SPU::GSREG_ACC_SRC_A1, "ACC_SRC_A1", "", 2 },
  { PS_SPU::GSREG_ACC_SRC_B0, "ACC_SRC_B0", "", 2 },
  { PS_SPU::GSREG_ACC_SRC_B1, "ACC_SRC_B1", "", 2 },
  { PS_SPU::GSREG_IIR_SRC_A0, "IIR_SRC_A0", "", 2 },
  { PS_SPU::GSREG_IIR_SRC_A1, "IIR_SRC_A1", "", 2 },
- { PS_SPU::GSREG_IIR_DEST_B0, "IIR_DEST_B0", "", 2 },
- { PS_SPU::GSREG_IIR_DEST_B1, "IIR_DEST_B1", "", 2 },
+ { PS_SPU::GSREG_IIR_DEST_B0, "IIR_DST_B0", "", 2 },
+ { PS_SPU::GSREG_IIR_DEST_B1, "IIR_DST_B1", "", 2 },
  { PS_SPU::GSREG_ACC_SRC_C0, "ACC_SRC_C0", "", 2 },
  { PS_SPU::GSREG_ACC_SRC_C1, "ACC_SRC_C1", "", 2 },
  { PS_SPU::GSREG_ACC_SRC_D0, "ACC_SRC_D0", "", 2 },
  { PS_SPU::GSREG_ACC_SRC_D1, "ACC_SRC_D1", "", 2 },
  { PS_SPU::GSREG_IIR_SRC_B1, "IIR_SRC_B1", "", 2 },
  { PS_SPU::GSREG_IIR_SRC_B0, "IIR_SRC_B0", "", 2 },
- { PS_SPU::GSREG_MIX_DEST_A0, "MIX_DEST_A0", "", 2 },
- { PS_SPU::GSREG_MIX_DEST_A1, "MIX_DEST_A1", "", 2 },
- { PS_SPU::GSREG_MIX_DEST_B0, "MIX_DEST_B0", "", 2 },
- { PS_SPU::GSREG_MIX_DEST_B1, "MIX_DEST_B1", "", 2 },
+ { PS_SPU::GSREG_MIX_DEST_A0, "MIX_DST_A0", "", 2 },
+ { PS_SPU::GSREG_MIX_DEST_A1, "MIX_DST_A1", "", 2 },
+ { PS_SPU::GSREG_MIX_DEST_B0, "MIX_DST_B0", "", 2 },
+ { PS_SPU::GSREG_MIX_DEST_B1, "MIX_DST_B1", "", 2 },
  { PS_SPU::GSREG_IN_COEF_L, "IN_COEF_L", "", 2 },
  { PS_SPU::GSREG_IN_COEF_R, "IN_COEF_R", "", 2 },
 
@@ -498,16 +549,21 @@ static RegType Regs_SPU[] =
 
 static RegType Regs_SPU_Voices[] =
 {
-#if 0
+#if 1
  VOICE_HELPER(0),
  VOICE_HELPER(1),
  VOICE_HELPER(2),
  VOICE_HELPER(3),
 #else
- VOICE_HELPER(20),
- VOICE_HELPER(21),
+ VOICE_HELPER(9),
+ VOICE_HELPER(12),
+ VOICE_HELPER(17),
  VOICE_HELPER(22),
- VOICE_HELPER(23),
+
+ //VOICE_HELPER(20),
+ //VOICE_HELPER(21),
+ //VOICE_HELPER(22),
+ //VOICE_HELPER(23),
 #endif
  { 0, "", "", 0 },
 };
@@ -580,6 +636,8 @@ static RegType Regs_CPU[] =
         { 0, "------", "", 0xFFFF },
 
 	{ PS_CPU::GSREG_SR,	"SR",	"Status Register", 4 },
+	{ PS_CPU::GSREG_CAUSE,	"CAU","Cause Register", 4 },
+	{ PS_CPU::GSREG_EPC,	"EPC",	"EPC Register", 4 },
 	{ 0, "", "", 0 }
 };
 
@@ -605,9 +663,13 @@ static RegGroupType CPURegsGroup =
 bool DBG_Init(void)
 {
  CPUHook = NULL;
- BPCallB = NULL;
+ CPUHookContinuous = false;
  FoundBPoint = false;
 
+ BTEnabled = false;
+ BTIndex = false;
+ memset(BTEntries, 0, sizeof(BTEntries));
+
  MDFNDBG_AddRegGroup(&CPURegsGroup);
  MDFNDBG_AddRegGroup(&MiscRegsGroup);
  MDFNDBG_AddRegGroup(&SPURegsGroup);
@@ -622,3 +684,4 @@ bool DBG_Init(void)
 
 
 }
+
