@@ -1,10 +1,31 @@
+/* Mednafen - Multi-system Emulator
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
 #include "psx.h"
 #include "frontio.h"
-#include "../FileWrapper.h"
 
 #include "input/gamepad.h"
 #include "input/dualanalog.h"
+#include "input/dualshock.h"
 #include "input/mouse.h"
+#include "input/negcon.h"
+#include "input/guncon.h"
+#include "input/justifier.h"
+
 #include "input/memcard.h"
 
 #include "input/multitap.h"
@@ -36,6 +57,26 @@ void InputDevice::ResetTS(void)
 
 }
 
+void InputDevice::SetAMCT(bool)
+{
+
+}
+
+void InputDevice::SetCrosshairsColor(uint32 color)
+{
+
+}
+
+bool InputDevice::RequireNoFrameskip(void)
+{
+ return(false);
+}
+
+pscpu_timestamp_t InputDevice::GPULineHook(const pscpu_timestamp_t timestamp, bool vsync, uint32 *pixels, const MDFN_PixelFormat* const format, const unsigned width, const unsigned pix_clock_offset, const unsigned pix_clock)
+{
+ return(PSX_EVENT_MAXTS);
+}
+
 
 void InputDevice::UpdateInput(const void *data)
 {
@@ -84,53 +125,80 @@ void InputDevice::ResetNVDirtyCount(void)
 
 }
 
-static INLINE unsigned EP_to_MP(unsigned ep)
+static unsigned EP_to_MP(bool emulate_multitap[2], unsigned ep)
 {
-#if 0
- return((bool)(ep & 0x4));
-#else
- return(ep == 1 || ep >= 5);
-#endif
+ if(!emulate_multitap[0] && emulate_multitap[1])
+ {
+  if(ep == 0 || ep >= 5)
+   return(0);
+  else
+   return(1);
+ }
+ else
+  return(ep >= 4);
 }
 
-static INLINE unsigned EP_to_SP(unsigned ep)
+static INLINE unsigned EP_to_SP(bool emulate_multitap[2], unsigned ep)
 {
-#if 0
- return(ep & 0x3);
-#else
- if(ep < 2)
-  return(0);
- else if(ep < 5)
-  return(ep - 2 + 1);
+ if(!emulate_multitap[0] && emulate_multitap[1])
+ {
+  if(ep == 0)
+   return(0);
+  else if(ep < 5)
+   return(ep - 1);
+  else
+   return(ep - 4);
+ }
  else
-  return(ep - 5 + 1);
-#endif
+  return(ep & 0x3);
 }
 
 void FrontIO::MapDevicesToPorts(void)
 {
- for(unsigned i = 0; i < 2; i++)
+ if(emulate_multitap[0] && emulate_multitap[1])
  {
-  if(emulate_multitap[i])
+  for(unsigned i = 0; i < 2; i++)
   {
    Ports[i] = DevicesTap[i];
    MCPorts[i] = DummyDevice;
   }
-  else
+ }
+ else if(!emulate_multitap[0] && emulate_multitap[1])
+ {
+  Ports[0] = Devices[0];
+  MCPorts[0] = emulate_memcards[0] ? DevicesMC[0] : DummyDevice;
+
+  Ports[1] = DevicesTap[1];
+  MCPorts[1] = DummyDevice;
+ }
+ else if(emulate_multitap[0] && !emulate_multitap[1])
+ {
+  Ports[0] = DevicesTap[0];
+  MCPorts[0] = DummyDevice;
+
+  Ports[1] = Devices[4];
+  MCPorts[1] = emulate_memcards[4] ? DevicesMC[4] : DummyDevice;
+ }
+ else
+ {
+  for(unsigned i = 0; i < 2; i++)
   {
    Ports[i] = Devices[i];
    MCPorts[i] = emulate_memcards[i] ? DevicesMC[i] : DummyDevice;
   }
  }
 
+ //printf("\n");
  for(unsigned i = 0; i < 8; i++)
  {
-  unsigned mp = EP_to_MP(i);
+  unsigned mp = EP_to_MP(emulate_multitap, i);
   
   if(emulate_multitap[mp])
-   DevicesTap[mp]->SetSubDevice(EP_to_SP(i), Devices[i], emulate_memcards[i] ? DevicesMC[i] : DummyDevice);
+   DevicesTap[mp]->SetSubDevice(EP_to_SP(emulate_multitap, i), Devices[i], emulate_memcards[i] ? DevicesMC[i] : DummyDevice);
   else
-   DevicesTap[mp]->SetSubDevice(EP_to_SP(i), DummyDevice, DummyDevice);
+   DevicesTap[mp]->SetSubDevice(EP_to_SP(emulate_multitap, i), DummyDevice, DummyDevice);
+
+  //printf("%d-> multitap: %d, sub-port: %d\n", i, mp, EP_to_SP(emulate_multitap, i));
  }
 }
 
@@ -139,23 +207,42 @@ FrontIO::FrontIO(bool emulate_memcards_[8], bool emulate_multitap_[2])
  memcpy(emulate_memcards, emulate_memcards_, sizeof(emulate_memcards));
  memcpy(emulate_multitap, emulate_multitap_, sizeof(emulate_multitap));
 
- DummyDevice = new InputDevice;
+ DummyDevice = new InputDevice();
 
- for(int i = 0; i < 8; i++)
+ for(unsigned i = 0; i < 8; i++)
  {
   DeviceData[i] = NULL;
-  Devices[i] = new InputDevice;
+  Devices[i] = new InputDevice();
   DevicesMC[i] = Device_Memcard_Create();
+  chair_colors[i] = 1 << 24;
+  Devices[i]->SetCrosshairsColor(chair_colors[i]);
  }
 
  for(unsigned i = 0; i < 2; i++)
  {
-  DevicesTap[i] = new InputDevice_Multitap;
+  DevicesTap[i] = new InputDevice_Multitap();
  }
 
  MapDevicesToPorts();
 }
 
+void FrontIO::SetAMCT(bool enabled)
+{
+ for(unsigned i = 0; i < 8; i++)
+ {
+  Devices[i]->SetAMCT(enabled);
+ }
+ amct_enabled = enabled;
+}
+
+void FrontIO::SetCrosshairsColor(unsigned port, uint32 color)
+{
+ assert(port >= 0 && port < 8);
+
+ chair_colors[port] = color;
+ Devices[port]->SetCrosshairsColor(color);
+}
+
 FrontIO::~FrontIO()
 {
  for(int i = 0; i < 8; i++)
@@ -180,10 +267,18 @@ FrontIO::~FrontIO()
    DevicesTap[i] = NULL;
   }
  }
+
+ if(DummyDevice)
+ {
+  delete DummyDevice;
+  DummyDevice = NULL;
+ }
 }
 
-int32 FrontIO::CalcNextEvent(int32 next_event)
+pscpu_timestamp_t FrontIO::CalcNextEventTS(pscpu_timestamp_t timestamp, int32 next_event)
 {
+ pscpu_timestamp_t ret;
+
  if(ClockDivider > 0 && ClockDivider < next_event)
   next_event = ClockDivider;
 
@@ -191,9 +286,19 @@ int32 FrontIO::CalcNextEvent(int32 next_event)
   if(dsr_pulse_delay[i] > 0 && next_event > dsr_pulse_delay[i])
    next_event = dsr_pulse_delay[i];
 
- return(next_event);
+ ret = timestamp + next_event;
+
+ if(irq10_pulse_ts[0] < ret)
+  ret = irq10_pulse_ts[0];
+
+ if(irq10_pulse_ts[1] < ret)
+  ret = irq10_pulse_ts[1];
+
+ return(ret);
 }
 
+static const uint8 ScaleShift[4] = { 0, 0, 4, 6 };
+
 void FrontIO::CheckStartStopPending(pscpu_timestamp_t timestamp, bool skip_event_set)
 {
  //const bool prior_ReceiveInProgress = ReceiveInProgress;
@@ -220,7 +325,8 @@ void FrontIO::CheckStartStopPending(pscpu_timestamp_t timestamp, bool skip_event
    TransmitBitCounter = 0;
   }
 
-  ClockDivider = 0x44;
+  ClockDivider = std::max<uint32>(0x20, (Baudrate << ScaleShift[Mode & 0x3]) & ~1); // Minimum of 0x20 is an emulation sanity check to prevent severe performance degradation.
+  //printf("CD: 0x%02x\n", ClockDivider);
  }
 
  if(!(Control & 0x5))
@@ -233,7 +339,7 @@ void FrontIO::CheckStartStopPending(pscpu_timestamp_t timestamp, bool skip_event
   ClockDivider = 0;
 
  if(!(skip_event_set))
-  PSX_SetEventNT(PSX_EVENT_FIO, timestamp + CalcNextEvent(0x10000000));
+  PSX_SetEventNT(PSX_EVENT_FIO, CalcNextEventTS(timestamp, 0x10000000));
 }
 
 // DSR IRQ bit setting appears(from indirect tests on real PS1) to be level-sensitive, not edge-sensitive
@@ -270,8 +376,9 @@ void FrontIO::Write(pscpu_timestamp_t timestamp, uint32 A, uint32 V)
 
   case 0xa:
 	if(ClockDivider > 0 && ((V & 0x2000) != (Control & 0x2000)) && ((Control & 0x2) == (V & 0x2))  )
-	 fprintf(stderr, "FIO device selection changed during comm %04x->%04x", Control, V);
+	 PSX_DBG(PSX_DBG_WARNING, "FIO device selection changed during comm %04x->%04x\n", Control, V);
 
+	//printf("Control: %d, %04x\n", timestamp, V);
 	Control = V & 0x3F2F;
 
 	if(V & 0x10)
@@ -338,6 +445,7 @@ if(!((Control & 0x2) && (Control & 0x2000)))
 
   case 0xe:
 	Baudrate = V;
+	//printf("%02x\n", V);
 	//MDFN_DispMessage("%02x\n", V);
 	break;
  }
@@ -357,6 +465,7 @@ uint32 FrontIO::Read(pscpu_timestamp_t timestamp, uint32 A)
  switch(A & 0xF)
  {
   case 0x0:
+	//printf("FIO Read: 0x%02x\n", ReceiveBuffer);
 	ret = ReceiveBuffer | (ReceiveBuffer << 8) | (ReceiveBuffer << 16) | (ReceiveBuffer << 24);
 	ReceiveBufferAvail = false;
 	ReceivePending = true;
@@ -416,6 +525,16 @@ pscpu_timestamp_t FrontIO::Update(pscpu_timestamp_t timestamp)
    }
   }
 
+ for(int i = 0; i < 2; i++)
+ {
+  if(timestamp >= irq10_pulse_ts[i])
+  {
+   //printf("Yay: %d %u\n", i, timestamp);
+   irq10_pulse_ts[i] = PSX_EVENT_MAXTS;
+   IRQ_Assert(IRQ_PIO, true);
+   IRQ_Assert(IRQ_PIO, false);
+  }
+ }
 
  if(ClockDivider > 0)
  {
@@ -471,7 +590,7 @@ pscpu_timestamp_t FrontIO::Update(pscpu_timestamp_t timestamp)
       }
      }
     }
-    ClockDivider += 0x44; //88; //99;
+    ClockDivider += std::max<uint32>(0x20, (Baudrate << ScaleShift[Mode & 0x3]) & ~1); // Minimum of 0x20 is an emulation sanity check to prevent severe performance degradation.
    }
    else
     break;
@@ -487,7 +606,7 @@ pscpu_timestamp_t FrontIO::Update(pscpu_timestamp_t timestamp)
   CheckStartStopPending(timestamp, true);
  }
 
- return(timestamp + CalcNextEvent(0x10000000));
+ return(CalcNextEventTS(timestamp, 0x10000000));
 }
 
 void FrontIO::ResetTS(void)
@@ -507,6 +626,12 @@ void FrontIO::ResetTS(void)
   DevicesTap[i]->ResetTS();
  }
 
+ for(int i = 0; i < 2; i++)
+ {
+  if(irq10_pulse_ts[i] != PSX_EVENT_MAXTS)
+   irq10_pulse_ts[i] -= lastts;
+ }
+
  for(int i = 0; i < 4; i++)
  {
   if(dsr_active_until_ts[i] >= 0)
@@ -527,6 +652,11 @@ void FrontIO::Power(void)
   dsr_active_until_ts[i] = -1;
  }
 
+ for(int i = 0; i < 2; i++)
+ {
+  irq10_pulse_ts[i] = PSX_EVENT_MAXTS;
+ }
+
  lastts = 0;
 
  //
@@ -572,17 +702,34 @@ void FrontIO::SetInput(unsigned int port, const char *type, void *ptr)
  delete Devices[port];
  Devices[port] = NULL;
 
- if(!strcmp(type, "gamepad"))
+ if(port < 2)
+  irq10_pulse_ts[port] = PSX_EVENT_MAXTS;
+
+ if(!strcmp(type, "gamepad") || !strcmp(type, "dancepad"))
   Devices[port] = Device_Gamepad_Create();
  else if(!strcmp(type, "dualanalog"))
   Devices[port] = Device_DualAnalog_Create(false);
  else if(!strcmp(type, "analogjoy"))
   Devices[port] = Device_DualAnalog_Create(true);
+ else if(!strcmp(type, "dualshock"))
+ {
+  char name[256];
+  trio_snprintf(name, 256, _("DualShock on port %u"), port + 1);
+  Devices[port] = Device_DualShock_Create(std::string(name));
+ }
  else if(!strcmp(type, "mouse"))
   Devices[port] = Device_Mouse_Create();
+ else if(!strcmp(type, "negcon"))
+  Devices[port] = Device_neGcon_Create();
+ else if(!strcmp(type, "guncon"))
+  Devices[port] = Device_GunCon_Create();
+ else if(!strcmp(type, "justifier"))
+  Devices[port] = Device_Justifier_Create();
  else
-  Devices[port] = new InputDevice;
+  Devices[port] = new InputDevice();
 
+ Devices[port]->SetAMCT(amct_enabled);
+ Devices[port]->SetCrosshairsColor(chair_colors[port]);
  DeviceData[port] = ptr;
 
  MapDevicesToPorts();
@@ -603,7 +750,7 @@ void FrontIO::LoadMemcard(unsigned int which, const char *path)
  {
   if(DevicesMC[which]->GetNVSize())
   {
-   FileWrapper mf(path, FileWrapper::MODE_READ);
+   FileStream mf(path, FileStream::MODE_READ);
    std::vector<uint8> tmpbuf;
 
    tmpbuf.resize(DevicesMC[which]->GetNVSize());
@@ -630,7 +777,7 @@ void FrontIO::SaveMemcard(unsigned int which, const char *path)
 
  if(DevicesMC[which]->GetNVSize() && DevicesMC[which]->GetNVDirtyCount())
  {
-  FileWrapper mf(path, FileWrapper::MODE_WRITE);	// TODO: MODE_WRITE_ATOMIC_OVERWRITE
+  FileStream mf(path, FileStream::MODE_WRITE);	// TODO: MODE_WRITE_ATOMIC_OVERWRITE
   std::vector<uint8> tmpbuf;
 
   tmpbuf.resize(DevicesMC[which]->GetNVSize());
@@ -644,7 +791,38 @@ void FrontIO::SaveMemcard(unsigned int which, const char *path)
  }
 }
 
+bool FrontIO::RequireNoFrameskip(void)
+{
+ for(unsigned i = 0; i < 8; i++)
+  if(Devices[i]->RequireNoFrameskip())
+   return(true);
+ 
+ return(false);
+}
 
+void FrontIO::GPULineHook(const pscpu_timestamp_t timestamp, const pscpu_timestamp_t line_timestamp, bool vsync, uint32 *pixels, const MDFN_PixelFormat* const format, const unsigned width, const unsigned pix_clock_offset, const unsigned pix_clock)
+{
+ Update(timestamp);
+
+ for(unsigned i = 0; i < 8; i++)
+ {
+  pscpu_timestamp_t plts = Devices[i]->GPULineHook(line_timestamp, vsync, pixels, format, width, pix_clock_offset, pix_clock);
+
+  if(i < 2)
+  {
+   irq10_pulse_ts[i] = plts;
+
+   if(irq10_pulse_ts[i] <= timestamp)
+   {
+    irq10_pulse_ts[i] = PSX_EVENT_MAXTS;
+    IRQ_Assert(IRQ_PIO, true);
+    IRQ_Assert(IRQ_PIO, false);
+   }
+  }
+ }
+
+ PSX_SetEventNT(PSX_EVENT_FIO, CalcNextEventTS(timestamp, 0x10000000));
+}
 
 static InputDeviceInfoStruct InputDeviceInfoPSXPort[] =
 {
@@ -653,6 +831,7 @@ static InputDeviceInfoStruct InputDeviceInfoPSXPort[] =
   "none",
   "none",
   NULL,
+  NULL,
   0,
   NULL 
  },
@@ -661,15 +840,27 @@ static InputDeviceInfoStruct InputDeviceInfoPSXPort[] =
  {
   "gamepad",
   "Digital Gamepad",
+  "PlayStation digital gamepad; SCPH-1080.",
   NULL,
   sizeof(Device_Gamepad_IDII) / sizeof(InputDeviceInputInfoStruct),
   Device_Gamepad_IDII,
  },
 
+ // Dual Shock Gamepad(SCPH-1200)
+ {
+  "dualshock",
+  "DualShock",
+  "DualShock gamepad; SCPH-1200.  Emulation in Mednafen includes the analog mode toggle button.  Rumble is emulated, but currently only supported on Linux, and MS Windows via the XInput API and XInput-compatible gamepads/joysticks.  If you're having trouble getting rumble to work on Linux, see if Mednafen is printing out error messages during startup regarding /dev/input/event*, and resolve the issue(s) as necessary.",
+  NULL,
+  sizeof(Device_DualShock_IDII) / sizeof(InputDeviceInputInfoStruct),
+  Device_DualShock_IDII,
+ },
+
  // Dual Analog Gamepad(SCPH-1180), forced to analog mode.
  {
   "dualanalog",
   "Dual Analog",
+  "Dual Analog gamepad; SCPH-1180.  It is the predecessor/prototype to the more advanced DualShock.  Emulated in Mednafen as forced to analog mode, and without rumble.",
   NULL,
   sizeof(Device_DualAnalog_IDII) / sizeof(InputDeviceInputInfoStruct),
   Device_DualAnalog_IDII,
@@ -680,6 +871,7 @@ static InputDeviceInfoStruct InputDeviceInfoPSXPort[] =
  {
   "analogjoy",
   "Analog Joystick",
+  "Flight-game-oriented dual-joystick controller; SCPH-1110.   Emulated in Mednafen as forced to analog mode.",
   NULL,
   sizeof(Device_AnalogJoy_IDII) / sizeof(InputDeviceInputInfoStruct),
   Device_AnalogJoy_IDII,
@@ -689,22 +881,59 @@ static InputDeviceInfoStruct InputDeviceInfoPSXPort[] =
   "mouse",
   "Mouse",
   NULL,
+  NULL,
   sizeof(Device_Mouse_IDII) / sizeof(InputDeviceInputInfoStruct),
   Device_Mouse_IDII,
  },
 
+ {
+  "negcon",
+  "neGcon",
+  "Namco's unconventional twisty racing-game-oriented gamepad; NPC-101.",
+  NULL,
+  sizeof(Device_neGcon_IDII) / sizeof(InputDeviceInputInfoStruct),
+  Device_neGcon_IDII,
+ },
+
+ {
+  "guncon",
+  "GunCon",
+  "Namco's light gun; NPC-103.",
+  NULL,
+  sizeof(Device_GunCon_IDII) / sizeof(InputDeviceInputInfoStruct),
+  Device_GunCon_IDII,
+ },
+
+ {
+  "justifier",
+  "Konami Justifier",
+  "Konami's light gun; SLUH-00017.  Rumored to be wrought of the coagulated rage of all who tried to shoot The Dog.  If the game you want to play supports the \"GunCon\", you should use that instead. NOTE: Currently does not work properly when on any of ports 1B-1D and 2B-2D.",
+  NULL,
+  sizeof(Device_Justifier_IDII) / sizeof(InputDeviceInputInfoStruct),
+  Device_Justifier_IDII,
+ },
+
+ {
+  "dancepad",
+  "Dance Pad",
+  "Dingo Dingo Rodeo!",
+  NULL,
+  sizeof(Device_Dancepad_IDII) / sizeof(InputDeviceInputInfoStruct),
+  Device_Dancepad_IDII,
+ },
+
 };
 
 static const InputPortInfoStruct PortInfo[] =
 {
- { 0, "port1", "Port 1/1A", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
- { 0, "port2", "Port 2/2A", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
- { 0, "port3", "Port 1B", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
- { 0, "port4", "Port 1C", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
- { 0, "port5", "Port 1D", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
- { 0, "port6", "Port 2B", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
- { 0, "port7", "Port 2C", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
- { 0, "port8", "Port 2D", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
+ { "port1", "Virtual Port 1", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
+ { "port2", "Virtual Port 2", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
+ { "port3", "Virtual Port 3", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
+ { "port4", "Virtual Port 4", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
+ { "port5", "Virtual Port 5", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
+ { "port6", "Virtual Port 6", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
+ { "port7", "Virtual Port 7", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
+ { "port8", "Virtual Port 8", sizeof(InputDeviceInfoPSXPort) / sizeof(InputDeviceInfoStruct), InputDeviceInfoPSXPort, "gamepad" },
 };
 
 InputInfoStruct FIO_InputInfo =
