@@ -54,7 +54,7 @@
 	~63.41486 us per line, average.
 
  FB X and Y display positions can be changed during active display; and Y display position appears to be treated as an offset to the current Y readout
- position that gets reset around vsync time.
+ position that gets reset around vblank time.
 
 */
 
@@ -76,6 +76,18 @@
 
 */
 
+/*
+ November 29, 2012 notes:
+
+  PAL mode can be turned on, and then off again, mid-frame(creates a neat effect).
+
+  Pixel clock can be changed mid-frame with effect(the effect is either instantaneous, or cached at some point in the scanline, not tested to see which);
+  interestingly, alignment is off on a PS1 when going 5MHz->10MHz>5MHz with a grid image.
+
+  Vertical start and end can be changed during active display, with effect(though it needs to be vs0->ve0->vs1->ve1->..., vs0->vs1->ve0 doesn't apparently do anything 
+  different from vs0->ve0.
+*/
+
 namespace MDFN_IEN_PSX
 {
 //FILE *fp;
@@ -88,11 +100,10 @@ static const int32 dither_table[4][4] =
  {  3, -1,  2, -2 },
 };
 
-PS_GPU::PS_GPU(bool pal_clock_and_tv) : BlitterFIFO(0x10)
+PS_GPU::PS_GPU(bool pal_clock_and_tv, int sls, int sle) : BlitterFIFO(0x20) // 0x10 on actual PS1 GPU, 0x20 here(see comment at top of gpu.h)	// 0x10)
 {
  HardwarePALType = pal_clock_and_tv;
 
-
  for(int y = 0; y < 4; y++)
   for(int x = 0; x < 4; x++)
    for(int v = 0; v < 512; v++)
@@ -125,6 +136,9 @@ PS_GPU::PS_GPU(bool pal_clock_and_tv) : BlitterFIFO(0x10)
   RGB8SAT[i] = i;
 
  memset(RGB8SAT_Over, 0xFF, sizeof(RGB8SAT_Over));
+
+ LineVisFirst = sls;
+ LineVisLast = sle;
 }
 
 PS_GPU::~PS_GPU()
@@ -136,23 +150,73 @@ void PS_GPU::SoftReset(void) // Control command 0x00
 {
  DMAControl = 0;
 
+ if(DrawTimeAvail < 0)
+  DrawTimeAvail = 0;
+
  BlitterFIFO.Flush();
+ InCmd = INCMD_NONE;
 
- ParsingLineOrPolygonCommand = false;
- InPLine = false;
- InFBRead = false;
- InFBWrite = false;
-
- DisplayMode = 0;
  DisplayOff = 1;
  DisplayFB_XStart = 0;
  DisplayFB_YStart = 0;
 
- HorizStart = 0;
- HorizEnd = 0;
+ if(HardwarePALType)
+ {
+  DisplayMode = 0x08;
 
- VertStart = 0;
- VertEnd = 0;
+  // FIXME, timing values(I need a PAL PS1 to derive them); for now, just copy the NTSC ones.
+  HorizStart = 0x200;
+  HorizEnd = 0xC00;
+
+  VertStart = 0x10;
+  VertEnd = 0x100;
+ }
+ else
+ {
+  DisplayMode = 0;
+
+  HorizStart = 0x200;
+  HorizEnd = 0xC00;
+
+  VertStart = 0x10;
+  VertEnd = 0x100;
+ }
+
+ //
+ TexPageX = 0;
+ TexPageY = 0;
+
+ SpriteFlip = 0;
+
+ abr = 0;
+ TexMode = 0;
+
+ dtd = 0;
+ dfe = 0;
+
+ //
+ tww = 0; 
+ twh = 0; 
+ twx = 0;
+ twy = 0;
+
+ RecalcTexWindowLUT();
+
+ //
+ ClipX0 = 0;
+ ClipY0 = 0;
+
+ //
+ ClipX1 = 0;
+ ClipY1 = 0;
+
+ //
+ OffsX = 0;
+ OffsY = 0;
+
+ //
+ MaskSetOR = 0;
+ MaskEvalAND = 0;
 }
 
 void PS_GPU::Power(void)
@@ -191,11 +255,7 @@ void PS_GPU::Power(void)
 
  BlitterFIFO.Flush();
 
- ParsingLineOrPolygonCommand = false;
- InPLine = false;
-
- InFBRead = false;
- InFBWrite = false;
+ InCmd = INCMD_NONE;
  FBRW_X = 0;
  FBRW_Y = 0;
  FBRW_W = 0;
@@ -219,22 +279,17 @@ void PS_GPU::Power(void)
  //
  DisplayFB_CurYOffset = 0;
  DisplayFB_CurLineYReadout = 0;
- DisplayHeightCounter = 0;
-
+ InVBlank = true;
 
  // TODO: factor out in a separate function.
  LinesPerField = 263;
- VisibleStartLine = 23;
 
  //
  //
  //
- FrameInterlaced = false;
- PALMode = false;
- HeightMode = false;
  scanline = 0;
  field = 0;
- field_atvs = 0;
+ field_ram_readout = 0;
  PhaseChange = 0;
 
  //
@@ -248,6 +303,11 @@ void PS_GPU::Power(void)
  DrawTimeAvail = 0;
 
  lastts = 0;
+
+ SoftReset();
+
+ IRQ_Assert(IRQ_VBLANK, InVBlank);
+ TIMER_SetVBlank(InVBlank);
 }
 
 void PS_GPU::ResetTS(void)
@@ -378,7 +438,7 @@ INLINE bool PS_GPU::LineSkipTest(unsigned y)
  if((DisplayMode & 0x24) != 0x24)
   return false;
 
- if(HeightMode && !dfe && ((y & 1) == ((DisplayFB_YStart + !field_atvs) & 1))/* && !DisplayOff*/) //&& (y >> 1) >= DisplayFB_YStart && (y >> 1) < (DisplayFB_YStart + (VertEnd - VertStart)))
+ if(!dfe && ((y & 1) == ((DisplayFB_YStart + field_ram_readout) & 1))/* && !DisplayOff*/) //&& (y >> 1) >= DisplayFB_YStart && (y >> 1) < (DisplayFB_YStart + (VertEnd - VertStart)))
   return true;
 
  return false;
@@ -403,17 +463,19 @@ void PS_GPU::Command_FBFill(const uint32 *cb)
  int32 height = (cb[2] >> 16) & 0x1FF;
 
  //printf("[GPU] FB Fill %d:%d w=%d, h=%d\n", destX, destY, width, height);
+ DrawTimeAvail -= 46;	// Approximate
  DrawTimeAvail -= ((width * height) >> 3) + (height * 9);
 
  for(int32 y = 0; y < height; y++)
  {
-  if(LineSkipTest(y))
+  const int32 d_y = (y + destY) & 511;
+
+  if(LineSkipTest(d_y))
    continue;
 
   for(int32 x = 0; x < width; x++)
   {
-   int32 d_y = (y + destY) & 511;
-   int32 d_x = (x + destX) & 1023;
+   const int32 d_x = (x + destX) & 1023;
 
    GPURAM[d_y][d_x] = fill_value;
   }
@@ -459,7 +521,7 @@ void PS_GPU::Command_FBCopy(const uint32 *cb)
 
 void PS_GPU::Command_FBWrite(const uint32 *cb)
 {
- assert(!InFBRead);
+ assert(InCmd == INCMD_NONE);
 
  FBRW_X = (cb[1] >>  0) & 0x3FF;
  FBRW_Y = (cb[1] >> 16) & 0x3FF;
@@ -476,15 +538,13 @@ void PS_GPU::Command_FBWrite(const uint32 *cb)
  FBRW_CurX = FBRW_X;
  FBRW_CurY = FBRW_Y;
 
- InFBWrite = true;
-
- if(FBRW_W == 0 || FBRW_H == 0)
-  InFBWrite = false;
+ if(FBRW_W != 0 && FBRW_H != 0)
+  InCmd = INCMD_FBWRITE;
 }
 
 void PS_GPU::Command_FBRead(const uint32 *cb)
 {
- assert(!InFBWrite);
+ assert(InCmd == INCMD_NONE);
 
  FBRW_X = (cb[1] >>  0) & 0x3FF;
  FBRW_Y = (cb[1] >> 16) & 0x3FF;
@@ -501,54 +561,30 @@ void PS_GPU::Command_FBRead(const uint32 *cb)
  FBRW_CurX = FBRW_X;
  FBRW_CurY = FBRW_Y;
 
- InFBRead = true;
-
- if(FBRW_W == 0 || FBRW_H == 0)
-  InFBRead = false;
+ if(FBRW_W != 0 && FBRW_H != 0)
+  InCmd = INCMD_FBREAD;
 }
 
 
 void PS_GPU::RecalcTexWindowLUT(void)
 {
+ const unsigned TexWindowX_AND = ~(tww << 3);
+ const unsigned TexWindowX_OR = (twx & tww) << 3;
+
+ const unsigned TexWindowY_AND = ~(twh << 3);
+ const unsigned TexWindowY_OR = (twy & twh) << 3;
+
+// printf("TWX: 0x%02x, TWW: 0x%02x\n", twx, tww);
+// printf("TWY: 0x%02x, TWH: 0x%02x\n", twy, twh);
+
+ for(unsigned x = 0; x < 256; x++)
  {
-  uint8 targ_x = 0;
-  uint8 targ_wc = 0;
-  uint8 x = twx * 8;
-
-  do
-  {
-   if(!targ_wc)
-   {
-    targ_x = twx * 8;
-    targ_wc = tww * 8;
-   }
-   TexWindowXLUT[x] = targ_x;
-
-   x++;
-   targ_x++;
-   targ_wc++;
-  } while(x != (twx * 8));
+  TexWindowXLUT[x] = (x & TexWindowX_AND) | TexWindowX_OR;
  }
 
+ for(unsigned y = 0; y < 256; y++)
  {
-  uint8 targ_y = 0;
-  uint8 targ_hc = 0;
-  uint8 y = twy * 8;
-
-  do
-  {
-   if(!targ_hc)
-   {
-    targ_y = twy * 8;
-    targ_hc = twh * 8;
-   }
-   //printf("Texture window y: %d %d\n", y, targ_y);
-   TexWindowYLUT[y] = targ_y;
-
-   y++;
-   targ_y++;
-   targ_hc++;
-  } while(y != (twy * 8));
+  TexWindowYLUT[y] = (y & TexWindowY_AND) | TexWindowY_OR;
  }
 
  memset(TexWindowXLUT_Pre, TexWindowXLUT[0], sizeof(TexWindowXLUT_Pre));
@@ -570,7 +606,7 @@ void PS_GPU::Command_DrawMode(const uint32 *cb)
 
  dtd = (*cb >> 9) & 1;
  dfe = (*cb >> 10) & 1;
- //printf("DFE: %d -- scanline=%d\n", dfe, scanline);
+ //printf("*******************DFE: %d -- scanline=%d\n", dfe, scanline);
 }
 
 void PS_GPU::Command_TexWindow(const uint32 *cb)
@@ -644,70 +680,109 @@ CTEntry PS_GPU::Commands[4][256] =
 
 static uint64 PrimitiveCounter[256] = { 0 }; // Debug
 
-void PS_GPU::ProcessFIFO(bool force)
+void PS_GPU::ProcessFIFO(void)
 {
  if(!BlitterFIFO.CanRead())
   return;
 
- if(DrawTimeAvail < 0 && !force)
-  return;
-
- if(InFBWrite)
+ switch(InCmd)
  {
-  uint32 InData = BlitterFIFO.ReadUnit();
+  default:
+	abort();
+	break;
 
-  for(int i = 0; i < 2; i++)
-  {
-   if(!(GPURAM[FBRW_CurY & 511][FBRW_CurX & 1023] & MaskEvalAND))
-    GPURAM[FBRW_CurY & 511][FBRW_CurX & 1023] = InData | MaskSetOR;
+  case INCMD_NONE:
+	break;
 
-   FBRW_CurX++;
-   if(FBRW_CurX == (FBRW_X + FBRW_W))
-   {
-    FBRW_CurX = FBRW_X;
-    FBRW_CurY++;
-    if(FBRW_CurY == (FBRW_Y + FBRW_H))
-    {
-     InFBWrite = false;
-     break;
-    }
-   }
-   InData >>= 16;
-  }
-  return;
- }
+  case INCMD_FBREAD:
+	puts("BOGUS SALAMANDERS, CAPTAIN!");
+	return;
 
- if(InPLine)
- {
-  const uint32 cc = InPLine;
-  const CTEntry *command = &Commands[abr][cc];
-  unsigned vl = 1 + (bool)(InPLine & 0x10);
-  uint32 CB[2];
+  case INCMD_FBWRITE:
+       {
+  	uint32 InData = BlitterFIFO.ReadUnit();
 
-  if((BlitterFIFO.ReadUnit(true) & 0xF000F000) == 0x50005000)
-  {
-   BlitterFIFO.ReadUnit();
-   InPLine = false;
-   ParsingLineOrPolygonCommand = false;
-   return;
-  }
+  	for(int i = 0; i < 2; i++)
+  	{
+   	 if(!(GPURAM[FBRW_CurY & 511][FBRW_CurX & 1023] & MaskEvalAND))
+    	  GPURAM[FBRW_CurY & 511][FBRW_CurX & 1023] = InData | MaskSetOR;
 
-  if(BlitterFIFO.CanRead() >= vl)
-  {
-   for(unsigned i = 0; i < vl; i++)
-   {
-    CB[i] = BlitterFIFO.ReadUnit();
-   }
+	 FBRW_CurX++;
+   	 if(FBRW_CurX == (FBRW_X + FBRW_W))
+	 {
+	  FBRW_CurX = FBRW_X;
+	  FBRW_CurY++;
+	  if(FBRW_CurY == (FBRW_Y + FBRW_H))
+	  {
+	   InCmd = INCMD_NONE;
+	   break;	// Break out of the for() loop.
+	  }
+	 }
+	 InData >>= 16;
+  	}
+  	return;
+       }
+       break;
 
-   ((this)->*(command->func[TexMode | (MaskEvalAND ? 0x4 : 0x0)]))(CB);
-  }
-  return;
+  case INCMD_QUAD:
+       {
+	if(DrawTimeAvail < 0)
+	 return;
+
+	const uint32 cc = InCmd_CC;
+	const CTEntry *command = &Commands[abr][cc];
+	unsigned vl = 1 + (bool)(cc & 0x4) + (bool)(cc & 0x10);
+	uint32 CB[3];
+
+	if(BlitterFIFO.CanRead() >= vl)
+	{
+	 for(unsigned i = 0; i < vl; i++)
+	 {
+	  CB[i] = BlitterFIFO.ReadUnit();
+	 }
+
+	 ((this)->*(command->func[TexMode | (MaskEvalAND ? 0x4 : 0x0)]))(CB);
+	}
+	return;
+       }
+       break;
+
+  case INCMD_PLINE:
+       {
+  	if(DrawTimeAvail < 0)
+	 return;
+
+	const uint32 cc = InCmd_CC;
+	const CTEntry *command = &Commands[abr][cc];
+	unsigned vl = 1 + (bool)(InCmd_CC & 0x10);
+	uint32 CB[2];
+
+  	if((BlitterFIFO.ReadUnit(true) & 0xF000F000) == 0x50005000)
+	{
+	 BlitterFIFO.ReadUnit();
+	 InCmd = INCMD_NONE;
+	 return;
+	}
+
+	if(BlitterFIFO.CanRead() >= vl)
+	{
+	 for(unsigned i = 0; i < vl; i++)
+	 {
+	  CB[i] = BlitterFIFO.ReadUnit();
+	 }
+
+	 ((this)->*(command->func[TexMode | (MaskEvalAND ? 0x4 : 0x0)]))(CB);
+	}
+	return;
+       }
+       break;
  }
 
  const uint32 cc = BlitterFIFO.ReadUnit(true) >> 24;
  const CTEntry *command = &Commands[0][cc];
 
- ParsingLineOrPolygonCommand = (cc >= 0x20 && cc < 0x60);
+ if(DrawTimeAvail < 0 && !command->ss_cmd)
+  return;
 
  if(BlitterFIFO.CanRead() >= command->len)
  {
@@ -716,14 +791,15 @@ void PS_GPU::ProcessFIFO(bool force)
   for(unsigned i = 0; i < command->len; i++)
    CB[i] = BlitterFIFO.ReadUnit();
 
-  DrawTimeAvail -= 2;
+  if(!command->ss_cmd)
+   DrawTimeAvail -= 2;
 
   PrimitiveCounter[cc]++;
 
   if(!command->func[TexMode])
   {
-   if(CB[0])
-    PSX_WARNING("[GPU] Unknown command: %08x, %d", CB[0], scanline);
+   //if(CB[0])
+   // PSX_WARNING("[GPU] Unknown command: %08x, %d", CB[0], scanline);
   }
   else
   {
@@ -759,16 +835,16 @@ void PS_GPU::ProcessFIFO(bool force)
    ((this)->*(command->func[TexMode | (MaskEvalAND ? 0x4 : 0x0)]))(CB);
    //printf("COMMAND: %08x -- %8d ---- scanline=%d -- adta=%8d\n", CB[0], DrawTimeAvail - olddt, scanline, DrawTimeAvail);
   }
-
-  if(!InPLine)
-   ParsingLineOrPolygonCommand = false;
  }
 }
 
 INLINE void PS_GPU::WriteCB(uint32 InData)
 {
- if(!BlitterFIFO.CanWrite())
-  ProcessFIFO(true);
+ if(BlitterFIFO.CanRead() >= 0x10 && (InCmd != INCMD_NONE || (BlitterFIFO.CanRead() - 0x10) >= Commands[0][BlitterFIFO.ReadUnit(true) >> 24].fifo_fb_len))
+ {
+  PSX_DBG(PSX_DBG_WARNING, "GPU FIFO overflow!!!\n");
+  return;
+ }
 
  BlitterFIFO.WriteUnit(InData);
  ProcessFIFO();
@@ -776,6 +852,8 @@ INLINE void PS_GPU::WriteCB(uint32 InData)
 
 void PS_GPU::Write(const pscpu_timestamp_t timestamp, uint32 A, uint32 V)
 {
+ V <<= (A & 3) * 8;
+
  if(A & 4)	// GP1 ("Control")
  {
   uint32 command = V >> 24;
@@ -794,15 +872,15 @@ void PS_GPU::Write(const pscpu_timestamp_t timestamp, uint32 A, uint32 V)
 	    break;
 
    case 0x00:	// Reset GPU
+	//printf("\n\n************ Soft Reset %u ********* \n\n", scanline);
 	SoftReset();
 	break;
 
    case 0x01:	// Reset command buffer
+	if(DrawTimeAvail < 0)
+	 DrawTimeAvail = 0;
 	BlitterFIFO.Flush();
-	ParsingLineOrPolygonCommand = false;
-	InPLine = false;
-	InFBRead = false;
-	InFBWrite = false;
+	InCmd = INCMD_NONE;
 	break;
 
    case 0x02: 	// Reset IRQ ???
@@ -817,9 +895,8 @@ void PS_GPU::Write(const pscpu_timestamp_t timestamp, uint32 A, uint32 V)
 	break;
 
    case 0x05:	// Start of display area in framebuffer
-	DisplayFB_XStart = V & 0x3FF;
+	DisplayFB_XStart = V & 0x3FE; // Lower bit is apparently ignored.
 	DisplayFB_YStart = (V >> 10) & 0x1FF;
-	//printf("Xstart: %d -- Ystart: %d -- scanline=%d\n", DisplayFB_XStart, DisplayFB_YStart, scanline);
 	break;
 
    case 0x06:	// Horizontal display range
@@ -829,22 +906,19 @@ void PS_GPU::Write(const pscpu_timestamp_t timestamp, uint32 A, uint32 V)
 
    case 0x07:
 	VertStart = V & 0x3FF;
-	VertEnd = (V >> 10) & 0x3FF;	// & 0x3FF or & 0x7FF?
-	//printf("VS: %04x\n", VertStart);
+	VertEnd = (V >> 10) & 0x3FF;
 	break;
 
    case 0x08:
+	//printf("\n\nDISPLAYMODE SET: 0x%02x, %u *************************\n\n\n", V & 0xFF, scanline);
 	DisplayMode = V & 0xFF;
-	//printf("DM: %02x, %d\n", V, scanline);
 	break;
 
    case 0x10:	// GPU info(?)
 	switch(V & 0xF)
 	{
-	 default: PSX_WARNING("[GPU] Unknown control command GPU info param - %01x", V & 0xF);
-		  DataReadBuffer = 0;
-		  //assert(0);
-		  break;
+	 // DataReadBuffer must remain unchanged for any unhandled GPU info index.
+	 default:  break;
 
 	 case 0x2: DataReadBuffer = (tww << 0) | (twh << 5) | (twx << 10) | (twy << 15);
 		   break;
@@ -855,12 +929,14 @@ void PS_GPU::Write(const pscpu_timestamp_t timestamp, uint32 A, uint32 V)
  	 case 0x4: DataReadBuffer = (ClipY1 << 10) | ClipX1;
 		   break;
 
-	 case 0x5:
-	 case 0x6: DataReadBuffer = (OffsX & 2047) | ((OffsY & 2047) << 11);
+	 case 0x5: DataReadBuffer = (OffsX & 2047) | ((OffsY & 2047) << 11);
 		   break;
 
 	 case 0x7: DataReadBuffer = 2;
 		   break;
+
+	 case 0x8: DataReadBuffer = 0;
+		   break;
 	}
 	break;
 
@@ -881,6 +957,37 @@ void PS_GPU::WriteDMA(uint32 V)
  WriteCB(V);
 }
 
+INLINE uint32 PS_GPU::ReadData(void)
+{
+ if(InCmd == INCMD_FBREAD)
+ {
+  DataReadBuffer = 0;
+  for(int i = 0; i < 2; i++)
+  {
+   DataReadBuffer |= GPURAM[FBRW_CurY & 511][FBRW_CurX & 1023] << (i * 16);
+
+   FBRW_CurX++;
+   if(FBRW_CurX == (FBRW_X + FBRW_W))
+   {
+    FBRW_CurX = FBRW_X;
+    FBRW_CurY++;
+    if(FBRW_CurY == (FBRW_Y + FBRW_H))
+    {
+     InCmd = INCMD_NONE;
+     break;
+    }
+   }
+  }
+ }
+
+ return DataReadBuffer;
+}
+
+uint32 PS_GPU::ReadDMA(void)
+{
+ return ReadData();
+}
+
 uint32 PS_GPU::Read(const pscpu_timestamp_t timestamp, uint32 A)
 {
  uint32 ret = 0;
@@ -893,21 +1000,20 @@ uint32 PS_GPU::Read(const pscpu_timestamp_t timestamp, uint32 A)
 
   ret |= (DisplayFB_CurLineYReadout & 1) << 31;
 
-  ret |= field << 13;
+  ret |= (!field) << 13;
 
   if(DMAControl & 0x02)
    ret |= 1 << 25;
 
   ret |= DisplayOff << 23;
 
-  if(!InFBRead && !InFBWrite && !InPLine && DrawTimeAvail >= 0 && BlitterFIFO.CanWrite() == 0x10)	// GPU idle bit.
+  if(InCmd == INCMD_NONE && DrawTimeAvail >= 0 && BlitterFIFO.CanRead() == 0x00)	// GPU idle bit.
    ret |= 1 << 26;
 
-  if(InFBRead)
+  if(InCmd == INCMD_FBREAD)	// Might want to more accurately emulate this in the future?
    ret |= (1 << 27);
 
-  if(BlitterFIFO.CanWrite() && !ParsingLineOrPolygonCommand)	// FIFO has room bit? (kinda).
-   ret |= (1 << 28);
+  ret |= CalcFIFOReadyBit() << 28;		// FIFO has room bit? (kinda).
 
   //
   //
@@ -926,39 +1032,25 @@ uint32 PS_GPU::Read(const pscpu_timestamp_t timestamp, uint32 A)
    ret |= 1 << 12;
  }
  else		// "Data"
- {
-  if(InFBRead)
-  {
-   DataReadBuffer = 0;
-   for(int i = 0; i < 2; i++)
-   {
-    DataReadBuffer |= GPURAM[FBRW_CurY & 511][FBRW_CurX & 1023] << (i * 16);
-
-    FBRW_CurX++;
-    if(FBRW_CurX == (FBRW_X + FBRW_W))
-    {
-     FBRW_CurX = FBRW_X;
-     FBRW_CurY++;
-     if(FBRW_CurY == (FBRW_Y + FBRW_H))
-     {
-      InFBRead = false;
-      break;
-     }
-    }
-   }
-  }
-
-  ret = DataReadBuffer;
- }
+  ret = ReadData();
 
  if(DMAControl & 2)
  {
   //PSX_WARNING("[GPU READ WHEN (DMACONTROL&2)] 0x%08x - ret=0x%08x, scanline=%d", A, ret, scanline);
  }
 
- return(ret);
+ return(ret >> ((A & 3) * 8));
 }
 
+/*
+static INLINE uint32 ShiftHelper(uint32 val, int shamt, uint32 mask)
+{
+ if(shamt < 0)
+  return((val >> (-shamt)) & mask);
+ else
+  return((val << shamt) & mask);
+}
+*/
 INLINE void PS_GPU::ReorderRGB_Var(uint32 out_Rshift, uint32 out_Gshift, uint32 out_Bshift, bool bpp24, const uint16 *src, uint32 *dest, const int32 dx_start, const int32 dx_end, int32 fb_x)
 {
      if(bpp24)	// 24bpp
@@ -982,9 +1074,10 @@ INLINE void PS_GPU::ReorderRGB_Var(uint32 out_Rshift, uint32 out_Gshift, uint32
       {
        uint32 srcpix = src[fb_x >> 1];
 
-       //dest[x] = (((srcpix >> 0) << out_Rshift) & (0xFF << out_Rshift)) | (((srcpix >> 8) << out_Gshift) & (0xFF << out_Gshift)) |
-       //		 (((srcpix >> 16) << out_Bshift) & (0xFF << out_Bshift));
        dest[x] = OutputLUT[srcpix & 0x7FFF];
+       //dest[x] = ShiftHelper(srcpix, out_Rshift + 3 -  0, (0xF8 << out_Rshift)) |
+//	         ShiftHelper(srcpix, out_Gshift + 3 -  5, (0xF8 << out_Gshift)) |
+//	         ShiftHelper(srcpix, out_Bshift + 3 - 10, (0xF8 << out_Bshift));
 
        fb_x = (fb_x + 2) & 0x7FF;
       }
@@ -1003,11 +1096,10 @@ pscpu_timestamp_t PS_GPU::Update(const pscpu_timestamp_t sys_timestamp)
 {
  static const uint32 DotClockRatios[5] = { 10, 8, 5, 4, 7 };
  const uint32 dmc = (DisplayMode & 0x40) ? 4 : (DisplayMode & 0x3);
- const uint32 dmw = 2720 / DotClockRatios[dmc];
+ const uint32 dmw = 2800 / DotClockRatios[dmc];	// Must be <= 768
 
  int32 sys_clocks = sys_timestamp - lastts;
  int32 gpu_clocks;
- int32 dot_clocks;
 
  //printf("GPUISH: %d\n", sys_timestamp - lastts);
 
@@ -1028,34 +1120,47 @@ pscpu_timestamp_t PS_GPU::Update(const pscpu_timestamp_t sys_timestamp)
  gpu_clocks = GPUClockCounter >> 16;
  GPUClockCounter -= gpu_clocks << 16;
 
- DotClockCounter += gpu_clocks;
- dot_clocks = DotClockCounter / DotClockRatios[DisplayMode & 0x3];
- DotClockCounter -= dot_clocks * DotClockRatios[DisplayMode & 0x3];
-
- TIMER_AddDotClocks(dot_clocks);
-
  while(gpu_clocks > 0)
  {
   int32 chunk_clocks = gpu_clocks;
+  int32 dot_clocks;
 
   if(chunk_clocks > LineClockCounter)
+  {
+   //printf("Chunk: %u, LCC: %u\n", chunk_clocks, LineClockCounter);
    chunk_clocks = LineClockCounter;
+  }
 
   gpu_clocks -= chunk_clocks;
   LineClockCounter -= chunk_clocks;
 
+  DotClockCounter += chunk_clocks;
+  dot_clocks = DotClockCounter / DotClockRatios[DisplayMode & 0x3];
+  DotClockCounter -= dot_clocks * DotClockRatios[DisplayMode & 0x3];
+
+  TIMER_AddDotClocks(dot_clocks);
+
+
   if(!LineClockCounter)
   {
+   PSX_SetEventNT(PSX_EVENT_TIMER, TIMER_Update(sys_timestamp));  // We could just call this at the top of GPU_Update(), but do it here for slightly less CPU usage(presumably).
+
    LinePhase = (LinePhase + 1) & 1;
 
    if(LinePhase)
    {
+    TIMER_SetHRetrace(true);
     LineClockCounter = 200;
     TIMER_ClockHRetrace();
    }
    else
    {
-    if(PALMode)
+    const unsigned int FirstVisibleLine = LineVisFirst + (HardwarePALType ? 20 : 16);
+    const unsigned int VisibleLineCount = LineVisLast + 1 - LineVisFirst; //HardwarePALType ? 288 : 240;
+
+    TIMER_SetHRetrace(false);
+
+    if(DisplayMode & 0x08)
      LineClockCounter = 3405 - 200;
     else
      LineClockCounter = 3412 + PhaseChange - 200;
@@ -1063,131 +1168,177 @@ pscpu_timestamp_t PS_GPU::Update(const pscpu_timestamp_t sys_timestamp)
     scanline = (scanline + 1) % LinesPerField;
     PhaseChange = !PhaseChange;
 
-    if(scanline == (LinesPerField - 1))
-    {
-     PSX_RequestMLExit();
-    }
-
+#ifdef WANT_DEBUGGER
     DBG_GPUScanlineHook(scanline);
-
-    // printf("[GPU] DTA scanline=%3d --- %8d\n", scanline, DrawTimeAvail);
-
-    if(scanline == 0)
-    {
-#if 0
-	printf("Primitive Counts for previous frame:\n");
-	for(unsigned i = 0; i < 256; i++)
-	 if(PrimitiveCounter[i])
-	 {
-	  printf("0x%02x: %8u\n", i, (unsigned int)PrimitiveCounter[i]);
-	 }
-
-	printf("\n");
-	memset(PrimitiveCounter, 0, sizeof(PrimitiveCounter));
 #endif
 
-     //MDFN_DispMessage("%8d %d %d %d", DrawTimeAvail, InFBRead, InFBWrite, DMA_GPUWriteActive());
-     IRQ_Assert(IRQ_VSYNC, true);
-     IRQ_Assert(IRQ_VSYNC, false);
+    //
+    //
+    //
+    if(scanline == (HardwarePALType ? 308 : 256))	// Will need to be redone if we ever allow for visible vertical overscan with NTSC.
+    {
+     if(sl_zero_reached)
+     {
+      //printf("Req Exit(visible fallthrough case): %u\n", scanline);
+      PSX_RequestMLExit();
+     }
+    }
+
+    if(scanline == (LinesPerField - 1))
+    {
+     if(sl_zero_reached)
+     {
+      //printf("Req Exit(final fallthrough case): %u\n", scanline);
+      PSX_RequestMLExit();
+     }
+
+     if(DisplayMode & 0x20)
+      field = !field;
+     else
+      field = 0;
     }
 
     if(scanline == 0)
     {
-     field_atvs = field;
-     DisplayHeightCounter = 0;
+     assert(sl_zero_reached == false);
+     sl_zero_reached = true;
 
-     PALMode = (bool)(DisplayMode & 0x08);
-
-     if(PALMode)	// PAL
-      VisibleStartLine = 26;
-     else	// NTSC
-      VisibleStartLine = 22;
-
-     FrameInterlaced = (bool)(DisplayMode & 0x20);
-     HeightMode = (bool)(DisplayMode & 0x04) && FrameInterlaced;
-
-     if(PALMode != HardwarePALType)
-     {
-      const uint32 black = surface->MakeColor(0, 0, 0);
-
-      DisplayRect->w = 384;
-      DisplayRect->h = (HardwarePALType ? 288 : 240);
-
-      for(int32 y = 0; y < DisplayRect->h; y++)
-      {
-       uint32 *dest = surface->pixels + y * surface->pitch32;
-
-       LineWidths[y].x = 0;
-       LineWidths[y].w = 384;
-
-       for(int32 x = 0; x < 384; x++)
-       {
-        dest[x] = black;
-       }
-      }
-      char buffer[256];
-
-      trio_snprintf(buffer, sizeof(buffer), _("VIDEO STANDARD MISMATCH"));
-      DrawTextTrans(surface->pixels + ((DisplayRect->h / 2) - (13 / 2)) * surface->pitch32, surface->pitch32 << 2, DisplayRect->w, (UTF8*)buffer,
-		surface->MakeColor(0x00, 0xFF, 0x00), true, MDFN_FONT_6x13_12x13);
-     }
-     else
-     {
-      espec->InterlaceOn = FrameInterlaced;
-      espec->InterlaceField = !field;
-      DisplayRect->h = (HardwarePALType ? 288 : 240) << FrameInterlaced;
-     }
-
-     DisplayFB_CurYOffset = 0;
-    }
-
-    const int32 VS_Adjust = 7; //PALMode ? (34 - 6) : 7;
-
-    if(scanline == (5 + field_atvs))
-    {
-     if(FrameInterlaced)
+     if(DisplayMode & 0x20)
      {
       skip = false;
-      field = !field;
 
-      if(PALMode)	// PAL
+      if(DisplayMode & 0x08)	// PAL
        LinesPerField = 313 - field;
       else			// NTSC
        LinesPerField = 263 - field;
      }
      else
      {
-      field = 0;	// May not be correct.
+      field = 0;	// May not be the correct place for this?
 
-      if(PALMode)	// PAL
+      if(DisplayMode & 0x08)	// PAL
        LinesPerField = 314;
       else			// NTSC
        LinesPerField = 263;
      }
+
+     if(espec)
+     {
+      if((bool)(DisplayMode & 0x08) != HardwarePALType)
+      {
+       const uint32 black = surface->MakeColor(0, 0, 0);
+
+       DisplayRect->x = 0;
+       DisplayRect->y = 0;
+       DisplayRect->w = 384;
+       DisplayRect->h = VisibleLineCount;
+
+       for(int32 y = 0; y < DisplayRect->h; y++)
+       {
+        uint32 *dest = surface->pixels + y * surface->pitch32;
+
+        LineWidths[y].x = 0;
+        LineWidths[y].w = 384;
+
+        for(int32 x = 0; x < 384; x++)
+        {
+         dest[x] = black;
+        }
+       }
+       char buffer[256];
+
+       trio_snprintf(buffer, sizeof(buffer), _("VIDEO STANDARD MISMATCH"));
+       DrawTextTrans(surface->pixels + ((DisplayRect->h / 2) - (13 / 2)) * surface->pitch32, surface->pitch32 << 2, DisplayRect->w, (UTF8*)buffer,
+		surface->MakeColor(0x00, 0xFF, 0x00), true, MDFN_FONT_6x13_12x13);
+      }
+      else
+      {
+       const uint32 black = surface->MakeColor(0, 0, 0);
+
+       espec->InterlaceOn = (bool)(DisplayMode & 0x20);
+       espec->InterlaceField = (bool)(DisplayMode & 0x20) && field;
+
+       DisplayRect->x = 0;
+       DisplayRect->y = 0;
+       DisplayRect->w = 0;
+       DisplayRect->h = VisibleLineCount << (bool)(DisplayMode & 0x20);
+
+       // Clear ~0 state.
+       LineWidths[0].x = LineWidths[0].w = 0;
+
+       for(int i = 0; i < (DisplayRect->y + DisplayRect->h); i++)
+       {
+	surface->pixels[i * surface->pitch32 + 0] =
+	surface->pixels[i * surface->pitch32 + 1] = black;
+        LineWidths[i].x = 0;
+        LineWidths[i].w = 2;
+       }
+      }
+     }
     }
 
-    if(scanline == (VertStart + VS_Adjust))
+    //
+    // Don't mess with the order of evaluation of these scanline == VertXXX && (InVblankwhatever) if statements and the following IRQ/timer vblank stuff
+    // unless you know what you're doing!!! (IE you've run further tests to refine the behavior)
+    //
+    if(scanline == VertEnd && !InVBlank)
     {
-     DisplayHeightCounter = (VertEnd - VertStart);
+     if(sl_zero_reached)
+     {
+      // Gameplay in Descent(NTSC) has vblank at scanline 236
+      if(scanline >= (FirstVisibleLine + VisibleLineCount) || (scanline >= (HardwarePALType ? 260 : 232)))
+      {
+       //printf("Req Exit(vblank case): %u\n", scanline);
+       PSX_RequestMLExit();
+      }
+      else
+      {
+       //printf("VBlank too early, chickening out early exit!\n");
+      }
+     }
+
+     //printf("VBLANK: %u\n", scanline);
+     InVBlank = true;
+
+     DisplayFB_CurYOffset = 0;
+
+     if((DisplayMode & 0x24) == 0x24)
+      field_ram_readout = !field;
+     else
+      field_ram_readout = 0;
+    }
+
+    if(scanline == VertStart && InVBlank)
+    {
+     InVBlank = false;
 
      // Note to self: X-Men Mutant Academy relies on this being set on the proper scanline in 480i mode(otherwise it locks up on startup).
-     if(HeightMode)
-      DisplayFB_CurYOffset = field;
-
-     //printf("%d\n", DisplayFB_CurYOffset);
+     //if(HeightMode)
+     // DisplayFB_CurYOffset = field;
     }
 
-    // Needs to occur even in vblank.
-    DisplayFB_CurLineYReadout = (DisplayFB_YStart + DisplayFB_CurYOffset) & 0x1FF;
+    IRQ_Assert(IRQ_VBLANK, InVBlank);
+    TIMER_SetVBlank(InVBlank);
+    //
+    //
+    //
 
-    if(scanline >= VisibleStartLine && scanline < (VisibleStartLine + (HardwarePALType ? 288 : 240)) && !skip && PALMode == HardwarePALType)
+    // Needs to occur even in vblank.
+    // Not particularly confident about the timing of this in regards to vblank and the upper bit(ODE) of the GPU status port, though the test that
+    // showed an oddity was pathological in that VertEnd < VertStart in it.
+    if((DisplayMode & 0x24) == 0x24)
+     DisplayFB_CurLineYReadout = (DisplayFB_YStart + (DisplayFB_CurYOffset << 1) + (InVBlank ? 0 : field_ram_readout)) & 0x1FF;
+    else
+     DisplayFB_CurLineYReadout = (DisplayFB_YStart + DisplayFB_CurYOffset) & 0x1FF;
+
+    if((bool)(DisplayMode & 0x08) == HardwarePALType && scanline >= FirstVisibleLine && scanline < (FirstVisibleLine + VisibleLineCount) && !skip && espec)
     {
      uint32 *dest;	// = surface->pixels + (scanline - VisibleStartLine) * surface->pitch32;
      int32 dest_line;
      int32 fb_x = DisplayFB_XStart * 2;
      int32 dx_start = HorizStart, dx_end = HorizEnd;
 
-     dest_line = ((scanline - VisibleStartLine) << FrameInterlaced) + field;
+     dest_line = ((scanline - FirstVisibleLine) << espec->InterlaceOn) + espec->InterlaceField;
      dest = surface->pixels + dest_line * surface->pitch32;
 
      if(dx_end < dx_start)
@@ -1196,8 +1347,8 @@ pscpu_timestamp_t PS_GPU::Update(const pscpu_timestamp_t sys_timestamp)
      dx_start = dx_start / DotClockRatios[dmc];
      dx_end = dx_end / DotClockRatios[dmc];
 
-     dx_start -= 528 / DotClockRatios[dmc];
-     dx_end -= 528 / DotClockRatios[dmc];
+     dx_start -= 488 / DotClockRatios[dmc];
+     dx_end -= 488 / DotClockRatios[dmc];
 
      if(dx_start < 0)
      {
@@ -1206,10 +1357,10 @@ pscpu_timestamp_t PS_GPU::Update(const pscpu_timestamp_t sys_timestamp)
       dx_start = 0;
      }
 
-     if(dx_end > dmw)
+     if((uint32)dx_end > dmw)
       dx_end = dmw;
 
-     if(!DisplayHeightCounter || DisplayOff)
+     if(InVBlank || DisplayOff)
       dx_start = dx_end = 0;
 
      // TODO, but there are problems with this, as not all blitter busy cycles(crudely abstracted with DrawTimeAvail) are GPU RAM access cycles.
@@ -1238,19 +1389,23 @@ pscpu_timestamp_t PS_GPU::Update(const pscpu_timestamp_t sys_timestamp)
       else
        ReorderRGB_Var(surface->format.Rshift, surface->format.Gshift, surface->format.Bshift, DisplayMode & 0x10, src, dest, dx_start, dx_end, fb_x);
 
-      for(int32 x = dx_end; x < dmw; x++)
+      for(uint32 x = dx_end; x < dmw; x++)
        dest[x] = black;
      }
+
+     //if(scanline == 64)
+     // printf("%u\n", sys_timestamp - ((uint64)gpu_clocks * 65536) / GPUClockRatio);
+
+     PSX_GPULineHook(sys_timestamp, sys_timestamp - ((uint64)gpu_clocks * 65536) / GPUClockRatio, scanline == 0, dest, &surface->format, dmw, (488 - 146) / DotClockRatios[dmc], (HardwarePALType ? 53203425 : 53693182) / DotClockRatios[dmc]);
+    }
+    else
+    {
+     PSX_GPULineHook(sys_timestamp, sys_timestamp - ((uint64)gpu_clocks * 65536) / GPUClockRatio, scanline == 0, NULL, &surface->format, 0, 0, 0);
     }
 
-    if(DisplayHeightCounter)
+    if(!InVBlank)
     {
-     if(HeightMode)
-      DisplayFB_CurYOffset = (DisplayFB_CurYOffset + 2) & 0x1FF;
-     else
-      DisplayFB_CurYOffset = (DisplayFB_CurYOffset + 1) & 0x1FF;
-
-     DisplayHeightCounter--;
+     DisplayFB_CurYOffset = (DisplayFB_CurYOffset + 1) & 0x1FF;
     }
    }
   }
@@ -1277,6 +1432,18 @@ pscpu_timestamp_t PS_GPU::Update(const pscpu_timestamp_t sys_timestamp)
 
 void PS_GPU::StartFrame(EmulateSpecStruct *espec_arg)
 {
+ sl_zero_reached = false;
+
+ if(!espec_arg)
+ {
+  espec = NULL;
+  surface = NULL;
+  DisplayRect = NULL;
+  LineWidths = NULL;
+  skip = true;
+  return;
+ }
+
  espec = espec_arg;
 
  surface = espec->surface;
@@ -1284,26 +1451,15 @@ void PS_GPU::StartFrame(EmulateSpecStruct *espec_arg)
  LineWidths = espec->LineWidths;
  skip = espec->skip;
 
- DisplayRect->x = 0;
- DisplayRect->y = 0;
- DisplayRect->w = 256;
- DisplayRect->h = 240;
-
- for(int i = 0; i < 240; i++)
- {
-  LineWidths[i].x = 0;
-  LineWidths[i].w = 0;
- }
-
  if(espec->VideoFormatChanged)
  {
   for(int rc = 0; rc < 0x8000; rc++)
   {
    uint32 r, g, b;
 
-   r = ((rc >> 0) & 0x1F) * 255 / 31;
-   g = ((rc >> 5) & 0x1F) * 255 / 31;
-   b = ((rc >> 10) & 0x1F) * 255 / 31;
+   r = ((rc >> 0) & 0x1F) << 3;
+   g = ((rc >> 5) & 0x1F) << 3;
+   b = ((rc >> 10) & 0x1F) << 3;
    OutputLUT[rc] = espec->surface->format.MakeColor(r, g, b, 0);
   }
  }
