@@ -1,3 +1,6 @@
+// WARNING WARNING WARNING:  ONLY use CanRead() method of BlitterFIFO, and NOT CanWrite(), since the FIFO is larger than the actual PS1 GPU FIFO to accommodate
+// our lack of fancy superscalarish command sequencer.
+
 #ifndef __MDFN_PSX_GPU_H
 #define __MDFN_PSX_GPU_H
 
@@ -10,12 +13,20 @@ class PS_GPU;
 
 struct CTEntry
 {
- uint32 len;
+ uint8 len;
+ uint8 fifo_fb_len;
+ bool ss_cmd;
  const char *name;
  void (PS_GPU::*func[8])(const uint32 *cb);
 };
 
-struct tri_vertex;
+struct tri_vertex
+{
+ int32 x, y;
+ int32 u, v;
+ int32 r, g, b;
+};
+
 struct i_group;
 struct i_deltas;
 
@@ -29,10 +40,10 @@ class PS_GPU
 {
  public:
 
- PS_GPU(bool pal_clock_and_tv);
- ~PS_GPU();
+ PS_GPU(bool pal_clock_and_tv, int sls, int sle) MDFN_COLD;
+ ~PS_GPU() MDFN_COLD;
 
- void Power(void);
+ void Power(void) MDFN_COLD;
 
  void ResetTS(void);
 
@@ -42,12 +53,30 @@ class PS_GPU
 
  void Write(const pscpu_timestamp_t timestamp, uint32 A, uint32 V);
 
+ INLINE bool CalcFIFOReadyBit(void)
+ {
+  if(InCmd & (INCMD_PLINE | INCMD_QUAD))
+   return(false);
+
+  if(BlitterFIFO.CanRead() == 0)
+   return(true);
+
+  if(InCmd & (INCMD_FBREAD | INCMD_FBWRITE))
+   return(false);
+
+  if(BlitterFIFO.CanRead() >= Commands[0][BlitterFIFO.ReadUnit(true) >> 24].fifo_fb_len)
+   return(false);
+
+  return(true);
+ }
+
  INLINE bool DMACanWrite(void)
  {
-  return BlitterFIFO.CanWrite(); // && (DMAControl & 2);
+  return CalcFIFOReadyBit();
  }
 
  void WriteDMA(uint32 V);
+ uint32 ReadDMA(void);
 
  uint32 Read(const pscpu_timestamp_t timestamp, uint32 A);
 
@@ -66,21 +95,11 @@ class PS_GPU
   GPURAM[(A >> 10) & 0x1FF][A & 0x3FF] = V;
  }
 
- INLINE void DTAResetKludge(void)
- {
-  DrawTimeAvail = 0;
- }
-
- INLINE int32 DTAGetWaitKludge(void)
- {
-  return std::max<int32>((0 - DrawTimeAvail) >> 1, 0);
- }
-
  private:
 
-
- void ProcessFIFO(bool force = false);
+ void ProcessFIFO(void);
  void WriteCB(uint32 data);
+ uint32 ReadData(void);
  void SoftReset(void);
 
  // Y, X
@@ -194,14 +213,26 @@ class PS_GPU
 
  uint32 DataReadBuffer;
 
- bool ParsingLineOrPolygonCommand;
+ //
+ //
+ //
+ // Powers of 2 for faster multiple equality testing(just for multi-testing; InCmd itself will only contain 0, or a power of 2).
+ enum
+ {
+  INCMD_NONE = 0,
+  INCMD_PLINE = (1 << 0),
+  INCMD_QUAD = (1 << 1),
+  INCMD_FBWRITE = (1 << 2),
+  INCMD_FBREAD = (1 << 3)
+ };
+ uint8 InCmd;
+ uint8 InCmd_CC;
+
+ tri_vertex InQuad_F3Vertices[3];
+ uint32 InQuad_clut;
 
- uint32 InPLine;
  line_point InPLine_PrevPoint;
 
- bool InFBWrite;
- bool InFBRead;
-
  uint32 FBRW_X;
  uint32 FBRW_Y;
  uint32 FBRW_W;
@@ -227,35 +258,18 @@ class PS_GPU
  //
  // Display work vars
  //
-/*
- uint32 DisplayMode_Latch;
-
- bool DisplayOff_Latch;
- uint32 DisplayFB_XStart_Latch;
- uint32 DisplayFB_YStart_Latch;
-
- uint32 HorizStart_Latch;
- uint32 HorizEnd_Latch;
-
- uint32 VertStart_Latch;
- uint32 VertEnd_Latch;
-*/
  uint32 DisplayFB_CurYOffset;
  uint32 DisplayFB_CurLineYReadout;
 
- uint32 DisplayHeightCounter;
+ bool InVBlank;
 
  //
  //
  //
  uint32 LinesPerField;
- uint32 VisibleStartLine;
- bool FrameInterlaced;
- bool PALMode;
- bool HeightMode;
  uint32 scanline;
  bool field;
- bool field_atvs;
+ bool field_ram_readout;
  bool PhaseChange;
 
  uint32 DotClockCounter;
@@ -272,12 +286,18 @@ class PS_GPU
  //
  //
  //
+
+ bool sl_zero_reached;
+ //
+ //
+
  EmulateSpecStruct *espec;
  MDFN_Surface *surface;
  MDFN_Rect *DisplayRect;
  MDFN_Rect *LineWidths;
  bool skip;
  bool HardwarePALType;
+ int LineVisFirst, LineVisLast;
 
  uint32 OutputLUT[32768];
  void ReorderRGB_Var(uint32 out_Rshift, uint32 out_Gshift, uint32 out_Bshift, bool bpp24, const uint16 *src, uint32 *dest, const int32 dx_start, const int32 dx_end, int32 fb_x);
