@@ -159,6 +159,8 @@ bool InputDevice_Gamepad::Clock(bool TxD, int32 &dsr_pulse_delay)
 	//assert(command == 0x42);
 	if(command == 0x42)
 	{
+	 //printf("PAD COmmand 0x42, sl=%u\n", GPU->GetScanlineNum());
+
 	 transmit_buffer[1] = 0xFF ^ buttons[0];
 	 transmit_buffer[2] = 0xFF ^ buttons[1];
          transmit_pos = 0;
@@ -211,6 +213,28 @@ InputDeviceInputInfoStruct Device_Gamepad_IDII[16] =
  { "square", "□ (left)", 8, IDIT_BUTTON_CAN_RAPID, NULL },
 };
 
+InputDeviceInputInfoStruct Device_Dancepad_IDII[16] =
+{
+ { "select", "SELECT", 0, IDIT_BUTTON, NULL },
+ { NULL, "empty", 0, IDIT_BUTTON },
+ { NULL, "empty", 0, IDIT_BUTTON },
+ { "start", "START", 1, IDIT_BUTTON, NULL },
+
+ { "up", "UP ↑", 3, IDIT_BUTTON, 	NULL },
+ { "right", "RIGHT →", 6, IDIT_BUTTON, 	NULL },
+ { "down", "DOWN ↓", 8, IDIT_BUTTON, 	NULL },
+ { "left", "LEFT ←", 5, IDIT_BUTTON, 	NULL },
+
+ { NULL, "empty", 0, IDIT_BUTTON, NULL },
+ { NULL, "empty", 0, IDIT_BUTTON, NULL },
+ { NULL, "empty", 0, IDIT_BUTTON, NULL },
+ { NULL, "empty", 0, IDIT_BUTTON, NULL },
+
+ { "triangle", "△ (lower left)", 7, IDIT_BUTTON, NULL },
+ { "circle", "○ (upper right)", 4, IDIT_BUTTON, NULL },
+ { "cross", "x (upper left)", 2, IDIT_BUTTON, NULL },
+ { "square", "□ (lower right)", 9, IDIT_BUTTON, NULL },
+};
 
 
 }
