@@ -1,3 +1,20 @@
+/* Mednafen - Multi-system Emulator
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
 #include "psx.h"
 
 namespace MDFN_IEN_PSX
@@ -21,6 +38,26 @@ void IRQ_Power(void)
  Recalc();
 }
 
+int IRQ_StateAction(StateMem *sm, int load, int data_only)
+{
+ SFORMAT StateRegs[] =
+ {
+  SFVAR(Asserted),
+  SFVAR(Mask),
+  SFVAR(Status),
+  SFEND
+ };
+ int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, "IRQ");
+
+ if(load)
+ {
+  Recalc();
+ }
+
+ return(ret);
+}
+
+
 void IRQ_Assert(int which, bool status)
 {
  uint32 old_Asserted = Asserted;
