@@ -14,9 +14,11 @@ uint32 MDEC_Read(const pscpu_timestamp_t timestamp, uint32 A);
 
 void MDEC_Power(void);
 
+bool MDEC_DMACanWrite(void);
 bool MDEC_DMACanRead(void);
 void MDEC_Run(int32 clocks);
 
+int MDEC_StateAction(StateMem *sm, int load, int data_only);
 }
 
 #endif
