@@ -16,7 +16,11 @@
  */
 
 /* TODO:
-	Proper IRQ semantics; output continuous level instead of pulse, support program ack, see if there's an IRQ asserted SPU status bit.
+	Note to self: Emulating the SPU at more timing accuracy than sample, and emulating the whole SPU RAM write port FIFO thing and hypothetical periodic FIFO commit to
+	SPU RAM(maybe every 32 CPU cycles, with exceptions?) will likely necessitate a much more timing-accurate CPU core, and emulation of the SPU delay register(or at least the
+	effects of the standard value written to it), to avoid game glitches.  Probably more trouble than it's worth....
+
+	SPU IRQ emulation isn't totally correct, behavior is kind of complex; run more tests on PS1.
 
 	Test reverb upsampler on the real thing.
 
@@ -28,8 +32,6 @@
 
 	See if sample flag & 0x8 does anything weird, like suppressing the program-readable block end flag setting.
 
-	Make sure SPU IRQ works correctly, test to see if it should be triggered on SPU RAM writes and/or reads by the CPU.
-
 	Determine the actual purpose of global register 0x2C(is it REALLY an address multiplier?  And if so, does it affect the reverb offsets too?)
 
 	For ADSR and volume sweep, should the divider be reset to 0 on &0x8000 == true, or should the upper bit be cleared?
@@ -66,6 +68,7 @@
  system, though this will obviously need to change if we ever emulate the SPU with better precision than per-sample(pair).
 */
 
+#define SPUIRQ_DBG(format, ...) { printf("[SPUIRQDBG] " format " -- Voice 22 CA=0x%06x,LA=0x%06x\n", ## __VA_ARGS__, Voices[22].CurAddr, Voices[22].LoopAddr); }
 
 #include "psx.h"
 #include "cdc.h"
@@ -99,8 +102,11 @@ PS_SPU::PS_SPU()
 
 PS_SPU::~PS_SPU()
 {
-
-
+ if(resampler)
+ {
+  speex_resampler_destroy(resampler);
+  resampler = NULL;
+ }
 }
 
 void PS_SPU::Power(void)
@@ -166,13 +172,7 @@ void PS_SPU::Power(void)
 
  BlockEnd = 0;
 
- CDAudioCWA = 0;
-
- memset(CDXA_ResampBuffer, 0, sizeof(CDXA_ResampBuffer));
- CDXA_CurPhase = 0;
-
- VoiceCWP[0] = 0;
- VoiceCWP[1] = 0;
+ CWA = 0;
 
  memset(Regs, 0, sizeof(Regs));
 
@@ -183,6 +183,8 @@ void PS_SPU::Power(void)
  RUSB_WP = 0;
 
  ReverbCur = ReverbWA;
+
+ IRQAsserted = false;
 }
 
 static INLINE void CalcVCDelta(const uint8 zs, uint8 speed, bool log_mode, bool dec_mode, int16 Current, int &increment, int &divinco)
@@ -286,39 +288,6 @@ INLINE void SPU_Sweep::WriteVolume(int16 value)
  Current = value;
 }
 
-// output should be readable at -2 and -1
-void PS_SPU::DecodeADPCM(const uint8 *input, int16 *output, const unsigned shift, const unsigned weight)
-{
- // 5 through 0xF appear to be 0 on the real thing.
- static const int32 Weights[16][2] =
- {
-  // s-1    s-2
-  {   0,    0 },
-  {  60,    0 },
-  { 115,  -52 },
-  {  98,  -55 },
-  { 122,  -60 },
- };
-
- for(int i = 0; i < 28; i++)
- {
-  int32 sample;
-
-  sample = (int16)(input[i] << 8);
-  sample >>= shift;
-
-  sample += ((output[i - 1] * Weights[weight][0]) >> 6) + ((output[i - 2] * Weights[weight][1]) >> 6);
-
-  if(sample < -32768)
-   sample = -32768;
-
-  if(sample > 32767)
-   sample = 32767;
-
-  output[i] = sample;
- }
-}
-
 void PS_SPU::DecodeSamples(SPU_Voice *voice)
 {
  // Handle delayed flags from the previously-decoded block.
@@ -326,12 +295,13 @@ void PS_SPU::DecodeSamples(SPU_Voice *voice)
  {
   BlockEnd |= 1 << (voice - Voices);
 
-  voice->CurAddr = voice->LoopAddr & ~0x7;
-
   if(!(voice->DecodeFlags & 0x2))	// Force enveloping to 0 if not "looping".  TODO: Should we reset the ADSR divider counter too?
   {
-   voice->ADSR.Phase = ADSR_RELEASE;
-   voice->ADSR.EnvLevel = 0;
+   if(!(Noise_Mode & (1 << (voice - Voices))))
+   {
+    voice->ADSR.Phase = ADSR_RELEASE;
+    voice->ADSR.EnvLevel = 0;
+   }
   }
  }
 
@@ -397,8 +367,9 @@ void PS_SPU::DecodeSamples(SPU_Voice *voice)
 
   if(TestIRQAddr >= PrevCurAddr && TestIRQAddr < voice->CurAddr)
   {
-   //puts("SPU IRQ");
-   IRQ_Assert(IRQ_SPU, true);
+   //SPUIRQ_DBG("SPU IRQ(sample decoding) on voice %d", (int)(voice - Voices));
+   IRQAsserted = true;
+   IRQ_Assert(IRQ_SPU, IRQAsserted);
   }
  }
 
@@ -406,7 +377,10 @@ void PS_SPU::DecodeSamples(SPU_Voice *voice)
 
  if(flags & 0x4)
  {
-  //PSX_WARNING("[SPU] Flag loop set for voice %d, 0x%08x", (int)(voice - Voices), PrevCurAddr);
+  //if((int)(voice - Voices) == 22)
+  //{
+  // SPUIRQ_DBG("[SPU] Flag loop set for voice %d, 0x%08x", (int)(voice - Voices), PrevCurAddr);
+  //}
   voice->LoopAddr = PrevCurAddr;
  }
 
@@ -532,16 +506,16 @@ INLINE void PS_SPU::CheckIRQAddr(uint32 addr)
  {
   if(IRQAddr == addr)
   {
-   //puts("SPU IRQ (ALT)");
-   IRQ_Assert(IRQ_SPU, true);
+   //SPUIRQ_DBG("SPU IRQ (ALT): 0x%06x", addr);
+   IRQAsserted = true;
+   IRQ_Assert(IRQ_SPU, IRQAsserted);
   }
  }
 }
 
-INLINE void PS_SPU::WriteSPURAM(uint32 addr, uint16 value, bool test_irq)
+INLINE void PS_SPU::WriteSPURAM(uint32 addr, uint16 value)
 {
- if(test_irq)
-  CheckIRQAddr(addr);
+ CheckIRQAddr(addr);
 
  SPURAM[addr] = value;
 }
@@ -554,57 +528,6 @@ INLINE uint16 PS_SPU::ReadSPURAM(uint32 addr)
 
 #include "spu_reverb.inc"
 
-INLINE bool PS_SPU::GetCDAudio(int32 &l, int32 &r)
-{
- unsigned freq = CDC->GetCDAudioFreq();
-
- if(!freq)
-  return false;
-
- if(freq == 7 || freq == 14)
- {
-  CDC->GetCDAudio(l, r);
-  if(freq == 14)
-   CDC->GetCDAudio(l, r);
- }
- else
- {
-  const int pi = CDXA_CurPhase * 37;
-  int32 out_tmp[2];
-
-  for(unsigned i = 0; i < 2; i++)
-  {
-   out_tmp[i] = ((CDXA_ResampBuffer[i][0] * FIR_Table[pi][0]) +
-	         (CDXA_ResampBuffer[i][1] * FIR_Table[pi][1]) +
-	         (CDXA_ResampBuffer[i][2] * FIR_Table[pi][2]) +   
-	         (CDXA_ResampBuffer[i][3] * FIR_Table[pi][3])) >> 15;
-  }
-
-  l = out_tmp[0];
-  r = out_tmp[1];
-
-  CDXA_CurPhase += freq;
-
-  if(CDXA_CurPhase >= 7)
-  {
-   int32 raw[2];
-   CDXA_CurPhase -= 7;
-
-   CDC->GetCDAudio(raw[0], raw[1]);
-
-   for(unsigned i = 0; i < 2; i++)
-   {
-    CDXA_ResampBuffer[i][0] = CDXA_ResampBuffer[i][1];
-    CDXA_ResampBuffer[i][1] = CDXA_ResampBuffer[i][2];
-    CDXA_ResampBuffer[i][2] = CDXA_ResampBuffer[i][3];
-    CDXA_ResampBuffer[i][3] = raw[i];
-   }
-  }
- }
-
- return true;
-}
-
 int32 PS_SPU::UpdateFromCDC(int32 clocks)
 //pscpu_timestamp_t PS_SPU::Update(const pscpu_timestamp_t timestamp)
 {
@@ -639,6 +562,37 @@ int32 PS_SPU::UpdateFromCDC(int32 clocks)
   int32 output_r = 0;
 
   const uint32 PhaseModCache = FM_Mode & ~ 1;
+/*
+**
+** 0x1F801DAE Notes and Conjecture:
+**   -------------------------------------------------------------------------------------
+**   |   15   14 | 13 | 12 | 11 | 10  | 9  | 8 |  7 |  6  | 5    4    3    2    1    0   |
+**   |      ?    | *13| ?  | ba | *10 | wrr|rdr| df |  is |      c                       |
+**   -------------------------------------------------------------------------------------
+**
+**	c - Appears to be delayed copy of lower 6 bits from 0x1F801DAA.
+**
+**     is - Interrupt asserted out status. (apparently not instantaneous status though...)
+**
+**     df - Related to (c & 0x30) == 0x20 or (c & 0x30) == 0x30, at least.
+**          0 = DMA busy(FIFO not empty when in DMA write mode?)?
+**	    1 = DMA ready?  Something to do with the FIFO?
+**
+**     rdr - Read(DMA read?) Ready?
+**
+**     wrr - Write(DMA write?) Ready?
+**
+**     *10 - Unknown.  Some sort of (FIFO?) busy status?(BIOS tests for this bit in places)
+**
+**     ba - Alternates between 0 and 1, even when SPUControl bit15 is 0; might be related to CD audio and voice 1 and 3 writing to SPU RAM.
+**
+**     *13 - Unknown, was set to 1 when testing with an SPU delay system reg value of 0x200921E1(test result might not be reliable, re-run).
+*/
+  SPUStatus = SPUControl & 0x3F;
+  SPUStatus |= IRQAsserted ? 0x40 : 0x00;
+
+  if(Regs[0xD6] == 0x4)	// TODO: Investigate more(case 0x2C in global regs r/w handler)
+   SPUStatus |= (CWA & 0x100) ? 0x800 : 0x000;
 
   for(int voice_num = 0; voice_num < 24; voice_num++)
   {
@@ -650,12 +604,18 @@ int32 PS_SPU::UpdateFromCDC(int32 clocks)
    //PSX_WARNING("[SPU] Voice %d CurPhase=%08x, pitch=%04x, CurAddr=%08x", voice_num, voice->CurPhase, voice->Pitch, voice->CurAddr);
 
    // Decode new samples if necessary.
+   if(voice->DecodeFlags & 0x1)
+    voice->CurAddr = voice->LoopAddr & ~0x7;
+
    if(voice->CurPhase_SD >= (24 << 12))
    {
     voice->CurPhase_SD -= 28 << 12;
     DecodeSamples(voice);
    }
-
+   else
+   {
+    CheckIRQAddr(voice->CurAddr);
+   }
    //
    //
    //
@@ -681,8 +641,7 @@ int32 PS_SPU::UpdateFromCDC(int32 clocks)
    {
     int index = voice_num >> 1;
 
-    WriteSPURAM(0x400 | (index * 0x200) | VoiceCWP[index], voice_pvs);
-    VoiceCWP[index] = (VoiceCWP[index] + 1) & 0x1FF;
+    WriteSPURAM(0x400 | (index * 0x200) | CWA, voice_pvs);
    }
 
 
@@ -707,21 +666,31 @@ int32 PS_SPU::UpdateFromCDC(int32 clocks)
 
    // Increment stuff
    {
-    int32 phase_inc = voice->Pitch;
+    int32 phase_inc;
 
     if(PhaseModCache & (1 << voice_num))
     {
-     phase_inc = (voice->Pitch * ((voice - 1)->PreLRSample + 0x8000)) >> 15;
+     // This old formula: phase_inc = (voice->Pitch * ((voice - 1)->PreLRSample + 0x8000)) >> 15;
+     // is incorrect, as it does not handle carrier pitches >= 0x8000 properly.
 
-     if(phase_inc > 0x3FFF)
-      phase_inc = 0x3FFF;
+     phase_inc = voice->Pitch + (((int16)voice->Pitch * ((voice - 1)->PreLRSample)) >> 15);
+     if(phase_inc < 0)
+     {
+      PSX_DBG(PSX_DBG_ERROR, "[SPU] phase_inc < 0 (THIS SHOULD NOT HAPPEN)\n");
+      phase_inc = 0;
+     }
     }
+    else
+     phase_inc = voice->Pitch;
+
+    if(phase_inc > 0x3FFF)
+     phase_inc = 0x3FFF;
 
     voice->CurPhase = (voice->CurPhase + phase_inc) & 0x1FFFF;
     voice->CurPhase_SD += phase_inc;
    }
 
-   if(!(SPUControl & 0x8000) || (VoiceOff & (1 << voice_num)))
+   if(VoiceOff & (1 << voice_num))
    {
     if(voice->ADSR.Phase != ADSR_RELEASE)
     {
@@ -729,7 +698,7 @@ int32 PS_SPU::UpdateFromCDC(int32 clocks)
     }
    }
 
-   if((SPUControl & 0x8000) && (VoiceOn & (1 << voice_num)))
+   if(VoiceOn & (1 << voice_num))
    {
     ResetEnvelope(voice);
 
@@ -746,44 +715,56 @@ int32 PS_SPU::UpdateFromCDC(int32 clocks)
     voice->CurPhase_SD = 28 << 12;	 // Trigger initial sample decode
 
     voice->CurAddr = voice->StartAddr & ~0x7;
+   }
 
-    if(voice_num == 1 || voice_num == 3)
-     VoiceCWP[voice_num >> 1] = 0;
+   if(!(SPUControl & 0x8000))
+   {
+    voice->ADSR.Phase = ADSR_RELEASE;
+    voice->ADSR.EnvLevel = 0;
    }
   }
 
   VoiceOff = 0;
   VoiceOn = 0; 
 
-  // Get CD-DA
+  // "Mute" control doesn't seem to affect CD audio(though CD audio reverb wasn't tested...)
+  // TODO: If we add sub-sample timing accuracy, see if it's checked for every channel at different times, or just once.
+  if(!(SPUControl & 0x4000))
   {
-   int32 cda_raw[2] = { 0 };
-
-   if(GetCDAudio(cda_raw[0], cda_raw[1]))
-   {
-    int32 cdav[2];
-
-    for(unsigned i = 0; i < 2; i++)
-     cdav[i] = (cda_raw[i] * CDVol[i]) >> 15;
-
-    if(SPUControl & 0x0001)
-    {
-     accum_l += cdav[0];
-     accum_r += cdav[1];
-
-     if(SPUControl & 0x0004)	// TODO: Test this bit(and see if it is really dependent on bit0)
-     {
-      accum_fv_l += cdav[0];
-      accum_fv_r += cdav[1];
-     }
-    }
-   }
-   WriteSPURAM(CDAudioCWA | 0x000, cda_raw[0]);
-   WriteSPURAM(CDAudioCWA | 0x200, cda_raw[1]);
-
-   CDAudioCWA = (CDAudioCWA + 1) & 0x1FF;
+   accum_l = 0;
+   accum_r = 0;
+   accum_fv_l = 0;
+   accum_fv_r = 0;
   }
 
+  // Get CD-DA
+  {
+   int32 cda_raw[2];
+   int32 cdav[2];
+
+   CDC->GetCDAudio(cda_raw);	// PS_CDC::GetCDAudio() guarantees the variables passed by reference will be set to 0,
+				// and that their range shall be -32768 through 32767.
+
+   WriteSPURAM(CWA | 0x000, cda_raw[0]);
+   WriteSPURAM(CWA | 0x200, cda_raw[1]);
+
+   for(unsigned i = 0; i < 2; i++)
+    cdav[i] = (cda_raw[i] * CDVol[i]) >> 15;
+
+   if(SPUControl & 0x0001)
+   {
+    accum_l += cdav[0];
+    accum_r += cdav[1];
+
+    if(SPUControl & 0x0004)	// TODO: Test this bit(and see if it is really dependent on bit0)
+    {
+     accum_fv_l += cdav[0];
+     accum_fv_r += cdav[1];
+    }
+   }
+  }
+
+  CWA = (CWA + 1) & 0x1FF;
 
   NoiseCounter += NoiseFreqTable[(SPUControl >> 8) & 0x3F];
   if(NoiseCounter >= 0x8000)
@@ -792,24 +773,6 @@ int32 PS_SPU::UpdateFromCDC(int32 clocks)
    LFSR = (LFSR << 1) | (((LFSR >> 15) ^ (LFSR >> 12) ^ (LFSR >> 11) ^ (LFSR >> 10) ^ 1) & 1);
   }
 
-  //accum_l >>= 5;
-  //accum_r >>= 5;
-
-  //accum_l >>= 1;
-  //accum_r >>= 1;
-
-  //accum_fv_l >>= 1;
-  //accum_fv_r >>= 1;
-
-  if(!(SPUControl & 0x4000)) // General output muted? TODO: Determine if it affects CD audio too; and if not, and we add sub-sample timing accuracy,
-			     // if it's checked for every channel at different times, or just once.
-  {
-   accum_l = 0;
-   accum_r = 0;
-   accum_fv_l = 0;
-   accum_fv_r = 0;
-  }
-
   clamp(&accum_l, -32768, 32767);
   clamp(&accum_r, -32768, 32767);
   clamp(&accum_fv_l, -32768, 32767);
@@ -836,14 +799,12 @@ int32 PS_SPU::UpdateFromCDC(int32 clocks)
   clamp(&output_l, -32768, 32767);
   clamp(&output_r, -32768, 32767);
 
-  assert(IntermediateBufferPos < 4096);
-  IntermediateBuffer[IntermediateBufferPos][0] = output_l;
-  IntermediateBuffer[IntermediateBufferPos][1] = output_r;
-  IntermediateBufferPos++;
-
-  //resampler.buffer()[0] = (int16)(rand() & 0xFFFF) >> 1;
-  //resampler.buffer()[1] = (int16)(rand() & 0xFFFF) >> 1;
-  //resampler.write(2);
+  if(IntermediateBufferPos < 4096)	// Overflow might occur in some debugger use cases.
+  {
+   IntermediateBuffer[IntermediateBufferPos][0] = output_l;
+   IntermediateBuffer[IntermediateBufferPos][1] = output_r;
+   IntermediateBufferPos++;
+  }
 
   sample_clocks--;
 
@@ -859,13 +820,15 @@ int32 PS_SPU::UpdateFromCDC(int32 clocks)
 
 void PS_SPU::WriteDMA(uint32 V)
 {
- //PSX_WARNING("[SPU] memWrite: %08x %08x\n", RWAddr, V);
-
- WriteSPURAM(RWAddr, V, false);
+ //SPUIRQ_DBG("DMA Write, RWAddr after=0x%06x", RWAddr);
+ WriteSPURAM(RWAddr, V);
  RWAddr = (RWAddr + 1) & 0x3FFFF;
 
- WriteSPURAM(RWAddr, V >> 16, false);
+ WriteSPURAM(RWAddr, V >> 16);
  RWAddr = (RWAddr + 1) & 0x3FFFF;
+
+
+ CheckIRQAddr(RWAddr);
 }
 
 uint32 PS_SPU::ReadDMA(void)
@@ -878,14 +841,33 @@ uint32 PS_SPU::ReadDMA(void)
  ret |= (uint32)(uint16)ReadSPURAM(RWAddr) << 16;
  RWAddr = (RWAddr + 1) & 0x3FFFF;
 
+ CheckIRQAddr(RWAddr);
+
+ //SPUIRQ_DBG("DMA Read, RWAddr after=0x%06x", RWAddr);
+
  return(ret);
 }
 
 void PS_SPU::Write(pscpu_timestamp_t timestamp, uint32 A, uint16 V)
 {
- //PSX_WARNING("[SPU] Write: %08x %04x", A, V);
+ //if((A & 0x3FF) < 0x180)
+ // PSX_WARNING("[SPU] Write: %08x %04x", A, V);
 
- A &= 0x1FF;
+ A &= 0x3FF;
+
+ if(A >= 0x200)
+ {
+  //printf("Write: %08x %04x\n", A, V);
+  if(A < 0x260)
+  {
+   SPU_Voice *voice = &Voices[(A - 0x200) >> 2];
+   voice->Sweep[(A & 2) >> 1].WriteVolume(V);
+  }
+  else if(A < 0x280)
+   AuxRegs[(A & 0x1F) >> 1] = V;
+
+  return;
+ }
 
  if(A < 0x180)
  {
@@ -898,7 +880,7 @@ void PS_SPU::Write(pscpu_timestamp_t timestamp, uint32 A, uint16 V)
 	     voice->Sweep[(A & 2) >> 1].WriteControl(V);
 	     break;
 
-   case 0x04: voice->Pitch = V & 0x3FFF;
+   case 0x04: voice->Pitch = V;
 	      break;
 
    case 0x06: voice->StartAddr = (V << 2) & 0x3FFFF;
@@ -918,7 +900,10 @@ void PS_SPU::Write(pscpu_timestamp_t timestamp, uint32 A, uint16 V)
 	      break;
 
    case 0x0E: voice->LoopAddr = (V << 2) & 0x3FFFF;
-	      //PSX_WARNING("[SPU] Manual loop address setting for voice %d: %04x", voice - Voices, V);
+	      //if((voice - Voices) == 22)
+	      //{
+	      // SPUIRQ_DBG("Manual loop address setting for voice %d: %04x", (int)(voice - Voices), V);
+	      //}
 	      break;
   }
  }
@@ -992,27 +977,37 @@ void PS_SPU::Write(pscpu_timestamp_t timestamp, uint32 A, uint16 V)
 	      break;
 
    case 0x24: IRQAddr = (V << 2) & 0x3FFFF;
-	      //PSX_WARNING("[SPU] IRQ addr reg 0x2c set: 0x%04x", V);
+	      CheckIRQAddr(RWAddr);
+	      //SPUIRQ_DBG("Set IRQAddr=0x%06x", IRQAddr);
 	      break;
 
-   case 0x26: RWAddr = (V << 2) & 0x3FFFF;
+   case 0x26: RWAddr = (V << 2) & 0x3FFFF;	      
+	      CheckIRQAddr(RWAddr);
+	      //SPUIRQ_DBG("Set RWAddr=0x%06x", RWAddr);
 	      break;
 
-   case 0x28: WriteSPURAM(RWAddr, V, false);
+   case 0x28: WriteSPURAM(RWAddr, V);
 	      RWAddr = (RWAddr + 1) & 0x3FFFF;
+	      CheckIRQAddr(RWAddr);
 	      break;
 
-   case 0x2A: SPUControl = V;
+   case 0x2A: //if((SPUControl & 0x80) && !(V & 0x80))
+	      // printf("\n\n\n\n ************** REVERB PROCESSING DISABLED\n\n\n\n");
+
+	      SPUControl = V;
+	      //SPUIRQ_DBG("Set SPUControl=0x%04x -- IRQA=%06x, RWA=%06x", V, IRQAddr, RWAddr);
 	      //printf("SPU control write: %04x\n", V);
 	      if(!(V & 0x40))
-	       IRQ_Assert(IRQ_SPU, false);
+	      {
+	       IRQAsserted = false;
+	       IRQ_Assert(IRQ_SPU, IRQAsserted);
+	      }
+	      CheckIRQAddr(RWAddr);
 	      break;
 
    case 0x2C: PSX_WARNING("[SPU] Global reg 0x2c set: 0x%04x", V);
 	      break;
 
-   //case 0x2E: // Read-only?
-
    case 0x30: CDVol[0] = V;
 	      break;
 
@@ -1036,10 +1031,27 @@ void PS_SPU::Write(pscpu_timestamp_t timestamp, uint32 A, uint16 V)
 
 uint16 PS_SPU::Read(pscpu_timestamp_t timestamp, uint32 A)
 {
- A &= 0x1FF;
+ A &= 0x3FF;
 
  PSX_DBGINFO("[SPU] Read: %08x", A);
 
+ if(A >= 0x200)
+ {
+  if(A < 0x260)
+  {
+   SPU_Voice *voice = &Voices[(A - 0x200) >> 2];
+
+   //printf("Read: %08x %04x\n", A, voice->Sweep[(A & 2) >> 1].ReadVolume());
+
+   return voice->Sweep[(A & 2) >> 1].ReadVolume();
+  }
+  else if(A < 0x280)
+   return(AuxRegs[(A & 0x1F) >> 1]);
+
+  return(0xFFFF);
+ }
+
+
  if(A < 0x180)
  {
   SPU_Voice *voice = &Voices[A >> 4];
@@ -1057,7 +1069,7 @@ uint16 PS_SPU::Read(pscpu_timestamp_t timestamp, uint32 A)
    case 0x1C: return(BlockEnd);
    case 0x1E: return(BlockEnd >> 16);
 
-   case 0x26: PSX_WARNING("[SPU] RWADDR Read");
+   case 0x26: //PSX_WARNING("[SPU] RWADDR Read");
 	      break;
 
    case 0x28: PSX_WARNING("[SPU] SPURAM Read port(?) Read");
@@ -1066,6 +1078,7 @@ uint16 PS_SPU::Read(pscpu_timestamp_t timestamp, uint32 A)
 	       uint16 ret = ReadSPURAM(RWAddr);
 
 	       RWAddr = (RWAddr + 1) & 0x3FFFF;
+	       CheckIRQAddr(RWAddr);
 
 	       return(ret);
 	      }
@@ -1073,8 +1086,7 @@ uint16 PS_SPU::Read(pscpu_timestamp_t timestamp, uint32 A)
    case 0x2a:
 	return(SPUControl);
 
-   // FIXME: What are these two for?
-   case 0x2e:
+/* FIXME: What is this used for? */
    case 0x3C:
 	//PSX_WARNING("[SPU] Read Unknown: %08x", A);
 	return(0);
@@ -1092,8 +1104,6 @@ void PS_SPU::StartFrame(double rate, uint32 quality)
 {
  if((int)rate != last_rate || quality != last_quality)
  {
-  //double ratio = (double)44100 / (rate ? rate : 44100);
-  //resampler.time_ratio(ratio, 0.9965);
   int err = 0;
 
   if(resampler)
@@ -1128,8 +1138,8 @@ int32 PS_SPU::EndFrame(int16 *SoundBuf)
  }
  else if(resampler)
  {
-  spx_uint32_t in_len;
-  spx_uint32_t out_len;
+  spx_uint32_t in_len; // "Number of input samples in the input buffer. Returns the number of samples processed. This is all per-channel."
+  spx_uint32_t out_len; // "Size of the output buffer. Returns the number of samples written. This is all per-channel."
 
   in_len = IntermediateBufferPos;
   out_len = 524288; //8192;	// FIXME, real size.
@@ -1152,12 +1162,143 @@ int32 PS_SPU::EndFrame(int16 *SoundBuf)
  }
 }
 
+int PS_SPU::StateAction(StateMem *sm, int load, int data_only)
+{
+ SFORMAT StateRegs[] =
+ {
+#define SFSWEEP(r) SFVAR((r).Control),	\
+		   SFVAR((r).Current),	\
+		   SFVAR((r).Divider)
+
+#define SFVOICE(n) SFARRAY32(&Voices[n].DecodeBuffer[0], sizeof(Voices[n].DecodeBuffer) / sizeof(Voices[n].DecodeBuffer[0])),	\
+		   SFVAR(Voices[n].DecodeWritePos),										\
+		   SFVAR(Voices[n].DecodeFlags),										\
+																\
+		   SFSWEEP(Voices[n].Sweep[0]),											\
+		   SFSWEEP(Voices[n].Sweep[1]),											\
+																\
+		   SFVAR(Voices[n].Pitch),											\
+		   SFVAR(Voices[n].CurPhase),											\
+		   SFVAR(Voices[n].CurPhase_SD),										\
+																\
+		   SFVAR(Voices[n].StartAddr),											\
+		   SFVAR(Voices[n].CurAddr),											\
+		   SFVAR(Voices[n].ADSRControl),										\
+		   SFVAR(Voices[n].LoopAddr),											\
+		   SFVAR(Voices[n].PreLRSample),										\
+																\
+		   SFVAR(Voices[n].ADSR.EnvLevel),										\
+		   SFVAR(Voices[n].ADSR.Divider),										\
+		   SFVAR(Voices[n].ADSR.Phase),											\
+																\
+		   SFVAR(Voices[n].ADSR.AttackExp),										\
+		   SFVAR(Voices[n].ADSR.SustainExp),										\
+		   SFVAR(Voices[n].ADSR.SustainDec),										\
+		   SFVAR(Voices[n].ADSR.ReleaseExp),										\
+																\
+		   SFVAR(Voices[n].ADSR.AttackRate),										\
+		   SFVAR(Voices[n].ADSR.DecayRate),										\
+		   SFVAR(Voices[n].ADSR.SustainRate),										\
+		   SFVAR(Voices[n].ADSR.ReleaseRate),										\
+																\
+		   SFVAR(Voices[n].ADSR.SustainLevel)
+
+  SFVOICE(0),
+  SFVOICE(1),
+  SFVOICE(2),
+  SFVOICE(3),
+  SFVOICE(4),
+  SFVOICE(5),
+  SFVOICE(6),
+  SFVOICE(7),
+  SFVOICE(8),
+  SFVOICE(9),
+  SFVOICE(10),
+  SFVOICE(11),
+  SFVOICE(12),
+  SFVOICE(13),
+  SFVOICE(14),
+  SFVOICE(15),
+  SFVOICE(16),
+  SFVOICE(17),
+  SFVOICE(18),
+  SFVOICE(19),
+  SFVOICE(20),
+  SFVOICE(21),
+  SFVOICE(22),
+  SFVOICE(23),
+#undef SFVOICE
+
+  SFVAR(NoiseCounter),
+  SFVAR(LFSR),
+
+  SFVAR(FM_Mode),
+  SFVAR(Noise_Mode),
+  SFVAR(Reverb_Mode),
+
+  SFVAR(ReverbWA),
+
+  SFSWEEP(GlobalSweep[0]),
+  SFSWEEP(GlobalSweep[1]),
+
+  SFARRAY32(ReverbVol, sizeof(ReverbVol) / sizeof(ReverbVol[0])),
+
+  SFARRAY32(CDVol, sizeof(CDVol) / sizeof(CDVol[0])),
+  SFARRAY32(ExternVol, sizeof(ExternVol) / sizeof(ExternVol[0])),
+ 
+  SFVAR(IRQAddr),
+
+  SFVAR(RWAddr),
+
+  SFVAR(SPUControl),
+
+  SFVAR(VoiceOn),
+  SFVAR(VoiceOff),
+
+  SFVAR(BlockEnd),
+
+  SFVAR(CWA),
+
+  SFARRAY16(Regs, sizeof(Regs) / sizeof(Regs[0])),
+  SFARRAY16(AuxRegs, sizeof(AuxRegs) / sizeof(AuxRegs[0])),
+
+  SFARRAY16(&RDSB[0][0], sizeof(RDSB) / sizeof(RDSB[0][0])),
+  SFVAR(RDSB_WP),
+
+  SFARRAY16(&RUSB[0][0], sizeof(RUSB) / sizeof(RUSB[0][0])),
+  SFVAR(RUSB_WP),
+
+  SFVAR(ReverbCur),
+  SFVAR(IRQAsserted),
+
+  SFVAR(clock_divider),
+
+  SFARRAY16(SPURAM, 524288 / sizeof(uint16)),
+  SFEND
+ };
+#undef SFSWEEP
+ int ret = 1;
+
+ ret &= MDFNSS_StateAction(sm, load, data_only, StateRegs, "SPU");
+
+ if(load)
+ {
+
+ }
+
+ return(ret);
+}
 
 uint16 PS_SPU::PeekSPURAM(uint32 address)
 {
  return(SPURAM[address & 0x3FFFF]);
 }
 
+void PS_SPU::PokeSPURAM(uint32 address, uint16 value)
+{
+ SPURAM[address & 0x3FFFF] = value;
+}
+
 uint32 PS_SPU::GetRegister(unsigned int which, char *special, const uint32 special_len)
 {
  uint32 ret = 0xDEADBEEF;
