@@ -34,8 +34,11 @@ struct SPU_ADSR
  int32 SustainLevel;	// (Sl + 1) << 11
 };
 
+class PS_SPU;
 class SPU_Sweep
 {
+ friend class PS_SPU;		// For save states - FIXME(remove in future?)
+
  public:
  SPU_Sweep() { }
  ~SPU_Sweep() { }
@@ -88,6 +91,8 @@ class PS_SPU
  PS_SPU();
  ~PS_SPU();
 
+ int StateAction(StateMem *sm, int load, int data_only);
+
  void Power(void);
  void Write(pscpu_timestamp_t timestamp, uint32 A, uint16 V);
  uint16 Read(pscpu_timestamp_t timestamp, uint32 A);
@@ -101,13 +106,10 @@ class PS_SPU
  int32 UpdateFromCDC(int32 clocks);
  //pscpu_timestamp_t Update(pscpu_timestamp_t timestamp);
 
-
- static void DecodeADPCM(const uint8 *input, int16 *output, const unsigned shift, const unsigned weight);
-
  private:
 
  void CheckIRQAddr(uint32 addr);
- void WriteSPURAM(uint32 addr, uint16 value, bool test_irq = true);
+ void WriteSPURAM(uint32 addr, uint16 value);
  uint16 ReadSPURAM(uint32 addr);
 
  void DecodeSamples(SPU_Voice *voice);
@@ -144,19 +146,13 @@ class PS_SPU
  uint32 RWAddr;
 
  uint16 SPUControl;
- uint16 AddressMult;	// Just a guess!
 
  uint32 VoiceOn;
  uint32 VoiceOff;
 
  uint32 BlockEnd;
 
- uint32 CDAudioCWA;
-
- int32 CDXA_ResampBuffer[2][4];
- int32 CDXA_CurPhase;
-
- uint32 VoiceCWP[2];
+ uint32 CWA;
 
  union
  {
@@ -164,7 +160,16 @@ class PS_SPU
   struct
   {
    uint16 VoiceRegs[0xC0];
-   uint16 GlobalRegs[0x20];
+   union
+   {
+    uint16 GlobalRegs[0x20];
+    struct
+    {
+     uint16 _Global0[0x17];
+     uint16 SPUStatus;
+     uint16 _Global1[0x08];
+    };
+   };
    union
    {
     int16 ReverbRegs[0x20];
@@ -208,6 +213,8 @@ class PS_SPU
   };
  };
 
+ uint16 AuxRegs[0x10];
+
  int16 RDSB[2][128];	// [40]
  int32 RDSB_WP;
 
@@ -220,17 +227,18 @@ class PS_SPU
  int32 RD_RVB(int16 raw_offs);
  void WR_RVB(int16 raw_offs, int32 sample, int32 extra_offs = 0);
 
- int last_rate;
- uint32 last_quality;
+ bool IRQAsserted;
 
  //pscpu_timestamp_t lastts;
  int32 clock_divider;
 
  uint16 SPURAM[524288 / sizeof(uint16)];
 
+ int last_rate;
+ uint32 last_quality;
  SpeexResamplerState *resampler;
 
- // Buffers 44.1KHz samples, should have enough for one video frame(~735 frames NTSC, ~882 PAL) plus jitter plus enough for the resampler leftovers.
+ // Buffers 44.1KHz samples, should have enough for two(worst-case scenario) video frames(2* ~735 frames NTSC, 2* ~882 PAL) plus jitter plus enough for the resampler leftovers.
  // We'll just go with 4096 because powers of 2 are AWESOME and such.
  uint32 IntermediateBufferPos;
  int16 IntermediateBuffer[4096][2];
@@ -318,6 +326,7 @@ class PS_SPU
  void SetRegister(unsigned int which, uint32 value);
 
  uint16 PeekSPURAM(uint32 address);
+ void PokeSPURAM(uint32 address, uint16 value);
 };
 
 
