@@ -22,9 +22,9 @@
  Notes(some of it may be incomplete or wrong in subtle ways)
 
  Control bits:
-	Lower 3 bits of mode, for timer1:
-		0x1 = don't count during vblank
-		0x3 = vblank going inactive triggers timer reset
+	Lower 3 bits of mode, for timer1(when mode is | 0x100):
+		0x1 = don't count while in vblank(except that the first count while in vblank does go through)
+		0x3 = vblank going inactive triggers timer reset, then some interesting behavior where counting again is delayed...
 		0x5 = vblank going inactive triggers timer reset, and only count within vblank.
 		0x7 = Wait until vblank goes active then inactive, then start counting?
 	For timer2:
@@ -67,17 +67,21 @@
  Dec. 26, 2011 Note
 	Due to problems I've had with my GPU timing test program, timer2 appears to be unreliable(clocks are skipped?) when target mode is enabled and the full
 	33MHz clock is used(rather than 33MHz / 8).  TODO: Investigate further and confirm(or not).
+
+ Jan. 15, 2013 Note:
+	Counters using GPU clock sources(hretrace,dot clock) reportedly will with a low probability return wrong count values on an actual PS1, so keep this in mind
+	when writing test programs(IE keep reading the count value until two consecutive reads return the same value).
 */
 
 /*
  FIXME: Clock appropriately(and update events) when using SetRegister() via the debugger.
+
+ TODO: If we ever return randomish values to "simulate" open bus, remember to change the return type and such of the TIMER_Read() function to full 32-bit too.
 */
 
 namespace MDFN_IEN_PSX
 {
 
-extern PS_GPU *GPU;
-
 struct Timer
 {
  uint32 Mode;
@@ -87,8 +91,11 @@ struct Timer
  int32 Div8Counter;
 
  bool IRQDone;
+ int32 DoZeCounting;
 };
 
+static bool vblank;
+static bool hretrace;
 static Timer Timers[3];
 static pscpu_timestamp_t lastts;
 
@@ -116,13 +123,16 @@ static int32 CalcNextEvent(int32 next_event)
   count_delta = target - Timers[i].Counter;
   if(count_delta <= 0)
   {
-   fprintf(stderr, "timer %d count_delta <= 0!!! %d %d\n", i, target, Timers[i].Counter);
+   PSX_DBG(PSX_DBG_ERROR, "timer %d count_delta <= 0!!! %d %d\n", i, target, Timers[i].Counter);
    continue;
   }
 
   {
    int32 tmp_clocks;
 
+   if(Timers[i].DoZeCounting <= 0)
+    continue;
+
    if((i == 0x2) && (Timers[i].Mode & 0x1))
     continue;
 
@@ -150,6 +160,9 @@ static void ClockTimer(int i, uint32 clocks)
  int32 target = 0x10000;
  bool zero_tm = false;
 
+ if(Timers[i].DoZeCounting <= 0)
+  clocks = 0;
+
  if(i == 0x2)
  {
   uint32 d8_clocks;
@@ -182,7 +195,7 @@ static void ClockTimer(int i, uint32 clocks)
   if(Timers[i].Mode & 0x10)
   {
    if((Timers[i].Counter - target) > 3)
-    printf("Timer %d IRQ trigger error: %d\n", i, Timers[i].Counter - target);
+    PSX_WARNING("Timer %d IRQ trigger error: %d", i, Timers[i].Counter - target);
   }
 
 #endif
@@ -218,6 +231,52 @@ static void ClockTimer(int i, uint32 clocks)
 
 }
 
+void TIMER_SetVBlank(bool status)
+{
+ switch(Timers[1].Mode & 0x7)
+ {
+  case 0x1:
+	Timers[1].DoZeCounting = !status;
+	break;
+
+  case 0x3:
+	if(vblank && !status)
+	 Timers[1].Counter = 0;
+	break;
+
+  case 0x5:
+	Timers[1].DoZeCounting = status;
+	if(vblank && !status)
+	 Timers[1].Counter = 0;
+	break;
+
+  case 0x7:
+	if(Timers[1].DoZeCounting == -1)
+	{
+	 if(!vblank && status)
+	  Timers[1].DoZeCounting = 0;
+	}
+	else if(Timers[1].DoZeCounting == 0)
+	{
+	 if(vblank && !status)
+	  Timers[1].DoZeCounting = 1;
+	}
+	break;
+ }
+ vblank = status;
+}
+
+void TIMER_SetHRetrace(bool status)
+{
+ if(hretrace && !status)
+ {
+  if((Timers[0].Mode & 0x7) == 0x3)
+   Timers[0].Counter = 0;
+ }
+
+ hretrace = status;
+}
+
 void TIMER_AddDotClocks(uint32 count)
 {
  if(Timers[0].Mode & 0x100)
@@ -249,15 +308,42 @@ pscpu_timestamp_t TIMER_Update(const pscpu_timestamp_t timestamp)
  return(timestamp + CalcNextEvent(1024));
 }
 
+static void CalcCountingStart(unsigned which)
+{
+ Timers[which].DoZeCounting = true;
+
+ switch(which)
+ {
+  case 1:
+	switch(Timers[which].Mode & 0x07)
+	{
+	 case 0x1:
+		Timers[which].DoZeCounting = !vblank;
+		break;
+
+	 case 0x5:
+		Timers[which].DoZeCounting = vblank;
+		break;
+
+	 case 0x7:
+		Timers[which].DoZeCounting = -1;
+		break;
+	}
+	break;
+
+
+ }
+}
+
 void TIMER_Write(const pscpu_timestamp_t timestamp, uint32 A, uint16 V)
 {
  TIMER_Update(timestamp);
 
  int which = (A >> 4) & 0x3;
 
- assert(!(A & 3));
+ V <<= (A & 3) * 8;
 
- PSX_DBGINFO("[TIMER] Write: %08x %04x", A, V);
+ PSX_DBGINFO("[TIMER] Write: %08x %04x\n", A, V);
 
  if(which >= 3)
   return;
@@ -296,12 +382,14 @@ void TIMER_Write(const pscpu_timestamp_t timestamp, uint32 A, uint16 V)
 	    }
 	    Timers[which].Counter = 0;
 #endif
+	    CalcCountingStart(which);	// Call after setting .Mode
 	    break;
 
   case 0x8: Timers[which].Target = V & 0xFFFF;
 	    break;
 
-  default: assert(0);
+  case 0xC: // Open bus
+	    break;
  }
 
  // TIMER_Update(timestamp);
@@ -314,10 +402,12 @@ uint16 TIMER_Read(const pscpu_timestamp_t timestamp, uint32 A)
  uint16 ret = 0;
  int which = (A >> 4) & 0x3;
 
- assert(!(A & 3));
-
  if(which >= 3)
-  assert(0);
+ {
+  PSX_WARNING("[TIMER] Open Bus Read: 0x%08x", A);
+
+  return(ret >> ((A & 3) * 8));
+ }
 
  TIMER_Update(timestamp);
 
@@ -333,10 +423,11 @@ uint16 TIMER_Read(const pscpu_timestamp_t timestamp, uint32 A)
   case 0x8: ret = Timers[which].Target;
 	    break;
 
-  default: assert(0);
+  case 0xC: PSX_WARNING("[TIMER] Open Bus Read: 0x%08x", A);
+	    break;
  }
 
- return(ret);
+ return(ret >> ((A & 3) * 8));
 }
 
 
@@ -350,9 +441,37 @@ void TIMER_Power(void)
 {
  lastts = 0;
 
+ hretrace = false;
+ vblank = false;
  memset(Timers, 0, sizeof(Timers));
 }
 
+int TIMER_StateAction(StateMem *sm, int load, int data_only)
+{
+ SFORMAT StateRegs[] =
+ {
+#define SFTIMER(n)	SFVARN(Timers[n].Mode, #n "Mode"),			\
+			SFVARN(Timers[n].Counter, #n "Counter"),		\
+			SFVARN(Timers[n].Target, #n "Target"),			\
+			SFVARN(Timers[n].Div8Counter, #n "Div8Counter"),	\
+			SFVARN(Timers[n].IRQDone, #n "IRQDone")
+  SFTIMER(0),
+  SFTIMER(1),
+  SFTIMER(2),
+#undef SFTIMER
+  SFVAR(lastts),
+  SFEND
+ };
+ int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, "TIMER");
+
+ if(load)
+ {
+
+ }
+
+ return(ret);
+}
+
 uint32 TIMER_GetRegister(unsigned int which, char *special, const uint32 special_len)
 {
  int tw = (which >> 4) & 0x3;
