@@ -200,7 +200,7 @@ void QTRecord::atom_end(void)
  qtfile.seek(cur_offset, SEEK_SET);
 }
 
-QTRecord::QTRecord(const char *path, const VideoSpec &spec) : qtfile(path, FileWrapper::MODE_WRITE_SAFE)
+QTRecord::QTRecord(const char *path, const VideoSpec &spec) : qtfile(path, FileWrapper::MODE_WRITE_SAFE), resampler(NULL)
 {
  Finished = false;
 
@@ -209,6 +209,44 @@ QTRecord::QTRecord(const char *path, const VideoSpec &spec) : qtfile(path, FileW
  SoundRate = spec.SoundRate;
  SoundChan = spec.SoundChan;
 
+ // If we change the rate threshold for resampling here, remember to change the std::min in the informational printf in
+ // mednafen.cpp
+ if(SoundRate > 64000)
+ {
+  int err = 0;
+
+  ResampInRate = spec.SoundRate;
+  SoundRate = 64000;
+  if(!(resampler = speex_resampler_init(spec.SoundChan, spec.SoundRate, SoundRate, 5, &err)))
+  {
+   throw MDFN_Error(0, _("Error initializing audio resampler."));
+  }
+  //printf("%f ms\n", 1000.0 * speex_resampler_get_input_latency(resampler) / spec.SoundRate);
+ }
+ else
+  resampler = NULL;
+
+ ResampInBufferFramesInCount = 0;
+ ResampInBuffer.resize(0);
+ ResampOutBuffer.resize(0);
+
+ //ResampInBuffer.resize((uint32)((100 * spec.SoundRate + 999) / 1000) * SoundChan);
+ //ResampOutBuffer.resize((uint32)((100 * SoundRate + 999) / 1000) * SoundChan);
+
+ //printf("%u %u\n", ResampInBuffer.size(), ReampOutBuffer.size());
+
+ TimeIndex = 0;
+ if(SoundRate && SoundChan)
+ {
+  TimeScale = SoundRate;
+ }
+ else
+ {
+  TimeScale = 10000;
+  MC = spec.MasterClock;
+  MCAccum = 0;
+ }
+
  QTVideoWidth = spec.VideoWidth;
  QTVideoHeight = spec.VideoHeight;
 
@@ -248,12 +286,18 @@ QTRecord::QTRecord(const char *path, const VideoSpec &spec) : qtfile(path, FileW
 
 
 void QTRecord::WriteFrame(const MDFN_Surface *surface, const MDFN_Rect &DisplayRect, const MDFN_Rect *LineWidths,
-			  const int16 *SoundBuf, const int32 SoundBufSize)
+			  const int16 *SoundBuf, const int32 SoundBufSize, const int64 MasterCycles)
 {
  QTChunk qts;
 
  memset(&qts, 0, sizeof(qts));
 
+ if(DisplayRect.h <= 0)
+ {
+  fprintf(stderr, "[BUG] qtrecord.cpp: DisplayRect.h <= 0\n");
+  return;
+ }
+
 
  qts.video_foffset = qtfile.tell();
 
@@ -304,6 +348,17 @@ void QTRecord::WriteFrame(const MDFN_Surface *surface, const MDFN_Rect &DisplayR
     dest_line++;
    }
 
+#if 0
+   while(dest_x < ((QTVideoWidth - (xscale_factor * width)) / 2))
+   {
+    dest_line[dest_x * 3 + 0] = 0;
+    dest_line[dest_x * 3 + 1] = 0;
+    dest_line[dest_x * 3 + 2] = 0;
+
+    dest_x++;
+   }
+#endif
+
    for(int x = x_start; x < x_start + width; x++)
    {
     for(int sub_x = 0; sub_x < xscale_factor; sub_x++)
@@ -412,21 +467,93 @@ void QTRecord::WriteFrame(const MDFN_Surface *surface, const MDFN_Rect &DisplayR
  qts.audio_foffset = qtfile.tell();
 
  // Write audio here
+ //
+ //
+ int32 SoundBufROSize;
+
+ if(resampler)
  {
-  int16 abuf[SoundBufSize * SoundChan];
+  spx_uint32_t in_len;
+  spx_uint32_t out_len;
+
+  if((ResampInBuffer.size() - (ResampInBufferFramesInCount * SoundChan)) < SoundBufSize * SoundChan)
+   ResampInBuffer.resize((ResampInBufferFramesInCount + SoundBufSize) * SoundChan);
 
   for(int i = 0; i < SoundBufSize * SoundChan; i++)
-   MDFN_en16msb((uint8 *)&abuf[i], SoundBuf[i]);
+   ResampInBuffer[ResampInBufferFramesInCount * SoundChan + i] = SoundBuf[i];
 
-  qtfile.write(abuf, sizeof(abuf));
+  ResampInBufferFramesInCount += SoundBufSize;
+
+  {
+   // *2 for padding for potential precision issues related to the resampling ratio.
+   size_t needed_out_size = (uint64_t)ResampInBuffer.size() / SoundChan * 2 * SoundRate / ResampInRate * SoundChan;
+
+   //printf("Zoom: %u %u\n", (unsigned)ResampInBuffer.size(), (unsigned)needed_out_size);
+
+   if(ResampOutBuffer.size() < needed_out_size)
+    ResampOutBuffer.resize(needed_out_size);
+
+   // Very crude, but works because we're only downsampling.  TODO make it more precise?
+   //if(ResampOutBuffer.size() < ResampInBuffer.size())
+   // ResampOutBuffer.resize(ResampInBuffer.size());
+  }
+
+  in_len = ResampInBufferFramesInCount;
+  out_len = ResampOutBuffer.size() / SoundChan;
+
+  speex_resampler_process_interleaved_int(resampler, (const spx_int16_t *)&ResampInBuffer[0], &in_len, (spx_int16_t *)&ResampOutBuffer[0], &out_len);
+
+  if((ResampInBufferFramesInCount - in_len) > 0)	// Shouldn't happen, unless there's not enough room in the output buffer.
+  {
+   printf("%u\n", ResampInBufferFramesInCount - in_len);
+   memmove(&ResampInBuffer[0], &ResampInBuffer[in_len * SoundChan], (ResampInBufferFramesInCount - in_len) * sizeof(int16) * SoundChan);
+  }
+
+  ResampInBufferFramesInCount -= in_len;
+  SoundBufROSize = out_len;
+
+  for(int i = 0; i < SoundBufROSize * SoundChan; i++)
+   MDFN_en16msb((uint8 *)&ResampOutBuffer[i], ResampOutBuffer[i]);
+ }
+ else
+ {
+  SoundBufROSize = SoundBufSize;
+
+  if(ResampOutBuffer.size() < SoundBufSize * SoundChan)
+   ResampOutBuffer.resize(SoundBufSize * SoundChan);
+
+  for(int i = 0; i < SoundBufROSize * SoundChan; i++)
+   MDFN_en16msb((uint8 *)&ResampOutBuffer[i], SoundBuf[i]);
  }
 
+ qtfile.write(&ResampOutBuffer[0], sizeof(int16) * SoundBufROSize * SoundChan);
+ //
+ //
+ //
  qts.audio_byte_size = qtfile.tell() - qts.audio_foffset;
 
+ SoundFramesWritten += SoundBufROSize;
+
+ if(SoundRate && SoundChan)
+ {
+  qts.time_length = qts.audio_byte_size / SoundChan / sizeof(int16);
+  TimeIndex += SoundBufROSize;
+ }
+ else
+ {
+  uint64 tnt;
+ 
+  MCAccum += (uint64)MasterCycles * TimeScale;
+  tnt = MCAccum / (MC >> 32);
+  MCAccum %= (MC >> 32);
+
+  //printf("%u\n", tnt);
+
+  qts.time_length = tnt;
+  TimeIndex += tnt;
+ }
 
  QTChunks.push_back(qts);
-
- SoundFramesWritten += SoundBufSize;
 }
 
 void QTRecord::Write_ftyp(void) // Leaf
@@ -454,9 +581,9 @@ void QTRecord::Write_mvhd(void)	// Leaf
  w32(0);		// Version/flags
  w32(CreationTS);	// Created Mac date
  w32(ModificationTS);	// Modified Mac date
- w32(SoundRate);	// Time scale
+ w32(TimeScale);	// Time scale
 
- w32(SoundFramesWritten); // Duration
+ w32(TimeIndex); 	// Duration
  w32(65536 * 1);	// Preferred rate
  w16(256 * 1);		// Preferred volume
 
@@ -479,11 +606,15 @@ void QTRecord::Write_mvhd(void)	// Leaf
  w32(0);	// Poster time
 
  w32(0);	// Selection time.
- w32(SoundFramesWritten);	// Selection duration.
+ w32(TimeIndex);	// Selection duration.
 
  w32(0);	// Current time
 
- w32(3);	// Next track id
+ // Next track id
+ if(SoundRate && SoundChan)
+  w32(3);
+ else
+  w32(2);
 
  atom_end();
 }
@@ -504,7 +635,7 @@ void QTRecord::Write_tkhd(void)	// Leaf
 
  w32(0);	// Reserved
 
- w32(SoundFramesWritten);	// Duration 
+ w32(TimeIndex); // Duration 
 
  w64(0);	// Reserved
 
@@ -635,7 +766,7 @@ void QTRecord::Write_stts(void)	// Leaf
   for(uint32 i = 0; i < QTChunks.size(); i++)
   {
    w32(1);
-   w32(QTChunks[i].audio_byte_size / SoundChan / sizeof(int16));
+   w32(QTChunks[i].time_length);
   }
  }
 
@@ -774,8 +905,8 @@ void QTRecord::Write_mdhd(void)	// Leaf
  w32(0);		  // Version/flags
  w32(CreationTS);	  // Creation date
  w32(ModificationTS);	  // Modification date
- w32(SoundRate);	  // Time scale
- w32(SoundFramesWritten); // Duration
+ w32(TimeScale);	  // Time scale
+ w32(TimeIndex);	  // Duration
 
  w16(0);		// Language
  w16(0);		// Quality
@@ -889,7 +1020,7 @@ void QTRecord::Write_edts(void)
   atom_begin("elst");
    w32(0); // version/flags
    w32(1); // Number of edits
-   w32(SoundFramesWritten);	// Duration
+   w32(TimeIndex);	// Duration
    w32(0);			// start time
    w32(65536 * 1);		// Rate
   atom_end();
@@ -922,7 +1053,7 @@ void QTRecord::Write_udta(void)
 
   atom_begin("@swr");
 
-   qtfile.put_string("Mednafen "MEDNAFEN_VERSION" -- qtrecord.cpp compiled " __DATE__ " " __TIME__);
+   qtfile.put_string("Mednafen " MEDNAFEN_VERSION " -- qtrecord.cpp compiled " __DATE__ " " __TIME__);
 
   atom_end();
 
@@ -938,8 +1069,11 @@ void QTRecord::Write_moov(void)
  OnAudioTrack = false;
  Write_trak();
 
- OnAudioTrack = true;
- Write_trak();
+ if(SoundRate && SoundChan)
+ {
+  OnAudioTrack = true;
+  Write_trak();
+ }
 
  Write_udta();
 
@@ -970,4 +1104,10 @@ QTRecord::~QTRecord(void)
  {
   MDFND_PrintError(e.what());
  }
+
+ if(resampler)
+ {
+  speex_resampler_destroy(resampler);
+  resampler = NULL;
+ }
 }
