@@ -318,6 +318,7 @@ static InputDeviceInfoStruct GGInputDeviceInfo[] =
   "gamepad",
   "Gamepad",
   NULL,
+  NULL,
   sizeof(GGGamepadIDII) / sizeof(InputDeviceInputInfoStruct),
   GGGamepadIDII,
  }
@@ -329,6 +330,7 @@ static InputDeviceInfoStruct SMSInputDeviceInfo[] =
   "gamepad",
   "Gamepad",
   NULL,
+  NULL,
   sizeof(SMSGamepadIDII) / sizeof(InputDeviceInputInfoStruct),
   SMSGamepadIDII,
  }
@@ -336,13 +338,13 @@ static InputDeviceInfoStruct SMSInputDeviceInfo[] =
 
 static const InputPortInfoStruct GGPortInfo[] =
 {
- { 0, "builtin", "Built-In", sizeof(GGInputDeviceInfo) / sizeof(InputDeviceInfoStruct), GGInputDeviceInfo, "gamepad" },
+ { "builtin", "Built-In", sizeof(GGInputDeviceInfo) / sizeof(InputDeviceInfoStruct), GGInputDeviceInfo, "gamepad" },
 };
 
 static const InputPortInfoStruct SMSPortInfo[] =
 {
- { 0, "port1", "Port 1", sizeof(SMSInputDeviceInfo) / sizeof(InputDeviceInfoStruct), SMSInputDeviceInfo, "gamepad" },
- { 0, "port2", "Port 2", sizeof(SMSInputDeviceInfo) / sizeof(InputDeviceInfoStruct), SMSInputDeviceInfo, "gamepad" }
+ { "port1", "Port 1", sizeof(SMSInputDeviceInfo) / sizeof(InputDeviceInfoStruct), SMSInputDeviceInfo, "gamepad" },
+ { "port2", "Port 2", sizeof(SMSInputDeviceInfo) / sizeof(InputDeviceInfoStruct), SMSInputDeviceInfo, "gamepad" }
 };
 
 static InputInfoStruct GGInputInfo =
@@ -373,11 +375,6 @@ static void DoSimpleCommand(int cmd)
  }
 }
 
-static bool ToggleLayer(int which)
-{
- return(TRUE);
-}
-
 static MDFNSetting_EnumList Territory_List[] =
 {
  { "domestic", TERRITORY_DOMESTIC, gettext_noop("Domestic(Japanese)") },
@@ -423,11 +420,15 @@ MDFNGI EmulatedSMS =
  NULL,
  NULL,
  CloseGame,
- ToggleLayer,
- "Hi\0Ho\0Mo\0", //"Background Scroll\0Foreground Scroll\0Sprites\0",
+ NULL, //ToggleLayer,
+ NULL, //"Hi\0Ho\0Mo\0", //"Background Scroll\0Foreground Scroll\0Sprites\0",
  NULL,
  NULL,
  NULL,
+ NULL,
+ NULL,
+ NULL,
+ false,
  StateAction,
  Emulate,
  SetInput,
@@ -463,11 +464,15 @@ MDFNGI EmulatedGG =
  NULL,
  NULL,
  CloseGame,
- ToggleLayer,
- "Hi\0Ho\0Mo\0", //"Background Scroll\0Foreground Scroll\0Sprites\0",
+ NULL, //ToggleLayer,
+ NULL, //"Hi\0Ho\0Mo\0", //"Background Scroll\0Foreground Scroll\0Sprites\0",
  NULL,
  NULL,
  NULL,
+ NULL,
+ NULL,
+ NULL,
+ false,
  StateAction,
  Emulate,
  SetInput,
