@@ -1,3 +1,5 @@
+// DO NOT REMOVE/DISABLE THESE MATH AND COMPILER SANITY TESTS.  THEY EXIST FOR A REASON.
+
 /* Mednafen - Multi-system Emulator
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,6 +29,7 @@
 
 #undef NDEBUG
 #include <assert.h>
+#include <math.h>
 
 #define FATALME	 { printf("Math test failed: %s:%d\n", __FILE__, __LINE__); fprintf(stderr, "Math test failed: %s:%d\n", __FILE__, __LINE__); return(0); }
 
@@ -39,7 +42,7 @@ typedef struct
  int32 mostnegresult;
 } MathTestEntry;
 
-#define ADD_MTE(_bits) { _bits, ((uint32)1 << _bits) - 1, (uint32)1 << (_bits - 1), 0 - ((uint32)1 << (_bits - 1)) }
+#define ADD_MTE(_bits) { _bits, ((uint32)1 << _bits) - 1, (uint32)1 << (_bits - 1), (int32)(0 - ((uint32)1 << (_bits - 1))) }
 
 MathTestEntry math_test_vals[] =
 {
@@ -359,6 +362,20 @@ static void DoAlignmentChecks(void)
  assert(((uint8 *)&g_aligned2[1] - (uint8 *)&g_aligned2[0]) == 4);
 }
 
+#include "masmem.h"
+
+static void QuickEndianRBOTest(void)
+{
+ uint32 test[2] = { 0xDEADBEEF, 0xCAFEBABE };
+ uint32 test2 = { 0xD00FD00F };
+
+ assert(LoadU32_RBO(&test[0]) == 0xEFBEADDE);
+ StoreU32_RBO(&test[1], 0x12341235);
+ assert(test[1] == 0x35123412);
+ assert(LoadU32_RBO(&test[1]) == 0x12341235);
+ assert(LoadU32_RBO(&test2) == 0x0FD00FD0);
+}
+
 // don't make this static, and don't make it local scope.  Whole-program optimization might defeat the purpose of this, though...
 unsigned int mdfn_shifty_test[4] =
 {
@@ -368,6 +385,41 @@ unsigned int mdfn_shifty_test[4] =
 
 #include "general.h"
 
+// Don't make static.
+double mdfn_fptest0_sub(double x, double n) MDFN_COLD NO_INLINE;
+double mdfn_fptest0_sub(double x, double n)
+{
+ double u = x / (n * n);
+
+ return(u);
+}
+
+static void fptest0(void)
+{
+ assert(mdfn_fptest0_sub(36, 2) == 9);
+}
+
+volatile double mdfn_fptest1_v;
+static void fptest1(void)
+{
+ mdfn_fptest1_v = 1.0;
+
+ for(int i = 0; i < 128; i++)
+  mdfn_fptest1_v *= 2;
+
+ assert(mdfn_fptest1_v == 340282366920938463463374607431768211456.0);
+}
+
+static void RunFPTests(void)
+{
+ fptest0();
+ fptest1();
+}
+
+const char* MDFN_tests_stringA = "AB\0C";
+const char* MDFN_tests_stringB = "AB\0CD";
+const char* MDFN_tests_stringC = "AB\0X";
+
 bool MDFN_RunMathTests(void)
 {
  MathTestEntry *itoo = math_test_vals;
@@ -375,6 +427,9 @@ bool MDFN_RunMathTests(void)
  if(!DoSizeofTests())
   return(0);
 
+ assert(MDFN_tests_stringA != MDFN_tests_stringB && MDFN_tests_stringA[3] == 'C' && MDFN_tests_stringB[4] == 'D');
+ assert(MDFN_tests_stringA != MDFN_tests_stringC && MDFN_tests_stringB != MDFN_tests_stringC && MDFN_tests_stringC[3] == 'X');
+
  // Make sure the "char" type is signed(pass -fsigned-char to gcc).  New code in Mednafen shouldn't be written with the
  // assumption that "char" is signed, but there likely is at least some code that does.
  {
@@ -608,6 +663,10 @@ bool MDFN_RunMathTests(void)
  assert(uilog2(4095) == 11);
  assert(uilog2(0xFFFFFFFF) == 31);
 
+ QuickEndianRBOTest();
+
+ RunFPTests();
+
 #if 0
 // Not really a math test.
  const char *test_paths[] = { "/meow", "/meow/cow", "\\meow", "\\meow\\cow", "\\\\meow", "\\\\meow\\cow",
