@@ -24,12 +24,11 @@
 #endif
 #include "input.h"
 #include "../general.h"
+#include "../string/trim.h"
 #include "../md5.h"
 #include "../mempatcher.h"
 #include <iconv.h>
 
-//#define VB_SUPPORT_BIN_EXT		// Even with this enabled, any *.bin file loaded must be in the internal database to be recognized as a VB game.
-
 namespace MDFN_IEN_VB
 {
 
@@ -233,15 +232,15 @@ uint16 MDFN_FASTCALL MemRead16(v810_timestamp_t &timestamp, uint32 A)
 
   case 4: break;
 
-  case 5: ret = le16toh(*(uint16 *)&WRAM[A & 0xFFFF]);
+  case 5: ret = LoadU16_LE((uint16 *)&WRAM[A & 0xFFFF]);
 	  break;
 
   case 6: if(GPRAM)
-           ret = le16toh(*(uint16 *)&GPRAM[A & GPRAM_Mask]);
+           ret = LoadU16_LE((uint16 *)&GPRAM[A & GPRAM_Mask]);
 	  else printf("GPRAM(Unmapped) Read: %08x\n", A);
 	  break;
 
-  case 7: ret = le16toh(*(uint16 *)&GPROM[A & GPROM_Mask]);
+  case 7: ret = LoadU16_LE((uint16 *)&GPROM[A & GPROM_Mask]);
 	  break;
  }
  return(ret);
@@ -303,11 +302,11 @@ void MDFN_FASTCALL MemWrite16(v810_timestamp_t &timestamp, uint32 A, uint16 V)
 
   case 4: break;
 
-  case 5: *(uint16 *)&WRAM[A & 0xFFFF] = htole16(V);
+  case 5: StoreU16_LE((uint16 *)&WRAM[A & 0xFFFF], V);
           break;
 
   case 6: if(GPRAM)
-           *(uint16 *)&GPRAM[A & GPRAM_Mask] = htole16(V);
+           StoreU16_LE((uint16 *)&GPRAM[A & GPRAM_Mask], V);
           break;
 
   case 7: // ROM, no writing allowed!
@@ -390,7 +389,8 @@ static int32 MDFN_FASTCALL EventHandler(const v810_timestamp_t timestamp)
  return(CalcNextTS());
 }
 
-static void ForceEventUpdates(const v810_timestamp_t timestamp)
+// Called externally from debug.cpp in some cases.
+void ForceEventUpdates(const v810_timestamp_t timestamp)
 {
  next_vip_ts = VIP_Update(timestamp);
  next_timer_ts = TIMER_Update(timestamp);
@@ -490,6 +490,7 @@ static void ReadHeader(MDFNFILE *fp, VB_HeaderInfo *hi)
 
   *out_ptr = 0;
 
+  MDFN_RemoveControlChars(hi->game_title);
   MDFN_trim(hi->game_title);
  }
  else
@@ -500,64 +501,11 @@ static void ReadHeader(MDFNFILE *fp, VB_HeaderInfo *hi)
  hi->version = fp->data[0xFFFFFDFF & (fp->size - 1)];
 }
 
-#ifdef VB_SUPPORT_BIN_EXT
-static bool FindGame(MDFNFILE *fp)
-{
- static const char *GameNames[] =
- {
-  "POLYGOBLOCK",	// 3D Tetris
-  "BOUND HIGH",
-  "GALACTIC PINBALL",
-  "GOLF",
-  "ｲﾝｽﾏｳｽ ﾉ ﾔｶﾀ",
-  "JACK BROS",
-  "MARIO CLASH",
-  "Mario's Tennis",
-  "NESTER'S FUNKY BOWLI",
-  "Panic Bomber",
-  "REDALARM",
-  "VB SDｶﾞﾝﾀﾞﾑ",
-  "SPACE INVADERS V.C.",
-  "SPACE SQUASH",
-  "TELEROBOXER",
-  "T&EVIRTUALGOLF",
-  "ﾄﾋﾞﾀﾞｾ!ﾊﾟﾆﾎﾞﾝ",
-  "VERTICAL FORCE",
-  "VIRTUAL BOWLING",
-  "VB ﾜﾘｵﾗﾝﾄﾞ",
-  "VIRTUAL FISHING JPN",
-  "ﾊﾞ-ﾁｬﾙLAB",
-  "VIRTUAL LEAGUE BB",
-  "ﾊﾞｰﾁｬﾙﾌﾟﾛﾔｷｭｳ'95",
-  "V･TETRIS",
-  "WATERWORLD",
-  NULL
- };
- VB_HeaderInfo hinfo;
-
- ReadHeader(fp, &hinfo);
-
- const char **gni = GameNames;
- while(*gni)
- {
-  if(!strcmp(hinfo.game_title, *gni))
-   return(true);
-  gni++;
- }
- return(false);
-}
-#endif
-
 static bool TestMagic(const char *name, MDFNFILE *fp)
 {
  if(!strcasecmp(fp->ext, "vb") || !strcasecmp(fp->ext, "vboy"))
   return(true);
 
- #ifdef VB_SUPPORT_BIN_EXT
- if(!strcasecmp(fp->ext, "bin") && FindGame(fp))
-  return(true);
- #endif
-
  return(false);
 }
 
@@ -865,7 +813,7 @@ static void Emulate(EmulateSpecStruct *espec)
 
  RebaseTS(v810_timestamp);
 
- VB_V810->ResetTS();
+ VB_V810->ResetTS(0);
 
 
 #if 0
@@ -892,7 +840,7 @@ static DebuggerInfoStruct DBGInfo =
  32,
  32,
  0x00000000,
- ~0,
+ ~0U,
 
  VBDBG_MemPeek,
  VBDBG_Disassemble,
@@ -902,7 +850,7 @@ static DebuggerInfoStruct DBGInfo =
  VBDBG_FlushBreakPoints,
  VBDBG_AddBreakPoint,
  VBDBG_SetCPUCallback,
- VBDBG_SetBPCallback,
+ VBDBG_EnableBranchTrace,
  VBDBG_GetBranchTrace,
  NULL, 	//KING_SetGraphicsDecode,
  VBDBG_SetLogFunc,
@@ -912,7 +860,9 @@ static DebuggerInfoStruct DBGInfo =
 
 static int StateAction(StateMem *sm, int load, int data_only)
 {
+ const v810_timestamp_t timestamp = VB_V810->v810_timestamp;
  int ret = 1;
+
  SFORMAT StateRegs[] =
  {
   SFARRAY(WRAM, 65536),
@@ -920,13 +870,6 @@ static int StateAction(StateMem *sm, int load, int data_only)
   SFVAR(WCR),
   SFVAR(IRQ_Asserted),
   SFVAR(VSU_CycleFix),
-
-
-  // TODO: Remove these(and recalc on state load)
-  SFVAR(next_vip_ts), 
-  SFVAR(next_timer_ts),
-  SFVAR(next_input_ts),
-
   SFEND
  };
 
@@ -941,14 +884,15 @@ static int StateAction(StateMem *sm, int load, int data_only)
 
  if(load)
  {
-
+  // Needed to recalculate next_*_ts since we don't bother storing their deltas in save states.
+  ForceEventUpdates(timestamp);
  }
  return(ret);
 }
 
-static bool ToggleLayer(int which)
+static void SetLayerEnableMask(uint64 mask)
 {
- return(1);
+
 }
 
 static void DoSimpleCommand(int cmd)
@@ -969,7 +913,7 @@ static const MDFNSetting_EnumList V810Mode_List[] =
 
 static const MDFNSetting_EnumList VB3DMode_List[] =
 {
- { "anaglyph", VB3DMODE_ANAGLYPH, gettext_noop("Anaglyph"), gettext_noop("Used in conjuction with classic dual-lens-color glasses.") },
+ { "anaglyph", VB3DMODE_ANAGLYPH, gettext_noop("Anaglyph"), gettext_noop("Used in conjunction with classic dual-lens-color glasses.") },
  { "cscope",  VB3DMODE_CSCOPE, gettext_noop("CyberScope"), gettext_noop("Intended for use with the CyberScope 3D device.") },
  { "sidebyside", VB3DMODE_SIDEBYSIDE, gettext_noop("Side-by-Side"), gettext_noop("The left-eye image is displayed on the left, and the right-eye image is displayed on the right.") },
 // { "overunder", VB3DMODE_OVERUNDER },
@@ -1047,6 +991,7 @@ static InputDeviceInfoStruct InputDeviceInfo[] =
   "gamepad",
   "Gamepad",
   NULL,
+  NULL,
   sizeof(IDII) / sizeof(InputDeviceInputInfoStruct),
   IDII,
  }
@@ -1054,7 +999,7 @@ static InputDeviceInfoStruct InputDeviceInfo[] =
 
 static const InputPortInfoStruct PortInfo[] =
 {
- { 0, "builtin", "Built-In", sizeof(InputDeviceInfo) / sizeof(InputDeviceInfoStruct), InputDeviceInfo, "gamepad" }
+ { "builtin", "Built-In", sizeof(InputDeviceInfo) / sizeof(InputDeviceInfoStruct), InputDeviceInfo, "gamepad" }
 };
 
 static InputInfoStruct InputInfo =
@@ -1068,9 +1013,6 @@ static const FileExtensionSpecStruct KnownExtensions[] =
 {
  { ".vb", gettext_noop("Nintendo Virtual Boy") },
  { ".vboy", gettext_noop("Nintendo Virtual Boy") },
- #ifdef VB_SUPPORT_BIN_EXT
- { ".bin", gettext_noop("Nintendo Virtual Boy (Deprecated)") },
- #endif
  { NULL, NULL }
 };
 
@@ -1091,11 +1033,15 @@ MDFNGI EmulatedVB =
  NULL,
  NULL,
  CloseGame,
- ToggleLayer,
+ SetLayerEnableMask,
  NULL,		// Layer names, null-delimited
  NULL,
  NULL,
  NULL,
+ NULL,
+ NULL,
+ NULL,
+ false,
  StateAction,
  Emulate,
  VBINPUT_SetInput,
