@@ -21,42 +21,49 @@
 
 typedef struct
 {
-        uint32 glyph_width;
-        uint32 glyph_height;
-        int extension;
+        uint8 glyph_width;
+        uint8 glyph_height;
+	int8 extension;
+        uint8 entry_bsize;
+	const uint8 *base_ptr;
 } FontDescriptor_t;
 
 static FontDescriptor_t FontDescriptors[_MDFN_FONT_COUNT] =
 {
  #ifdef WANT_INTERNAL_CJK
- { 9, 18, MDFN_FONT_18x18 },
- { 5, 7, -1 },
- { 4, 5, -1 },
- { 6, 13, MDFN_FONT_12x13 },
- { 12, 13, -1 },
- { 18, 18, -1 },
+ { 9, 18,	MDFN_FONT_18x18,	sizeof(FontData9x18[0]),	&FontData9x18[0].data[0] },
+ { 5, 7, 	-1,			sizeof(FontData5x7[0]),		&FontData5x7[0].data[0] },
+ { 4, 5, 	-1,			sizeof(FontData4x5[0]),		&FontData4x5[0].data[0] },
+ { 6, 13,	MDFN_FONT_12x13,	sizeof(FontData6x13[0]),	&FontData6x13[0].data[0] },
+ { 12, 13, 	-1,			sizeof(FontData12x13[0]),	&FontData12x13[0].data[0] },
+ { 18, 18, 	-1,			sizeof(FontData18x18[0]),	&FontData18x18[0].data[0] },
  #else
- { 9, 18, -1 },
- { 5, 7, -1 },
- { 4, 5, -1 },
- { 6, 13, -1 },
+ { 9, 18,	-1,			sizeof(FontData9x18[0]),	&FontData9x18[0].data[0] },
+ { 5, 7, 	-1,			sizeof(FontData5x7[0]),		&FontData5x7[0].data[0] },
+ { 4, 5,	-1,			sizeof(FontData4x5[0]),		&FontData4x5[0].data[0] },
+ { 6, 13, 	-1,			sizeof(FontData6x13[0]),	&FontData6x13[0].data[0] },
  #endif
 };
 
-static const uint8 *FontDataCache[_MDFN_FONT_COUNT][65536];
+static uint16 FontDataIndexCache[_MDFN_FONT_COUNT][65536];
+
+uint32 GetFontHeight(unsigned which_font)
+{
+ return(FontDescriptors[which_font].glyph_height);
+}
 
 void MDFN_InitFontData(void)
 {
  unsigned int x;
  unsigned int inx;
 
- memset(FontDataCache, 0, sizeof(FontDataCache));
+ memset(FontDataIndexCache, 0xFF, sizeof(FontDataIndexCache));
 
  for(inx=x=0;x<65536;x++)
  {
   if(inx < (FontData4x5_Size / sizeof(font4x5)) && FontData4x5[inx].glyph_num == x)
   {
-   FontDataCache[MDFN_FONT_4x5][x] = FontData4x5[inx].data;
+   FontDataIndexCache[MDFN_FONT_4x5][x] = inx;
    inx++;
   }
  }
@@ -65,7 +72,7 @@ void MDFN_InitFontData(void)
  {
   if(inx < (FontData5x7_Size / sizeof(font5x7)) && FontData5x7[inx].glyph_num == x)
   {
-   FontDataCache[MDFN_FONT_5x7][x] = FontData5x7[inx].data;
+   FontDataIndexCache[MDFN_FONT_5x7][x] = inx;
    inx++;
   }
  }
@@ -74,7 +81,7 @@ void MDFN_InitFontData(void)
  {
   if(inx < (FontData6x13_Size / sizeof(font6x13)) && FontData6x13[inx].glyph_num == x)
   {
-   FontDataCache[MDFN_FONT_6x13_12x13][x] = FontData6x13[inx].data;
+   FontDataIndexCache[MDFN_FONT_6x13_12x13][x] = inx;
    inx++;
   }
  }
@@ -83,7 +90,7 @@ void MDFN_InitFontData(void)
  {
   if(inx < (FontData9x18_Size / sizeof(font9x18)) && FontData9x18[inx].glyph_num == x)
   {
-   FontDataCache[MDFN_FONT_9x18_18x18][x] = FontData9x18[inx].data;
+   FontDataIndexCache[MDFN_FONT_9x18_18x18][x] = inx;
    inx++;
   }
  }
@@ -93,7 +100,7 @@ void MDFN_InitFontData(void)
  {
   if(inx < (FontData12x13_Size / sizeof(font12x13)) && FontData12x13[inx].glyph_num == x)
   {
-   FontDataCache[MDFN_FONT_12x13][x] =  FontData12x13[inx].data;
+   FontDataIndexCache[MDFN_FONT_12x13][x] = inx;
    inx++;
   }
  }
@@ -102,7 +109,7 @@ void MDFN_InitFontData(void)
  {
   if(inx < (FontData18x18_Size / sizeof(font18x18)) && FontData18x18[inx].glyph_num == x)
   {
-   FontDataCache[MDFN_FONT_18x18][x] =  FontData18x18[inx].data;
+   FontDataIndexCache[MDFN_FONT_18x18][x] = inx;
    inx++;
   }
  }
@@ -130,9 +137,9 @@ static void DrawTextSub(const UTF32 *utf32_buf, uint32 &slen, const uint8 **glyp
 
   while(!GlyphFound)
   {
-   if(FontDataCache[recurse_which_font][thisglyph])
+   if(FontDataIndexCache[recurse_which_font][thisglyph] != 0xFFFF)
    {
-    glyph_ptrs[x] = FontDataCache[recurse_which_font][thisglyph];
+    glyph_ptrs[x] = FontDescriptors[recurse_which_font].base_ptr + (FontDescriptors[recurse_which_font].entry_bsize * FontDataIndexCache[recurse_which_font][thisglyph]);
     glyph_width[x] = FontDescriptors[recurse_which_font].glyph_width;
     GlyphFound = TRUE;
    }
@@ -144,7 +151,7 @@ static void DrawTextSub(const UTF32 *utf32_buf, uint32 &slen, const uint8 **glyp
 
   if(!GlyphFound)
   {
-   glyph_ptrs[x] = FontDataCache[which_font][(unsigned char)'?'];
+   glyph_ptrs[x] = FontDescriptors[which_font].base_ptr + (FontDescriptors[which_font].entry_bsize * FontDataIndexCache[which_font][(unsigned char)'?']);
    glyph_width[x] = FontDescriptors[which_font].glyph_width;
   }
 
@@ -256,7 +263,6 @@ static uint32 DoRealDraw(uint32 *dest, uint32 pitch, uint32 width_limit, uint32
  return(pixwidth);
 }
 
-// FIXME/TODO: maximum text pixel width should be signed, not unsigned.
 uint32 DrawTextTrans(uint32 *dest, int pitch, uint32 width, const UTF8 *msg, uint32 fgcolor, int centered, uint32 which_font)
 {
  uint32 slen;
@@ -294,7 +300,7 @@ uint32 DrawTextTrans(uint32 *dest, int pitch, uint32 width, const UTF32 *msg, ui
 
 uint32 DrawTextTransShadow(uint32 *dest, int pitch, uint32 width, const UTF8 *textmsg, uint32 fgcolor, uint32 shadcolor, int centered, uint32 which_font)
 {
- DrawTextTrans(dest + 1 + (pitch >> 2), pitch, width, textmsg, shadcolor, centered, which_font);
+ DrawTextTrans(dest + 1 + (pitch >> 2), pitch, width ? (width - 1) : 0, textmsg, shadcolor, centered, which_font);
  return(DrawTextTrans(dest, pitch, width, textmsg, fgcolor, centered, which_font));
 }
 
@@ -302,16 +308,19 @@ uint32 DrawTextTransShadow(uint32 *dest, int pitch, uint32 width, const std::str
 {
  const char *tmp = textmsg.c_str();
 
- DrawTextTrans(dest + 1 + (pitch >> 2), pitch, width, (const UTF8 *)tmp, shadcolor, centered, which_font);
+ DrawTextTrans(dest + 1 + (pitch >> 2), pitch, width ? (width - 1) : 0, (const UTF8 *)tmp, shadcolor, centered, which_font);
  return(DrawTextTrans(dest, pitch, width, (const UTF8 *)tmp, fgcolor, centered, which_font));
 }
 
 #if 0
-uint32 DrawText(MDFN_Surface *surface, const MDFN_Rect &rect, const std::string &textmsg, uint32 color,
+uint32 DrawText(MDFN_Surface *surface, const MDFN_Rect &rect, const char *textmsg, uint32 color,
 		bool centered, uint32 which_font)
 {
  MDFN_Rect tr = rect;
 
+ if(tr.w < 0 || tr.h < 0)
+  return;
+
  if((tr.x + tr.w) > surface->w)
   tr.w = surface->w - tr.x;
 
