@@ -464,6 +464,7 @@ static InputDeviceInfoStruct InputDeviceInfo[] =
   "gamepad",
   "Gamepad",
   NULL,
+  NULL,
   sizeof(IDII) / sizeof(InputDeviceInputInfoStruct),
   IDII,
  }
@@ -471,7 +472,7 @@ static InputDeviceInfoStruct InputDeviceInfo[] =
 
 static const InputPortInfoStruct PortInfo[] =
 {
- { 0, "builtin", "Built-In", sizeof(InputDeviceInfo) / sizeof(InputDeviceInfoStruct), InputDeviceInfo, "gamepad" }
+ { "builtin", "Built-In", sizeof(InputDeviceInfo) / sizeof(InputDeviceInfoStruct), InputDeviceInfo, "gamepad" }
 };
 
 static InputInfoStruct InputInfo =
@@ -490,7 +491,7 @@ static DebuggerInfoStruct DBGInfo =
  16,
  20,
  0x0000,
- ~0,
+ ~0U,
 
  WSwanDBG_MemPeek,
  WSwanDBG_Disassemble,
@@ -500,7 +501,7 @@ static DebuggerInfoStruct DBGInfo =
  WSwanDBG_FlushBreakPoints,
  WSwanDBG_AddBreakPoint,
  WSwanDBG_SetCPUCallback,
- WSwanDBG_SetBPCallback,
+ WSwanDBG_EnableBranchTrace,
  WSwanDBG_GetBranchTrace,
  WSwan_GfxSetGraphicsDecode,
 };
@@ -535,11 +536,15 @@ MDFNGI EmulatedWSwan =
  NULL,
  NULL,
  CloseGame,
- WSwan_GfxToggleLayer,
+ WSwan_SetLayerEnableMask,
  "Background\0Foreground\0Sprites\0",
  NULL,
  NULL,
  NULL,
+ NULL,
+ NULL,
+ NULL,
+ false,
  StateAction,
  Emulate,
  SetInput,
